/**
 * Created by 王友生 on 2018-04-13.
 */
import "./index.less"

import React, { PureComponent } from "react"
import { getItemActualValue, getItemSourceByKey, checkFieldCondition, downloadFromServer } from "../../utils"
import { itemLayout, sysTheme, fileServerUrl } from "../../common/config"
import KeyValuePaired from "../KeyValuePaired"
import PropTypes from "prop-types"
import { Row, Col, Table, Icon, Button, Input, Popover, Modal as AntModal, Popconfirm, Dropdown, Menu, message } from "antd"
import { selectedModeConst, buttonType, formChildSelectedRowKeysConst, detailDisplayTypeConst } from "../../common/const"
import classnames from "classnames"
import { connect } from "react-redux"
import CheckableTags from "../CheckableTags"
import UploadThumb from "../UploadThumb"
import PicturesWall from "../PicturesWall"
import Division from "../Division"
import Stage from "./stage"
import ResizeableTitle from "../../components/ResizeableTitle"
import EmployeeSelectorLabel from "../EmployeeSelector/EmployeeSelectorLabel"
import DraggableModal from "../DraggableModal"
import OrganizationModal from "../OrganizationModal"
import _ from "lodash"
import { download } from "../../service/api"
import BDMap from "../BDMap"
import { FormattedMessage } from "react-intl"
import QRCode from "qrcode.react"
import { isImage, isVideo, isMP3 } from "./ext/fileUtil"
import { request } from "../../utils/request"
import { convertListToTreeSource, numFormat } from "../../utils"

const { Search } = Input

const formTypeLayout = {
  textArea: {
    xs: 24,
    md: 24,
    lg: 24,
    xl: 24
  }
}
@connect((state, route) => {
  const functionCode = route && route.funCode
  const id = route && route.source && route.source.id
  const cache = functionCode && state.get("list").get(functionCode)
  const formC = functionCode && state.get("form").get(functionCode)
  const mainTableName = formC && formC.get("mainTableName")
  const reference = state.get("reference")
  const dictionary = state.get("dictionary")
  const system = state.get("system")
  const config = cache && cache.get("config")
  const configF = formC && formC.get("config")
  const searchConfig = config && config.get("searchConfig")
  const fromConfig = configF && configF.get("formConfig");

  return {
    reference,
    searchConfig,
    dictionary,
    // cache,
    mainTableName,
    functionCode,
    listData: cache && cache.get("listData"),
    id,
    tabs: system && system.get("tabs"),
    showMore: system && system.get("showMore"),
    fromConfig
  }
})
export default class AutoGenerationDetail extends PureComponent {
  static defaultProps = {
    config: [],
    source: {},
    border: false
  }

  static propTypes = {
    config: PropTypes.array,
    source: PropTypes.object
  }

  state = {
    update: false,
    fullscreen: -1,
    previewModalVisible: false,
    dropDownVisible: false,
    pagination: undefined,
    hl: false
  }

  preventSort = false

  getItemActualValue(item, label) {
    const { dispatch, reference, dictionary } = this.props
    // let itemSource = item.source || item.sourceKey && getItemSourceFromCache(item.sourceKey, dictionary, dispatch);
    // if (itemSource && itemSource.length > 0) {
    //   const filterResult = itemSource.filter(({ value }) => value === label);
    //   label = getValueFromSource(itemSource, label);
    // }
    // label = getReferenceByItem(reference, item, label, dispatch).value;

    // let newLabel = getItemActualValue(item, label, dispatch, reference, dictionary);
    // return newLabel === '-' ? label : newLabel;
    return getItemActualValue(item, label, dispatch, reference, dictionary)
  }

  components = {
    header: {
      cell: ResizeableTitle
    }
  }

  handleResize =
    index =>
      (e, { size }) => {
        const { columns } = this.props
        let totalWidth = this.state.totalWidth || this.props.totalWidth
        totalWidth = totalWidth - (columns[index].width || 0) + size.width
        columns[index].width = size.width
        this.setState({ update: !this.state.update, totalWidth })
      }
  getReferenceConditions(item, getItemValue, source, childIndex) {
    if (item.referenceScenario) {
      return item.referenceScenario
    }

    if (!item.referenceConditions) {
      return {}
    }
    let scenario = {}
    let tableData
    if (childIndex >= 0) {
      tableData = source.getIn([item.tableName, childIndex])
    }
    item.referenceConditions &&
      item.referenceConditions.map(condition => {
        const formField = condition["formField"]
        if (formField && formField.length > 1 && condition.referenceField) {
          if (tableData) {
            if (item.tableName === formField[0]) {
              scenario[condition.referenceField] = tableData.get(formField[1]) || ""
            } else {
              let value =
                (getItemValue &&
                  getItemValue(
                    {
                      tableName: formField[0],
                      dataIndex: formField[1]
                    },
                    source
                  )) ||
                ""
              scenario[condition.referenceField] = value
            }
          } else {
            let value =
              (getItemValue &&
                getItemValue(
                  {
                    tableName: formField[0],
                    dataIndex: formField[1]
                  },
                  source
                )) ||
              ""
            if (!value) {
              value = source.get(formField[1])
            }
            scenario[condition.referenceField] = value
          }
        }
        return true
      })
    return scenario
  }
  renderTableButton(wrapper, filterTableRecord = "") {
    const { onTableChange, getItemValue, source, deleteTableRecords, size, lang } = this.props
    let buttonsComponent = null
    //循环按钮
    if (wrapper.buttons && wrapper.buttons.length > 0) {
      buttonsComponent = (
        <Button.Group size="small" style={{ marginLeft: "5px" }}>
          {wrapper.buttons.map((tableButton, index) => {
            if (tableButton.detailShow == "1") {
              this.setState({ rowSelectionShow: true })
              return (
                <Button
                  size="small"
                  onClick={e => {
                    if (tableButton.btnType === buttonType.customPopModal.value) {
                      //自定义弹窗
                      this.props.dispatch({
                        type: "system/visibleCustomPopEdit",
                        payload: {
                          key: tableButton.btnUrl,
                          record: { edit: "-1" }
                        }
                      })
                    }
                    this.props.onButtonClick(e, tableButton)
                  }}
                >
                  {tableButton.btnName}
                </Button>
              )
            }
          })}
        </Button.Group>
      )
    }
    //无新增
    if (wrapper.hideActions === true || wrapper.hideActions === "1") {
      return (
        <div style={{ display: "flex" }}>
          {/* {lang === 'en' ? wrapper.titleEN : wrapper.title}{wrapper.error} */}
          <div>{buttonsComponent}</div>
        </div>
      )
    }
    //全部删除
    let mutableDeleteComponent = null
    if ((wrapper.mutableDelete || wrapper.mutableDelete === "1") && deleteTableRecords) {
      mutableDeleteComponent = (
        <Popconfirm
          onClick={e => {
            e.preventDefault()
            e.stopPropagation()
          }}
          title="确认要删除全部数据？"
          onConfirm={e => {
            e.preventDefault()
            e.stopPropagation()
            deleteTableRecords(wrapper.tableName)
          }}
          onCancel={e => {
            e.preventDefault()
            e.stopPropagation()
          }}
        >
          <Button size={size || "small"} type="danger" style={{ marginLeft: "5px" }} icon="delete"></Button>
        </Popconfirm>
      )
    }
    //referenceKey弹出选择
    if (wrapper.referenceKey || wrapper.customReferenceKey) {
      const clickProps = {
        onClick: e => {
          wrapper.error = false
          e.preventDefault()
          e.stopPropagation()
          if (wrapper.customReferenceKey) {
            this.props.dispatch({
              type: "system/showCustomPop",
              payload: {
                name: wrapper.customReferenceKey
              }
            })
          } else {
            let extraSearchScenario = this.getReferenceConditions(wrapper, getItemValue, source)
            this.props.dispatch({
              type: "reference/visibleReference",
              payload: {
                id: wrapper.referenceKey,
                item: wrapper,
                selectedMode: selectedModeConst.multiple,
                extraSearchScenario,
                filterTableRecord,
                objectHash: this.state.objectHash
              }
            })
          }
        }
      }
      // 隐藏新增空白行
      if (wrapper.hideEmpty === true || wrapper.hideEmpty === "1") {
        return (
          <div style={{ display: "flex" }}>
            {/*添加*/}
            {/* <Button
             size={size || "small"} icon="plus" type='primary' style={{ marginRight: '10px' }}
             {...clickProps}>
           </Button>
           {lang === 'en' ? wrapper.titleEN : wrapper.title}{wrapper.error} */}
            <div>
              {buttonsComponent}
              {mutableDeleteComponent}
            </div>
          </div>
        )
      }
      // 新增空白行
      const menu = (
        <Button.Group size={size || "small"}>
          <Button type="primary" icon="plus" {...clickProps}>
            新增
          </Button>
          <Button
            type="primary"
            onClick={e => {
              e.preventDefault()
              e.stopPropagation()
              onTableChange && onTableChange(wrapper, [wrapper.defaultObj], false, wrapper.newTriggerRef)
            }}
          >
            空白行
          </Button>
        </Button.Group>
      )
      // const menus = <AntMenu>
      //   <AntMenu.Item>
      //     <a href="" {...clickProps}>选参照</a>
      //   </AntMenu.Item>
      //   <AntMenu.Item>
      //     <a href="" onClick={(e) => {
      //       e.preventDefault();
      //       e.stopPropagation();
      //       onTableChange && onTableChange(wrapper, [wrapper.defaultObj], false, wrapper.newTriggerRef);
      //     }}>空白行</a>
      //   </AntMenu.Item>
      // </AntMenu>
      return (
        <span style={{ display: "flex" }}>
          {/* {menu} */}
          {/* <Dropdown overlay={menu}>
           <Button style={{ marginLeft: '10px' }} size={size || "small"} icon="plus" type='primary'>添加</Button>
         </Dropdown> */}
          {/* {mutableDeleteComponent} */}
          {buttonsComponent}
        </span>
      )
    }
    //常规
    return (
      <div style={{ display: "flex" }}>
        {/*添加*/}
        {/* <Button
         size={size || "small"} icon="plus" type='primary' style={{ marginRight: '10px' }}
         onClick={() => {
           wrapper.error = false;
           onTableChange && onTableChange(wrapper, [wrapper.defaultObj], false, wrapper.newTriggerRef);
         }}>
       </Button>
       {lang === 'en' ? wrapper.titleEN : wrapper.title}{wrapper.error} */}
        <div>
          {buttonsComponent}
          {/* {mutableDeleteComponent} */}
        </div>
      </div>
    )
  }
  // 渲染表格
  renderTable(wrapper, wrapperIndex) {
    const {
      getItemValue,
      source,
      onTableItemValueChange,
      deleteTable,
      dispatch,
      lang = "zh",
      onSelectedRowsChange,
      selectedRowKeys
    } = this.props
    const { sorter, rowSelectionShow } = this.state
    let value = source[wrapper.tableName] || []
    if (wrapper.detailFilterCond && wrapper.detailFilterField) {
      value = value.filter(item => {
        return checkFieldCondition(wrapper.detailFilterCond, item[wrapper.detailFilterField])
      })
    }
    const displayColumns = [
      {
        title: <FormattedMessage id="locale_index" />,
        width: 45,
        render: record => {
          return <div style={{ width: "42px" }}>{value.indexOf(record) + 1}</div>
        }
      }
    ]
    const sumColumns = []
    wrapper.children.map((item, index) => {
      if (!item.titleCh) {
        item.titleCh = item.title
      }
      item.title = lang === "en" ? item.titleEN : item.titleCh

      if (sorter && sorter.sortKey === item.dataIndex) {
        item.sortOrder = sorter.sortOrder
      } else {
        item.sortOrder = ""
      }
      item.key = item.dataIndex
      if (item.hide === true || item.hide === "1" || ((item.hideInEdit === true) === item.hideInEdit) === "1") {
        return
      }
      item.onHeaderCell = () => ({
        width: item.width,
        onResize: (e, { size }) => {
          this.preventSort = true
          let totalWidth = wrapper.resizeTotalWidth || wrapper.totalWidth
          totalWidth = totalWidth - (item.width || 0) + size.width
          item.width = size.width
          wrapper.resizeTotalWidth = totalWidth
          this.setState({ update: !this.state.update })
        }
      })
      if (item.sumChildField === "1") {
        sumColumns.push({ ...item, total: 0 })
      }
      item.render = (text, record) => {
        let rowSpan = 1
        let children = ""
        const index = value.indexOf(record)
        if (item.group === "1" && text) {
          if (index > 0) {
            const lastIndex = index - 1
            const lastValue = value[lastIndex][item["dataIndex"]]
            if (lastValue === text) {
              rowSpan = 0
            }
          }
          let newRecordCount = 0
          if (rowSpan !== 0) {
            for (let nextIndex = index + 1; nextIndex < value.length; nextIndex++) {
              const nextRecord = value[nextIndex]
              if (nextRecord[item["dataIndex"]] === text) {
                newRecordCount++
              } else {
                break
              }
            }
            rowSpan = newRecordCount + 1
          }
        }
        let label = ""
        if (item.type === "multipleSelect") {
          let labelArray = null
          let vLabelArray = []
          if (text) {
            labelArray = text.split(",")
          }
          labelArray &&
            labelArray.map(v => {
              vLabelArray.push(this.getItemActualValue(item, v))
            })
          if (vLabelArray) label = vLabelArray.join()
        } else if (item.type === "tags") {
          let labelArray = null
          let vLabelArray = []
          if (text) {
            labelArray = text.split(",")
          }
          labelArray &&
            labelArray.map(v => {
              vLabelArray.push(this.getItemActualValue(item, v))
            })
          if (vLabelArray) label = vLabelArray.join()
        } else {
          label = this.getItemActualValue(item, text)
        }
        const style = { textOverflow: "ellipsis", color: "#403335" }
        if (item.width) {
          style.width = item.width - 3
          style.whiteSpace = "nowrap"
          style.overflow = "hidden"
          style.maxWidth = "100%"
          if (!sysTheme) {
            style.lineHeight = "1.5"
          }
        }
        if (item.eventFunctionCode && item.eventTableName && item.eventDataIndex) {
          let relationFieldValue = ""
          if (wrapper.tableName === item.eventTableName) {
            relationFieldValue = record[item.eventDataIndex]
          } else {
            const relationField = { tableName: item.eventTableName, dataIndex: item.eventDataIndex }
            relationFieldValue = (getItemValue && getItemValue(relationField, source)) || source[relationField.dataIndex] || ""
          }
          children = (
            <a
              href=""
              onClick={e => {
                e.preventDefault()
                e.stopPropagation()
                dispatch({
                  type: "system/jumpToDetailPage",
                  payload: {
                    functionCode: item.eventFunctionCode,
                    id: relationFieldValue
                  }
                })
              }}
              style={{ ...style, display: "block" }}
              title={label}
            >
              {label}
            </a>
          )
        } else {
          children = (
            <div style={style} title={label}>
              {label}
            </div>
          )
        }

        return {
          props: {
            rowSpan
          },
          children
        }
      }
      item.fixed = item.fixed === "none" ? "" : item.fixed
      displayColumns.push(item)
      return true
    })
    let totalDesc = ""
    if (sumColumns.length > 0) {
      value.map(record => {
        sumColumns.map(item => {
          item.total = item.total + Number(record[item.dataIndex])
          return true
        })
        return true
      })
      sumColumns.map(item => {
        totalDesc = totalDesc + item.title + `: ${item.total.toFixed(window.numberFixedConst)} `
        return true
      })
    }

    let displayDataSource = value
    if (sorter) {
      displayDataSource = _.sortBy(displayDataSource, sorter.field)
      if (sorter.sortOrder === "descend") {
        displayDataSource = displayDataSource.reverse()
      }
    }
    let rowSelection = undefined
    if ((wrapper.supportBatch === true && rowSelectionShow) || (wrapper.supportBatch === "1" && rowSelectionShow)) {
      rowSelection = {
        columnWidth: "35px",
        type: "checkbox",
        // selectedRowKeys: selectedRowKeys && selectedRowKeys.get(wrapper.tableName) || source[wrapper.tableName + formChildSelectedRowKeysConst] || [],
        onChange: selectedRowKeys => {
          onSelectedRowsChange && onSelectedRowsChange(wrapper.tableName, selectedRowKeys)
        }
      }
      // console.log('formChildSelectedRowKeysConst', wrapper.tableName, formChildSelectedRowKeysConst, source);
    }
    return (
      <div key={wrapperIndex} style={{ padding: "0 5px" }}>
        <h3 className="title">
          {this.state.fullscreen === wrapperIndex ? (
            <Icon
              type="fullscreen-exit"
              onClick={() => {
                this.setState({ fullscreen: -1 })
              }}
            />
          ) : (
            <Icon
              type="fullscreen"
              onClick={() => {
                this.setState({ fullscreen: wrapperIndex })
              }}
            />
          )}

          <div style={{ marginRight: "10px", alignItems: "center" }}>{`${lang === "en" ? wrapper.titleEN : wrapper.title}(${value.length
            })`}</div>
          {this.renderTableButton(wrapper)}
          <span style={{ marginLeft: "120px", color: "black" }}>{totalDesc}</span>
        </h3>
        <Table
          rowKey={record => value.indexOf(record)}
          locale={{ emptyText: <span></span> }}
          components={this.components}
          rowClassName={(record, index) => (index % 2 == 0 ? "" : "zebraCross")}
          rowSelection={rowSelection}
          bordered={true}
          scroll={{ x: Math.max(wrapper.totalWidth, wrapper.resizeTotalWidth || 0) + 60, y: 260 }}
          pagination={wrapper.forbidPagination ? false : { pageSize: 15 }}
          size="small"
          columns={displayColumns.concat({})}
          onChange={(pagination, filters, sorter) => {
            if (this.preventSort) {
              this.preventSort = false
              return
            }
            this.setState({ sorter: { sortKey: sorter.field, sortOrder: sorter.order } })
          }}
          dataSource={displayDataSource}
        />
      </div>
    )
  }
  switchShowMore() {
    let { showMore } = this.props
    this.props.dispatch({ type: 'system/switchShowMore', payload: !showMore })
  }
  handleMenuClick = e => {
    //if (e.key === '3') {
    //console.log(e);
    this.setState({ dropDownVisible: false })
    let tabs = this.props.tabs.toJSON()
    //console.log(511,tabs);
    let findTab = undefined
    if (tabs && tabs.length > 0) {
      tabs.forEach(item => {
        if (item.funCode === this.props.funCode && item.type === 'detail') {
          findTab = item

          findTab.cache = {
            id: e.key
          }

          //console.log('&&&',findTab);
          //findTab.cache.id  = e.key
        }
      })
      if (window.detailDisplayType === detailDisplayTypeConst.left.value) {
        this.props.dispatch({
          type: "list/changeSelectedId",
          payload: {
            selectedId: findTab && findTab.cache && findTab.cache.id,
            functionCode: this.props.functionCode
          }
        })
      }
      if (window.detailDisplayType === detailDisplayTypeConst.tab.value) {
        this.props.dispatch({
          type: "system/tabChange",
          payload: findTab
        })
      }
    }

    //}
  }

  handleVisibleChange = flag => {
    this.setState({ dropDownVisible: flag })
  }
  showPrev = () => {
    //console.log('上一条数据')
    let { listData, id } = this.props
    let tabs = this.props.tabs.toJSON()
    let findTab = undefined
    let flag = undefined
    if (tabs && tabs.length > 0) {
      tabs.forEach(item => {
        if (item.funCode === this.props.funCode) {
          findTab = item
        }
      })
    }
    if (listData && listData.length > 0) {
      listData.forEach((item, idx) => {
        if (item.id === id) {
          //console.log('idx',idx);
        }
        if (item.id === id && idx > 0) {
          findTab.cache = {
            id: listData[idx - 1].id
          }
          flag = true
        }
      })
    }
    if (findTab && flag) {
      if (window.detailDisplayType === detailDisplayTypeConst.left.value) {
        this.props.dispatch({
          type: "list/changeSelectedId",
          payload: {
            selectedId: findTab && findTab.cache && findTab.cache.id,
            functionCode: this.props.functionCode
          }
        })
      }
      if (window.detailDisplayType === detailDisplayTypeConst.tab.value) {
        this.props.dispatch({
          type: "system/tabChange",
          payload: findTab
        })
      }
    } else {
      message.warning("已经到首条记录")
      return
    }
  }
  showNext = () => {
    //console.log('下一条数据');
    let { listData, id } = this.props
    let tabs = this.props.tabs.toJSON()
    let findTab = undefined
    let flag = undefined
    if (tabs && tabs.length > 0) {
      tabs.forEach(item => {
        if (item.funCode === this.props.funCode) {
          findTab = item
        }
      })
    }
    if (listData && listData.length > 0) {
      listData.forEach((item, idx) => {
        if (item.id === id) {
          //console.log('idx',idx);
        }
        if (item.id === id && idx < listData.length - 1) {
          findTab.cache = {
            id: listData[idx + 1].id
          }
          flag = true
        }
      })
    }
    if (findTab && flag) {
      if (window.detailDisplayType === detailDisplayTypeConst.left.value) {
        this.props.dispatch({
          type: "list/changeSelectedId",
          payload: {
            selectedId: findTab && findTab.cache && findTab.cache.id,
            functionCode: this.props.functionCode
          }
        })
      }
      if (window.detailDisplayType === detailDisplayTypeConst.tab.value) {
        this.props.dispatch({
          type: "system/tabChange",
          payload: findTab
        })
      }
    } else {
      message.warning("已经到末条记录")
      return
    }
  }
  showPrevData() {
    //console.log('前一页数据')
    let pagination = _.cloneDeep(this.state.pagination)
    if (pagination.pageNo <= 1) {
      message.warn("已到首页")
      return
    }
    let { listData } = this.props
    pagination.nowPageData = listData.slice((pagination.pageNo - 2) * pagination.pageSize, (pagination.pageNo - 1) * pagination.pageSize)
    pagination.pageNo = pagination.pageNo - 1
    //console.log(524,pagination.nowPageData);
    this.setState({ pagination })
  }
  showNextData() {
    //console.log("后一页数据")
    let pagination = _.cloneDeep(this.state.pagination)
    //pagination.pageNo !== Math.floor(pagination.total/pagination.pageSize
    if (pagination.pageNo === Math.ceil(pagination.total / pagination.pageSize)) {
      message.warn("已到尾页")
      return
    }
    let { listData } = this.props
    pagination.nowPageData = listData.slice(pagination.pageNo * pagination.pageSize, (pagination.pageNo + 1) * pagination.pageSize)
    pagination.pageNo = pagination.pageNo + 1
    //console.log(534,pagination.nowPageData);
    this.setState({ pagination })
  }
  componentWillReceiveProps(newProps) {
    //console.log(525,newProps);
    //console.log(526,this.props);
    // if(newProps.displayType && (newProps.id !== this.props.id || newProps.listData !== this.props.listData)){
    //   console.log('执行了！')
    //   this.setCustomer(newProps)
    // }
    if (newProps) {
      //console.log('执行了！')
      this.setCustomer(newProps)
    }

    if (newProps.formConfig && this.state.hl == false) {
      let showStage = newProps.formConfig && this.showStageItem(newProps.formConfig)
      if (showStage && showStage.referenceKey) {

        request('/sysCache/getCacheReference', { key: showStage.referenceKey }).then(res => {
          const { refName, refType, content } = res.result;
          const contentJSON = content ? JSON.parse(content) : {};
          const config = {
            field: {
              value: contentJSON.value,
              label: contentJSON.label,
              pid: contentJSON.pid,
              id: contentJSON.id,
              foreignKey: contentJSON.foreignKey
            },
            refName,
            refType
          };
          let treeData = undefined;
          request('/senior/getReferenceTreeAndDropDown', { id: showStage.referenceKey }).then(res => {
            treeData = res ? res.result : [];
            this.props.dispatch({
              type: "reference/loadReferenceConfigSuccess",
              payload: {
                id: showStage.referenceKey,
                config,
                treeData
              }
            })
            this.setState({ hl: true })
          })
        })


      }
    }

  }
  componentDidMount() {
    this.setCustomer()
  }
  setCustomer(newProps) {
    //console.log('shandian')
    const { listData, id, config } = newProps || this.props
    if (listData && listData.length > 0 && id) {
      let pagination = {
        pageSize: 10,
        total: listData.length,
        pageNo: undefined,
        nowIndex: undefined,
        nowPageData: []
      }
      listData.forEach((item, idx) => {
        if (item.id === id) {
          pagination.nowIndex = item._ || idx
          const caclPageNo = Math.ceil((idx + 1) / pagination.pageSize)
          pagination.pageNo = caclPageNo
          pagination.nowPageData = listData.slice((caclPageNo - 1) * pagination.pageSize, caclPageNo * pagination.pageSize)
        }
      })
      this.setState({
        pagination
      })
      //console.log('552',pagination)
      //console.log('550',config[0][0]);//tableName 000 dataIndex
    }
  }

  //TODO:渲染企业详情列表
  showMes(list, obg) {
    let newobg = JSON.parse(obg)
    return (
      <div style={{ listStyle: "none", width: 400, height: 450, overflow: "hidden auto" }}>
        {list.map((item, index) => {
          return (
            <div key={index} style={{ padding: "5px 0" }}>
              <p
                style={{
                  float: "left",
                  textAlign: "right",
                  fontStyle: "normal",
                  width: 80,
                  marginRight: 10,
                  marginBottom: 0
                }}
              >
                {item}:
              </p>
              <p style={{
                overflow: "hidden",
                minHeight: '18px',
                marginBottom: 0
              }}>{newobg[item]}</p>
            </div>
          )
        })}
      </div>
    )
  }

  // 阶段条件获取
  showStageItem(formConfig) {
    let showStage = undefined;
    formConfig.map(item => {
      if (Array.isArray(item)) {
        item.map(itemC => {
          if (itemC.showStage == "1") {
            showStage = itemC
          }
        })
      }
    })
    return showStage
  }

  // TODO:渲染阶段内容
  renderStage() {
    const { searchConfig, fromConfig, dispatch, reference, dictionary, functionCode, mainTableName, source, config } = this.props;

    let showStage = fromConfig && this.showStageItem(fromConfig)

    if (showStage && showStage.showStage && showStage.showStage != 0) {
      let stateC;
      let stateD;
      let rk = "";
      if (showStage && showStage.referenceKey) {
        rk = showStage.referenceKey;
        if (reference && reference.get(rk)) {
          stateC = reference.get(rk).get("config");
          stateD = reference.get(rk).get("treeData");
        }
      }

      if (showStage.sourceKey) {
        rk = showStage.sourceKey;
        stateC = {
          field: {
            label: "label",
            value: "value",
          }
        };
        stateD = dictionary.get(rk);
      }

      return (
        <Stage stateC={stateC} showStage={showStage} stateD={stateD} id={source[mainTableName].id} fcode={functionCode} ></Stage>
      )
    } else {
      return false;
    }


  }

  goToDetail(item, relationFieldValue) {
    const { dispatch } = this.props;

    if (item.documentContentFunCode == "1") {
      request("/senior/getRealFunCode", { id: relationFieldValue, funCode: item.eventFunctionCode }, "", "", item.eventFunctionCode).then(res => {
        dispatch({
          type: "system/jumpToDetailPage",
          payload: {
            functionCode: res.result.realFunCode,
            id: relationFieldValue
          }
        })
      })
      return
    }
    dispatch({
      type: "system/jumpToDetailPage",
      payload: {
        functionCode: item.eventFunctionCode,
        id: relationFieldValue
      }
    })
  }

  // 判断返回是否为负数
  setIsNegative(itemnum) {
    let nNum = parseFloat(itemnum);
    if (nNum) {
      return Math.sign(nNum);
    }
    return ""
  }

  render() {
    const {
      config,
      layout,
      border,
      className,
      style,
      getItemValue,
      reference,
      dispatch,
      dictionary,
      source,
      lang,
      showAll,
      displayType,
      listData,
      id,
      functionCode,
      showMore
    } = this.props
    const borderStyle = border ? "row" : "noborder"
    const { pagination } = this.state

    const newConfig = config.filter(item => Array.isArray(item)) || []
    // 获取标题项及值
    let detailQuickItem = undefined
    if (config && config[0]) {
      for (let i = 0; i < config[0].length; i++) {
        if (config[0][i].detailQuick) {
          detailQuickItem = config[0][i]
          let label = (getItemValue && getItemValue(config[0][i], source)) || source[config[0][i].dataIndex] || ""
          detailQuickItem.referAfterValue = this.getItemActualValue(config[0][i], label)
          break
        }
      }
    }
    const menu = detailQuickItem && (
      <Menu onClick={this.handleMenuClick}>
        {pagination &&
          pagination.nowPageData &&
          pagination.nowPageData.length > 0 &&
          pagination.nowPageData.map(item => {
            let idx = detailQuickItem.tableName + "000" + detailQuickItem.dataIndex
            let label = item[idx] || ""
            label = this.getItemActualValue(detailQuickItem, label)
            if (item.id === this.props.id) {
              return (
                <Menu.Item key={item.id}>
                  <span style={{ fontWeight: "bold" }}>{label}</span>
                </Menu.Item>
              )
            } else {
              return <Menu.Item key={item.id}>{label}</Menu.Item>
            }
          })}
        {/* <Menu.Item key="1">Clicking me will not close the menu.</Menu.Item>
         <Menu.Item key="2">Clicking me will not close the menu also.</Menu.Item> */}
        <Menu.Divider />
        {/* <Menu.Item key="3"> */}
        <div className="control">
          <div className={pagination && pagination.pageNo > 1 ? "active" : "fail"} onClick={() => this.showPrevData()}>
            上10条
          </div>
          <div
            className={
              pagination && pagination.pageNo > 0 && pagination.pageNo !== Math.ceil(pagination.total / pagination.pageSize)
                ? "active"
                : "fail"
            }
            onClick={() => this.showNextData()}
          >
            下10条
          </div>
        </div>
        {/* </Menu.Item> */}
      </Menu>
    )

    let firmMes, firmMesList, content, content1

    if (source.cs_customer) {
      firmMes = source.cs_customer.businessInfo && source.cs_customer.businessInfo //从form存贮状态中取出用户信息
      if (firmMes) {
        //如果有用户信息提取对象的key变成数组可遍历
        firmMesList = Object.keys(JSON.parse(firmMes))
      }

      content = (
        <div style={{ textAlign: "center", padding: "50px 0", width: 300 }}>
          <h4 style={{ fontSize: 14 }}>暂无企业信息</h4>
        </div>
      )

      if (firmMes) {
        content1 = this.showMes(firmMesList, firmMes) //有用户信息渲染返回
      }
    }

    return (
      <div className={classnames("detailWrapper", className)} style={style}>
        {/* FIXME:↓↓↓↓↓↓详情状态↓↓↓↓↓↓ */}
        {this.renderStage()}
        {/* ↑↑↑↑↑↑详情状态↑↑↑↑↑↑ */}

        <div
          className="w98"
          style={{
            overflow: "hidden",
            height: (showMore || showAll || newConfig.length == 1) ? "auto" : "126px"
          }}
        >
          {/* (showMore ? idx>-1 : idx<2) && */}
          {newConfig.map((wrapper, idx) => {
            if (idx == 0) {
              // console.log(detailQuickItem, idx, pagination)
            }
            return (
              <div key={idx}>
                {wrapper.filter(el => el.hide != "1").length > 0 && <Row
                  key={idx}
                  className={classnames(borderStyle, this.state.fullscreen === idx ? "fullscreen" : "")}
                  style={{ borderBottom: showMore ? "1px dashed #c7adad" : "none" }}
                >
                  {
                    // 标题快速切换
                    detailQuickItem && idx === 0 && (
                      <Col key={"detailQuickItem"} span={24}>
                        {pagination && pagination.nowPageData && pagination.nowPageData.length > 0 && (
                          <div className="customer">
                            <i onClick={this.showPrev} style={{ color: "#828282" }} className="icon iconfont icon-ic_circle_prev-copy"></i>
                            <Dropdown
                              overlay={menu}
                              overlayStyle={{ minWidth: 270 }}
                              onVisibleChange={this.handleVisibleChange}
                              visible={this.state.dropDownVisible}
                            >
                              <a className="ant-dropdown-link" onClick={e => e.preventDefault()}>
                                {detailQuickItem.title}：{detailQuickItem.referAfterValue} <Icon type="down" />
                              </a>
                            </Dropdown>
                            <i onClick={this.showNext} style={{ color: "#828282" }} className="icon iconfont icon-ic_circle_next"></i>
                            {/* {functionCode}{id} */}
                            {firmMes && (
                              <Popover content={firmMes ? content1 : content} title="企业信息" trigger="click">
                                <i
                                  style={{ color: firmMes ? "#1a98f1" : "#F30", fontSize: "20px", verticalAlign: "middle" }}
                                  className="iconfont icon-gongshang"
                                ></i>
                              </Popover>
                            )}
                            {/* 客户组织图 */}
                            {detailQuickItem.organization && (
                              <OrganizationModal url={detailQuickItem.organizationUrl} id={id} dispatch={dispatch} />
                            )}
                          </div>
                        )}
                      </Col>
                    )
                  }
                  {wrapper.map((item, index) => {
                    try {
                      if (item.showTitle) {
                        return <h3 key={index} className="showTitle">{item.showTitle}</h3>
                      }
                      if (item.hide === true || item.hide === "1" || ((item.hideInEdit === true) === item.hideInEdit) === "1") {
                        return null
                      }
                      if (item.detailQuick && pagination && pagination.nowPageData && pagination.nowPageData.length > 0) {
                        return null
                      }
                      let label = (getItemValue && getItemValue(item, source)) || source[item.dataIndex] || ""
                      let layoutProps = {}
                      let isNegative = null
                      if (item.width) {
                        layoutProps.span = item.width
                      } else {
                        layoutProps = { ...(formTypeLayout[item.type] || layout || itemLayout) }
                      }
                      if (item.type === "htmlEditor" || item.type === "tinyMCE") {
                        return (
                          <Col key={index} {...layoutProps} style={{ padding: "0 5px" }}>
                            <h3 className="title">{lang === "en" ? item.titleEN : item.title}</h3>
                            <div dangerouslySetInnerHTML={{ __html: label }}></div>
                          </Col>
                        )
                      } else if (item.type === "tags") {
                        const { itemSource, config } = getItemSourceByKey(item, dispatch, reference, dictionary)
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title}>
                              <CheckableTags disabled source={itemSource} field={item.field} value={label} onlyShowChecked />
                            </KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === "thumb" || item.type === "signature") {
                        return (
                          <Col {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title}>
                              <UploadThumb disabled value={label} />
                            </KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === "picturesWall") {
                        var pictures = []
                        if (label) {
                          pictures = label.split(",")
                        }
                        return (
                          <Col {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title}>
                              <PicturesWall disabled defaultFileList={pictures} />
                            </KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === "textArea") {
                        label = this.getItemActualValue(item, label)
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth}
                              wrap
                              style={{ wordWrap: "break-all", whiteSpace: "pre-wrap", lineHeight: "25px" }}
                              // wordWrap={"break-all"}
                              // whiteSpace={"pre-wrap"}
                              title={lang === "en" ? item.titleEN : item.title}
                              value={label}
                            ></KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === "employeeSelector") {
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth}
                              wrap
                              title={lang === "en" ? item.titleEN : item.title}
                              value={<EmployeeSelectorLabel value={label} />}
                            ></KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type == "dateTimeRange" || item.type == "dateRange" || item.type == "customDateRange" || item.type == "customWeekRange") {
                        console.log("dateTimeRange", item, label, label.replace(/["\[\]]/g, ''));
                        let reWords = label.replace(/["\[\]]/g, '');
                        let reWordsList = reWords.split(",");
                        let newWords = reWordsList[0] + " ~ " + reWordsList[1];
                        if (item.type == "customWeekRange") {
                          newWords = reWordsList[1] + " ~ " + reWordsList[2] + " " + reWordsList[0] + "月 第" + reWordsList[3] + "周";
                        }
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth}
                              wrap
                              title={lang === "en" ? item.titleEN : item.title}
                              value={newWords}
                            ></KeyValuePaired>
                          </Col>
                        )

                      } else if (item.type === "multipleSelect" || item.type == "multipleTreeSelect") {
                        let labelArray = null
                        let vLabelArray = []
                        if (label) {
                          if (label.indexOf('value') != -1) {
                            labelArray = JSON.parse(label).value;
                          } else {
                            labelArray = label.split(',')
                          }
                        }
                        labelArray &&
                          labelArray.map(v => {
                            vLabelArray.push(this.getItemActualValue(item, v))
                          })
                        if (vLabelArray) label = vLabelArray.join()
                      } else if (item.type === "BDMap") {
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} wrap title={lang === "en" ? item.titleEN : item.title}>
                              <BDMap value={label} detail={true} />
                            </KeyValuePaired>
                          </Col>
                        )
                        // return <BDMap value={props.value} onChange={(value) => onChange(value)} />
                      } else if (item.type === "Geocoder") {
                        let value
                        if (label) {
                          value = JSON.parse(label)[0]
                        }
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title} value={value}></KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === "uploadFile") {
                        let fileList = []
                        if (label) {
                          fileList = JSON.parse(label)
                        }
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} wrap title={lang === "en" ? item.titleEN : item.title}>
                              {fileList.map((item, index) => (
                                <div key={index} className={"attachments2"}>
                                  <p
                                    title={item.oldFileName}
                                    onClick={e => {
                                      e.preventDefault()
                                      const previewLink = `${fileServerUrl}${item.fileUrl}${item.newFileName}`
                                      if (isImage(previewLink) || isMP3(previewLink) || isVideo(previewLink)) {
                                        this.setState({
                                          previewModalVisible: true,
                                          previewLink
                                        })
                                      } else {
                                        message.warning('非图片/视频/音频文件,无法预览!', 2);
                                        //window.open(previewLink)
                                      }
                                    }}
                                  >
                                    <Icon type="paper-clip" style={{ marginRight: "2px" }} />
                                    {item.oldFileName}
                                  </p>
                                  <div>
                                    <Icon
                                      type="cloud-download"
                                      onClick={e => {
                                        e.preventDefault()
                                        downloadFromServer("/attach/downloadFile", item, item.oldFileName)
                                      }}
                                    />
                                  </div>
                                </div>
                              ))}
                            </KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === "customPopup") {
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title}>
                              <div>
                                <span>{label}</span>
                                <Button
                                  size="small"
                                  onClick={() =>
                                    dispatch({
                                      type: "system/visibleCustomPopEdit",
                                      payload: {
                                        key: item.customReferenceKey,
                                        record: source
                                      }
                                    })
                                  }
                                >
                                  查看
                                </Button>
                              </div>
                            </KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === "passWord") {
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title}>
                              <Input className="passWord" type="passWord" disabled="disabled" value={this.getItemActualValue(item, label)} />
                            </KeyValuePaired>
                          </Col>
                        )
                      }
                      // else if (item.type === "time") {
                      // }
                      else if (item.type === "wechatIdQR") {
                        label = this.getItemActualValue(item, label)
                        const thirdPartyConfiguration = localStorage.getItem("thirdPartyConfiguration")
                          ? JSON.parse(localStorage.getItem("thirdPartyConfiguration"))
                          : ""
                        const wechatUrl = thirdPartyConfiguration
                          ? thirdPartyConfiguration.offical && thirdPartyConfiguration.offical.officalurl
                          : ""
                        const content = (
                          <QRCode
                            value={`${wechatUrl}/eap/sysWechatoffical/loginWechat?referralCode=${source["id"]}`}
                            size={177}
                            bgColor={"#ffffff"}
                            fgColor={"#000000"}
                            level={"L"}
                            includeMargin={false}
                            renderAs={"svg"}
                          />
                        )
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title}>
                              <Popover
                                content={wechatUrl ? content : <p>没有配置微信公众号地址</p>}
                                title={<div style={{ textAlign: "center" }}>请使用微信扫描</div>}
                                placement="right"
                              >
                                <Button type="link" size="small">
                                  显示二维码
                                </Button>
                              </Popover>
                            </KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === "marketIdQR") {
                        label = this.getItemActualValue(item, label)
                        const thirdPartyConfiguration = localStorage.getItem("thirdPartyConfiguration")
                          ? JSON.parse(localStorage.getItem("thirdPartyConfiguration"))
                          : ""
                        const wechatUrl = thirdPartyConfiguration
                          ? thirdPartyConfiguration.offical && thirdPartyConfiguration.offical.officalurl
                          : ""
                        const content = (
                          <QRCode
                            value={`${wechatUrl}/eap/sysWechatoffical/loginmarket?activityid=${source["id"]}`}
                            size={177}
                            bgColor={"#ffffff"}
                            fgColor={"#000000"}
                            level={"L"}
                            includeMargin={false}
                            renderAs={"svg"}
                          />
                        )
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title}>
                              <Popover
                                content={wechatUrl ? content : <p>没有配置微信公众号地址</p>}
                                title={<div style={{ textAlign: "center" }}>请使用微信扫描</div>}
                                placement="right"
                              >
                                <Button type="link" size="small">
                                  显示二维码
                                </Button>
                              </Popover>
                            </KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === "division") {
                        label = this.getItemActualValue(item, label)
                        //console.log('962',label);
                        let value = []
                        if (label) {
                          let temp = JSON.parse(label)[0]
                          value = temp.split(",")
                        }
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title}>
                              <Division value={value}></Division>
                            </KeyValuePaired>
                          </Col>
                        )
                      } else if (item.type === 'coordinates') {
                        if (label && label.split(",") && label.split(",")[2]) {
                          label = label.split(",")[2]
                        }
                      } else if (item.type === 'number' || item.type === 'amount') {
                        isNegative = this.setIsNegative(label);
                        label = this.getItemActualValue(item, label);
                        label = numFormat(label);
                      } else {
                        //console.log('before',item,label);
                        label = this.getItemActualValue(item, label)
                        //console.log('after',label);
                      }
                      if (item.eventFunctionCode && item.eventTableName && item.eventDataIndex) {
                        const relationField = { tableName: item.eventTableName, dataIndex: item.eventDataIndex }
                        const relationFieldValue =
                          (getItemValue && getItemValue(relationField, source)) || source[relationField.dataIndex] || ""
                        return (
                          <Col key={index} {...layoutProps}>
                            <KeyValuePaired titleWidth={item.titleWidth} title={lang === "en" ? item.titleEN : item.title}>
                              <a
                                href=""
                                style={{ color: "#1890ff" }}
                                onClick={e => {
                                  e.preventDefault()
                                  e.stopPropagation()
                                  this.goToDetail(item, relationFieldValue);
                                  // dispatch({
                                  //   type: "system/jumpToDetailPage",
                                  //   payload: {
                                  //     functionCode: item.eventFunctionCode,
                                  //     id: relationFieldValue
                                  //   }
                                  // })
                                }}
                                title={label}
                              >
                                {label}
                              </a>
                            </KeyValuePaired>
                          </Col>
                        )
                      }
                      return (
                        <Col key={index} {...layoutProps}>
                          <div style={{ display: "flex", alignItems: "baseline" }}>
                            <KeyValuePaired titleWidth={item.titleWidth}
                              title={lang === "en" ? item.titleEN : item.title}
                              value={label}
                              isNegative={isNegative}
                              align={item.align ? item.align : item.type === "amount" ? "left" : ""}
                            />
                            {/* 组织结构图 */}
                            {item.organization && (
                              <div style={{ marginLeft: "10px" }}>
                                <OrganizationModal url={item.organizationUrl} id={id} dispatch={dispatch} />
                              </div>
                            )}
                          </div>
                        </Col>
                      )
                    } catch (e) {
                      console.log("itemErr", item, e)
                    }
                  })}
                </Row>
                }
              </div>
            )
          })}
        </div>
        {/* TODO:收起展开按钮 */}
        {/* className={classnames('bottomFunc')} */}
        {!showAll && (newConfig.length > 1) && (
          <div className={`bottomFunc ${showMore ? "a" : "b"}`}>
            {showMore ? (
              <span onClick={() => this.switchShowMore()}>
                {" "}
                收起 <i className="iconfont icon-shouqi" />
              </span>
            ) : (
              <span onClick={() => this.switchShowMore()}>
                {" "}
                展开 <i className="iconfont icon-xiala"></i>
              </span>
            )}
          </div>
        )}

        <AntModal
          visible={this.state.previewModalVisible}
          title="预览"
          width={900}
          bodyStyle={{
            height: "70vh",
            overflow: "auto"
          }}
          className="preview-modal"
          onCancel={() => this.setState({ big: false, previewModalVisible: false, previewLink: "" })}
          footer={null}
        >
          {isImage(this.state.previewLink) && (
            <img
              style={
                this.state.big
                  ? { maxWidth: "1000px", maxHeight: "none", margin: "0 auto", display: "block" }
                  : { margin: "0 auto", display: "block" }
              }
              onClick={() => this.setState({ big: !this.state.big })}
              alt=""
              src={this.state.previewLink}
            />
          )}
          {isMP3(this.state.previewLink) && (
            <audio controls controlsList="nodownload">
              <source src={this.state.previewLink} />
              您的浏览器不支持音频播放
            </audio>
          )}
          {isVideo(this.state.previewLink) && (
            <video controls controlsList="nodownload">
              <source src={this.state.previewLink} />
              您的浏览器不支持视频播放
            </video>
          )}
        </AntModal>
      </div>
    )
  }
}
