
import "./index1.less"
import "rc-color-picker/assets/index.css"
import React, { Component, PureComponent } from "react"
import {
  Form,
  Input,
  Row,
  Col,
  Radio,
  message,
  Table,
  Button,
  Icon,
  Popconfirm,
  Tooltip,
  Tabs,
  Alert as AntAlert,
  Modal,
  Popover
} from "antd"
import PropTypes from "prop-types"
import { itemLayout } from "../../common/config"
import {
  selectedModeConst,
  groupType,
  formFieldType,
  formChildSelectedRowKeysConst,
  buttonType,
  wfConditionConst
} from "../../common/const"
import { getItemSourceFromCache, getValueFromSource, getReferenceByItem, newGuid, getBeginAndEndDate, returnMinusWidth } from "../../utils"
import { connect } from "react-redux"
import { List, Map } from "immutable"
import EllipsisText from "../EllipsisText"
import _ from "lodash"
import classnames from "classnames"
import ResizeableTitle from "../ResizeableTitle"
import { FormattedMessage } from "react-intl"
import renderItemFunctions from './modules'
const FormItem = Form.Item
let cacheChangeAction = undefined
let referenceTranslateCache = {}
const formTypeLayout = {
  textArea: {
    xs: 24,
    md: 24,
    lg: 24,
    xl: 24
  }
}

@connect(
  (state, route) => {
    const reference = state.get("reference")
    const dictionary = state.get("dictionary")
    const activeTab = state.get("system").get("activeTab")
    return {
      reference,
      dictionary,
      selectedRows: reference.get("selectedRows"),
      selectedSuccess: reference.get("selectedSuccess"),
      currentItem: reference.get("currentItem"),
      objectHash: reference.get("objectHash"),
      currentId: reference.get("currentId"),
      functionCode: activeTab.functionCode
    }
  },
  null,
  null,
  { withRef: true }
)
class AutogenerationForm extends Component {
  groupRecord = {}

  static defaultProps = {
    config: [],
    source: {},
    border: false
    // onItemValueChange: (value, index) => {},
    // onSubmitForm: () => {}
  }

  preventSort = false

  static propTypes = {
    config: PropTypes.array,
    onItemValueChange: PropTypes.func,
    source: PropTypes.object,
    cache: PropTypes.object
  }
  // state = {
  //   update: false
  // }

  constructor(props) {
    super(props)
    this.state = {
      update: false,
      objectHash: newGuid(),
      isFilter: localStorage.getItem("locIsFilter") ? JSON.parse(localStorage.getItem("locIsFilter")) : {},
      isHideLabel: false
    }
  }

  componentDidMount() {
    const { config } = this.props
    this.filterDataIndex(config)
  }

  filterDataIndex(data) {
    let { isFilter } = this.state
    data.map(item => {
      if (Array.isArray(item)) {
        this.filterDataIndex(item)
      } else {
        if (item.children) {
          this.filterDataIndex(item.children)
        }
      }
      if (item.filter) {
        isFilter[item.dataIndex] = true
        this.setState({ isFilter }, () => {
          localStorage.setItem("locIsFilter", JSON.stringify(this.state.isFilter))
        })
      }
    })
  }

  filterSwitch(dataIndex) {
    let { isFilter } = this.state
    isFilter[dataIndex] = !isFilter[dataIndex]
    this.setState({ isFilter }, () => {
      localStorage.setItem("locIsFilter", JSON.stringify(this.state.isFilter))
    })
  }

  componentWillReceiveProps(newProps) {
    if (this.state.objectHash === newProps.objectHash) {
      if (!newProps.currentId && this.props.currentId) {//关闭弹窗时清空
        cacheChangeAction = undefined
      }
      if (newProps.selectedSuccess && !this.props.selectedSuccess) {
        const { currentItem, selectedRows, dispatch, onItemValueChange, onTableChange, onTableItemValueChange } = newProps
        if (currentItem) {
          if (selectedRows && selectedRows.length > 0) {
            let value = undefined
            if (selectedRows.length > 1) {//多选
              let arr = [];
              selectedRows.map(item => {
                arr.push(item[currentItem.referenceField])
              })
              value = arr.join(",")
            } else {
              value = selectedRows[0][currentItem.referenceField]
            }
            if (currentItem.groupType === groupType.table) {
              if (currentItem.dataIndex && typeof cacheChangeAction === "function") {
                cacheChangeAction(value, selectedRows[0], selectedRows.length > 1 ? selectedRows : undefined)
                cacheChangeAction = undefined
              } else {
                onTableChange && onTableChange(currentItem, selectedRows, true, currentItem.newTriggerRef)
              }
            } else if (currentItem.referenceField) {
              if (typeof cacheChangeAction === "function") {
                cacheChangeAction(value, selectedRows[0], selectedRows.length > 1 ? selectedRows : undefined)
                cacheChangeAction = undefined
              } else {//正常返回
                onItemValueChange(value, currentItem.dataIndex, currentItem, selectedRows[0])
              }
            }
          }
        }
        this.props.dispatch({
          type: "reference/resetReference"
        })
      }
    }
  }

  getRequiredMessageByType(type, title) {
    //TODO 不通的type输入不通的错误消息
    return `请输入${title}`
  }

  onItemValueChange(value, dataIndex, item) {
    this.props.onItemValueChange(value, dataIndex, item)
  }
  // 校验
  validateItems() {
    const { config, source, extraConfig, getItemValue, errMsg } = this.props
    let hasError = false
    let errorMessage = ""
    let patternError = false;
    let mesList = [];
    config.map(wrapper => {
      if (Array.isArray(wrapper)) {//主表
        wrapper.map(item => {
          let value = getItemValue && getItemValue(item, source)
          if (!value && value !== 0) {
            value = source.get(item.dataIndex)
            if (typeof value === "object") {
              hasError = hasError ? hasError : this.isRequired(item.required)
            }
          } else {
            // 校验提示
            // message.error(`请将${item.title}填写完整！`);
            // hasError = true
            errorMessage += item.title + ","
          }
          item.error = this.isRequired(item.required) && !value && value !== 0
          hasError = hasError ? hasError : item.error

          // patternError = false
          if (item.pattern && value) {
            //正则校验
            try {
              const reg = eval(item.pattern)
              if (reg && typeof reg.test === "function") {
                patternError = !reg.test(value)
                item.error = item.error || !reg.test(value)
              }
            } catch (e) {
              message.error(`${item.title} 正则表达式有问题`)
            }
          }
          hasError = item.error || hasError
          if (item.error) {
            patternError && item.errMsg ? message.error(`${item.title}${item.errMsg}`) : message.error(`请正确填写${item.title}`)
          }
        })
      } else {//子表
        const tables = source.get(wrapper.tableName)
        if (this.isRequired(wrapper.required) && (!tables || tables.size === 0)) {
          wrapper.error = true
          hasError = true
          message.error(`请填写${wrapper.title}`)//表必填
        } else if (tables) {
          const requiredFields = wrapper.children.filter(x => this.isRequired(x.required) || x.pattern)
          if (requiredFields.length > 0 || (wrapper.itemRequireds && wrapper.itemRequireds.length > 0)) {
            tables.map((table, index) => {
              //子表独立必填
              if (wrapper.itemRequireds && wrapper.itemRequireds.length > 0) {
                try {
                  wrapper.itemRequireds.map(req => {
                    req = req.split(',')
                    if (req[0] == index) {
                      let value = table.get(req[1])
                      if (!value) {
                        let newBog = wrapper.children.filter(x => x.dataIndex == req[1])[0]
                        newBog.TbName = wrapper.title
                        let errors = newBog.errors || []
                        errors.push(index)
                        errors
                        newBog.errors = errors
                        console.log('newBog', newBog);
                        mesList.push(newBog);
                        // hasError = true
                      }
                    }
                  })
                } catch (err) {
                  message.error(`${wrapper.title}计算公式影响配置出现问题`)
                }
              }

              requiredFields.map(field => {
                const value = table.get(field.dataIndex)
                let itemError = this.isRequired(field.required)
                  && !table.get(field.dataIndex)
                  && table.get(field.dataIndex) !== 0

                if (itemError) {
                  let newBog = _.cloneDeep(field);
                  newBog.TbName = wrapper.title
                  // console.log('newBogB', newBog);

                  mesList.push(newBog);
                  // console.log('mesList1', mesList);
                }
                if (field.pattern && value) {
                  //子表正则
                  try {
                    const reg = eval(field.pattern)
                    if (reg && typeof reg.test === "function") {
                      itemError = !reg.test(value)
                      patternError = !reg.test(value)
                      field.error = field.error || !reg.test(value)
                    }
                  } catch (e) {
                    message.error(`${field.title} 正则表达式有问题`)
                  }
                }
                hasError = hasError || itemError
                hasError = field.error || hasError
                if (field.error) {
                  patternError && field.errMsg ? message.error(`${wrapper.title}-${field.title}-${field.errMsg}`) : message.error(`请填写${field.title}`)
                }
                if (itemError) {
                  let errors = field.errors || []
                  errors.push(index)
                  errors = _.uniqWith(errors, _.isEqual);
                  field.errors = errors
                }
              })
            })
          }
        }
      }
    })

    if (hasError && !patternError) {
      let lMes = "请完善页面中"
      let list1 = mesList.map(item => ({                   //取两个关机字段准备去重
        dataIndex: item.dataIndex,
        tableName: item.tableName
      }))
      let list2 = _.uniqWith(list1, _.isEqual);           //去重
      let mesListN = [];
      list2.forEach(el => {
        let nList = mesList.filter(item => item.dataIndex == el.dataIndex && item.tableName == el.tableName)
        mesListN.push(nList[0]);
      })
      mesListN.forEach(item => {
        let title = ''
        if (typeof item.title == 'object') {
          title = item.title.props && item.title.props.children && item.title.props.children[0] + item.title.props.children[1]
        } else {
          title = item.title
        }
        lMes = lMes + " '" + item.TbName + "' 表中 '" + title + "' 字段,"
      })
      // "请将页面填写完整！"
      message.error(errMsg ? errMsg : lMes)
    }
    this.setState({ hasError })
    // console.log('最终', hasError);
    return !hasError
  }

  focus() {
    // this.firstItem && this.firstItem.focus();
  }

  onSubmitForm(e) {
    e.preventDefault()
    if (this.validateItems()) {
      this.props.onSubmitForm && this.props.onSubmitForm()
    }
  }

  isRequired(required) {
    return required && required !== "0"
  }

  //TODO: 判断是否需要隐藏
  isSetNoLabel(list) {
    const { isHideLabel } = this.state
    if (list) {
      list.forEach(item => {
        item.forEach(item1 => {
          if (item1.dataIndex && item1.dataIndex.indexOf("cbx_") > -1 && isHideLabel === false) {
            this.setState({
              isHideLabel: true
            })
          }
        })
      })
    }
  }

  // 设置搜索条件的列数
  setWidth = (winW, yuanW, longWList) => {
    if (longWList.length > 0 && winW > 1679) {
      return 8
    }

    if (longWList.length > 0 && winW > 1279) {
      return 12
    } else if (winW > 1800) {
      return yuanW
    } else if (winW > 1360) {
      return 8
    } else if (winW > 930) {
      return 12
    } else if (winW > 0) {
      return 24
    }

    return yuanW
  }

  editDetailAddress = (val1, val2, val3) => {
    const { onItemValueChange } = this.props;
    Modal.confirm({
      title: '确认',
      content: '系统发现你更改了定位地址,是否同步更新详细地址?',
      okText: '确认',
      cancelText: '取消',
      onOk() {
        onItemValueChange(val1, val2, val3)
      }
    });
  }

  renderItemWrapper = (item, index) => {
    const {
      source,
      extraConfig,
      layout,
      getItemValue,
      onItemValueChange,
      labelWidthSwitch,
      scale,
      winWidth,
      hanLong,
      forbidHide,
      lang = "zh",
      dataChange,
      smallTop
    } = this.props


    const { isHideLabel } = this.state
    if (item.showTitle) {
      return (
        <h3 className="showTitle" style={{ fontSize: item.fontSize ? item.fontSize : "initial" }}>
          {item.showTitle}
        </h3>
      )
    }
    try {
      if (extraConfig && extraConfig[item.dataIndex]) {
        item = { ...item, ...extraConfig[item.dataIndex] }
      }
      let layoutProps = {}
      if (item.dataIndex == "queryViewSql") {
        console.log("item", item)
      }
      // 隐藏显示
      if (
        item.hideInEdit === true ||
        item.hideInEdit === "1" ||
        item.hide === true ||
        item.hide === "1" ||
        (!forbidHide && item.defaultHide === "1")
      ) {
        return null
      }
      if (item.dataIndex == "queryViewSql") {
        console.log("item2", item)
      }
      // 宽度
      if (item.width) {
        // layoutProps.span = scale ? scale * item.width : item.width;
        layoutProps.span = scale ? scale * item.width : winWidth ? this.setWidth(winWidth, item.width, hanLong) : item.width
        // layoutProps.span = scale ? scale * item.width : this.setWidth(winWidth,item.width, hanLong);
      } else {
        layoutProps = { ...(formTypeLayout[item.type] || layout || itemLayout) }
      }
      // 表单回调
      const onChange = (value, selectRow, selectedRows) => {
        item.error = false
        if (selectRow && item.referenceDisplayField) {
          if (selectedRows) {
            let arr = []
            selectedRows.map(row => {
              arr.push(row[item.referenceDisplayField]
              )
            })
            referenceTranslateCache[item.dataIndex + "_" + value] = arr.join(',')
          } else {
            referenceTranslateCache[item.dataIndex + "_" + value] = selectRow[item.referenceDisplayField]
          }
        }
        if (item.type === "BDMap") {
          if (!item.tableName) {
            return
          }
          let tableNameSource = Object.keys(source.get(item.tableName).toJSON())
          const transform = (key) => {
            if (tableNameSource.indexOf(key + '_') > 0) {
              return key + '_'
            } else {
              return key
            }
          }
          if (JSON.parse(value)[3]) {
            onItemValueChange(JSON.parse(value)[1], "detailedAddress", { tableName: item.tableName, dataIndex: transform("detailedAddress") })
          }
          //备份 onItemValueChange(JSON.parse(value)[1], "detailedAddress", { tableName: item.tableName, dataIndex: transform("detailedAddress") })
          onItemValueChange(JSON.parse(value)[0], "mapLocation", { tableName: item.tableName, dataIndex: transform("mapLocation") })
          onItemValueChange(JSON.parse(value)[2].split(",")[0], "province", { tableName: item.tableName, dataIndex: transform("province") })
          onItemValueChange(JSON.parse(value)[2].split(",")[1], "city", { tableName: item.tableName, dataIndex: transform("city") })
          onItemValueChange(JSON.parse(value)[2].split(",")[2], "area", { tableName: item.tableName, dataIndex: transform("area") })
        }
        // console.log('value, item.dataIndex', value, item.dataIndex);

        onItemValueChange(value, item.dataIndex, item, selectRow)
      }
      // 取值
      let value = getItemValue && getItemValue(item, source)
      if (value === undefined) {
        value = source.get(item.dataIndex)
      }
      if (value === undefined) {
        value = item.defaultValue
      }
      if (item.type && item.type === 'popAlert') {
        return <AntAlert message="注意!"
          description={<div dangerouslySetInnerHTML={{ __html: item.title }} />}
          type="warning"
          showIcon />
      }
      // 数组标签循环渲染
      return (
        <Col key={index} style={{ marginTop: smallTop ? smallTop + 'px' : "12px", lineHeight: '32px', minHeight: "32px" }} {...layoutProps}>
          <FormItem
            // style={{marginTop:'10px', height:'30px', lineHeight:'30px'}}
            colon={false}
            hasFeedback
            className="changCss"
            // labelCol={{span:6}}
            validateStatus={item.error ? "error" : ""}
            // labelAlign={"right"}
            // required={this.isRequired(item.required)}
            label={
              item.type === "htmlEditor" || item.type === "tinyMCE" || labelWidthSwitch === "hide" ? null : (
                <span
                  title={lang === "en" ? item.titleEN : item.title}
                  data-dataindex={item.dataIndex}
                  data-type={item.type}
                  style={
                    this.isRequired(item.required)
                      ? {

                        color: "#427dce",
                        textAlign: item.align ? item.align : "right",
                        width: (labelWidthSwitch === "off" || isHideLabel) ? "auto" : item.titleWidth ? item.titleWidth + "px" : "125px"
                      }
                      : {
                        textAlign: item.align ? item.align : "right",
                        width: (labelWidthSwitch === "off" || isHideLabel) ? "auto" : item.titleWidth ? item.titleWidth + "px" : "125px"
                      }
                  }
                >
                  {lang === "en" ? item.titleEN : item.title}&nbsp;
                  {item.help && (
                    <Tooltip title={item.help}>
                      <Icon type="question-circle-o" />
                    </Tooltip>
                  )}
                </span>
              )
            }
          >
            {/* <CustomToolTip text={item.errMsg} visible={item.error}> */}
            {<div style={{ display: 'flex', alignItems: 'center' }}><div style={{ flex: '1' }}>
              {this.renderItem(item, value, onChange, index)}
            </div>
              {/* 新增修改记录||历史记录 同名dataIndex + _old*/}
              {dataChange && this.blast(item, source, value) && <Popover content={<p style={{ padding: "5px" }}>{this.blast(item, source, value)}</p>} title="变更">
                <Icon style={{ marginLeft: '5px', color: '#1890ff', fontSize: '14px' }} type="info-circle" />
              </Popover>}
            </div>}
            {/* </CustomToolTip> */}
          </FormItem>
        </Col>
      )
    } catch (e) {
      console.log("1092", item.title, e)
    }
  }
  blast(item, source, val) {
    let oldVal = source.get(item.dataIndex + '_old');
    if (oldVal) {
      oldVal = this.getItemActualValue(item, oldVal)
      val = this.getItemActualValue(item, val)
      if (oldVal == val) {
        return false
      } else {
        return `由 ${oldVal || '空'} 变更为 ${val}`
      }
    } else {
      return false;
    }
  }
  renderItem(item, value, onChange, index, childIndex, disabled) {
    try {
      const { source, fromPage, extraConfig, dispatch, dictionary, getItemValue, size, isEdit, lang = "zh" } = this.props
      let { isFilter } = this.state
      // 额外配置
      if (extraConfig && extraConfig[item.dataIndex]) {
        item = { ...item, ...extraConfig[item.dataIndex] }
      }
      //
      if (item.hidden === true || item.hidden === "1") return null

      if (item.referenceDisplayField) {
        value = referenceTranslateCache[item.dataIndex + "_" + value]
      }
      const commonProps = {
        ...{
          lang,
          size: size || "small",
          placeholder: item.placeholder || item.title,
          value: value !== "null" ? value : "",
          disabled: (item.forbidEdit === "1" && isEdit) || disabled || item.disabled === true || item.disabled === "1",
          objectHash: this.state.objectHash,
          employeeSupport: item.employeeSupport,
          align: item.align
        },
        ...item.extraProps
      }
      // if (index === 0) commonProps.ref = 'firstItem';
      let itemSource = item.source || (item.sourceKey && getItemSourceFromCache(item.sourceKey, dictionary, dispatch))
      if (item.sourceKey && !itemSource) {
        console.log(`找不到key为${item.sourceKey}的数据源`)
        return null
      }
      itemSource = this.getReferenceSource(item, itemSource);
      const sourceScenario = this.getReferenceConditions(item, getItemValue, source, childIndex)

      if (!renderItemFunctions(lang, cacheChangeAction)[item.type]) {
        console.log(`暂不支持${item.type}类型配置`)
        return null
      }

      if (item.type === formFieldType.amount.value && item.disabled === "1") {
        commonProps.value = Number(commonProps.value) ? Number(commonProps.value).toFixed(window.numberFixedConst) : commonProps.value
      }


      // if(item.dataIndex=="tableName"&&commonProps.value){
      //   let newlist = itemSource.filter(el=>el.tableName==commonProps.value);
      //   if(newlist[0] && newlist[0].tableType && newlist[0].tableType=="001"){
      //     itemSource = itemSource.filter(el=>el.tableName==commonProps.value||el.tableType=="003")
      //   }else{
      //     itemSource = itemSource.filter(el=>el.tableName==commonProps.value)
      //   }
      // }
      return item.renderFunc
        ? item.renderFunc({
          props: commonProps,
          onChange,
          lang,
          item,
          sourceScenario,
          source: itemSource || [],
          field: item.field,
          affectOther: item.affectFields,
          referenceKey: item.referenceKey,
          customReferenceKey: item.customReferenceKey,
          refreshReferenceFunc: () => this.refreshReference(item),
          editDictionaryFunc: () => this.editDictionary(item),
          filterSwitch: () => {
            this.filterSwitch(item.dataIndex)
          },
          isFilter: isFilter[item.dataIndex]
        })
        : renderItemFunctions(lang)[item.type]({
          props: commonProps,
          onChange,
          source: itemSource || [],
          field: item.field,
          dispatch,
          item,
          fromPage,
          affectOther: item.affectFields,
          sourceScenario,
          referenceKey: item.referenceKey,
          customReferenceKey: item.customReferenceKey,
          refreshReferenceFunc: () => this.refreshReference(item),
          editDictionaryFunc: () => this.editDictionary(item),
          filterSwitch: () => {
            this.filterSwitch(item.dataIndex)
          },
          changeCacheChangeAction: (fun) => { cacheChangeAction = fun },
          isFilter: isFilter[item.dataIndex],
          functionCode: this.props.functionCode
        })
    } catch (e) {
      message.error(`${item.title}配置错误`)
    }
  }

  refreshReference(item) {
    if (item.referenceKey) {
      this.props.dispatch({
        type: "reference/refreshReference",
        payload: item.referenceKey
      })
    }
  }

  editDictionary(item) {
    this.props.dispatch({
      type: "dictionary/changeChangedDicNo",
      payload: {
        key: item.sourceKey,
        title: item.title
      }
    })
  }

  getReferenceConditions(item, getItemValue, source, childIndex) {
    if (item.referenceScenario) {
      return item.referenceScenario
    }

    if (!item.referenceConditions) {
      return {}
    }
    let scenario = {}
    let tableData
    if (childIndex >= 0) {
      tableData = source.getIn([item.tableName, childIndex])
    }
    item.referenceConditions &&
      item.referenceConditions.map(condition => {
        const formField = condition["formField"]
        if (formField && formField.length > 1 && condition.referenceField) {
          if (tableData) {
            if (item.tableName === formField[0]) {
              scenario[condition.referenceField] = tableData.get(formField[1]) || ""
            } else {
              let value =
                (getItemValue &&
                  getItemValue(
                    {
                      tableName: formField[0],
                      dataIndex: formField[1]
                    },
                    source
                  )) ||
                ""
              scenario[condition.referenceField] = value
            }
          } else {
            let value =
              (getItemValue &&
                getItemValue(
                  {
                    tableName: formField[0],
                    dataIndex: formField[1]
                  },
                  source
                )) ||
              ""
            if (!value) {
              value = source.get(formField[1])
            }
            scenario[condition.referenceField] = value
          }
        }
        return true
      })
    return scenario
  }

  getReferenceSource(item, itemSource) {
    const { reference, dispatch } = this.props
    if (item.referenceKey && (item.triggerReference || item.triggerReference === "1")) {
      if (reference.get(item.referenceKey)) {
        itemSource = reference.get(item.referenceKey).get("treeData") || []
        const config = reference.get(item.referenceKey).get("config")
        item.field = config.field
        if (config.field && config.field.id) {
          const referenceIdFieldConfig = _.find(item.affectFields, { referenceField: config.field.id })
          if (referenceIdFieldConfig && referenceIdFieldConfig.formField && referenceIdFieldConfig.formField.length > 1) {
            item.referenceIdField = referenceIdFieldConfig.formField[1]
          }
        }
        // item.type = referenceStyleFormStyleMapping[config.refType] || item.type;
      } else {
        dispatch({
          type: "reference/initReference",
          payload: item.referenceKey
        })
        itemSource = []
      }
    }
    return itemSource
  }

  // 表单——表格上标题设置
  renderTableButton(wrapper, filterTableRecord) {
    const { onTableChange, getItemValue, source, deleteTableRecords, size, lang } = this.props
    let buttonsComponent = null
    //循环按钮
    if (wrapper.buttons && wrapper.buttons.length > 0) {
      buttonsComponent = (
        <Button.Group size="small" style={{ marginLeft: "5px" }}>
          {wrapper.buttons.map((tableButton, index) => {
            return (
              <Button
                size="small"
                key={index}
                onClick={e => {
                  if (tableButton.btnType === buttonType.customPopModal.value) {
                    //自定义弹窗
                    this.props.dispatch({
                      type: "system/visibleCustomPopEdit",
                      payload: {
                        key: tableButton.btnUrl,
                        record: { edit: "-1" }
                      }
                    })
                  }
                  this.props.onButtonClick(e, tableButton)
                }}
              >
                {tableButton.btnName}
              </Button>
            )
          })}
        </Button.Group>
      )
    }
    //无新增
    if ((wrapper.hideActions === true || wrapper.hideActions === "1") && (wrapper.hideEmpty === true || wrapper.hideEmpty === "1")) {
      return (
        <div style={{ display: "flex" }}>
          {lang === "en" ? wrapper.titleEN : wrapper.title}
          {wrapper.error}
          <div>{buttonsComponent}</div>
        </div>
      )
    }
    //全部删除
    let mutableDeleteComponent = null
    if ((wrapper.mutableDelete || wrapper.mutableDelete === "1") && deleteTableRecords) {
      mutableDeleteComponent = (
        <Popconfirm
          onClick={e => {
            e.preventDefault()
            e.stopPropagation()
          }}
          title="确认要删除全部数据？"
          getPopupContainer={(trigger) => trigger.parentNode}
          onConfirm={e => {
            e.preventDefault()
            e.stopPropagation()
            deleteTableRecords(wrapper.tableName)
          }}
          onCancel={e => {
            e.preventDefault()
            e.stopPropagation()
          }}
        >
          <Button size={size || "small"} type="danger" style={{ marginLeft: "5px" }} icon="delete"></Button>
        </Popconfirm>
      )
    }
    //referenceKey弹出选择
    if (wrapper.referenceKey || wrapper.customReferenceKey) {
      const clickProps = {
        onClick: e => {
          wrapper.error = false
          e.preventDefault()
          e.stopPropagation()
          if (wrapper.customReferenceKey) {
            this.props.dispatch({
              type: "system/showCustomPop",
              payload: {
                name: wrapper.customReferenceKey
              }
            })
          } else {
            let extraSearchScenario = this.getReferenceConditions(wrapper, getItemValue, source)
            this.props.dispatch({
              type: "reference/visibleReference",
              payload: {
                id: wrapper.referenceKey,
                item: wrapper,
                selectedMode: selectedModeConst.multiple,
                extraSearchScenario,
                filterTableRecord,
                objectHash: this.state.objectHash
              }
            })
          }
        }
      }
      // 隐藏新增空白行
      if (wrapper.hideEmpty === true || wrapper.hideEmpty === "1") {
        return (
          <div style={{ display: "flex" }}>
            {/* {lang === 'en' ? wrapper.titleEN : wrapper.title}{wrapper.error} */}
            <Button size={size || "small"} icon="plus" type="primary" style={{ height: "auto", padding: "0 10px" }} {...clickProps}>
              新增
            </Button>
            <div>
              {mutableDeleteComponent}
              {buttonsComponent}
            </div>
          </div>
        )
      }
      // 新增空白行
      const menu = (
        <Button.Group size={size || "small"}>
          {!wrapper.hideActions && (
            <Button type="primary" icon="plus" {...clickProps}>
              新增
            </Button>
          )}
          {!wrapper.hideEmpty && (
            <Button
              type="primary"
              onClick={e => {
                e.preventDefault()
                e.stopPropagation()
                onTableChange && onTableChange(wrapper, [wrapper.defaultObj], false, wrapper.newTriggerRef)
              }}
            >
              空白行
            </Button>
          )}
        </Button.Group>
      )
      // const menus = <AntMenu>
      //   <AntMenu.Item>
      //     <a href="" {...clickProps}>选参照</a>
      //   </AntMenu.Item>
      //   <AntMenu.Item>
      //     <a href="" onClick={(e) => {
      //       e.preventDefault();
      //       e.stopPropagation();
      //       onTableChange && onTableChange(wrapper, [wrapper.defaultObj], false, wrapper.newTriggerRef);
      //     }}>空白行</a>
      //   </AntMenu.Item>
      // </AntMenu>
      return (
        <span style={{ display: "flex" }}>
          {menu}

          {/* <Dropdown overlay={menu}>
          <Button style={{ marginLeft: '10px' }} size={size || "small"} icon="plus" type='primary'>添加</Button>
        </Dropdown> */}
          {mutableDeleteComponent}
          {buttonsComponent}
        </span>
      )
    }
    //常规
    return (
      <div style={{ display: "flex" }}>
        <Button
          size={size || "small"}
          icon="plus"
          type="primary"
          style={{ marginRight: "10px" }}
          onClick={() => {
            wrapper.error = false
            onTableChange && onTableChange(wrapper, [wrapper.defaultObj], false, wrapper.newTriggerRef)
          }}
        ></Button>
        {lang === "en" ? wrapper.titleEN : wrapper.title}
        {wrapper.error}
        <div>
          {buttonsComponent}
          {mutableDeleteComponent}
        </div>
      </div>
    )
  }

  getItemActualValue(item, label) {
    const { getItemValue, dispatch, reference, dictionary } = this.props
    let itemSource = item.source || (item.sourceKey && getItemSourceFromCache(item.sourceKey, dictionary, dispatch))
    if (itemSource && itemSource.length > 0) {
      const filterResult = itemSource.filter(({ value }) => value === label)
      label = getValueFromSource(itemSource, label)
    }
    label = getReferenceByItem(reference, item, label, dispatch).value
    return label === "null" ? "-" : label
  }

  components = {
    header: {
      cell: ResizeableTitle
    }
  }

  // tab标签组件绘制
  // reanderTabs(wrapper){
  //   return(
  //     <Tabs defaultActiveKey="1" onChange={callback}>
  //       {
  //         wrapper.map()
  //       }
  //     </Tabs>
  //   )
  // }

  // TODO: 判断关联表的配置对象是否是全隐藏配置
  isNotAllHide(list) {
    let newlist = [];
    list.forEach(item => {
      if (item.children && item.children.length > 0) {
        let nl = item.children.filter(el => el.hide != "1")
        if (nl.length > 0 && !item.hideSubTable) {
          newlist.push(item);
        }
      }
    })
    return newlist
  }
  // 计算配置
  calculateConfig(formulas, type, dataIndex, record) {
    let is = false
    // console.log('formulas', formulas);
    formulas.map(formula => {
      formula.newKeys && formula.newKeys.map(item => {
        if (item.dataIndex[1] == dataIndex
          && item[type] !== undefined
          && wfConditionConst[formula['condition']].operation(record[formula['result'][1]], formula['conValue'])) {
          // console.log('item[type]', dataIndex, type, item[type]);
          if (item[type]) {
            is = true
          }
        }
      })
    })
    return is
  }
  // 表格组件绘制
  renderTable(wrapper) {
    const {
      getItemValue,
      source,
      onTableItemValueChange,
      deleteTable,
      selectedRowKeys,
      onSelectedRowsChange,
      size,
      onChildTableSort,
      lang = "zh"
    } = this.props;
    const { sorter } = this.state
    if (!wrapper.children) {
      return null
    }
    try {
      const value = source.get(wrapper.tableName) || wrapper.defaultValue || List([])
      this.getReferenceSource(wrapper, [])
      let filterTableRecord = []
      const dataSource = value.toJSON()
      if (wrapper.referenceIdField) {
        filterTableRecord = Array.from(dataSource, item => item[wrapper.referenceIdField])
      }
      const displayColumns = [
        {
          title: <FormattedMessage id="locale_index" />,
          width: 45,
          fixed: "left",
          render: record => {
            return <div style={{ width: 28, height: 30, lineHeight: "30px", textAlign: "center" }}>{dataSource.indexOf(record) + 1}</div>
          }
        }
      ]
      let itemRequireds = [];
      wrapper.children.map((item, index) => {
        if (!item.titleCh) {
          item.titleCh = item.title
        }
        item.placeholder = item.placeholder ? item.placeholder : item.title;
        // item.title = <span style={{ color: item.required == '1' ? '#427dce' : '#444' }}> {lang === "en" ? item.titleEN : item.titleCh}</span>
        item.title = lang === "en" ? item.titleEN : item.titleCh
        if (sorter && sorter.sortKey === item.dataIndex) {
          item.sortOrder = sorter.sortOrder
        } else {
          item.sortOrder = ""
        }
        item.key = item.dataIndex
        if (item.hide === true || item.hide === "1" || item.hideInEdit === true || item.hideInEdit === "1") {
          return
        }
        item.onHeaderCell = () => ({
          width: item.width,
          onResize: (e, { size }) => {
            this.preventSort = true
            let totalWidth = wrapper.resizeTotalWidth || wrapper.totalWidth
            totalWidth = totalWidth - (item.width || 0) + size.width
            item.width = size.width
            wrapper.resizeTotalWidth = totalWidth
            this.setState({ update: !this.state.update })
          }
        })
        item.render = (text, record, idx) => {
          let rowSpan = 1
          const index = dataSource.indexOf(record) //获取总数据下标
          if (item.group === "1" && text) {
            let jiequList1 = wrapper.children.filter(el=>el.groupMain=="1");
            //群组且有值
            if(jiequList1.length>0){
              if (index > 0) {
                //第二个数据
                const lastIndex = index - 1 //上一条
                const lastValue = dataSource[lastIndex][item["dataIndex"]] //上一条数据
                if (lastValue === text && record[jiequList1[0].dataIndex]===dataSource[lastIndex][jiequList1[0].dataIndex]) {
                  //上一条数据等于当前数据
                  rowSpan = 0 //不显示
                }
              }
              let newRecordCount = 0
              if (rowSpan !== 0) {
                //显示
                for (let nextIndex = index + 1; nextIndex < dataSource.length; nextIndex++) {
                  //向下循环
                  const nextRecord = dataSource[nextIndex]
                  if (nextRecord[item["dataIndex"]] === text  && record[jiequList1[0].dataIndex]===dataSource[nextIndex][jiequList1[0].dataIndex]) {
                    //值相等时加一
                    newRecordCount++
                  } else {
                    break
                  }
                }

                if(jiequList1.length>0){
                  let newS = dataSource.filter(el=>el[jiequList1[0].dataIndex]== record[jiequList1[0].dataIndex])
                  if(newS.length>(newRecordCount + 1)){
                    rowSpan = newS.length
                  }else{
                    rowSpan = newRecordCount + 1
                  }
                }
                else{
                  rowSpan = newRecordCount + 1
                }
                
              }
            }else{
              if (index > 0) {
                //第二个数据
                const lastIndex = index - 1 //上一条
                const lastValue = dataSource[lastIndex][item["dataIndex"]] //上一条数据
                if (lastValue === text) {
                  //上一条数据等于当前数据
                  rowSpan = 0 //不显示
                }
              }
              let newRecordCount = 0
              if (rowSpan !== 0) {
                //显示
                for (let nextIndex = index + 1; nextIndex < dataSource.length; nextIndex++) {
                  //向下循环
                  const nextRecord = dataSource[nextIndex]
                  if (nextRecord[item["dataIndex"]] === text) {
                    //值相等时加一
                    newRecordCount++
                  } else {
                    break
                  }
                }
                rowSpan = newRecordCount + 1
                
              }
            }
          }
          // table回调
          const onChange = (value, selectItem) => {
            item.error = false
            if (item.errors) {
              const errorIndex = item.errors.indexOf(index)
              errorIndex >= 0 && item.errors.splice(errorIndex, 1)
            }
            if (item.type === "BDMap") {
              let tableNameSource = Object.keys(source.get(item.tableName).toJS())
              const transform = (key) => {
                if (tableNameSource.indexOf(key + '_') > 0) {
                  return key + '_'
                } else {
                  return key
                }
              }
              if (JSON.parse(value)[3]) {
                onTableItemValueChange(JSON.parse(value)[1], { tableName: item.tableName, dataIndex: transform("detailedAddress") }, index)
              }
              //备份 onTableItemValueChange(JSON.parse(value)[1], { tableName: item.tableName, dataIndex: transform("detailedAddress") }, index)
              onTableItemValueChange(JSON.parse(value)[0], { tableName: item.tableName, dataIndex: transform("mapLocation") }, index)
              onTableItemValueChange(JSON.parse(value)[2].split(",")[0], { tableName: item.tableName, dataIndex: transform("province") }, index)
              onTableItemValueChange(JSON.parse(value)[2].split(",")[1], { tableName: item.tableName, dataIndex: transform("city") }, index)
              onTableItemValueChange(JSON.parse(value)[2].split(",")[2], { tableName: item.tableName, dataIndex: transform("area") }, index)
            }
            onTableItemValueChange && onTableItemValueChange(value, item, index, selectItem)
          }
          // 计算公式子表影响当前行配置
          if (wrapper.formulas && wrapper.formulas.length > 0) {
            if (this.calculateConfig(wrapper.formulas, 'hide', item.dataIndex, record)) {
              item.disabled = true;
            } else {
              item.disabled = false;
              item.renderFunc = undefined;
            }
            if (this.calculateConfig(wrapper.formulas, 'required', item.dataIndex, record)) {
              item.itemRequired = true;
              if (itemRequireds.indexOf(`${idx},${item.dataIndex}`) < 0) {
                itemRequireds.push(`${idx},${item.dataIndex}`)
              }
              if (itemRequireds.length > 0) {
                wrapper.itemRequireds = itemRequireds
              }
            } else {
              item.itemRequired = false;
            }
            // console.log('itemRequireds', itemRequireds);

          }
          if (item.disabled === true || item.disabled === "1") {
            if (item.type === "tags") {
              let labelArray = null
              let vLabelArray = []
              if (text) {
                labelArray = text.split(",")
              }
              labelArray &&
                labelArray.map(v => {
                  vLabelArray.push(this.getItemActualValue(item, v) || "-")
                })
              item.renderFunc = ({ props }) => <EllipsisText>{vLabelArray.join()}</EllipsisText>
            } else {
              // style={{ width: item.width - 19 }}
              item.renderFunc = ({ props }) => (
                <EllipsisText>{this.getItemActualValue(item, props.value)}</EllipsisText>
              )
            }
          }

          return {
            props: {
              rowSpan
            },
            children: (
              <FormItem
                key={item.dataIndex}
                style={{ width: item.width - 17 }}
                colon={false}
                validateStatus={item.errors && item.errors.indexOf(index) >= 0 ? "error" : ""}
                required={this.isRequired(item.required) || item.itemRequired}//必填
              >
                {item.itemRequired && <span style={{ zIndex: '10', position: 'absolute', right: '2px', color: 'red' }}>*</span>}
                {/* <CustomToolTip text={item.errMsg} visible={item.errors && item.errors.indexOf(index) >= 0}> */}
                {this.renderItem(item, text, onChange, -1, index, wrapper.forbidEditOld ? !!record.id : false)}
                {/* </CustomToolTip> */}
              </FormItem>
            )
          }
        }
        item.fixed = item.fixed === "none" ? "" : item.fixed
        displayColumns.push(_.cloneDeep(item))
      })

      const deleteColumn = {
        key: "delete",
        fixed: "right",
        width: 40,
        align: "center",
        render: (text, record) => (
          <Popconfirm
            title="您确认要删除此数据?"
            overlayClassName="gouridetanceng"             //增加定义弹层的class名 使用 “overlayClassName”
            placement="leftTop"
            autoAdjustOverflow
            size="small"
            // getPopupContainer={(trigger) => trigger.parentNode}
            onConfirm={() => deleteTable && deleteTable(wrapper.tableName, dataSource.indexOf(record))}
            okText="删除"
            cancelText="取消"
          >
            <Icon className="deleteIcon" type="delete" />
          </Popconfirm>
        )
      }
      const extraColumns = [{}]
      if (wrapper.recordButtons && wrapper.recordButtons.length > 0) {
        extraColumns.push({
          key: "buttons",
          fixed: "right",
          width: wrapper.recordButtons.length * 60,
          align: "center",
          render: (text, record) => (
            <div>
              {wrapper.recordButtons.map((recordButton, index) => {
                return (
                  <a
                    key={index}
                    style={{ marginRight: "5px" }}
                    onClick={e => {
                      e.stopPropagation()
                      e.preventDefault()
                      if (recordButton.btnType === buttonType.customPopModal.value) {
                        this.props.dispatch({
                          type: "system/visibleCustomPopEdit",
                          payload: {
                            key: recordButton.btnUrl,
                            record: { rowSelectedKey: dataSource.indexOf(record) }
                          }
                        })
                        return
                      }
                      this.props.onButtonClick(e, recordButton, { selectedIndex: dataSource.indexOf(record) })
                    }}
                  >
                    {recordButton.btnName}
                  </a>
                )
              })}
            </div>
          )
        })
      }
      if (!wrapper.hideDeleteAction || wrapper.hideDeleteAction === "0") {
        extraColumns.push(deleteColumn)
      }
      let rowSelection = undefined
      if (wrapper.supportBatch === true || wrapper.supportBatch === "1") {
        let RowKeys = []
        if (selectedRowKeys && selectedRowKeys.get(wrapper.tableName)) {
          RowKeys = selectedRowKeys.get(wrapper.tableName)
        } else if (source.get(wrapper.tableName + formChildSelectedRowKeysConst)) {
          if (List.isList(source.get(wrapper.tableName + formChildSelectedRowKeysConst))) {
            RowKeys = source.get(wrapper.tableName + formChildSelectedRowKeysConst).toJS()
          } else {
            RowKeys = source.get(wrapper.tableName + formChildSelectedRowKeysConst) || []
          }
        }
        rowSelection = {
          columnWidth: "35px",
          type: "checkbox",
          selectedRowKeys: RowKeys,
          // (selectedRowKeys && selectedRowKeys.get(wrapper.tableName)) ||
          //   List.isList(source.get(wrapper.tableName + formChildSelectedRowKeysConst))
          //   ? source.get(wrapper.tableName + formChildSelectedRowKeysConst).toJS()
          //   : source.get(wrapper.tableName + formChildSelectedRowKeysConst) || [],
          onChange: selectedRowKeys => {
            onSelectedRowsChange && onSelectedRowsChange(wrapper.tableName, selectedRowKeys)
          }
        }
      }

      const tableScroll = { x: Math.max(wrapper.totalWidth, wrapper.resizeTotalWidth || 0) + 124 }
      if (wrapper.forbidChildYScroll !== "1") {
        tableScroll.y = 850
      }
      return (
        <div>
          {/*{lang === 'en' ? wrapper.titleEN : wrapper.title}{wrapper.error}*/}
          {/* NOTE:表格标题以及新增功能键 */}
          <h3 className="title">{this.renderTableButton(wrapper, filterTableRecord)}</h3>
          <Table
            rowKey={record => dataSource.indexOf(record)}
            bordered={true}
            sortDirections={["descend", "ascend"]}
            rowClassName={(record, index) => (index % 2 == 0 ? "" : "zebraCross")}
            rowSelection={rowSelection}
            pagination={wrapper.forbidPagination ? false : { pageSize: 15 }}
            size={size || "small"}
            components={this.components}
            className={classnames("childTable", wrapper.error ? "errorTable" : "")}
            scroll={tableScroll}
            // locale={{ emptyText: <span></span> }}
            columns={displayColumns.concat(extraColumns)}
            onChange={(pagination, filters, sorter) => {
              if (this.preventSort) {
                this.preventSort = false
                return
              }
              this.setState({ sorter: { sortKey: sorter.field, sortOrder: sorter.order } })
              let newDataSource = _.sortBy(dataSource, sorter.field)
              if (sorter.order === "descend") {
                newDataSource = newDataSource.reverse()
              }
              onChildTableSort && onChildTableSort(wrapper.tableName, newDataSource)
            }}
            dataSource={dataSource}
          />
          <div style={{ height: window.detailDisplayType == "1" ? "65px" : "32px" }}></div>
        </div>
      )
    } catch (e) {
      console.log("1673", e)
    }
  }

  render() {
    const { config, showListType, border, extra, style, extraItemLayout, source, borderClass } = this.props
    if (!Map.isMap(source)) {
      return null
    }
    // 简单模式
    let configC = _.clone(config)

    let listForm = configC.filter(el => Array.isArray(el))
    let arr = []
    listForm.forEach(aaa => {
      aaa.forEach(bbb => {
        if (bbb.hide === "1") {
          arr.push(bbb.dataIndex)
        }
      })
    })
    window.localStorage.setItem("hideList", JSON.stringify(arr))
    let obgForm = configC.filter(el => !Array.isArray(el))
    this.isSetNoLabel(listForm)
    const borderStyle = borderClass ? borderClass : border ? "row" : ""

    const { TabPane } = Tabs
    // 表单内容渲染
    return (
      <Form className="formWrapper" style={style} onSubmit={e => this.onSubmitForm(e)}>
        {showListType == "1" ? (
          <div>
            {configC.map((wrapper, index) => (
              <div className="cl">
                {wrapper.filter(el => el.hide != "1").length > 0 && <Row key={index} className={borderStyle}>
                  {Array.isArray(wrapper) ? wrapper.map(this.renderItemWrapper) : this.renderTable(wrapper)}
                  <Col {...itemLayout} {...extraItemLayout}>
                    {extra && extra}
                  </Col>
                </Row>
                }
              </div>
            ))}
          </div>
        ) : (
          <div>
            {listForm.length > 0 &&
              listForm.map((wrapper, index) => (
                <div key={index}>
                  {wrapper.filter(el => el.hide != "1").length > 0 && <Row
                    style={{ borderBottom: listForm.length - 1 == index ? "none" : "1px dashed #dedede" }}
                    className={borderStyle}
                  >
                    {wrapper.map(this.renderItemWrapper)}
                    <Col {...itemLayout} {...extraItemLayout}>
                      {extra && extra}
                    </Col>
                  </Row>
                  }
                </div>
              ))}
            {obgForm.length > 0 && (
              <div style={{ padding: "10px" }}>
                <Tabs defaultActiveKey={1}>
                  {this.isNotAllHide(obgForm).map((wrapper, index) => (
                    <TabPane tab={wrapper.title} key={index}>
                      {this.renderTable(wrapper)}
                    </TabPane>
                  ))}
                </Tabs>
              </div>
            )}
          </div>
        )}
        <input type="submit" value="Submit" hidden />
      </Form>
    )
  }
}

export default AutogenerationForm
