
import "./index1.less"
import "rc-color-picker/assets/index.css"
import React from "react"
import ColorPicker from "rc-color-picker"
import {
    Form,
    Input,
    DatePicker,
    Radio,
    Checkbox,
    InputNumber,
    Button,
    Icon,
    TimePicker,
    Tooltip,
} from "antd"
import {
    selectedModeConst,
    dateFormatConst,
    timeFormatConst,
    dateTimeFormatConst,
} from "../../common/const"
import { getBeginAndEndDate, returnMinusWidth } from "../../utils"
import moment from "moment"
import CustomTreeSelect from "../CustomTreeSelect"
import UEditor from "../UEditor"
import TinyMCE from "../TinyMCE"
import CheckableTags from "../CheckableTags"
import PicturesWall from "../PicturesWall"
import Division from "../Division"
import UploadThumb from "../UploadThumb"
import _ from "lodash"
import CustomSelect from "../CustomSelect"
import classnames from "classnames"
import EmployeeSelectorLabel from "../EmployeeSelector/EmployeeSelectorLabel"
import UploadFile from "../UploadFile"
import LoginThumb from "../LoginThumb"
import BDMap from "../BDMap"
import Geocoder from "../Geocoder"
import CustomSearch from "../CustomSearch"
import CustomDateRange from "../CustomDateRange"
import CustomDateTime from "../CustomDateTime"
import CustomWeekRange from "../CustomWeekRange"
import PriceRange from "../PriceRange"
import ICinfo from "../ICinfo"
import AsyncTextArea from "../AsyncTextArea"
import AsyncInput from "../AsyncInput"
import YearPicker from "../YearPicker"
import { quickButtons } from "../../utils/quick-button"
import { formatMessage } from "../../intl"
const RadioGroup = Radio.Group
const renderItemFunctions = (lang, cacheChangeAction) => ({
    text: ({ props, onChange }) => <AsyncInput {...props} onChange={value => onChange(value)} />,
    asyncText: ({ props, onChange }) => <AsyncInput {...props} async onChange={value => onChange(value)} />,
    // <Input {...props} onChange={e => onChange(e.target.value)} />,
    passWord: ({ props, onChange }) => <Input {...props} type="password" onChange={e => onChange(e.target.value)} />,
    //  <InputNumber {...props} type={"passWord"} onChange={value => onChange(value)} />,
    number: ({ props, onChange }) => (
        <InputNumber {...props} style={{ width: "100%" }} onChange={value => value !== props.value && onChange(value)} />
    ),
    colorPicker: ({ props, onChange }) => {
        return (
            <div className={"colorPicker"}>
                <ColorPicker
                    animation="slide-up"
                    color={props.value || "#fff"}
                    onChange={({ color }) => {
                        console.log("color===>", color)
                        onChange(color)
                    }}
                />
                {props.value && (
                    <Icon
                        type="close"
                        onClick={() => {
                            onChange("")
                        }}
                    />
                )}
            </div>
        )
    },
    amount: ({ props, onChange }) => {
        // 数字转格式千分位
        return (
            <div className={props.align || "left"}>
                <InputNumber
                    {...props}
                    style={{ width: "100%" }}
                    // formatter={value=>qinfen(value)}
                    formatter={value => `${value}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}
                    // parser={value => value.replace(/\$\s?|(,*)/g, '')}
                    precision={window.numberFixedConst}
                    onChange={value => {
                        value !== props.value && onChange(value)
                    }}
                />
            </div>
        )
    },
    select: ({
        props,
        fromPage,
        onChange,
        source,
        field,
        sourceScenario,
        affectOther,
        item,
        refreshReferenceFunc,
        editDictionaryFunc,
        filterSwitch,
        isFilter
    }) => {
        const newSource = sourceScenario ? _.filter(source, sourceScenario) : source
        return (
            <div style={{ width: "100%" }}>
                <CustomSelect
                    style={{ width: `calc(100% - ${returnMinusWidth(item)}px)` }}
                    needSelectItem={affectOther}
                    filter={item.filter ? item.filter : false}
                    isFilter={isFilter}
                    selectProps={props}
                    field={field}
                    fromPage={fromPage}
                    source={newSource}
                    onChange={(value, selectItem) => onChange(value, selectItem)}
                />
                {item.filter && item.filter.filterReg && (
                    <Tooltip placement="top" title={item.filter.filterName}>
                        <Icon
                            type="filter"
                            style={isFilter ? { color: "#1890ff" } : {}}
                            className="formSelectRefreshIcon"
                            onClick={() => filterSwitch()}
                        />
                    </Tooltip>
                )}
                {item.referenceRefresh === "1" && <Icon type="reload" className="formSelectRefreshIcon" onClick={() => refreshReferenceFunc()} />}
                {item.editDictionary == "1" && <Icon type="setting" className="formSelectRefreshIcon" onClick={() => editDictionaryFunc()} />}
            </div>
        )
    },
    multipleSelect: ({
        props,
        onChange,
        source,
        field,
        sourceScenario,
        affectOther,
        item,
        refreshReferenceFunc,
        editDictionaryFunc,
        filterSwitch,
        isFilter
    }) => {
        const newSource = sourceScenario ? _.filter(source, sourceScenario) : source

        return (
            <div style={{ width: "100%", height: 32, display: "flex", alignItems: "center" }}>
                <CustomSelect
                    style={{ width: `calc(100% - ${returnMinusWidth(item)}px)` }}
                    needSelectItem={affectOther}
                    selectProps={{ ...props, mode: "multiple", value: props.value ? props.value.split(",") : [] }}
                    field={field}
                    filter={item.filter ? item.filter : false}
                    isFilter={isFilter}
                    source={newSource}
                    onChange={(value, selectItem) => {
                        onChange(value.join(), selectItem)
                    }}
                />
                {item.filter && item.filter.filterReg && (
                    <Tooltip placement="top" title={item.filter.filterName}>
                        <Icon
                            type="filter"
                            style={isFilter ? { color: "#1890ff" } : {}}
                            className="formSelectRefreshIcon"
                            onClick={() => filterSwitch()}
                        />
                    </Tooltip>
                )}
                {item.referenceRefresh === "1" && <Icon type="reload" className="formSelectRefreshIcon" onClick={() => refreshReferenceFunc()} />}
                {item.editDictionary === "1" && <Icon type="setting" className="formSelectRefreshIcon" onClick={() => editDictionaryFunc()} />}
            </div>
        )
    },
    date: ({ props, onChange }) => {
        props.value = props.value && moment(props.value)
        return (
            <DatePicker disabled={props.disabled} style={{ width: "100%" }} {...props} onChange={(moment, dateString) => onChange(dateString)} />
        )
    },
    time: ({ props, onChange }) => {
        props.value = props.value && moment(props.value, "HH:mm:ss")
        return (
            <TimePicker {...props} disabled={props.disabled} style={{ width: "100%" }} onChange={(moment, timeString) => onChange(timeString)} />
        )
    },
    dateTime: ({ props, onChange }) => {
        props.value = props.value && moment(props.value)
        return (
            <DatePicker
                disabled={props.disabled}
                showTime
                format={dateTimeFormatConst}
                style={{ width: "100%" }}
                {...props}
                onChange={(moment, dateString) => onChange(dateString)}
            />
        )
    },
    customWeekRange: ({ props, onChange }) => {
        let value = []
        if (props.value) {
            try {
                value = JSON.parse(props.value)
            } catch (e) {
                console.error('customWeekRange数据转换', e)
            }
        }

        return (
            <CustomWeekRange
                {...props}
                disabled={props.disabled}
                value={value}
                placeholder={[formatMessage({ id: "datepicker.start", lang }), formatMessage({ id: "datepicker.end", lang })]}
                onChange={data => {
                    onChange(data || "")
                }}
            />
        )
    },
    priceRange: ({ props, onChange }) => {
        let value = []
        if (props.value) {
            try {
                value = JSON.parse(props.value)
            } catch (e) {
                console.error('priceRange数据转换', e)
            }
        }
        return (
            <PriceRange
                {...props}
                value={value}
                placeholder={["￥", "￥"]}
                onChange={data => {
                    onChange(data || "")
                }}
            />
        )
    },
    customDateRange: ({ props, onChange }) => {
        const value = []
        if (props.value) {
            try {
                const data = JSON.parse(props.value)
                if (data.length >= 2) {
                    value.push(data[0])
                    value.push(data[1])
                    value.push(data[2] || "")
                }
            } catch (e) {
                console.error('自定义日期数据转换1', e)
            }
        }

        return (
            <CustomDateRange
                {...props}
                disabled={props.disabled}
                format={dateFormatConst}
                value={value}
                placeholder={[formatMessage({ id: "datepicker.start", lang }), formatMessage({ id: "datepicker.end", lang })]}
                onChange={data => {
                    onChange(data || "")
                }}
            />
        )
    },
    customDateTime: ({ props, onChange }) => {
        let value = "",
            format = 0
        if (props.value) {
            try {
                value = props.value
            } catch (e) {
                console.error('customDateTime数据转换', e)
            }
        }
        if (props.format) {
            try {
                if (props.format < 4) {
                    format = Number(props.format)
                }
            } catch (e) {
                console.error('自定义日期数据转换', e)
            }
        }

        return (
            <CustomDateTime
                {...props}
                disabled={props.disabled}
                format={format}
                value={value}
                style={{ width: "100%" }}
                size={props.size}
                //placeholder={[formatMessage({ id: 'datepicker.start', lang }), formatMessage({ id: 'datepicker.end', lang })]}
                onChange={data => {
                    onChange(data || "")
                }}
            />
        )
    },
    customDateTime1: ({ props, onChange }) => {
        let value = "",
            format = 1
        if (props.value) {
            try {
                value = props.value
            } catch (e) {
                console.error('自定义日期1数据转换', e)
            }
        }

        return (
            <CustomDateTime
                {...props}
                disabled={props.disabled}
                format={format}
                value={value}
                style={{ width: "100%" }}
                size={props.size}
                //placeholder={[formatMessage({ id: 'datepicker.start', lang }), formatMessage({ id: 'datepicker.end', lang })]}
                onChange={data => {
                    onChange(data || "")
                }}
            />
        )
    },
    customDateTime2: ({ props, onChange }) => {
        let value = "",
            format = 2
        if (props.value) {
            try {
                value = props.value
            } catch (e) {
                console.error('自定义日期2数据转换', e)
            }
        }

        return (
            <CustomDateTime
                {...props}
                disabled={props.disabled}
                format={format}
                value={value}
                style={{ width: "100%" }}
                size={props.size}
                //placeholder={[formatMessage({ id: 'datepicker.start', lang }), formatMessage({ id: 'datepicker.end', lang })]}
                onChange={data => {
                    onChange(data || "")
                }}
            />
        )
    },
    customDateTime3: ({ props, onChange }) => {
        let value = "",
            format = 3
        if (props.value) {
            try {
                value = props.value
            } catch (e) {
                console.error('自定义日期3数据转换', e)
            }
        }

        return (
            <CustomDateTime
                {...props}
                disabled={props.disabled}
                format={format}
                value={value}
                style={{ width: "100%" }}
                size={props.size}
                //placeholder={[formatMessage({ id: 'datepicker.start', lang }), formatMessage({ id: 'datepicker.end', lang })]}
                onChange={data => {
                    onChange(data || "")
                }}
            />
        )
    },
    dateRange: ({ props, onChange }) => {
        const value = []
        if (props.value) {
            try {
                const date = JSON.parse(props.value)
                if (date.length >= 2) {
                    value.push(moment(date[0]))
                    value.push(moment(date[1]))
                }
            } catch (e) {
                console.error('dateRange数据转换', e)
            }
        }

        const ranges = quickButtons.reduce((prev, current) => {
            prev[formatMessage({ id: current.label, lang })] = getBeginAndEndDate(current.value)
            return prev
        }, {})
        return (
            <DatePicker.RangePicker
                {...props}
                disabled={props.disabled}
                style={{ width: "100%" }}
                format={dateFormatConst}
                value={value}
                ranges={ranges}
                placeholder={[formatMessage({ id: "datepicker.start", lang }), formatMessage({ id: "datepicker.end", lang })]}
                onChange={(value, date) => {
                    if (date && date[0] && date[1]) {
                        onChange(JSON.stringify(date))
                    } else {
                        onChange("")
                    }
                }}
            />
        )
    },
    dateTimeRange: ({ props, onChange }) => {
        const value = []
        if (props.value) {
            const date = JSON.parse(props.value)
            if (date.length === 2) {
                value.push(moment(date[0]))
                value.push(moment(date[1]))
            }
        }
        return (
            <DatePicker.RangePicker
                {...props}
                disabled={props.disabled}
                showTime={{ format: timeFormatConst }}
                format={dateTimeFormatConst}
                value={value}
                style={{ width: "100%" }}
                placeholder={[formatMessage({ id: "datepicker.start", lang }), formatMessage({ id: "datepicker.end", lang })]}
                onChange={(value, date) => {
                    if (date && date[0] && date[1]) {
                        onChange(JSON.stringify(date))
                    } else {
                        onChange("")
                    }
                }}
            />
        )
    },
    monthPicker: ({ props, onChange }) => {
        props.value = props.value && moment(props.value)
        return (
            <DatePicker.MonthPicker
                disabled={props.disabled}
                style={{ width: "100%" }}
                {...props}
                onChange={(moment, dateString) => onChange(dateString)}
            />
        )
    },
    yearPicker: ({ props, onChange }) => {
        props.value = props.value && moment(props.value)
        return (
            //选择年从新封装
            <YearPicker onChange={onChange} params={props} />
            // <DatePicker
            //   mode="year"
            //   format="YYYY"
            //   disabled={props.disabled}
            //   style={{ width: "100%" }}
            //   {...props}
            //   onPanelChange={value => {
            //     onChange(value && value.format("YYYY"))
            //   }}
            // />
        )
    },
    textArea: ({ props, onChange }) => <AsyncTextArea {...props} onChange={val => onChange(val)} />,
    treeSelect: ({ props, onChange, source, field, sourceScenario, affectOther, item, refreshReferenceFunc, filterSwitch, isFilter }) => {
        return (
            <div style={{ width: "100%" }}>
                <CustomTreeSelect
                    antProps={{ ...props }}
                    style={{ width: `calc(100% - ${returnMinusWidth(item)}px)` }}
                    needSelectItem={affectOther}
                    disabled={props.disabled}
                    treeField={
                        field
                            ? { id: field.value, name: field.label, pid: field.pid }
                            : {
                                id: "value",
                                name: "label",
                                pid: "pid"
                            }
                    }
                    treeExpand={item.treeExpand}
                    filter={item.filter ? item.filter : false}
                    isFilter={isFilter}
                    onChange={(value, selectItem) => {
                        onChange(value, selectItem)
                    }}
                    sourceScenario={sourceScenario}
                    placeholder={props.placeholder}
                    value={props.value}
                    data={source}
                />
                {item.filter && item.filter.filterReg && (
                    <Tooltip placement="top" title={item.filter.filterName}>
                        <Icon
                            type="filter"
                            style={isFilter ? { color: "#1890ff" } : {}}
                            className="formSelectRefreshIcon"
                            onClick={() => filterSwitch()}
                        />
                    </Tooltip>
                )}
                {item.referenceRefresh === "1" && <Icon type="reload" className="formSelectRefreshIcon" onClick={() => refreshReferenceFunc()} />}
            </div>
        )
    },
    treeSelectForbidParent: ({
        props,
        onChange,
        source,
        field,
        sourceScenario,
        affectOther,
        item,
        refreshReferenceFunc,
        editDictionaryFunc,
        filterSwitch,
        isFilter
    }) => {
        return (
            <div style={{ width: "100%" }}>
                <CustomTreeSelect
                    antProps={{ ...props }}
                    forbitParent
                    style={{ width: `calc(100% - ${returnMinusWidth(item)}px)` }}
                    needSelectItem={affectOther}
                    disabled={props.disabled}
                    treeField={
                        field
                            ? { id: field.value, name: field.label, pid: field.pid }
                            : {
                                id: "value",
                                name: "label",
                                pid: "pid"
                            }
                    }
                    treeExpand={item.treeExpand}
                    filter={item.filter ? item.filter : false}
                    isFilter={isFilter}
                    onChange={(value, selectItem) => {
                        onChange(value, selectItem)
                    }}
                    sourceScenario={sourceScenario}
                    placeholder={props.placeholder}
                    value={props.value}
                    data={source}
                />
                {item.filter && item.filter.filterReg && (
                    <Tooltip placement="top" title={item.filter.filterName}>
                        <Icon
                            type="filter"
                            style={isFilter ? { color: "#1890ff" } : {}}
                            className="formSelectRefreshIcon"
                            onClick={() => filterSwitch()}
                        />
                    </Tooltip>
                )}
                {item.referenceRefresh === "1" && <Icon type="reload" className="formSelectRefreshIcon" onClick={() => refreshReferenceFunc()} />}
                {item.editDictionary === "1" && <Icon type="setting" className="formSelectRefreshIcon" onClick={() => editDictionaryFunc()} />}
            </div>
        )
    },
    multipleTreeSelect: ({
        props,
        onChange,
        source,
        field,
        sourceScenario,
        affectOther,
        item,
        refreshReferenceFunc,
        editDictionaryFunc,
        changeFilter,
        filterSwitch,
        isFilter
    }) => {
        let actualValue = []
        let displayValue = ""
        if (props.value) {
            const valueObj = JSON.parse(props.value)
            if (valueObj.value) {
                actualValue = valueObj.value
            }
            if (valueObj.displayValue) {
                displayValue = valueObj.displayValue
            }
        }
        return (
            <div style={{ width: "100%" }}>
                <CustomTreeSelect
                    antProps={{ ...props, treeCheckable: true }}
                    multiple
                    forbitParent={item.forbitParent}                //多选树结构增加不选择父级
                    style={{ width: `calc(100% - ${returnMinusWidth(item, true)}px)` }}
                    needSelectItem={affectOther}
                    treeExpand={item.treeExpand}
                    disabled={props.disabled}
                    filter={item.filter ? item.filter : false}
                    isFilter={isFilter}
                    treeField={
                        field
                            ? { id: field.value, name: field.label, pid: field.pid }
                            : {
                                id: "value",
                                name: "label",
                                pid: "pid"
                            }
                    }
                    onChange={value => {
                        onChange(value ? JSON.stringify(value) : "")
                    }}
                    sourceScenario={sourceScenario}
                    placeholder={props.placeholder}
                    value={actualValue}
                    data={source}
                />
                {item.filter && item.filter.filterReg && (
                    <Tooltip placement="top" title={item.filter.filterName}>
                        <Icon
                            type="filter"
                            style={isFilter ? { color: "#1890ff" } : {}}
                            className="formSelectRefreshIcon"
                            onClick={() => filterSwitch()}
                        />
                    </Tooltip>
                )}
                {item.referenceRefresh === "1" && <Icon type="reload" className="formSelectRefreshIcon" onClick={() => refreshReferenceFunc()} />}
                {item.editDictionary === "1" && <Icon type="setting" className="formSelectRefreshIcon" onClick={() => editDictionaryFunc()} />}
                {item.sourceKey && (
                    <Tooltip title={displayValue}>
                        <Icon type="info-circle" className="formSelectRefreshIcon" onClick={() => editDictionaryFunc()} />
                    </Tooltip>
                )}
            </div>
        )
    },
    popupTreeSelect: ({ props, onChange, dispatch, referenceKey, item, sourceScenario, changeCacheChangeAction }) => {
        return (
            <CustomSearch
                value={props.value}
                disabled={props.disabled}
                onClear={() => onChange("", {})}
                onSearch={value => {
                    if (props.disabled) {
                        return
                    }
                    // cacheChangeAction = onChange
                    changeCacheChangeAction(onChange);
                    dispatch({
                        type: "reference/showReferenceWithFuzzy",
                        payload: {
                            id: referenceKey,
                            item,
                            selectedMode: selectedModeConst.single,
                            extraSearchScenario: sourceScenario,
                            objectHash: props.objectHash,
                            fuzzy: value,
                            singleTree: true
                        }
                    })
                }}
            />
        )
    },
    radio: ({ props, onChange, source, sourceScenario, field }) => {
        const newSource = sourceScenario ? _.filter(source, sourceScenario) : source
        field = field || { value: "value", label: "label" }
        return (
            <div style={{ position: "relative", top: "-3px" }}>
                <RadioGroup
                    disabled={props.disabled}
                    onChange={e => onChange(e.target.value, _.find(newSource, { [field.value]: e.target.value }))}
                    {...props}
                >
                    {newSource.map((item, index) => (
                        <Radio key={index} value={item[field.value]}>
                            {item[field.label]}
                        </Radio>
                    ))}
                </RadioGroup>
            </div>
        )
    },
    checkbox: ({ props, onChange }) => {
        return <Checkbox disabled={props.disabled} checked={props.value === "1"} onChange={e => onChange(e.target.checked ? "1" : "0")} />
    },
    checkboxBool: ({ props, onChange }) => {
        return <Checkbox disabled={props.disabled} checked={props.value} onChange={e => onChange(e.target.checked)} />
    },
    popup: ({ props, onChange, dispatch, referenceKey, item, sourceScenario, changeCacheChangeAction }) => {
        return (
            <CustomSearch
                value={props.value}
                disabled={props.disabled}
                onClear={() => {
                    if (props.disabled) {
                        return
                    }
                    onChange("", {})
                }}
                referenceKey={referenceKey}
                item={item}
                objectHash={props.objectHash}
                matching={() => {
                    changeCacheChangeAction(onChange);
                }}
                onSearch={value => {
                    if (props.disabled) {
                        return
                    }
                    // cacheChangeAction = onChange
                    changeCacheChangeAction(onChange);
                    dispatch({
                        type: "reference/showReferenceWithFuzzy",
                        payload: {
                            id: referenceKey,
                            item,
                            selectedMode: selectedModeConst.single,
                            extraSearchScenario: sourceScenario,
                            objectHash: props.objectHash,
                            fuzzy: value
                        }
                    })
                }}
            />
        )
    },
    popupMultiple: ({ props, onChange, dispatch, referenceKey, item, sourceScenario, changeCacheChangeAction }) => {
        return (
            <CustomSearch
                value={props.value}
                disabled={props.disabled}
                // onClear={() => onChange("", {})}
                onClear={() => {
                    if (props.disabled) {
                        return
                    }
                    onChange("", {})
                }}
                referenceKey={referenceKey}
                item={item}
                objectHash={props.objectHash}
                matching={() => {
                    changeCacheChangeAction(onChange);
                }}
                onSearch={value => {
                    if (props.disabled) {
                        return
                    }
                    // cacheChangeAction = onChange
                    changeCacheChangeAction(onChange);
                    dispatch({
                        type: "reference/showReferenceWithFuzzy",
                        payload: {
                            id: referenceKey,
                            item,
                            selectedMode: selectedModeConst.multiple,
                            extraSearchScenario: sourceScenario,
                            objectHash: props.objectHash,
                            fuzzy: value
                        }
                    })
                }}
            />
        )
    },
    popupSync: ({ props, onChange, dispatch, referenceKey, item, sourceScenario, changeCacheChangeAction }) => {
        return (
            <CustomSearch
                value={props.value}
                manualChange={1}
                disabled={props.disabled}
                onClear={() => onChange("", { action: "clear" })}
                referenceKey={referenceKey}
                item={item}
                objectHash={props.objectHash}
                matching={() => {
                    changeCacheChangeAction(onChange);
                }}
                onSearch={value => {
                    if (props.disabled) {
                        return
                    }
                    // cacheChangeAction = onChange
                    changeCacheChangeAction(onChange);
                    dispatch({
                        type: "reference/showReferenceWithFuzzy",
                        payload: {
                            id: referenceKey,
                            item,
                            selectedMode: selectedModeConst.single,
                            extraSearchScenario: sourceScenario,
                            objectHash: props.objectHash,
                            fuzzy: value
                        }
                    })
                }}
                onManualChange={value => {
                    onChange(value, { action: "clear" })//清除关联项
                }}
            />
        )
    },
    popupU8: ({ props, onChange, dispatch, referenceKey, item, sourceScenario }) => {
        return (
            <CustomSearch
                value={props.value}
                manualChange={1}
                disabled={props.disabled}
                onClear={() => onChange("", { action: "clear" })}
                onSearch={value => {
                    //console.log(props,  referenceKey, item, sourceScenario);
                    //u8新增的类型 针对u8自定义列表
                    if (props.disabled) {
                        return
                    }
                    onChange("", { action: "showModal", data: item, value: value })
                }}
                onManualChange={value => {
                    onChange(value, {})
                }}
            />
        )
    },

    htmlEditor: ({ props, onChange, item }) => (
        <div>
            <h3 className="title">{item.title}</h3>
            <UEditor content={props.value} onChange={content => onChange(content)} />
        </div>
    ),
    tinyMCE: ({ props, onChange, item, functionCode }) => (
        <div>
            <h3 className="title">{item.title}</h3>
            <TinyMCE functionCode={functionCode} value={props.value} onChange={content => onChange(content)}></TinyMCE>
        </div>
    ),
    tags: ({ props, onChange, source, field, sourceScenario }) => {
        const newSource = sourceScenario ? _.filter(source, sourceScenario) : source
        return (
            <CheckableTags source={newSource} field={field} value={props.value} disabled={props.disabled} onChange={value => onChange(value)} />
        )
    },
    picturesWall: ({ props, onChange, functionCode }) => {
        var pictures = []
        if (props.value) {
            pictures = props.value.split(",")
        }
        return (
            <PicturesWall
                max={9}
                disabled={props.disabled}
                defaultFileList={pictures}
                functionCode={functionCode}
                uploadSuccess={file => {
                    pictures.push(file)
                    onChange(pictures.toString())
                }}
                deleteFile={file => {
                    pictures.splice(file, 1)
                    onChange(pictures.toString())
                }}
            />
        )
    },
    division: ({ props, onChange, functionCode }) => {
        let value = []
        if (props.value) {
            let temp = JSON.parse(props.value)[0]
            value = temp.split(",")
        }
        return <Division functionCode={functionCode} disabled={props.disabled} value={value} onChange={value => onChange(value)} />
    },
    onlyCamera: ({ props, onChange, functionCode }) => {
        var pictures = []
        if (props.value) {
            pictures = props.value.split(",")
        }
        return (
            <PicturesWall
                max={9}
                disabled={props.disabled}
                defaultFileList={pictures}
                functionCode={functionCode}
                uploadSuccess={file => {
                    pictures.push(file)
                    onChange(pictures.toString())
                }}
                deleteFile={file => {
                    pictures.splice(file, 1)
                    onChange(pictures.toString())
                }}
            />
        )
    },
    thumb: ({ props, onChange, functionCode }) => {
        return <UploadThumb functionCode={functionCode} disabled={props.disabled} value={props.value} onChange={value => onChange(value)} />
    },
    employeeSelector: ({ props, onChange }) => {
        return (
            <div>
                <EmployeeSelectorLabel
                    edit
                    employeeSupport={props.employeeSupport}
                    disabled={props.disabled}
                    value={props.value}
                    onSelectEnd={value => onChange(value)}
                />
            </div>
        )
    },
    loginThumb: ({ props, onChange, functionCode }) => {
        var pictures = []
        if (props.value) {
            pictures = props.value.split(",")
        }
        return (
            <LoginThumb
                max={1}
                disabled={props.disabled}
                defaultFileList={pictures}
                functionCode={functionCode}
                uploadSuccess={file => {
                    pictures.push(file)
                    onChange(pictures.toString())
                }}
                deleteFile={file => {
                    pictures.splice(file, 1)
                    onChange(pictures.toString())
                }}
            />
        )
    },
    uploadFile: ({ props, onChange, functionCode }) => {
        let fileList = []
        if (props.value) {
            fileList = JSON.parse(props.value)
        }
        return <UploadFile functionCode={functionCode} value={fileList} onChange={newFileList => onChange(JSON.stringify(newFileList))} />
    },
    BDMap: ({ props, onChange, item }) => {
        // let value;
        // if (props.value) {
        //   value = JSON.parse(props.value)[0];
        // }
        //console.log('BDMap value',props.value);
        return <BDMap value={props.value} configItem={item} onChange={value => onChange(value)} />
    },
    Geocoder: ({ props, onChange }) => {
        let value
        if (props.value) {
            value = JSON.parse(props.value)[0]
        }
        return <Geocoder value={value} onChange={value => onChange(value)} />
    },
    customPopup: ({ props, customReferenceKey, dispatch }) => {
        return (
            <div>
                <span>{props.value}</span>
                <Button
                    size="small"
                    onClick={() => {
                        if (props.disabled) {
                            return
                        }
                        dispatch({
                            type: "system/visibleCustomPopEdit",
                            payload: {
                                key: customReferenceKey,
                                record: { edit: "1" }
                            }
                        })
                    }}
                >
                    选择
                </Button>
            </div>
        )
    },
    ICinfo: ({ props, onChange, source, functionCode, affectOther, field, sourceScenario }) => {
        const newSource = sourceScenario ? _.filter(source, sourceScenario) : source
        return (
            <ICinfo
                source={newSource}
                needSelectItem={affectOther}
                field={field}
                functionCode={functionCode}
                value={props.value}
                disabled={props.disabled}
                onChange={(value, selectItem) => onChange(value, selectItem)}
            />
        )
    },
    wechatIdQR: ({ props, onChange }) => {
        return <Input {...props} onChange={e => onChange(e.target.value)} />
    },
    marketIdQR: ({ props, onChange }) => {
        return <Input {...props} onChange={e => onChange(e.target.value)} />
    }
})

export default renderItemFunctions