import React, { Component } from "react"
import "./index.less"
import { Button, Radio, Menu, Dropdown, DatePicker } from "antd"
import BBox from "./BBox"
import WBox from "./WBox"
import MBox from "./MBox"
import { connect } from "react-redux"
import TimeTool from "./tool/index"
import _ from "lodash"
import richeng from "./data/richeng.json"
import shishi from "./data/shishi.json"
import moment from "moment"
import { request } from "./../../utils/request";
import AutogenerationForm from "./../AutoGenerationForm"
import { Map } from "immutable";

const ButtonGroup = Button.Group

const { MonthPicker, RangePicker, WeekPicker } = DatePicker

@connect((state, index) => {
  const basicInfo = state.get("system").get("basicInfo")
  const calendarState = state.get("system").get("calendarState")
  return {
    basicInfo,
    calendarState
  }
})
export default class Calendar extends Component {
  constructor(props) {
    super(props)
    this.state = {
      mode: "d",
      today: null,
      weekDay: null,
      weekFirst: null,
      weekLast: null,
      year: null,
      mouth: null,
      mouthFirst: null,
      mouthLast: null,
      weekTopList: [],
      mouthList: [],
      richengList: [],
      shishiList: [],
      typesList: [
        { title: "日程计划", functionCode: "", color: "#34b30b" }
        // { title: "实施计划", functionCode: "", color: "#0089ff" }
      ],
      peoList:[],
      configs:[[
        {
          dataIndex: 'userName',
          title: '选择参与人员',
          type: 'multipleTreeSelect',
          triggerReference: true,
          // forbitParent:true,
          referenceKey: 'f6453658d405413da7865bb454abf2ee',
          field:{'label':'name','value':'code','pid':'pcode'},
          width:24,
          treeExpand:"openEnd",
          filter: { filterName: "包含停用", filterColumn: "ynEnable", filterReg: "/1/g" },
        }
      ]],
      values:Map({}),
      nowChoBeignD:[],
      nowChoEndD:[]
    }
  }

  componentDidMount() {
    this.getFormConfig();
    this.getData();
  }

  componentWillUpdate(newprop){
    if((newprop.detailLoad!=this.props.detailLoad)||(newprop.list.listConfig!=this.props.list.listConfig)){
      this.getFormConfig();
      this.getData();
    }
  
  }

  getFormConfig(){
    request('/sysCache/getCacheDictionary',{key:"036"}).then(res=>{
      this.setState({peoList:res.result})
    })
  }

  getData(){
    const { information, calendarState } = this.props
    const shijian = _.cloneDeep(information)
    const { today } = shijian
    const { mode } = this.state

    //  页面刚进来初始化,没有redux存储状态数据的今天today是标准格式
    let week = TimeTool.getWeek(today)
    let yearN = today.getFullYear()
    let mouthN = today.getMonth() + 1
    let weekB = today.getTime() - 24 * parseInt(parseInt(week) - 1) * 60 * 60 * 1000 //周第一天
    let weekE = today.getTime() + 24 * parseInt(7 - parseInt(week)) * 60 * 60 * 1000 //周最后一天
    let beignD = [];
    let endD = [];
    //如果有保存状态 today 是时间戳模式
    if (Object.getOwnPropertyNames(calendarState).length != 0) {
      // 将扎状态先存储到state中然后去拿取数据
      this.setState(calendarState, () => {
        const { today, weekDay, mode, weekFirst, weekLast, mouthList } = this.state
        console.log("当前数据", this.state)
        
        if (mode == "d") {
          let todayG = new Date(today)
          beignD =[ todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0"),todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0")];
          endD =[ todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0"),todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0")];
          
        }

        if (mode == "w") {
          
          beignD = TimeTool.timeSpaceStandardTime(weekFirst, weekLast);
          endD = TimeTool.timeSpaceStandardTime(weekFirst, weekLast);
        }

        if (mode == "m") {
         
          beignD = TimeTool.timeSpaceStandardTime(mouthList[0], mouthList[mouthList.length - 1]);
          endD = TimeTool.timeSpaceStandardTime(mouthList[0], mouthList[mouthList.length - 1]);
        }

        this.setState({
          nowChoBeignD:beignD,
          nowChoEndD:endD
        })

        this.props.getList({days:{beignD:beignD,endD:endD}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng")
      })
      return
    }

    if (mode == "d") {
      beignD =[ today.getFullYear() + "-" + (today.getMonth() + 1).toString().padStart(2, "0")+"-"+today.getDate().toString().padStart(2, "0"),today.getFullYear() + "-" + (today.getMonth() + 1).toString().padStart(2, "0")+"-"+today.getDate().toString().padStart(2, "0")];
      endD = [today.getFullYear() + "-" + (today.getMonth() + 1).toString().padStart(2, "0")+"-"+today.getDate().toString().padStart(2, "0"),today.getFullYear() + "-" + (today.getMonth() + 1).toString().padStart(2, "0")+"-"+today.getDate().toString().padStart(2, "0")];
      
    }

    this.setState({
      nowChoBeignD:beignD,
      nowChoEndD:endD
    })
    
    this.props.getList({days:{beignD:beignD,endD:endD}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng")

    this.setState(
      {
        today: today.getTime(),
        weekFirst: weekB,
        weekLast: weekE,
        mouth: mouthN,
        year: yearN
      },
      () => {
        // const{today, weekFirst, weekLast} = this.state;
        this.saveState()
        this.setWeekData()
        this.setMouthData()
      }
    )
  }

  saveState() {
    const { dispatch } = this.props
    dispatch({
      type: "system/calendarState",
      payload: this.state
    })
  }

  // TODO:获取列表内容
  // getList(tiaojian,aid, tp){
  //   const { basicInfo } = this.props;
  //   request('/senior/getListPaging', tiaojian, "", "", tiaojian.funCode, aid).then(res=>{
  //     this.setState({
  //       [tp=="richeng"?"richengList":"shishiList"]:res.result.data1.filter(el=>{return el[tp=="richeng"?"bu_plan000userName":"bu_activity_plan000userId"]==basicInfo.id})
  //     })
  //   })

  // }

  // TODO:日周月按钮切换
  handleModeChange = e => {
    const { today, weekDay, weekFirst, weekLast, mouthList, conditionS } = this.state
    const mode = e.target.value
    this.setState(
      {
        mode,
        weekDay: weekDay ? weekDay : today
      },
      () => {
        this.saveState()
        let beignD = [];
        let endD =[];
        if (mode == "d") {
          let todayG = new Date(today)
          beignD =[ todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0"),todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0")];
          endD =[ todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0"),todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0")];
          
        }

        if (mode == "w") {
          
          beignD = TimeTool.timeSpaceStandardTime(weekFirst, weekLast);
          endD = TimeTool.timeSpaceStandardTime(weekFirst, weekLast);
        }

        if (mode == "m") {
          richeng.searchScenario.bu_plan.beginDate.value = TimeTool.timeSpaceStandardTime(mouthList[0], mouthList[mouthList.length - 1])
          
          shishi.searchScenario.bu_activity_plan.beginDate.value = TimeTool.timeSpaceStandardTime(
            mouthList[0],
            mouthList[mouthList.length - 1]
          )
          
          
          beignD = TimeTool.timeSpaceStandardTime(mouthList[0], mouthList[mouthList.length - 1]);
          endD = TimeTool.timeSpaceStandardTime(mouthList[0], mouthList[mouthList.length - 1]);
        }

        this.setState({
          nowChoBeignD:beignD,
          nowChoEndD:endD
        })
        this.props.getList({days:{beignD:beignD,endD:endD}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng", conditionS)
      }
    )
  }

  // TODO:单日增加减少
  chageOneDay(tp) {
    const { today } = this.state
    // console.log("今日时间", tp);
    let shijian = today
    if (tp === "up") {
      shijian = shijian - 24 * 60 * 60 * 1000
    } else {
      shijian = shijian + 24 * 60 * 60 * 1000
    }
    this.setState(
      {
        today: shijian
      },
      () => {
        const { today } = this.state
        this.saveState()
      
        let todayG = new Date(today)
        


        let beignD =[ todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0"),todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0")+"-"+todayG.getDate().toString().padStart(2, "0")];

        this.setState({
          nowChoBeignD:beignD,
          nowChoEndD:beignD
        })
        
        this.props.getList({days:{beignD:beignD,endD:beignD}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng")



      }
    )
  }

  // TODO:周增加减少
  chageWeekDay(tp) {
    const { weekFirst, weekLast, weekDay } = this.state
    // console.log("今日时间", tp);
    let shijian1 = weekFirst
    let shijian2 = weekLast
    let wd = weekDay
    if (tp === "up") {
      shijian1 = shijian1 - 24 * 7 * 60 * 60 * 1000
      shijian2 = shijian2 - 24 * 7 * 60 * 60 * 1000
      wd = wd - 24 * 7 * 60 * 60 * 1000
    } else {
      shijian1 = shijian1 + 24 * 7 * 60 * 60 * 1000
      shijian2 = shijian2 + 24 * 7 * 60 * 60 * 1000
      wd = wd + 24 * 7 * 60 * 60 * 1000
    }
    this.setState(
      {
        weekFirst: shijian1,
        weekLast: shijian2,
        weekDay: wd
      },
      () => {
        const { weekFirst, weekLast, conditionS } = this.state
        this.setWeekData()

        let beignD = TimeTool.timeSpaceStandardTime(weekFirst, weekLast);
        let endD = TimeTool.timeSpaceStandardTime(weekFirst, weekLast);
        this.setState({
          nowChoBeignD:beignD,
          nowChoEndD:endD
        })
        this.props.getList({days:{beignD:beignD,endD:endD}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng", conditionS)

      }
    )
  }

  // TODO:设置周时间列表
  setWeekData() {
    const { weekFirst } = this.state
    let list = new Array(7).fill(0)
    let list1 = list.map((el, index) => {
      return index == 0 ? weekFirst : weekFirst + index * 24 * 60 * 60 * 1000
    })
    this.setState(
      {
        weekTopList: list1
      },
      () => {
        this.saveState()
      }
    )
  }

  // TODO:月份变更
  chageMouth(tp) {
    const { mouth, year } = this.state
    let mouthN = _.cloneDeep(mouth)
    let yearN = _.cloneDeep(year)
    let m, y
    if (tp === "up") {
      if (mouthN == 1) {
        m = 12
        y = yearN - 1
      } else {
        m = mouthN - 1
        y = yearN
      }
    } else {
      if (mouthN == 12) {
        m = 1
        y = yearN + 1
      } else {
        m = mouthN + 1
        y = yearN
      }
    }
    this.setState(
      {
        year: y,
        mouth: m
      },
      () => {
        this.setMouthData(true)
      }
    )
  }

  // TODO:设置月份列表
  setMouthData(load) {
    const { mouth, year } = this.state
    // console.log(mouth,year);
    // return
    let shijian1 = year + "-" + mouth.toString().padStart(2, "0") + "-" + "01" //设当月的第一天
    let time = new Date(shijian1)
    let timeW = time.getDay() == 0 ? 7 : time.getDay() == 1 ? 2 : time.getDay() //当月的第一天的周
    let beiginTime = time.getTime() - parseInt(parseInt(timeW) - 1) * 24 * 60 * 60 * 1000 //月份列表中第一个空格内容的日期

    let shijian2 = (mouth == 12 ? year + 1 : year) + "-" + ((mouth == 12 ? 0 : mouth) + 1).toString().padStart(2, "0") + "-" + "01" //下个月的第一天

    let endTime = new Date(shijian2).getTime() - 24 * 60 * 60 * 1000 //当月的最后一天, 暂时表格中最后一格
    let timeW1 = new Date(endTime).getDay() //最后一天所在的周
    if (timeW1 != 0) {
      endTime = endTime + parseInt(7 - parseInt(timeW1)) * 24 * 60 * 60 * 1000 //表格中最后一格日期
    }
    let changdu = (endTime - beiginTime) / 24 / 60 / 60 / 1000

    let list = new Array(changdu + 1).fill("")
    let list1 = list.map((el, index) => {
      return index == 0 ? beiginTime : beiginTime + index * 24 * 60 * 60 * 1000
    })
    // console.log("现内容",shijian2, TimeTool.timestampToTime(endTime), timeW1, changdu);
    this.setState(
      {
        mouthList: list1
      },
      () => {
        const { mouthList, conditionS } = this.state
        this.saveState()
        
        if (load) {
          
          let beignD = TimeTool.timeSpaceStandardTime(mouthList[0], mouthList[mouthList.length - 1]);
          let endD = TimeTool.timeSpaceStandardTime(mouthList[0], mouthList[mouthList.length - 1]);
          this.setState({
            nowChoBeignD:beignD,
            nowChoEndD:endD
          })
          this.props.getList({days:{beignD:beignD,endD:endD}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng", conditionS)
        }
      }
    )
  }

  // 设置查询条件 传入开始时间,和结束时间,条件类型("1":richeng条件,"2":shishi条件)
  setQueryCriteria(beiginTime, endTime, tp) {
    richeng.searchScenario.bu_plan.beginDate.value = TimeTool.timeSpaceStandardTime(beiginTime, endTime)
    
    shishi.searchScenario.bu_activity_plan.beginDate.value = TimeTool.timeSpaceStandardTime(beiginTime, endTime)
    
    return tp === "1" ? richeng : shishi
  }

  // TODO:新增计划(弹层)
  addNew(fc) {
    this.props.dispatch({
      type: "system/jumpToEditPage",
      payload: "-1",
      functionCode: fc,
      popup: "popup"
    })
  }

  // TODO:修改月份年月（月份使用）
  changYM(e) {
    this.setState(
      {
        year: e._d.getFullYear(),
        mouth: e._d.getMonth() + 1
      },
      () => {
        this.saveState()
        this.setMouthData(true)
      }
    )
  }

  // TODO:日期选择框选择修改日期
  dataChange(e) {
    console.log("日期选择")
    // return
    this.setState(
      {
        today: e._d.getTime()
      },
      () => {
        const { today, conditionS } = this.state
        this.saveState()
        let todayG = new Date(today)
        let beignD = todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0") + "-" + todayG.getDate().toString().padStart(2, "0");
        
        // let firstD = todayG.getFullYear() + "-" + (todayG.getMonth() + 1).toString().padStart(2, "0") + "-" + "01 00:00:00"
        // let nextFirstD =
        //   todayG.getFullYear() +
        //   "-" +
        //   (todayG.getMonth() + 2 > 12 ? 1 : todayG.getMonth() + 2).toString().padStart(2, "0") +
        //   "-" +
        //   "01 00:00:00"
        // let endD = new Date(nextFirstD).getTime() - 24 * 60 * 60 * 1000
        this.props.getList({days:{beignD:[beignD,beignD],endD:[beignD, beignD]}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng", conditionS)
        this.props.getList({days:{beignD:beignD,endD:beignD}}, "ab9600046c1e45f8b07b73311cdee172", "shishi", conditionS)
      }
    )
  }

  // TODO:日期选择框选择修改日期（周）
  dataChangeW(e) {
    let w = e._d.getDay()
    let time = e._d.getTime()
    let shijian1 = time - 24 * parseInt(parseInt(w) - 1) * 60 * 60 * 1000
    let shijian2 = time + 24 * parseInt(7 - parseInt(w)) * 60 * 60 * 1000
    this.setState(
      {
        weekFirst: shijian1,
        weekLast: shijian2,
        weekDay: time
      },
      () => {
        const { weekFirst, weekLast, conditionS } = this.state
        this.setWeekData()
        this.saveState()

        let beignD = TimeTool.timeSpaceStandardTime(weekFirst, weekLast);
        let endD = TimeTool.timeSpaceStandardTime(weekFirst, weekLast);
        this.setState({
          nowChoBeignD:beignD,
          nowChoEndD:endD
        })
        this.props.getList({days:{beignD:beignD,endD:endD}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng", conditionS)
      }
    )
  }

  // NOTE:修改搜素内容
  onItemValueChange(value, dataIndex, currentItem, selectedRows) {
    let { values } = this.state;
    let newValue = JSON.parse(value);
    if(newValue.value.length>0){
      let conditionS = {
        indexName:dataIndex,
        operator:"belong",
        tableName:'bu_plan',
        type:"multipleTreeSelect",
        value:value
      }
      this.setState({
        values:values.set(dataIndex,value),
        conditionS
      },()=>{
        const { values, nowChoBeignD, nowChoEndD, conditionS } = this.state;
        
        // let conditionS = {
        //   indexName:dataIndex,
        //   operator:"belong",
        //   tableName:'bu_plan',
        //   type:"multipleTreeSelect",
        //   value:value
        // }
        this.props.getList({days:{beignD:nowChoBeignD,endD:nowChoEndD}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng", conditionS)
      });
    }else{
      this.setState({
        values:values.delete(dataIndex),
        conditionS:{}
      },()=>{
        const { nowChoEndD, nowChoBeignD, conditionS } = this.state;
        this.props.getList({days:{beignD:nowChoBeignD,endD:nowChoEndD}}, "be00408439a745a2ba4af0e51fbc1dd2", "richeng", conditionS)
      });
    }
    
    
  }

  setTimess(date){
    let rDate = new Date(date);
    let rDate1 = rDate.getFullYear()+"-"+(rDate.getMonth()+1).toString().padStart(2,"0")+"-"+(rDate.getDate().toString().padStart(2,"0")) + " 00:00:00";
    let rDate2 = new Date(rDate1)
    let rDate3 = rDate2.getTime();
    date = new Date(rDate3);
    var date2 = new Date(date.getFullYear(), 0, 1);
    var day1 = date.getDay();
    if (day1 == 0) day1 = 7;
    var day2 = date2.getDay();
    if (day2 == 0) day2 = 7;
    let d = Math.round((date.getTime() - date2.getTime() + (day2 - day1) * (24 * 60 * 60 * 1000)) / 86400000);
    //当周数大于52则为下一年的第一周
    if((Math.ceil(d / 7) + 1)>52){
      return (date.getFullYear()+1)+"第1周"
    }
    return date.getFullYear()+"-"+(Math.ceil(d / 7) + 1)+"周";
  }
  

  render() {
    const { mode, today, weekFirst, weekLast, weekTopList, mouthList, year, mouth, weekDay, typesList, configs } = this.state
    const { workflow, information, basicInfo } = this.props
    const { timestampToTime } = TimeTool

    let allD = 20
    let yearList = new Array(allD).fill(0)
    let yearList1 = yearList.map((item, index) => {
      return year - allD / 2 + index
    })

    let mList = new Array(12).fill(0)
    let mList1 = mList.map((item, index) => {
      return index + 1
    })

    const menu = (
      <Menu>
        {yearList1.map((item, index) => {
          return (
            <Menu.Item key={index} onClick={() => this.changYM("year", item)}>
              {item}
            </Menu.Item>
          )
        })}
      </Menu>
    )

    const menu1 = (
      <Menu>
        {mList1.map((item, index) => {
          return (
            <Menu.Item key={index} onClick={() => this.changYM("mouth", item)}>
              {item.toString().padStart(2, "0")}
            </Menu.Item>
          )
        })}
      </Menu>
    )

    return (
      <div className="calendarBox">
        <div className="calendarBoxTop">
          <div className="calendarBoxTopLeft">
            <Radio.Group onChange={this.handleModeChange} value={mode}>
              <Radio.Button value="d">日</Radio.Button>
              <Radio.Button value="w">周</Radio.Button>
              <Radio.Button value="m">月</Radio.Button>
            </Radio.Group>

            <ButtonGroup style={{ marginLeft: 20 }}>
              <Button onClick={() => this.addNew("001900140001")}>新增日程计划</Button>
              {/* <Button onClick={() => this.addNew("001900040001")}>新增实施计划</Button> */}
            </ButtonGroup>
            {basicInfo.departmentLeaders&&<Button onClick={()=>this.props.watchUnSetntPeo(mode)} style={{ marginLeft:10 }}>查看未发布人员</Button>}
            
            <AutogenerationForm labelWidthSwitch={"off"} style={{display:"inline-block", width:400, margin:"-15px 0 0 10px", verticalAlign:"middle"}} config={configs} onItemValueChange={(value, dataIndex, currentItem, selectedRows) => this.onItemValueChange(value, dataIndex, currentItem, selectedRows)} source={this.state.values} />
            
          </div>
          <div className="calendarBoxTopRight">
            <ul>
              {typesList.map((el, index) => {
                return (
                  <li key={index}>
                    <span style={{ background: el.color }}></span>
                    {el.title}
                  </li>
                )
              })}
            </ul>
          </div>
          {mode == "d" && (
            <div className="calendarBoxTopCenter">
              <Button shape="circle" onClick={() => this.chageOneDay("up")} size="small" icon="caret-left" />
              {/* <span>{timestampToTime(today)}</span> */}
              <DatePicker
                style={{ margin: "0 10px", fontSize: 14 }}
                allowClear={false}
                value={moment(timestampToTime(today), "YYYY-MM-DD")}
                onChange={e => this.dataChange(e)}
                format={"YYYY-MM-DD"}
              />
              <Button shape="circle" onClick={() => this.chageOneDay("next")} size="small" icon="caret-right" />
            </div>
          )}
          {mode == "w" && (
            <div className="calendarBoxTopCenter">
              <Button shape="circle" onClick={() => this.chageWeekDay("up")} size="small" icon="caret-left" />
              {/* <span>{timestampToTime(weekFirst)}~{timestampToTime(weekLast)}</span> */}
              <WeekPicker
                style={{ margin: "0 10px", fontSize: 14 }}
                allowClear={false}
                value={moment(timestampToTime(weekDay), "YYYY-MM-DD")}
                format = {moment().format(this.setTimess(weekDay))}
                onChange={e => this.dataChangeW(e)}
                placeholder="Select week"
              />
              <Button shape="circle" onClick={() => this.chageWeekDay("next")} size="small" icon="caret-right" />
            </div>
          )}
          {mode == "m" && (
            <div className="calendarBoxTopCenter">
              <Button shape="circle" onClick={() => this.chageMouth("up")} size="small" icon="caret-left" />
              {/* <span>
              <Dropdown overlay={menu} placement="bottomCenter" trigger={['click']}>
                <em className="ant-dropdown-link">
                  {year}
                </em>
              </Dropdown>
                ~
                <Dropdown overlay={menu1} placement="bottomCenter" trigger={['click']}>
                  <em className="ant-dropdown-link">
                    {mouth.toString().padStart(2,"0")}
                  </em>
                </Dropdown>
              </span> */}
              <MonthPicker
                style={{ margin: "0 10px", fontSize: 14 }}
                allowClear={false}
                value={moment(`${year}-${mouth.toString().padStart(2, "0")}`, "YYYY-MM-DD")}
                onChange={e => this.changYM(e)}
              ></MonthPicker>
              <Button shape="circle" onClick={() => this.chageMouth("next")} size="small" icon="caret-right" />
            </div>
          )}
        </div>
        <div className="calendarBoxBody">
          {mode == "d" && (
            <BBox
              list={this.props.list}
              stateConfig = {this.state}
              changeUnSentShow={(jg)=>this.props.changeUnSentShow(jg)}
              today={today}
              goToDetail={(obg, fc) => this.props.goToDetail(obg, fc)}
              addNewN={fc => this.addNew(fc)}
            ></BBox>
          )}
          {mode == "w" && (
            <WBox
              list={this.props.list}
              stateConfig = {this.state}
              changeUnSentShow={(jg)=>this.props.changeUnSentShow(jg)}
              goToDetail={(obg, fc) => this.props.goToDetail(obg, fc)}
              addNewN={fc => this.addNew(fc)}
              today={information.today}
              weekList={weekTopList}
            ></WBox>
          )}
          {mode == "m" && (
            <MBox
              list={this.state}
              goToDetail={(obg, fc) => this.props.goToDetail(obg, fc)}
              listF={this.props.list}
              changeUnSentShow={(jg)=>this.props.changeUnSentShow(jg)}
              addNewN={fc => this.addNew(fc)}
            ></MBox>
          )}
        </div>
      </div>
    )
  }
}
