/**
 * Created by 王友生 on 2018-07-19.
 */

import React, { PureComponent } from 'react';
import CardItem from './CardItem';
import { connect } from 'react-redux';
import { getItemActualValue } from '../../utils';
import { Row, Col } from 'antd';

@connect((state, route) => {
  const reference = state.get('reference');
  const dictionary = state.get('dictionary');
  return {
    reference,
    dictionary
  }
})
export default class CardCollection extends PureComponent {

  renderCardItem(itemData, index) {
    const { dispatch, reference, dictionary, cardColumnsConfig, onItemDoubleClick } = this.props;
    const { titleConfig, tagConfig, contentConfig, descConfig } = cardColumnsConfig;
    const title = titleConfig ? getItemActualValue(titleConfig, itemData[titleConfig.dataIndex], dispatch, reference, dictionary) : ' ';
    const tag = tagConfig ? getItemActualValue(tagConfig, itemData[tagConfig.dataIndex], dispatch, reference, dictionary) : ' ';
    const content = contentConfig ? getItemActualValue(contentConfig, itemData[contentConfig.dataIndex], dispatch, reference, dictionary) : ' ';
    const desc = descConfig ? getItemActualValue(descConfig, itemData[descConfig.dataIndex], dispatch, reference, dictionary) : ' ';
    return <CardItem
      title={title} tag={tag} content={content} desc={desc} index={index + 1}
      actions={{
        onDoubleClick: () => { onItemDoubleClick && onItemDoubleClick(itemData) }
      }} />
  }

  render() {
    const { dataSource, cardColumnsConfig } = this.props;
    if (! cardColumnsConfig || ! dataSource) {
      return <div>暂无数据</div>
    }
    return <Row>
      {
        dataSource.map((itemData, index) => <Col key={index} span={8}>
          {this.renderCardItem(itemData, index)}
        </Col>)
      }
    </Row>
  }
}
