/**
 * Created by 王友生 on 2018-04-17.
 */
import "./index.less"
import React, { PureComponent } from "react"
import { Popover, Tooltip, Icon, Tree, Badge, Input, List } from "antd"
import classnames from "classnames"
import NavIcon from "../../assets/crm/nav.png"
import FavIcon from "../../assets/crm/fav.png"
import NavLogout from "../../assets/crm/logout.png"
import MsgIcon from "../../assets/crm/msg.png"
import WarnIcon from "../../assets/crm/warn.png"
import lockImg from "../../assets/crm/password.png"
import settingImg from "../../assets/crm/setting.png"
import reportImg from "../../assets/crm/report.png"
import qyhlImg from "../../assets/crm/qyhl.png"
import eisImg from "../../assets/crm/eis.png"
import treeNodeEndImg from "../../assets/crm/icon/tree_node_end.png"
import QuickPanel from "../QuickPanel/CrmQuickPanel"
import connect from "react-redux/es/connect/connect"
import { timeFormatConst } from "../../common/const"

const DirectoryTree = Tree
@connect(state => {
  const system = state.get("system")
  return {
    basicInfo: system.get("basicInfo"),
    warningConfig: system.get("warningConfig"),
    allWarningCount: system.get("allWarningCount")
  }
})
export default class MainMenu extends PureComponent {
  constructor(props) {
    super(props)
    this.state = {
      visibleKey: "",
      searchText: "",
      expandedKeys: props.defaultOpenKeys,
      selectedWarningId: ""
    }
  }

  renderMenuPopContent(menuItem, activeKey) {
    if (menuItem.items && menuItem.items.length > 0) {
      return (
        <Popover
          overlayClassName="popover"
          key={menuItem.id}
          visible={menuItem.id == this.state.visibleKey}
          onVisibleChange={visible => this.setState({ visibleKey: visible ? menuItem.id : "" })}
          placement="right"
          content={menuItem.items.map(item => {
            return (
              <div key={item.id} className="subMenu">
                <span
                  className={classnames("title", item.id === activeKey ? "active" : "")}
                  onClick={() => (!item.items || item.items.length === 0) && this.onMenuClick([item.id, menuItem.id])}
                >
                  {item.menuName}
                </span>
                {item.items && item.items.length > 0 && (
                  <div className="content">
                    {item.items.map(childItem => (
                      <div
                        key={childItem.id}
                        className={childItem.id === activeKey ? "active" : ""}
                        onClick={() => this.onMenuClick([childItem.id, item.id, menuItem.id])}
                      >
                        {childItem.menuName}
                      </div>
                    ))}
                  </div>
                )}
              </div>
            )
          })}
        >
          <li className="menuName">
            <i className={"icon iconfont AntMenuIcon " + menuItem.icon + " " + "icon"} />
            {menuItem.menuName.substring(0, 2)}
          </li>

          {/*<img style={{ width: '20px', height: '20px', marginRight: '5px' }}*/}
          {/*src={menuItem.icon}*/}
          {/*alt='' />*/}
        </Popover>
      )
    } else {
      return (
        <Tooltip placement="right" key={menuItem.id} title={menuItem.menuName}>
          <li
            className={classnames("menuName", menuItem.id === activeKey ? "activeMenu" : "")}
            onClick={() => this.onMenuClick([menuItem.id])}
          >
            <i className={"icon iconfont AntMenuIcon " + menuItem.icon + " " + "icon"} />
            {menuItem.menuName.substring(0, 2)}
          </li>
          {/*<img style={{ width: '20px', height: '20px', marginRight: '5px' }}*/}
          {/*src={menuItem.icon}*/}
          {/*alt='' />*/}
          {/*<i className={"icon iconfont " + menuItem.icon }></i>*/}
        </Tooltip>
      )
    }
  }

  onMenuClick(keyPath) {
    const { onMenuClick } = this.props
    this.setState({ visibleKey: "", displayIndex: 0 })
    onMenuClick && onMenuClick(keyPath)
  }

  renderTreeNode(menus, keyPath) {
    return menus.map((item, index) => {
      if (item.ynHidden === "1") {
        return <Tree.TreeNode style={{ display: "none" }} />
      }
      const currentKeyPath = [...keyPath]
      currentKeyPath.splice(0, 0, item["id"])
      const hasChildren = item["items"] && item["items"].length > 0
      const name = item["menuName"]
      var searchIndex = name.indexOf(this.state.searchText)
      const beforeStr = name&&name.substr(0, searchIndex)
      const afterStr = name&&name.substr(searchIndex + this.state.searchText.length)
      return (
        <Tree.TreeNode
          isLeaf={!hasChildren}
          icon={<Icon type="meh-o" />}
          className="treeNodeWrapper"
          key={JSON.stringify({ hasChildren, currentKeyPath })}
          title={
            searchIndex > -1 ? (
              <span className="treeNodeText">
                {hasChildren || <img className="treeNodeEnd" src={treeNodeEndImg} />}
                {beforeStr}
                <span style={{ color: "#BD6100" }}>{this.state.searchText}</span>
                {afterStr}
              </span>
            ) : (
              <span className="treeNodeText">
                {hasChildren || <img className="treeNodeEnd" src={treeNodeEndImg} />}
                {name}
              </span>
            )
          }
        >
          {hasChildren && this.renderTreeNode(item["items"], currentKeyPath)}
        </Tree.TreeNode>
      )
    })
  }

  renderQuickMenus() {
    const { quickMenus, menuList, dispatch, originalMenu } = this.props
    return (
      /*quickMenus && quickMenus.length > 0 &&*/
      <QuickPanel
        onItemClick={item =>
          dispatch({
            type: "system/addTab",
            payload: {
              ...item,
              menuName: item.menuName
            }
          })
        }
        config={quickMenus}
        menuList={menuList}
        originalMenu={originalMenu}
        saveQuickPanel={quickMenuItems => dispatch({ type: "system/saveQuickMenu", payload: { quickMenuItems } })}
      />
    )
  }

  onSearch() {
    const { searchText, data } = this.state
    const { menus } = this.props
    if (!searchText) {
      this.setState({ filterData: null })
      return
    }
    const expandedKeys = []
    const loop = (menus, keyPath) => {
      let newList = []
      menus.map(item => {
        const currentKeyPath = [...keyPath]
        currentKeyPath.splice(0, 0, item["id"])
        const hasChildren = item["items"] && item["items"].length > 0
        const childrenList = hasChildren && loop(item["items"], currentKeyPath)
        let newItem = { ...item, items: childrenList }
        if (newItem.menuName.indexOf(searchText) >= 0) {
          newList.push(newItem)
          if (childrenList && childrenList.length > 0) {
            expandedKeys.push(JSON.stringify({ hasChildren, currentKeyPath }))
          }
        } else if (childrenList && childrenList.length > 0) {
          newList.push(newItem)
          expandedKeys.push(JSON.stringify({ hasChildren, currentKeyPath }))
        }
      })
      return newList
    }

    this.setState({ filterData: loop(menus, []), expandedKeys })
  }

  jumpMessage() {
    this.props.dispatch({
      type: "system/jumpToFunctionWidthScenario",
      payload: {
        functionCode: "002006"
      }
    })
  }

  render() {
    const { originalMenu, menus, basicInfo, messageCount, dispatch, reportMenus, defaultOpenKeys } = this.props
    return (
      <div className={classnames("nav", "menuWrapper")}>
        <div
          className={classnames("navIconCt", "menuWrapper")}
          onClick={() => this.setState({ displayIndex: this.state.displayIndex === 1 ? 0 : 1 })}
        >
          <img className="navIconCtImg" src={NavIcon} />
          <div className="navIconCtTxt">业务导航</div>
        </div>
        <div
          className="menuContentWrapper"
          style={{ display: this.state.displayIndex === 1 ? "block" : "none" }}
          onClick={e => this.setState({ displayIndex: 0 })}
        >
          <div className="menuContent" onClick={e => e.stopPropagation()}>
            <div className="menuTitle">
              <span>业务导航</span>
              <span className="close" onClick={() => this.setState({ displayIndex: 0 })}>
                <Icon type="close" />
              </span>
            </div>
            <div className="searchWrapper">
              <Input.Search
                size="small"
                placeholder="请输入"
                onSearch={() => {
                  this.onSearch()
                }}
                onChange={e => this.setState({ searchText: e.target.value })}
                value={this.state.searchText}
              />
            </div>
            <div className="treeWrapper">
              <DirectoryTree
                expandedKeys={this.state.expandedKeys}
                onExpand={expandedKeys => {
                  this.setState({ expandedKeys })
                }}
                onSelect={(
                  selectKeys,
                  {
                    selected,
                    node: {
                      props: { eventKey: key }
                    }
                  }
                ) => {
                  const { hasChildren, currentKeyPath } = JSON.parse(key)
                  let expandedKeys = this.state.expandedKeys || []
                  if (hasChildren) {
                    let index = expandedKeys.indexOf(key)
                    index < 0 ? expandedKeys.push(key) : expandedKeys.splice(index, 1)
                    this.setState({ expandedKeys: [...expandedKeys] })
                  } else {
                    !hasChildren && this.onMenuClick(currentKeyPath)
                  }
                }}
              >
                {this.renderTreeNode(this.state.filterData || menus, [])}
              </DirectoryTree>
            </div>
          </div>
        </div>
        <div
          className={classnames("navIconCt", "menuWrapper")}
          onClick={() => this.setState({ displayIndex: this.state.displayIndex === 2 ? 0 : 2 })}
        >
          <img className="navIconCtImg" src={FavIcon} />
          <div className="navIconCtTxt">常用功能</div>
        </div>
        <div
          className="menuContentWrapper"
          style={{ display: this.state.displayIndex === 2 ? "block" : "none" }}
          onClick={e => this.setState({ displayIndex: 0 })}
        >
          <div className="menuContent">
            <div className="menuTitle">
              <span>常用功能</span>
              <span className="close" onClick={() => this.setState({ displayIndex: 0 })}>
                <Icon type="close" />
              </span>
            </div>
            {this.renderQuickMenus()}
          </div>
        </div>
        <div className="navIconCt">
          <div style={{ position: "absolute", right: "15px", marginTop: "-2px" }}>
            <Badge count={messageCount}>
              <a href="#" className="head-example" />
            </Badge>
          </div>
          <img className="navIconCtImg" src={MsgIcon} onClick={() => this.jumpMessage()} />
          <div className="navIconCtTxt">消息任务</div>
        </div>
        <div className="navIconCt" onClick={() => this.setState({ displayIndex: this.state.displayIndex === 4 ? 0 : 4 })}>
          <img className="navIconCtImg" src={reportImg} />
          <div className="navIconCtTxt">报表中心</div>
        </div>
        <div
          className="menuContentWrapper"
          style={{ display: this.state.displayIndex === 4 ? "block" : "none" }}
          onClick={e => this.setState({ displayIndex: 0 })}
        >
          <div className="menuContent" onClick={e => e.stopPropagation()}>
            <div className="menuTitle">
              <span>报表中心</span>
              <span className="close" onClick={() => this.setState({ displayIndex: 0 })}>
                <Icon type="close" />
              </span>
            </div>
            <div className="treeWrapper">
              <DirectoryTree
                expandedKeys={this.state.expandedReportKeys}
                onExpand={expandedReportKeys => {
                  this.setState({ expandedReportKeys })
                }}
                onSelect={(
                  selectKeys,
                  {
                    selected,
                    node: {
                      props: { eventKey: key }
                    }
                  }
                ) => {
                  const { hasChildren, currentKeyPath } = JSON.parse(key)
                  let expandedReportKeys = this.state.expandedReportKeys || []
                  if (hasChildren) {
                    let index = expandedReportKeys.indexOf(key)
                    index < 0 ? expandedReportKeys.push(key) : expandedReportKeys.splice(index, 1)
                    this.setState({ expandedReportKeys })
                  } else {
                    !hasChildren && this.onMenuClick(currentKeyPath)
                  }
                }}
              >
                {this.renderTreeNode(reportMenus || [], [])}
              </DirectoryTree>
            </div>
          </div>
        </div>
        <div className="navIconCt" onClick={() => this.setState({ displayIndex: this.state.displayIndex === 5 ? 0 : 5 })}>
          <div style={{ position: "absolute", right: "15px", marginTop: "-2px" }}>
            <Badge count={this.props.allWarningCount}>
              <a href="#" className="head-example" />
            </Badge>
          </div>
          <img className="navIconCtImg" src={WarnIcon} />
          <div className="navIconCtTxt">预警信息</div>
        </div>
        <div
          className="menuContentWrapper"
          style={{ display: this.state.displayIndex === 5 ? "block" : "none" }}
          onClick={e => this.setState({ displayIndex: 0 })}
        >
          <div className="menuContent" onClick={e => e.stopPropagation()}>
            <div className="menuTitle">
              <span>预警信息</span>
              <span className="warningRefresh" onClick={() => this.props.dispatch({ type: "system/initWarningConfig" })}>
                <Icon type="reload" />
              </span>
              <span className="close" onClick={() => this.setState({ displayIndex: 0, selectedWarningId: "" })}>
                <Icon type="close" />
              </span>
            </div>
            <div className="menuWarningWrapper">
              {this.props.warningConfig.map((item, index) => (
                <div
                  key={index}
                  onClick={() => {
                    this.setState({ selectedWarningId: item.btnId })
                    this.props.dispatch({ type: "system/goToWarningDetail", payload: { item } })
                  }}
                  className={`menuWaringContent ${this.state.selectedWarningId === item.btnId ? "selected" : ""}`}
                >
                  <span className="menuWarningTitle">{item.btnName}</span>
                  <span className="menuWarningNum">{item.num}</span>
                </div>
              ))}
            </div>
          </div>
        </div>
        <div className="leftBarBottom">
          <div className="leftBarItem" onClick={() => this.props.logoutSystem()}>
            <img className="itemLogout" src={NavLogout} />
            <span>退出</span>
          </div>
        </div>
      </div>
    )
  }
}
