import './index.less';
import React, { PureComponent } from 'react';
import { DatePicker, Popover, Button, message } from 'antd';
import moment from 'moment';
import { getBeginAndEndDate } from '../../utils'
import { dateFormatConst } from '../../common/const'
import { quickButtonGroups, quickButtons } from '../../utils/quick-button';
import { FormattedMessage } from 'react-intl';


class CustomDateRange extends PureComponent {
  constructor(props) {
    super(props);

    this.state = {
      beginDate: undefined,
      endDate: undefined,
      selectedLabel: '',
      selectedValue: '',
      layerShow: false
    }
  }

  componentDidMount() {
    const { value } = this.props;
    this.handleChangeState(value);
  }

  componentWillReceiveProps(nextProps) {
    const { value } = nextProps;

    if (value
      && this.props.value
      && (value[0] !== this.props.value[0]
        || value[1] !== this.props.value[1]
        || value[2] !== this.props.value[2])) {

      this.handleChangeState(value);
    }
  }

  handleChangeState = (value) => {
    let selectedLabel = '';
    let selectedValue = '';

    if (value && value.length === 3) {
      selectedValue = value[2];
      const quickButton = quickButtons.find(v => v.value === selectedValue);
      selectedLabel = quickButton ? quickButton.label : '';
    }

    this.setState({
      beginDate: value && value[0] ? moment(value && value[0]) : undefined,
      endDate: value && value[1] ? moment(value && value[1]) : undefined,
      selectedLabel,
      selectedValue,
    })
  }

  handleQuickButtonClick = ({ label, value }) => {
    const [beginDate, endDate] = getBeginAndEndDate(value);
    this.handleDateChange(beginDate, endDate, value);
  }

  handleDateChange = (begin, end, operate) => {
    const beginDateString = begin ? begin.format(dateFormatConst) : '';
    const endDateString = end ? end.format(dateFormatConst) : '';

    this.props.onChange && this.props.onChange(JSON.stringify([beginDateString, endDateString, operate]));
  }

  render() {
    const { size, disabled, format, placeholder = [], lang } = this.props;
    const { layerShow } = this.state;

    return (
      <div className="custom-date-picker">
        <DatePicker
          disabled={disabled}
          format={format}
          size={size}
          value={this.state.beginDate}
          onChange={date => {
            let endDate = this.state.endDate;

            if (!date) {
              this.handleDateChange('', endDate, '');
              return;
            }

            if (!endDate || date.isAfter(this.state.endDate)) {
              endDate = date;
            }

            this.setState({
              beginDate: date,
              endDate,
              selectedLabel: '',
              selectedValue: ''
            });

            this.handleDateChange(date, endDate, '');
          }}
          placeholder={placeholder[0] || '请输入开始时间'} />
        <span style={{ display: "inline-block", verticalAlign: 'middle', margin: '-5px 5px 0' }}>~</span>
        <DatePicker
          disabled={disabled}
          format={format}
          size={size}
          value={this.state.endDate}
          onChange={date => {
            let beginDate = this.state.beginDate;

            if (!date) {
              this.handleDateChange(beginDate, '', '');
              return;
            }

            if (!beginDate || date.isBefore(this.state.beginDate)) {
              beginDate = date;
            }

            this.setState({
              beginDate,
              endDate: date,
              selectedLabel: '',
              selectedValue: ''
            });

            this.handleDateChange(beginDate, date, '');
          }}
          placeholder={placeholder[1] || '请输入结束时间'} />
        <Popover
          overlayClassName="btn-panel-popover"
          overlayStyle={{ zIndex: 9999 }}
          visible={layerShow}
          trigger="click"
          onVisibleChange={() => this.setState({ layerShow: !layerShow })}
          content={
            <div className="btn-panel">
              {quickButtonGroups.map(group => {
                return (
                  <div key={group.label} className="btn-group">
                    <span className="btn-group-title">
                      <FormattedMessage id={group.label} />
                    </span>
                    {group.children.map(child => {
                      return (
                        <Button
                          key={child.value}
                          type={this.state.selectedValue === child.value ? 'primary' : 'default'}
                          onClick={() => { this.handleQuickButtonClick(child); this.setState({ layerShow: false }) }}
                          size="small">
                          <FormattedMessage id={child.label} />
                        </Button>
                      )
                    })}
                  </div>
                )
              })}
            </div>
          }>
          <span className="quick-btn ant-input">
            {this.state.selectedLabel && <FormattedMessage id={this.state.selectedLabel} />}
          </span>
        </Popover>
      </div>
    )
  }
}

export default CustomDateRange;