/**
 * Created by 王友生 on 2018-06-14.
 */

import React, { Component } from "react"
import { Upload, message, Icon,Popover } from "antd"
import CustomPopover from "../CustomPopover"
import { serverUrl } from "../../common/config"
import _ from "lodash"
import { FormattedMessage } from "react-intl"
import { connect } from "react-redux"
import { systemCommonFileFunctionCode } from "../../common/const"

const Dragger = Upload.Dragger

export default class CustomDraggerUpload extends Component {
  constructor(props) {
    super(props)

    const { defaultFileList } = props
    if (defaultFileList) {
      const originalFileList = []
      defaultFileList.map(file => {
        originalFileList.push({
          name: file.oldFileName,
          uid: file.id,
          status: "done"
        })
        return
      })
      this.state = { fileList: originalFileList }
    } else {
      this.state = { fileList: [] }
    }
  }

  componentWillReceiveProps(newProps) {
    if (newProps.defaultFileList !== this.props.defaultFileList && (!this.state.fileList || this.state.fileList.length === 0)) {
      const defaultFileList = newProps.defaultFileList
      if (defaultFileList) {
        const originalFileList = []
        defaultFileList.map(file => {
          originalFileList.push({
            name: file.oldFileName,
            uid: file.id,
            status: "done"
          })
          return
        })
        this.setState({ fileList: originalFileList })
      }
    }
  }

  render() {
    const dragProps = {
      name: "file",
      multiple: true,
      defaultFileList: this.state.fileList,
      headers: { authorization: localStorage.getItem("token") || "", accessType: "backend" },
      action: `${serverUrl}/attach/uploadFiles?funCode=${this.props.functionCode || systemCommonFileFunctionCode}`,
      onChange: info => {
        const status = info.file.status
        // if (status !== "uploading") {
        // }
        if (status === "done") {
          if (info.file.response && info.file.response.code === "0") {
            message.success(`${info.file.name} 上传成功！`)
            // message.success(<p>{info.file.name}<FormattedMessage id="locale_uploadSucceed"/></p>);
            if (info.file.response.data && info.file.response.data.result && info.file.response.data.result.length > 0) {
              this.setState({ fileList: info.fileList })
              this.props.uploadSuccess && this.props.uploadSuccess(info.file.response.data.result)
            }
          }
        } else if (status === "error") {
          message.error(`${info.file.name} 上传失败！`)
        } else if (status === "removed") {
          const deletedIndex = _.findIndex(this.state.fileList, { uid: info.file.uid })
          // const deletedIndex = this.state.fileList.indexOf(info.file);
          this.setState({ fileList: info.fileList })
          this.props.deleteFile && this.props.deleteFile(deletedIndex)
        }
      }
    }
    return (
      <CustomPopover
        trigger="click"
        placement="bottomRight"
        overlayStyle={{ zIndex: 9999 }}
        content={
          <div style={{ width: "400px" }}>
            <Dragger {...dragProps} multiple>
              <p className="ant-upload-drag-icon">
                <Icon type="inbox" />
              </p>
              <p className="ant-upload-text">
                <FormattedMessage id="locale_uploadingText" />
              </p>
            </Dragger>
          </div>
        }
      >
        {this.props.children ? (
          this.props.children
        ) : (
          <a>
            <Icon type="paper-clip" style={{ marginRight: "5px" }} />
            <FormattedMessage id="locale_accessory" />
          </a>
        )}
      </CustomPopover>
    )
  }
}
