import React, { PureComponent, Fragment } from 'react';
import { connect } from 'react-redux';
import { Input, Icon, Dropdown, Menu } from 'antd';
import _ from "lodash"

/**
 *
 *
 * @export
 * @class CustomSearch
 * onManualChange:同步输入
 * @extends {PureComponent}
 */
@connect((state, route) => {

    const referenceCache = state.get('reference');
    return {
        referenceCache,
        extraSearchScenario: referenceCache.get('extraSearchScenario')
    }
})
export default class CustomSearch extends PureComponent {

    constructor(props) {
        super(props);
        this.state = {
            value: props.value,
            manualChange: props.manualChange
        };
    }

    componentWillReceiveProps(newProps) {
        if (newProps.value !== this.props.value) {
            this.setState({ value: newProps.value });
        }
    }
    getTableData(referenceKey, val) {
        const { dispatch, referenceCache, objectHash, extraSearchScenario } = this.props
        val = val.replace(/\'/g, "")
        let activeRelected = referenceCache && referenceCache.get(referenceKey) || undefined;
        if (activeRelected) {
            // console.log('displayedColumns2', activeRelected.get('config')['tableConfig']['displayedColumns']);

            this.setState({ displayedColumns: activeRelected.get('config')['tableConfig']['displayedColumns'] })
            dispatch({
                type: 'reference/fetchReferenceTableData',
                payload: {
                    id: referenceKey,
                    fuzzy: val,
                    condition: extraSearchScenario,
                    loading: false,
                    fuzzySearchField: activeRelected.get('config') && activeRelected.get('config')['searchField'],
                    pagination: {
                        pageNo: 1,
                        pageSize: 10
                    }
                }
            })
        }
    }
    dGetTableData = _.debounce(this.getTableData, 500)
    renderItem() {
        const { referenceCache, referenceKey, item, objectHash } = this.props;
        const { displayedColumns, value } = this.state;
        let arr = undefined
        let selectedSuccess = referenceCache.get('selectedSuccess')
        if (referenceKey && referenceCache.getIn([referenceKey, 'tableData'])) {
            const tableData = referenceKey && referenceCache.getIn([referenceKey, 'tableData']) || undefined;
            arr = tableData.data1
        }
        return <Menu style={{ maxHeight: "200px", overflowY: "auto" }}>
            {arr && value &&
                arr.length && displayedColumns &&
                arr.map(row => {
                    return (
                        <Menu.Item onClick={() => {
                            this.props.matching()
                            this.props.dispatch({
                                type: 'reference/onChangeSuccess',
                                payload: {
                                    selectedRows: [row],
                                    currentItem: item,
                                    selectedSuccess: !selectedSuccess,
                                    objectHash
                                }
                            });
                            this.setState({ visible: false })
                        }} key={row.id}>
                            <a>{`${row[displayedColumns[0]['dataIndex']]}`}</a>
                        </Menu.Item>
                    )
                })}
        </Menu>

    }
    render() {
        const { value, manualChange, visible } = this.state;
        const { onSearch, onClear, disabled, onManualChange, referenceKey, referenceCache } = this.props;
        const suffix = value ? <Icon style={{ marginRight: '5px' }} onClick={() => onClear && onClear()} type="close-circle" /> : null;
        return <Fragment><Dropdown
            disabled={disabled}
            onVisibleChange={(vis) => {
                this.setState({ visible: vis })
            }}
            visible={visible} trigger={"click"} overlay={this.renderItem()}>
            <Input.Search
                value={value}
                disabled={disabled}
                suffix={suffix}
                style={{ height: '24px' }}
                size="small"
                onChange={(e) => {
                    this.setState({ value: e.target.value })
                    if (!e.target.value) {
                        onClear && onClear()
                    }
                    if (referenceKey && e.target.value.length > 1) {
                        this.dGetTableData(referenceKey, e.target.value)
                    }
                    if (manualChange) {
                        onManualChange(e.target.value);//手填也更新对应字段
                    }
                }}
                onSearch={(value, e) => {
                    e.stopPropagation();
                    e.preventDefault();
                    this.setState({ visible: false })
                    onSearch(value);
                }}
            // onFocus={(e) => {
            //   e.target.blur();
            // cacheChangeAction = onChange;
            // dispatch({
            //   type: 'reference/visibleReference',
            //   payload: {
            //     id: referenceKey,
            //     item,
            //     selectedMode: selectedModeConst.single,
            //     extraSearchScenario: sourceScenario,
            //     objectHash: props.objectHash
            //   }
            // })
            // }}
            />
        </Dropdown>
        </Fragment>
    }
}
