/**
 * Created by 王友生 on 2018-04-27.
 * param： url， extraParams查询条件， selectedMode ：single， multiple， onDelete（record）
 * onRowSelected： （record，selected, allSelectedRecords）
 * onSelectAll: ( select, selectedRows, changedRows )
 * selectedKeys: 所有已勾选的key
 * defaultLoading
 * highlightKeys: 高亮的行
 */
import './index.less';
import React, { Component } from 'react';
import { pageInfo } from '../../common/config';
import { selectedModeConst, pageSizeOptions } from '../../common/const';
import ResizeableTitle from '../../components/ResizeableTitle';
import { Table, Popconfirm } from 'antd';
import { request } from '../../utils/request';
import { connect } from 'react-redux';
import { getItemSourceFromCache, getValueFromSource, getReferenceByItem, getItemActualValue } from '../../utils';
import { Map } from 'immutable';

export const paginationTypeConst = {
  none: '',
  async: 'async',
  sync: 'sync'
}

@connect((state, route) => {
  const reference = state.get('reference');
  const dictionary = state.get('dictionary');
  return {
    reference,
    dictionary
  }
}, null, null, { withRef: true })
export default class CustomTable extends Component {

  constructor(props) {
    super(props);
    this.state = {
      pageNo: pageInfo.pageNo,
      pageSize: window.defaultPageSize || pageInfo.pageSize,
      //pageSize:'',
      total: pageInfo.total,
      data: [],
      left: '-1000',
      top: '-1000',
      visible: false,
      selectedKeys: props.selectedKeys || [],
      selectedRows: props.selectedRows || [],
      deletedRecord: null,
      totalWidth: 0,
      update: false
    };
  }

  componentDidMount() {
    this.props.defaultLoading && this.fetch();
    this.setColumn(this.props.columns)
  }

  setColumn(columns) {
    const { pageNo, pageSize } = this.state;
    if (this.props.serialNumber) {                      //判断是否有使用序号的字段如果加在表配置前面加上“序号的字段”
      let pageRecord = (pageNo - 1) * pageSize;
      pageRecord = isNaN(pageRecord) ? 0 : pageRecord;
      const sn = [{
        dataIndex: 'serialNumber',
        key: "serialNumber",
        title: '序号',
        width: window.listIndexWidth || (window.themeStyle === "tiny" ? 80 : 120),
        render: (text, record, index) => `${index + 1 + pageRecord}`
      }];
      columns = sn.concat(columns);
    }
    this.setState({
      columns: columns
    })
  }
  componentWillReceiveProps(newProps) {
    const { pageNo, pageSize } = this.state;
    if (newProps.selectedKeys !== this.props.selectedKeys) {
      this.setState({ selectedKeys: newProps.selectedKeys });
    } else if (newProps.selectedRows !== this.props.selectedRows) {
      this.setState({ selectedKeys: Array.from(newProps.selectedRows, row => row[newProps['idField'] || 'id']), selectedRows: newProps.selectedRows });
    }
    if (newProps.columns !== this.props.columns) {
      this.setColumn(newProps.columns)//切换columns刷新table
    }
  }
  components = {
    header: {
      cell: ResizeableTitle,
    },
  }

  handleResize = index => (e, { size }) => {
    const { columns } = this.state;
    let totalWidth = this.state.totalWidth || this.props.totalWidth;
    totalWidth = totalWidth - (columns[index].width || 0) + size.width;
    columns[index].width = size.width;
    this.setState({ update: !this.state.update, totalWidth });
  };

  onPageChange(pageNo, pageSize) {
    this.setState({ pageNo, pageSize }, () => {
      this.fetch();
    });
  }

  fetch(currentPageNo) {
    const { pageNo, pageSize } = this.state;
    const newPageNo = currentPageNo || pageNo
    let { url, extraParams, paginationType, callback, dispatch, setTopDealWidth } = this.props;
    if (Map.isMap(extraParams)) {
      extraParams = extraParams.toJSON();
    }
    const actualPageSize = paginationType === paginationTypeConst.sync ? 500 : pageSize
    url && request(url, { ...extraParams, ...{ pageNo: newPageNo || pageNo, pageSize: actualPageSize } })
      .then(data => {
        this.setState({ data: data.result, pageNo: data.pageNo, total: data.total }, () => { callback && callback() });
        if (url === '/crmImplementProject/getSrImplementProjectBoard') {
          dispatch({
            type: "list/customSwitchData",
            payload: {
              customSwitchData: data.result,
            }
          })
        }
        // dispatch({
        //   type:"dashboard/fetchDealWithChartSuccess",
        //   payload:{
        //       chartData:data.result,
        //       totalCount:data.result.length,
        //     }
        //   },()=>{
        //   })
        setTopDealWidth && setTopDealWidth(data.result);
      });
  }

  onRowRightClick(column, e) {
    e.preventDefault();
    e.persist();
    this.setState({ left: e.pageX, top: e.pageY, visible: true, deletedRecord: column });
  }

  onSingleRowSelect(record) {
    let { selectedKeys, selectedRows } = this.state;
    var index = selectedKeys.indexOf(record[this.props.idField || 'id']);
    if (index >= 0) {
      selectedKeys.splice(index, 1);
      selectedRows.splice(index, 1);
    } else {
      selectedKeys.pop();
      selectedKeys.push(record[this.props.idField || 'id']);
      selectedRows.pop();
      selectedRows.push(record);
    }
    this.setState({ selectedKeys, selectedRows });
    this.props.onRowSelected && this.props.onRowSelected(record, index < 0, selectedRows);
  }

  onMultipleSelect(record) {
    let { selectedKeys, selectedRows } = this.state;
    var index = selectedKeys.indexOf(record[this.props.idField || 'id']);
    if (index >= 0) {
      selectedKeys.splice(index, 1);
      selectedRows.splice(index, 1);
    } else {
      selectedKeys.push(record[this.props.idField || 'id']);
      selectedRows.push(record);
    }
    this.setState({ selectedKeys, selectedRows });
    this.props.onRowSelected && this.props.onRowSelected(record, index < 0, selectedRows);
  }

  onMultipleSelectAll(selected, changeRows) {
    let { selectedKeys, selectedRows } = this.state;
    changeRows.map(record => {
      var index = selectedKeys.indexOf(record[this.props.idField || 'id']);
      if (index >= 0) {
        selectedKeys.splice(index, 1);
        selectedRows.splice(index, 1);
      } else {
        selectedKeys.push(record[this.props.idField || 'id']);
        selectedRows.push(record);
      }
      return true;
    });
    this.setState({ selectedKeys, selectedRows });
    this.props.onSelectAll && this.props.onSelectAll(selected, selectedRows, changeRows);
  }

  getItemActualValue(item, label) {
    const { dispatch, reference, dictionary } = this.props;
    return getItemActualValue(item, label, dispatch, reference, dictionary);
  }

  render() {
    const { pageNo, pageSize, total, data, top, left, visible, selectedKeys, deletedRecord, selectedRows } = this.state;
    const {
      idField, selectedMode, onDelete, asyncPagination, paginationType,
      pagination, dataSource, totalWidth, onSelectAll, antProps, onDoubleClick, totalHeight, serialNumber, highlightKeys, customRowKeyProps, searchMes
    } = this.props;
    const actualDataSource = dataSource || data;
    let columns1 = _.cloneDeep(this.state.columns);
    // console.log("表格内容", columns1);
    // if (serialNumber) {
    //   let pageRecord = (pageNo - 1) * pageSize;
    //   pageRecord = isNaN(pageRecord) ? 0 : pageRecord;
    //   const sn = [{
    //     dataIndex: 'serialNumber',
    //     key: "serialNumber",
    //     title: '序号',
    //     width: window.listIndexWidth || (window.themeStyle === "tiny" ? 80 : 120),
    //     render: (text, record, index) => `${index + 1 + pageRecord}`
    //   }];
    //   columns = sn.concat(columns);
    // }


    const gotoDetail = (obgMes) => {
      const { dispatch } = this.props;
      let newObg = searchMes.toJSON();
      newObg.record = obgMes;
      dispatch({
        type: "system/setSingleDetail",
        payload: newObg
      })
      dispatch({
        type: "system/addTab",
        payload: {
          menuUrl: "/report/reportDetailSingle",
          menuName: "回款业绩报表详情",
          funType: "3"
        }
      })
    }

    const searchM = () => {
      let newObg = searchMes.toJSON();
      return newObg
    }
    columns1 && columns1.map((column, index) => {
      if (!column.render || column.needRender === '1') {
        column.needRender = '1';
        column.onHeaderCell = () => ({
          width: column.width,
          onResize: this.handleResize(index),
        });
        column.render = (text, record) => {
          let rowSpan = 1;
          const index = actualDataSource.indexOf(record);
          if (column.group === '1' && text) {
            if (index > 0) {
              const lastIndex = index - 1;
              const lastValue = actualDataSource[lastIndex][column['dataIndex']];
              if (lastValue === text) {
                rowSpan = 0;
              }
            }
            let newRecordCount = 0;
            if (rowSpan !== 0) {
              for (let nextIndex = index + 1; nextIndex < actualDataSource.length; nextIndex++) {
                const nextRecord = actualDataSource[nextIndex];
                if (nextRecord[column['dataIndex']] === text) {
                  newRecordCount++;
                } else {
                  break;
                }
              }
              rowSpan = newRecordCount + 1;
            }
          }
          let label;
          if (column.type === "multipleSelect" || column.type == "multipleTreeSelect") {
            let labelArray = null
            let vLabelArray = []
            if (text) {
              if (text.indexOf('displayValue') != -1) {
                label = JSON.parse(text).displayValue;
              } else {
                labelArray = text.split(',')
              }
            }
            labelArray &&
              labelArray.map(v => {
                vLabelArray.push(this.getItemActualValue(column, v) || '-');
              })
            if (vLabelArray && vLabelArray.length > 0) label = vLabelArray.join()
          } else if (column.type === 'tags') {
            let labelArray = null;
            let vLabelArray = [];
            if (text) {
              labelArray = text.split(',')
            }
            labelArray && labelArray.map((v) => {
              vLabelArray.push(this.getItemActualValue(column, v) || '-');
            })
            if (vLabelArray) label = vLabelArray.join();
          } else if (column.type === 'tinyMCE') {//富文本
            if (text) {
              let regx = /<[^>]*>|<\/[^>]*>|&nbsp;/gm;
              label = text.replace(regx, "");
            }
          } else if (column.type === 'amount') {
            label = text&&parseFloat(text).toFixed(2).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')
          }else {
            label = this.getItemActualValue(column, text) || "-";
          }

          // let label = this.getItemActualValue(column, text) || '-';
          const style = { textOverflow: 'ellipsis' };
          if (column.width) {
            style.width = "100%";
            // style.width = column.width - 10;
          }
          return {
            props: {
              rowSpan
            },
            children: <div title={label} style={style} className="ellipseText">{(column.enterDetail && column.enterDetail == "1" && searchM().achievementType === "0002") ? <a onClick={() => gotoDetail(actualDataSource[index])}>{label}</a> : label}</div>

          }
        }
      }
    });
    let rowSelection = null;
    if (selectedMode) {
      if (selectedMode === selectedModeConst.single) {
        rowSelection = {
          columnWidth: '35px',
          type: 'radio',
          selectedRowKeys: selectedKeys,
          onSelect: (record) => this.onSingleRowSelect(record)
        }
      } else if (selectedMode === selectedModeConst.multiple) {
        rowSelection = {
          columnWidth: '35px',
          type: 'checkbox',
          selectedRowKeys: selectedKeys,
          onSelect: (record) => this.onMultipleSelect(record),
          onSelectAll: (selected, selectedRows, changeRows) => this.onMultipleSelectAll(selected, changeRows)
        }
      }
    }

    let paginationProps = false;
    if (paginationType === paginationTypeConst.async) {
      paginationProps = {
        size: "small",
        showSizeChanger: true,
        current: pageNo,
        pageSize,
        total,
        showTotal: total => `共 ${total} 条`,
        onChange: (pageNo, pageSize) => this.onPageChange(pageNo, pageSize),
        onShowSizeChange: (pageNo, pageSize) => this.onPageChange(pageNo, pageSize)
      }
    } else if (paginationType === paginationTypeConst.sync) {
      paginationProps = {
        size: "small",
        showTotal: total => `共 ${total} 条`,
      }
    }

    let scrollProps = {};
    if (totalWidth) {
      scrollProps.scroll = serialNumber ? { x: Math.max(totalWidth + 45, this.state.totalWidth + 45) } : { x: Math.max(this.state.totalWidth, totalWidth) };
      if (totalHeight) {
        scrollProps.scroll.y = totalHeight;
      }
    } else if (totalHeight) {
      scrollProps.scroll = { y: totalHeight }
    }

    return <div>
      <Table
        {...antProps}
        rowClassName={(record, index) => highlightKeys && highlightKeys.indexOf(record[[this.props.idField || 'id']]) >= 0 ? 'rowSelectClass' : index % 2 == 0 ? '' : 'zebraCross'}
        bordered={true}
        components={this.components}
        rowSelection={rowSelection}
        onRow={(record) => {
          return {
            onClick: (e) => {
              e.preventDefault();
              e.stopPropagation();
              if (selectedMode === selectedModeConst.single) {
                this.onSingleRowSelect(record);
              } else if (selectedMode === selectedModeConst.multiple) {
                this.onMultipleSelect(record);
              } else {
                this.props.onRowSelected && this.props.onRowSelected(record);
              }
            },
            onContextMenu: (e) => onDelete && this.onRowRightClick(record, e),
            onDoubleClick: (e) => {
              e.preventDefault();
              e.stopPropagation();
              onDoubleClick && onDoubleClick(record)
            }
          };
        }}
        columns={columns1}
        rowKey={(record, index) => {
          return record[idField || 'id'] || index
        }}
        {...customRowKeyProps}
        dataSource={actualDataSource}
        pagination={(pagination || paginationProps) ? { ...(pagination || paginationProps), pageSizeOptions } : false}
        {...scrollProps} />
      {
        onDelete && <Popconfirm
          trigger="hover"
          visible={visible}
          onVisibleChange={visible => {
            if (!visible) {
              this.setState({ visible, top: '-1000', left: '-1000', deletedRecord: null });
            }
          }}
          title="您确定要删除此条记录？"
          onConfirm={() => onDelete(deletedRecord)}
          okText="删除"
          cancelText="取消">
          <div style={{
            position: 'fixed',
            top: top,
            left: left - 10,
            width: '20px',
            height: '40px',
          }} />
        </Popconfirm>
      }
    </div>
  }
}
