/**
 * Created by 王友生 on 2018-04-26.
 * onDragFinished当拖拽结束后，会返回重新排序的父节点
 * onCheck 返回所有被选中的行 checkedKeys所有被选中的key, halfCheckedRows 所有半选的行 checkedRows, halfCheckedRows, currentCheckedRows, currentHalfCheckedRows, checked
 * onNodeSelect 选中单个选中的行
 * extraActions 添加额外的按钮
 * mark 添加颜色标注['字段','颜色'] 字段为真时染色
 * hidden 过滤条件
 */

import "./index.less"

import React, { PureComponent, Component } from "react"
import { Tree, Icon, Input, Popconfirm, Tooltip } from "antd"
import { convertListToTreeSource } from "../../utils"
import { fromJS, List, Map } from "immutable"
import CustomTreeSelect from "../CustomTreeSelect"
import DraggableModal from "../DraggableModal"
import AutoGenerationForm from "../AutoGenerationForm"
import _ from "lodash"


export default class CustomTree extends PureComponent {
  keyPathMapping = {}

  constructor(props) {
    super(props)
    this.state = {
      data: fromJS(this.toData(props.data, props.treeField, props.groupSort, props.hidden)),
      searchText: "",
      filterKeyPaths: [],
      selectedKey: props.selectedKey || "",
      expandedKeys: [],
      checkedKeys: props.checkedKeys || [],
      halfCheckedKeys: [],
      isFilter: true,
      filterSource: fromJS([]),
    }
  }
  componentDidMount() {
    const { filter, treeField, treeExpand, data, groupSort, hidden } = this.props
    let { data: dataTree } = this.state
    if (filter && filter.filterReg && data.length > 0) {
      let filterSource = data.filter(item => {
        return eval(filter.filterReg).test(item[filter.filterColumn])
      })
      this.setState({ filterSource: fromJS(this.toData(filterSource, treeField, groupSort, hidden)) })
    }
    this.controlExpand(dataTree.toJS(), treeExpand)
  }
  toData(data, treeField, groupSort, hidden) {
    let tree = convertListToTreeSource(data, treeField, "", groupSort)
    if (hidden) {
      return tree.filter(item => !item[hidden])
    } else {
      return tree
    }

  }
  controlExpand(dataSource, treeExpand) {
    // 5.10父节点控制
    if (Array.isArray(dataSource)) {
      if (treeExpand == "openFirst") {
        let expandedKeys = []
        dataSource.map(item => {
          if (item.children && item.children.length > 0) {
            expandedKeys.push(item.code)
          }
        })
        this.setState({
          expandedKeys
        })
      } else if (treeExpand == "openEnd") {
        // let expandedKeys = []
        // function recursion(dataItem) {
        //   dataItem.map(item => {
        //     if (item.children && item.children.length > 0) {
        //       expandedKeys.push(item.code)
        //       recursion(item.children)
        //     }
        //   })
        // }
        // recursion(dataSource)
        let expandedKeys = this.setListKey(dataSource)
        this.setState({
          expandedKeys
        })
      }
    }
  }

  setListKey(yuanList) {
    let expandedKeys = []
    const recursion = dataItem => {
      dataItem.map(item => {
        if (item.children && item.children.length > 0) {
          expandedKeys.push(item.code)
          recursion(item.children)
        }
      })
    }
    recursion(yuanList)
    return expandedKeys
  }

  componentWillReceiveProps(newProps) {
    if (newProps.data !== this.props.data) {
      this.setState({
        data: fromJS(this.toData(newProps.data, newProps.treeField, newProps.groupSort, newProps.hidden)),
        filterKeyPaths: []
      })
      this.controlExpand(this.toData(newProps.data, newProps.treeField, newProps.groupSort, newProps.hidden), newProps.treeExpand)
    }
    if (newProps.filter && newProps.filter.filterReg && newProps.data.length > 0) {
      let filterSource = newProps.data.filter(item => {
        return eval(newProps.filter.filterReg).test(item[newProps.filter.filterColumn])
      })
      this.setState({ filterSource: fromJS(this.toData(filterSource, newProps.treeField, newProps.groupSort, newProps.hidden)) })
    }
    if (newProps.selectedKey !== this.props.selectedKey) {
      this.setState({ selectedKey: newProps.selectedKey })
    }
    if (newProps.checkedKeys !== this.props.checkedKeys) {
      const { antProps } = newProps
      const newCheckedKeys = []
      const halfCheckedKeys = []
      newProps.checkedKeys.map(key => {
        const keyPath = this.keyPathMapping[key]
        if (keyPath) {
          const currentNode = this.state.data.getIn(keyPath)
          if (currentNode.get("children") && (!antProps || !antProps.checkStrictly)) {
            halfCheckedKeys.push(key)
          } else {
            newCheckedKeys.push(key)
          }
        }
        return true
      })
      this.setState({ checkedKeys: newCheckedKeys })
    }
  }

  onDrop(info) {
    console.log('info', info);

    const { data } = this.state
    const dropKeyPath = this.keyPathMapping[info.node.props.eventKey]
    const dropNode = data.getIn(dropKeyPath).set("children", null)
    const dragKeyPath = this.keyPathMapping[info.dragNode.props.eventKey]
    const dropPos = info.node.props.pos.split("-")
    const dropPosition = info.dropPosition - Number(dropPos[dropPos.length - 1])
    const dragNode = data.getIn(dragKeyPath)
    let targetIndex = 0
    if (info.dropToGap) {
      const lastKey = dropKeyPath.pop()
      targetIndex = dropPosition === 1 ? lastKey + dropPosition : lastKey
    } else {
      // let lastNum = parseInt( dropKeyPath[dropKeyPath.length-1] );
      // if(lastNum==0){
      //   dropKeyPath[dropKeyPath.length-1] = 0
      // }else{

      // 移动到同级别的内容，判断当前位置和要去的位置的索引大小如果当前位置在前面，需要将移动到的位置-1,（当前位置会去除，下面的位置会每个向上提一个）
      if (dropKeyPath.length == dragKeyPath.length) {
        if (parseInt(dragKeyPath[dragKeyPath.length - 1]) > parseInt(dropKeyPath[dropKeyPath.length - 1])) {
          dropKeyPath[dropKeyPath.length - 1] = parseInt(dropKeyPath[dropKeyPath.length - 1])
        } else {
          dropKeyPath[dropKeyPath.length - 1] = parseInt(dropKeyPath[dropKeyPath.length - 1]) - 1
        }
      }
      // }
      dropKeyPath.push("children")
    }
    const newData = data.deleteIn(dragKeyPath).updateIn(dropKeyPath, children => {
      let newChildren = children || List([])
      return newChildren.insert(targetIndex, dragNode)
    })
    this.setState({
      data: newData
    })
    dropKeyPath.pop()
    let parentNode = newData.getIn(dropKeyPath)
    if (List.isList(parentNode)) {
      parentNode = Map({
        id: -1,
        children: parentNode
      })
    }
    parentNode = parentNode.update("children", children => {
      let newChildren = List([])
      children &&
        children.map((item, index) => {
          if (targetIndex !== index) {
            newChildren = newChildren.push(item.set("children", null))
          } else {
            newChildren = newChildren.push(item)
          }
          return true
        })
      return newChildren
    })
    this.props.onDragFinished && this.props.onDragFinished(dragNode.toJSON(), parentNode.toJSON())
  }

  deleteNode(keyPath) {
    const { onDeleteNode } = this.props
    const { searchText, filterData } = this.state
    const data = searchText ? filterData : this.state.data
    onDeleteNode && onDeleteNode(data.getIn(keyPath).toJSON())
    if (searchText) {
      this.setState({ filterData: data.deleteIn(keyPath) })
    } else {
      this.setState({ data: data.deleteIn(keyPath) })
    }
  }

  // 树结构列表渲染
  renderTreeNode(data, keyPath) {
    const { manipulate, manipulateFun } = this.props;
    const { modalData } = this.state;
    return data.map((item, index) => {
      keyPath.pop()
      keyPath.push(index)
      const currentKeyPath = [...keyPath]
      const key = item.get(this.props.treeField["id"]);
      const name = item.get(this.props.treeField["name"]);
      let mark;
      if (this.props.mark) {
        if (item.get(this.props.mark[0]) && item.get(this.props.mark[0]) !== "0") {
          mark = this.props.mark[1]
        }
      }
      var searchIndex = name ? name.indexOf(this.state.searchText) : -1
      // var searchIndex  = 1;
      const beforeStr = name && name.substr(0, searchIndex)
      const afterStr = name && name.substr(searchIndex + this.state.searchText.length)
      this.keyPathMapping[key] = currentKeyPath
      return (
        <Tree.TreeNode
          key={key}
          title={
            manipulate ? <div className="treeTitleWrapper" title={item.get(this.props.treeField["name"])}>
              {searchIndex > -1 ? (
                <div style={mark ? { color: mark } : {}}>
                  {beforeStr}
                  <span style={{ color: "#1890ff" }}>{this.state.searchText}</span>
                  {afterStr}
                </div>
              ) : (
                <div>{name}</div>
              )}
              <div className="manipulate">
                <span onClick={() => manipulateFun(item, 'add')}>添加</span>
                <span onClick={() => manipulateFun(item, 'edit')}>修改</span>
                {item.get("children") && item.get("children").size > 0 ? <span style={{ color: '#999' }}>删除</span>
                  : <Popconfirm
                    placement="right"
                    title="您确认要删除此节点？"
                    onConfirm={e => {
                      e.stopPropagation()
                      this.deleteNode(currentKeyPath)
                    }}
                    onCancel={e => {
                      e.stopPropagation()
                    }}
                    okText="删除"
                    cancelText="取消"
                  >
                    <span >删除</span>
                  </Popconfirm>}

              </div>
            </div> :
              <div className="treeTitleWrapper" title={item.get(this.props.treeField["name"])}>
                {searchIndex > -1 ? (
                  <div style={mark ? { color: mark } : {}}>
                    {beforeStr}
                    <span style={{ color: "#1890ff" }}>{this.state.searchText}</span>
                    {afterStr}
                  </div>
                ) : (
                  <div>{name}</div>
                )}
                {this.props.onDeleteNode && (
                  <Popconfirm
                    placement="right"
                    title="您确认要删除此节点？"
                    onConfirm={e => {
                      e.stopPropagation()
                      this.deleteNode(currentKeyPath)
                    }}
                    onCancel={e => {
                      e.stopPropagation()
                    }}
                    okText="删除"
                    cancelText="取消"
                  >
                    <Icon type="delete" />
                  </Popconfirm>
                )}
              </div>
          }
        >
          {item.get("children") && this.renderTreeNode(item.get("children"), [...keyPath, ...["children", 0]])}
        </Tree.TreeNode>
      )
    })
  }

  onSearch() {
    const { searchText, data } = this.state
    if (!searchText) {
      this.setState({ filterData: null })
      return
    }
    const expandedKeys = []
    const loop = data => {
      let newList = List([])
      data.map(item => {
        const childrenList = item.get("children") && loop(item.get("children"))
        let newItem = item.set("children", childrenList)
        if (newItem.get(this.props.treeField["name"]).indexOf(searchText) >= 0) {
          newList = newList.push(newItem)
        } else if (childrenList && childrenList.size > 0) {
          newList = newList.push(newItem)
          expandedKeys.push(newItem.get(this.props.treeField["id"]))
        }
      })
      return newList
    }

    this.setState({ filterData: loop(data), expandedKeys })
  }

  onSelect(selectedKeys, e) {
    const { filter, treeField, onNodeSelect, invert, } = this.props
    const { isFilter, filterSource, searchText, filterData } = this.state
    console.log('e.nativeEvent.path', e.nativeEvent);

    // if (e.nativeEvent.path[0].tagName == "path" || e.nativeEvent.path[0].tagName == "svg") {
    //   return
    // }
    const source = searchText ? filterData : this.state.data
    if (!selectedKeys || selectedKeys.length === 0) {
      if (!invert) {
        return
      }
      this.setState({ selectedKey: "" })
      onNodeSelect && onNodeSelect("", e)
      return
    }
    const currentselectedKey = selectedKeys[0]
    this.setState({ selectedKey: selectedKeys[0] })
    const keyPath = this.keyPathMapping[currentselectedKey]
    //1.16过滤时数据源未切换
    // if (source.getIn(keyPath)) {
    //   onNodeSelect && onNodeSelect(source.getIn(keyPath).toJS(), e)
    // } else {
    //2022-10-21 09:26:15 数据计算错误新方案
    let node = this.props.data.filter(item => {
      return item[treeField["id"]] == currentselectedKey
    })
    node[0] && onNodeSelect && onNodeSelect(node[0], e)
    // }
  }

  onCheck(checkedKeys, e) {
    const { halfCheckedKeys, checked } = e
    const { data, checkedKeys: originalCheckedKeys, halfCheckedKeys: originalHalfCheckedKeys, searchText, filterData } = this.state
    const source = searchText ? filterData : data
    const { onCheck, antProps } = this.props
    let currentCheckedRows = []
    let currentHalfCheckedRows = []
    let checkedRows = []
    let halfCheckedRows = []
    if (antProps && antProps.checkStrictly) {
      this.setState({ checkedKeys: checkedKeys.checked })
      if (checked) {
        checkedRows = Array.from(checkedKeys.checked, x => {
          const row = source.getIn(this.keyPathMapping[x]).toJSON()
          if (originalCheckedKeys.indexOf(x) < 0) {
            currentCheckedRows.push(row)
          }
          return row
        })
        onCheck(checkedRows, undefined, currentCheckedRows, undefined, checked)
        return
      } else {
        checkedRows = Array.from(checkedKeys.checked, x => {
          const row = source.getIn(this.keyPathMapping[x]).toJSON()
          return row
        })
        originalCheckedKeys.map(x => {
          if (checkedKeys.checked.indexOf(x) < 0) {
            if (this.keyPathMapping[x]) {
              currentCheckedRows.push(source.getIn(this.keyPathMapping[x]).toJSON())
            }
          }
          return true
        })
      }
      onCheck(checkedRows, undefined, currentCheckedRows, undefined, checked)
      return
    }
    this.setState({ checkedKeys, halfCheckedKeys })
    if (checked) {
      let treeData = searchText ? filterData : data;
      checkedRows = Array.from(checkedKeys, x => {
        const row = treeData.getIn(this.keyPathMapping[x]).toJSON()
        if (originalCheckedKeys.indexOf(x) < 0 && originalHalfCheckedKeys.indexOf(x) < 0) {
          currentCheckedRows.push(row)
        }
        return row
      })
      halfCheckedRows = Array.from(halfCheckedKeys, x => {
        const row = treeData.getIn(this.keyPathMapping[x]).toJSON()
        if (originalHalfCheckedKeys.indexOf(x) < 0) {
          currentHalfCheckedRows.push(row)
        }
        return row
      })
    } else {
      let treeData = searchText ? filterData : data;
      checkedRows = Array.from(checkedKeys, x => treeData.getIn(this.keyPathMapping[x]).toJSON())
      halfCheckedRows = Array.from(halfCheckedKeys, x => {
        const row = treeData.getIn(this.keyPathMapping[x]).toJSON()
        if (originalHalfCheckedKeys.indexOf(x) < 0) {
          currentHalfCheckedRows.push(row)
        }
        return row
      })
      originalCheckedKeys.map(x => {
        if (checkedKeys.indexOf(x) < 0 && halfCheckedKeys.indexOf(x) < 0) {
          if (this.keyPathMapping[x]) {
            currentCheckedRows.push(treeData.getIn(this.keyPathMapping[x]).toJSON())
          }
        }
        return true
      })
      originalHalfCheckedKeys.map(x => {
        if (halfCheckedKeys.indexOf(x) < 0) currentCheckedRows.push(data.getIn(this.keyPathMapping[x]).toJSON())
        return true
      })
    }
    onCheck && onCheck(checkedRows, halfCheckedRows, currentCheckedRows, currentHalfCheckedRows, checked)
  }

  render() {
    if (!this.state.data) return null
    const { selectedKey, expandedKeys, checkedKeys, isFilter, filterSource } = this.state
    this.keyPathMapping = {}
    const {
      placeholder,
      treeProps,
      allowSearch,
      draggable,
      onCheck,
      extraActions,
      antProps,
      onExpand,
      filterFromAll,
      expandedKeys: propExpandedKeys,
      filter
    } = this.props
    //1.4增加过滤filter
    // const data = filterSource
    const data = filter && isFilter ? filterSource : this.state.data
    return (
      <div className="customTree" style={this.props.style}>
        {allowSearch && (
          <Input.Search
            style={{
              width: filter ? "calc(100% - 20px)" : "100%",
              minWidth: filter ? "calc(100% - 20px)" : "100%"
            }}
            size="small"
            placeholder={placeholder}
            onSearch={() => {
              this.onSearch()
            }}
            onChange={e => this.setState({ searchText: e.target.value })}
            value={this.state.searchText}
          />
        )}
        {filter && (
          <Tooltip placement="top" title={filter.filterName}>
            <Icon
              type="filter"
              style={isFilter ? { color: "#1890ff", marginLeft: "3px" } : { marginLeft: "3px" }}
              onClick={() => {
                this.setState({ isFilter: !isFilter })
              }}
            />
          </Tooltip>
        )}
        {extraActions}
        {data && data.size > 0 && (
          <Tree
            {...antProps}
            // defaultExpandAll//默认展开所有树节点
            showLine //是否展示连接线
            checkable={onCheck} //是否支持选中
            // defaultExpandParent={false}//默认展开对应树节点
            // autoExpandParent={true}//是否自动展开父节点
            draggable={draggable && !this.state.filterData} //设置节点可拖拽
            selectedKeys={[selectedKey]} //设置选中的树节点
            expandedKeys={expandedKeys && expandedKeys.length > 0 ? expandedKeys : propExpandedKeys ? propExpandedKeys : []} //展开指定的树节点,搜索state优先
            checkedKeys={checkedKeys} //选中复选框的树节点
            onCheck={(checkedKeys, e) => this.onCheck(checkedKeys, e)} //点击复选框触发
            onExpand={expandedKeys => {
              //展开/收起节点时触发
              this.setState({ expandedKeys })
              onExpand && onExpand(expandedKeys)
            }}
            onSelect={(selectedKeys, e) => this.onSelect(selectedKeys, e)} //点击树节点触发
            onDrop={info => this.onDrop(info)}
          >
            {/* 树列表渲染 */}
            {this.renderTreeNode(!filterFromAll && this.state.filterData ? this.state.filterData : data, [0])}
          </Tree>
        )}
      </div >
    )
  }
}
