/**
 * Created by 王友生 on 2018-12-06.
 */

import React, { PureComponent } from 'react';
import { Modal, Spin, Icon } from 'antd';
import './index.less';
import { connect } from 'react-redux';

@connect((state) => {
  return {
    loading: state.getIn(['system', 'loading'])
  }
})
export default class DraggableModal extends PureComponent {
  state = {
    top: undefined,
    left: undefined,
    offsetX: undefined,
    offsetY: undefined,
    full: false
  }

  dragStart(e) {
    e.stopPropagation();
    let offsetX = e.nativeEvent.offsetX;
    let offsetY = e.nativeEvent.offsetY;
    this.setState({ top: e.clientY - offsetY, left: e.screenX - offsetX, offsetY, offsetX });
    let x = e.offsetX || e.layerX;
    let y = e.offsetY || e.layerY;
  }

  dragOver(e) {
    const { offsetY, offsetX } = this.state;
    this.setState({ top: e.clientY - offsetY, left: e.screenX - offsetX });
  }

  dragEnd(e) {
    const { offsetY, offsetX } = this.state;
    let expectedTop = e.clientY - offsetY;
    let actualTop = Math.max(20, expectedTop);
    actualTop = Math.min(actualTop, document.body.clientHeight - 40);
    let expectedLeft = e.screenX - offsetX;
    this.setState({ top: actualTop, left: expectedLeft });
  }

  render() {
    const { top, left, full } = this.state;
    const { bodyStyle,destroyOnClose } = this.props;
    const style = this.props.style || {};
    if (top !== undefined) {
      style.top = top;
      style.position = "fixed";
    }
    if (left !== undefined) {
      style.left = left;
      style.position = "fixed";
    }
    return <Modal
      {...this.props}
      style={style}
      destroyOnClose
      bodyStyle={full ? { ...bodyStyle, height: "75vh" } : bodyStyle}
      maskClosable={false}
      okButtonProps={{
        size: 'small',
        type: 'default'
      }}
      cancelButtonProps={{
        size: 'small'
      }}
      title={<div
        draggable
        onDragStart={e => this.dragStart(e)}
        onDragOver={e => this.dragOver(e)}
        onDragEnd={e => this.dragEnd(e)}
        className="popTitle">
        {this.props.title}<Icon className="full" type={full ? "fullscreen-exit" : "fullscreen"} onClick={() => { this.setState({ full: !full }) }} />
      </div>}
      width={full ? '90%' : this.props.width}
    >
      <Spin size="large" tip="加载中..." spinning={this.props.loading}>
        {this.props.children}
      </Spin>
    </Modal>
  }
}
