/**
 * Created by 王友生 on 2018-04-10.
 */
import './index.less';

import PropTypes from 'prop-types';
import React, { PureComponent } from 'react';
import { Tabs, Button, Radio, Input } from 'antd';
import { workFlowSelectorType, workflowNodeType, workFlowNodeAttrModeConst } from '../../common/const';
import { request, getStatusMappingItemByValue } from '../../utils';
import DraggableModal from "../DraggableModal";
import CustomTree from "../CustomTree";
import CustomTreeSelect from '../CustomTreeSelect';

const TabPane = Tabs.TabPane;
const RadioGroup = Radio.Group;

export default class EmployeeSelector extends PureComponent {

  static propTypes = {
    visible: PropTypes.bool,
  }

  constructor(props) {
    super(props);
    let basicInfo = window.getUserInfo();
    let supportKey = [workflowNodeType.corporation.value, workflowNodeType.department.value, workflowNodeType.post.value, workflowNodeType.duties.value, workflowNodeType.role.value, workflowNodeType.allDepartment.value, workflowNodeType.allCorpration.value];
    if (props.employeeSupport) {
      supportKey = props.employeeSupport.split(',');
    }
    const supportTabs = Array.from(supportKey, key => getStatusMappingItemByValue(workflowNodeType, key));
    this.state = {
      leftEmployee: [],
      leftItems: [],
      leftRoles: [],
      leftDuties: [],
      leftPost: [],
      leftTopSelectedItem: {},
      corporations: [],
      leftAllCorporations: [],
      leftDepartments: [],
      leftAllDepartments: [],
      leftSelectedItems: [],
      rightItems: [],
      leftTopSource: [],
      leftBottomSource: [],
      supportTabs,
      currentCorporation: basicInfo ? basicInfo.corpCode : '',
      rightSelectedItems: [],
      activeTabKey: supportKey && supportKey.length > 0 ? supportKey[0] : '',
      type: workFlowSelectorType.supervene.key,
    }
  }

  componentDidMount() {
    request('/sysComponent/getSysCorporationList', {})
      .then(data => {
        if (data && data.result) {
          this.setState({
            corporations: Array.from(data.result, x => {
              return {
                id: x.corpCode,
                name: x.corpName,
                pid: x.pcode,
                type: workflowNodeType.corporation.value
              }
            })
          }, () => this.fetchLeftTopSource())
        }
      });
  }

  componentWillReceiveProps(newProps) {
    if (newProps.visible && !this.props.visible) {
      this.setState({ rightItems: newProps.value || [] });
    }
  }

  fetchLeftTopSource() {
    const { currentCorporation, activeTabKey } = this.state;
    if (activeTabKey === workflowNodeType.department.value) {
      request('/sysComponent/getSysDepartmentList', { longCode: currentCorporation })
        .then(data => {
          if (data && data.result) {
            const leftDepartments = Array.from(data.result, x => {
              return {
                name: x.deptName,
                id: x.longCode,
                pid: x.longPcode,
                type: workflowNodeType.department.value
              }
            });
            this.setState({
              leftTopSource: leftDepartments
            })
          }
        });
    } else if (activeTabKey === workflowNodeType.allDepartment.value) {
      request('/sysComponent/getSysDepartmentCompleteList', {})
        .then(data => {
          if (data && data.result) {
            const leftAllDepartments = Array.from(data.result, x => {
              return {
                name: x.deptName,
                id: x.longCode,
                pid: x.longPcode,
                type: workflowNodeType.department.value
              }
            });
            this.setState({
              leftTopSource: leftAllDepartments
            })
          }
        });
    } else if (activeTabKey === workflowNodeType.duties.value) {
      request('/sysComponent/getSysDutiesList', { longCode: currentCorporation })
        .then(data => {
          if (data && data.result) {
            this.setState({
              leftTopSource: Array.from(data.result, x => {
                return {
                  name: x.dutiesName,
                  id: x.longCode,
                  pid: x.longPcode,
                  type: workflowNodeType.duties.value
                }
              })
            })
          }
        });
    } else if (activeTabKey === workflowNodeType.post.value) {
      request('/sysComponent/getSysPostList', { longCode: currentCorporation })
        .then(data => {
          if (data && data.result) {
            this.setState({
              leftTopSource: Array.from(data.result, x => {
                return {
                  name: x.postName,
                  id: x.longCode,
                  pid: x.longPcode,
                  type: workflowNodeType.post.value
                }
              })
            })
          }
        });
    } else if (activeTabKey === workflowNodeType.role.value) {
      request('/sysComponent/getSysRoleList', { findCode: currentCorporation })
        .then(data => {
          if (data && data.result) {
            this.setState({
              leftTopSource: Array.from(data.result, x => {
                return {
                  name: x.name,
                  id: x.code,
                  pid: x.pCode,
                  type: workflowNodeType.role.value
                }
              }).concat(this.props.extraRoleItems || [])
            })
          }
        });
    } else if (activeTabKey === workflowNodeType.corporation.value) {
      this.setState({ leftTopSource: this.state.corporations });
    } else if (activeTabKey === workflowNodeType.allCorpration.value) {
      request('/sysComponent/getSysCorporationCompleteList', {})
        .then(data => {
          if (data && data.result) {
            this.setState({
              leftTopSource: Array.from(data.result, x => {
                return {
                  id: x.corpCode,
                  name: x.corpName,
                  pid: x.pcode,
                  type: workflowNodeType.corporation.value
                }
              })
            })
          }
        });
    }
  }

  fetchPost(selectedItem) {
    request('/sysComponent/getSysPostList', { longCode: selectedItem.id })
      .then(data => {
        if (data && data.result) {
          this.setState({
            leftBottomSource: Array.from(data.result, x => {
              return {
                name: x.postName,
                id: x.longCode,
                pid: x.longPcode,
                type: workflowNodeType.post.value
              }
            })
          })
        }
      });
  }

  fetchEmployee(selectedItem) {
    let params = {};
    if (selectedItem.type === workflowNodeType.department.value) {
      params.deptLongCode = selectedItem.id;
    } else if (selectedItem.type === workflowNodeType.duties.value) {
      params.dutiesLongCode = selectedItem.id;
    } else if (selectedItem.type === workflowNodeType.post.value) {
      params.postLongCode = selectedItem.id;
    } else if (selectedItem.type === workflowNodeType.corporation.value) {
      params.corpCode = selectedItem.id;
    }
    request('/sysComponent/getSysUserList', params)
      .then(data => {
        if (data && data.result) {
          this.setState({
            leftBottomSource: Array.from(data.result, x => {
              return {
                name: x.userName,
                id: x.id,
                type: workflowNodeType.person.value
              }
            })
          })
        }
      });
  }

  moveToRight() {
    const { leftSelectedItems, rightItems, leftTopValid, leftTopSelected } = this.state;
    const selectedItems = leftTopValid ? [{
      name: leftTopSelected.name,
      id: leftTopSelected.id,
      type: leftTopSelected.type
    }] : leftSelectedItems;
    const newSelectedItems = selectedItems.filter(x => {
      return rightItems.filter(y => y.id === x.id).length === 0;
    });
    this.setState({ rightItems: rightItems.concat(newSelectedItems) });
  }

  doubleClickLeftBottomItem(item) {
    const { rightItems } = this.state;
    const selectedItems = [item];
    const newSelectedItems = selectedItems.filter(x => {
      return rightItems.filter(y => y.id === x.id).length === 0;
    });
    this.setState({ rightItems: rightItems.concat(newSelectedItems) });
  }

  doubleClickRightItem(index) {
    const { rightItems } = this.state;
    rightItems.splice(index, 1);
    this.setState({ rightItems, rightSelectedItems: [] });
  }

  removeFromRight() {
    const { rightItems, rightSelectedItems } = this.state;
    rightSelectedItems.map(item => {
      rightItems.splice(rightItems.indexOf(item), 1);
    });
    this.setState({ rightItems, rightSelectedItems: [] });
  }

  leftTopSelected(selectedItem) {
    const { activeTabKey, childeType } = this.state;
    if (activeTabKey === workflowNodeType.department.value && childeType === workflowNodeType.post.value) {
      this.fetchPost(selectedItem);
    } else {
      activeTabKey !== workflowNodeType.role.value && this.fetchEmployee(selectedItem);
    }
    this.setState({ leftTopSelected: selectedItem, leftTopValid: true });
  }


  renderTabContent() {
    const {
      rightItems,
      corporations,
      currentCorporation,
      leftTopSource,
      activeTabKey,
      leftTopSelected,
      filterPerson
    } = this.state;
    //人员搜索过滤
    let { leftBottomSource } = this.state
    if (filterPerson) {
      leftBottomSource = leftBottomSource.filter(i => {
        return i.name.indexOf(filterPerson) > -1
      })
    }
    return <div className="tabContent">
      <div className="leftPanel">

        {
          // 部门
          activeTabKey !== workflowNodeType.corporation.value && activeTabKey !== workflowNodeType.allCorpration.value &&
          <CustomTreeSelect
            antProps={{ size: 'small' }}
            data={corporations}
            style={{ width: '100%', marginBottom: '5px' }}
            treeField={{ id: 'id', name: 'name', pid: 'pid' }}
            onChange={(value) => {
              this.setState({ currentCorporation: value }, () => this.fetchLeftTopSource());
            }}
            placeholder="请选择单位"
            value={currentCorporation}
          />
        }
        <div className="leftTopSelector" style={activeTabKey === workflowNodeType.role.value ? { height: '370px' } : {}}>
          <CustomTree
            data={leftTopSource}
            onNodeSelect={(node) => this.leftTopSelected(node)}
            treeField={{ id: 'id', name: 'name', pid: 'pid' }} />
        </div>
        {//角色
          activeTabKey !== workflowNodeType.role.value &&
          <div className="filter">
            <RadioGroup defaultValue={workflowNodeType.person.value}
              onChange={e => this.setState({ childeType: e.target.value }, () => leftTopSelected && (e.target.value === workflowNodeType.person.value ? this.fetchEmployee(leftTopSelected) : this.fetchPost(leftTopSelected)))}>
              <Radio value={workflowNodeType.person.value}>{workflowNodeType.person.label}</Radio>
              {activeTabKey === workflowNodeType.department.value &&
                <Radio value={workflowNodeType.post.value}>{workflowNodeType.post.label}</Radio>}
            </RadioGroup>

            <Input placeholder={'搜索'} onChange={(e) => this.setState({ filterPerson: e.target.value })} />
          </div>
        }
        {
          activeTabKey !== workflowNodeType.role.value && <div className="leftBottomSelector">
            <select multiple
              defaultValue={[]}
              style={{ height: leftBottomSource.length * 25 + 'px' }}
              size={leftBottomSource.length}
              onChange={(e) => this.setState({
                leftTopValid: false,
                leftSelectedItems: Array.from(e.target.selectedOptions, option => leftBottomSource.filter(x => x.id === option.value)[0])
              })}>
              {
                leftBottomSource.map((item, index) => <option key={index} value={item.id}
                  onDoubleClick={() => this.doubleClickLeftBottomItem(item)}>{item.name}</option>)
              }
            </select>
          </div>
        }
      </div>
      <div className="controlPanel">
        <div>
          <Button icon="right" onClick={() => this.moveToRight()} />
          <Button icon="left" onClick={() => this.removeFromRight()} />
        </div>
      </div>
      <div className="rightPanel">
        <select multiple
          style={{ height: rightItems.length * 25 + 'px' }}
          size={rightItems.length}
          onChange={(e) => this.setState({ rightSelectedItems: Array.from(e.target.selectedOptions, option => rightItems.filter(x => x.id === option.value)[0]) })}>
          {
            rightItems.map((item, index) => <option key={index} value={item.id}
              onDoubleClick={() => this.doubleClickRightItem(item)}>{item.name}</option>)
          }
        </select>
      </div>
    </div>
  }

  onSelectEnd() {
    const { onSelectEnd, itemExtra } = this.props;
    const { type, rightItems } = this.state;
    const items = Array.from(rightItems, x => {
      let newItem;
      if (typeof (itemExtra) === 'function') {
        newItem = { ...x, ...itemExtra() };
      } else if (typeof (itemExtra) === 'object') {
        newItem = { ...x, ...itemExtra };
      } else {
        newItem = { ...x };
      }
      return newItem;
    });
    onSelectEnd && onSelectEnd(type, items);
  }

  render() {
    const { visible, onClose, workFlowTypeShow } = this.props;
    return (
      <DraggableModal title='人员选择'
        visible={visible}
        bodyStyle={{ padding: '0 24px' }}
        className="employeeSelector"
        onOk={() => this.onSelectEnd()}
        onCancel={() => {
          this.setState({ leftSelectedItems: [], rightItems: [], rightSelectedItems: [], })
          onClose && onClose()
        }}
        width={800}>
        <div>
          <Tabs
            size="small"
            value={this.state.activeTabKey}
            tabBarStyle={{ margin: 0 }}
            onChange={(activeKey) => this.setState({ activeTabKey: activeKey }, () => this.fetchLeftTopSource())}
            tabBarExtraContent={workFlowTypeShow && <RadioGroup defaultValue={workFlowSelectorType.supervene.key}
              onChange={e => this.setState({ type: e.target.value })}>
              <Radio value={workFlowSelectorType.supervene.key}>{workFlowSelectorType.supervene.title}</Radio>
              <Radio value={workFlowSelectorType.series.key}>{workFlowSelectorType.series.title}</Radio>
              <Radio value={workFlowSelectorType.jointly.key}>{workFlowSelectorType.jointly.title}</Radio>
            </RadioGroup>}>
            {
              this.state.supportTabs.map(supportTab => <TabPane tab={supportTab.label}
                key={supportTab.value}></TabPane>)
            }
          </Tabs>
          <div>
            {this.renderTabContent(this.state.leftDuties)}
          </div>
        </div>
      </DraggableModal>
    )
  }
}
