import "./index.less"

import React, { PureComponent } from "react"
import DraggableModal from "../DraggableModal"
import { Button, Input, message } from "antd"
import { get } from "../../utils"
import provinces1 from "../../common/provinces.json"
import cities1 from "../../common/cities.json"
import areas1 from "../../common/areas.json"
import _ from "lodash"
import fetchJsonp from "fetch-jsonp"

const { Search } = Input
const ak = "gvy9USNcoa9E1lNrTcY01pYOv8QnHnax"
export default class Geocoder extends PureComponent {
  constructor(props) {
    super(props)
    this.state = {
      address: "",
      options: null, //三级数据
      pos: null, //坐标值
      division: null //三级值
    }
    this.nextFunc = _.debounce(this.nextFunc, 1000)
  }

  componentDidMount() {
    if (this.props.value) {
      this.setState({ address: this.props.value })
    }
    this.props.onChange && this.getProvinceInfos()
  }
  componentWillReceiveProps(newProps) {
    if (newProps.value !== this.props.value) {
      this.setState({ address: newProps.value })
    }
  }

  getProvinceInfos() {
    get("/sysPlatformCfg/getProvinceInfos", {}).then(data => {
      if (data) {
        //console.log('1111');
        this.setState({ provinces: JSON.parse(data.result) }, () => this.getCityInfos())
      }
      if (!data) {
        //console.log('11');
        this.setState({ provinces: provinces1 }, () => this.getCityInfos())
      }
    })
  }
  getCityInfos() {
    get("/sysPlatformCfg/getCityInfos", {}).then(data => {
      if (data) {
        //console.log('2222');
        this.setState({ cities: JSON.parse(data.result) }, () => this.getAreaInfos())
      }
      if (!data) {
        //console.log('22');
        this.setState({ cities: cities1 }, () => this.getAreaInfos())
      }
    })
  }
  getAreaInfos() {
    get("/sysPlatformCfg/getAreaInfos", {}).then(data => {
      if (data) {
        //console.log('333');
        this.setState({ areas: JSON.parse(data.result) }, () => this.setOpitions())
      }
      if (!data) {
        //console.log('33');
        this.setState({ areas: areas1 }, () => this.setOpitions())
      }
    })
  }
  setOpitions() {
    let provinces = _.cloneDeep(this.state.provinces)
    let cities = _.cloneDeep(this.state.cities)
    let areas = _.cloneDeep(this.state.areas)
    areas.forEach(area => {
      const matchCity = cities.filter(city => city.code === area.cityCode)[0]
      if (matchCity) {
        matchCity.children = matchCity.children || []
        matchCity.children.push({
          label: area.name,
          value: area.code
        })
      }
    })

    cities.forEach(city => {
      const matchProvince = provinces.filter(province => province.code === city.provinceCode)[0]
      if (matchProvince) {
        matchProvince.children = matchProvince.children || []
        matchProvince.children.push({
          label: city.name,
          value: city.code,
          children: city.children
        })
      }
    })

    const options = provinces.map(province => ({
      label: province.name,
      value: province.code,
      children: province.children
    }))
    //console.log('80&',options);
    this.setState({ options })
  }

  onChange(val) {
    this.setState({ address: val })
    this.nextFunc(val)
  }

  nextFunc(param1) {
    let arr = [param1, "", ""]
    //加载了 抖动函数
    //this.props.onChange && this.props.onChange(JSON.stringify([pos,fullAddress,res,DetailedAddress]));
    this.props.onChange && this.props.onChange(JSON.stringify(arr)) //三段数据 1. 详细地址 2.坐标 3.三级结构
  }

  getInMpAddress(e) {
    //console.log('70',e);

    let address = ""
    address += e.address.province
    address += e.address.city
    address += e.address.district
    address += e.address.street
    address += e.address.streetNumber

    this.setState({ addressState: address.replace(/undefined/g, "") })
    // return address.replace(/undefined/g,"");
    return e.address
  }
  addressToPosAndCascader(val) {
    //   const {options} = this.state;
    //   if(!options){
    // 	this.getProvinceInfos();
    //   }
    let url = `https://api.map.baidu.com/geocoding/v3/?address=${val}&output=json&ak=${ak}&callback=showLocation`
    fetchJsonp(url, {
      headers: { Accept: "application/json" },
      jsonCallbackFunction: "showLocation"
    })
      .then(response => response.json())
      .then(data => {
        if (data.status === 0) {
          if (data.result && data.result.location) {
            this.setState(
              {
                pos: data.result.location.lng + "," + data.result.location.lat
              },
              () => this.addressToCascader(data.result.location.lat, data.result.location.lng)
            )

            //data.result.location.lat 34.26702439574392
            //data.result.location.lng 117.16555349555169
          }
        } else {
          message.warning("您填写的地址不准确!百度地图未能查找到")
        }
      })
    //地址 转换 坐标 和 三级 联动数据
  }

  addressToCascader(lat, lng) {
    let url = `https://api.map.baidu.com/reverse_geocoding/v3/?ak=${ak}&output=json&coordtype=wgs84ll&location=${lat},${lng}&callback=showLocation`
    fetchJsonp(url, {
      headers: { Accept: "application/json" },
      jsonCallbackFunction: "showLocation"
    })
      .then(response => response.json())
      .then(data => {
        if (data.status === 0) {
          let addressComponent = data.result.addressComponent
          let address = {
            province: addressComponent.province,
            city: addressComponent.city,
            district: addressComponent.district
          }
          //console.log('转换前',address);
          let res = ""
          res = this.BDMapDataTranscode(address).toString()
          //console.log('转换后',res);
          this.setState({ division: res }, () => this.transToChange())
          //data.result.addressComponent // province city district
        }
      })
  }

  transToChange() {
    const { address, pos, division } = this.state
    //console.log('sasss');
    //console.log(address,pos,division);
    this.props.onChange && this.props.onChange(JSON.stringify([address, pos, division]))
  }

  BDMapDataTranscode(address) {
    let arr = []
    //{"streetNumber":"","street…,"district":"泉山区","city":"徐州市","province":"江苏省"}
    const { options } = this.state
    if (address.province && options) {
      options.forEach(item => {
        if (address.province === item.label) {
          arr[0] = item.value
          if (address.city) {
            //有城市的
            if (address.city === "北京市" || address.city === "天津市" || address.city === "上海市" || address.city === "重庆市") {
              //四个直辖市  因为结构不同 走特殊
              item.children.forEach(item1 => {
                if (address.district) {
                  item1.children.forEach(item2 => {
                    if (address.district === item2.label) {
                      arr[2] = item2.value
                      arr[1] = item1.value
                    }
                  })
                }
              })
            } else {
              //其他城市走正常的流程
              item.children.forEach(item1 => {
                if (address.city === item1.label) {
                  arr[1] = item1.value
                  if (address.district) {
                    item1.children.forEach(item2 => {
                      if (address.district === item2.label) {
                        arr[2] = item2.value
                      }
                    })
                  }
                } else {
                  //假设二级找不到所在市 继续寻找三级县/区
                  if (address.district) {
                    item1.children.forEach(item2 => {
                      if (address.district === item2.label) {
                        arr[2] = item2.value
                        arr[1] = item1.value
                      }
                    })
                  }
                }
              })
            }
          } else {
            //没有设置市的二级 所挂在县
            item.children.forEach(item1 => {
              if (address.district) {
                item1.children.forEach(item2 => {
                  if (address.district === item2.label) {
                    arr[2] = item2.value
                    arr[1] = item1.value
                  }
                })
              }
            })
          }
        }
      })
    }

    return arr
  }

  render() {
    return (
      <Search
        placeholder="请输入详细地址"
        enterButton="地址解析"
        size="small"
        value={this.state.address}
        onChange={e => this.onChange(e.target.value)} //debounce((e)=>{console.log(e.target.value)},500)
        //onChange={_.debounce((e)=>{console.log(e.target.value)},500)}//debounce((e)=>{console.log(e.target.value)},500)
        onSearch={value => this.addressToPosAndCascader(value)}
      />
    )
  }
}
