/**
 * Created by 王友生 on 2018-06-27.
 */

import styles from "./index.less"

import React, { PureComponent } from "react"
import { Badge, Popover, List, Avatar, Modal, Input, Button } from "antd"
import { request } from "../../utils/request"
import InfiniteScroll from "react-infinite-scroller"
import OneMessage from "./OneMessage"
import { connect } from "react-redux"

let currentUser = {}
const basicInfoString = localStorage.getItem("basicInfo")
if (basicInfoString) {
  currentUser = JSON.parse(basicInfoString)
}

@connect(state => {
  const messageState = state.get("message")
  const chatUser = messageState.get("chatUser")
  if (!chatUser) {
    return { chatUser: {}, messageState, unreadCount: messageState.get("unreadCount") }
  }
  return {
    chatUser,
    messageState,
    messages: messageState.getIn(["message", chatUser.id, "messages"]),
    messageContent: messageState.getIn(["message", chatUser.id, "messageContent"]),
    unreadCount: messageState.get("unreadCount")
  }
})
export default class IMessage extends PureComponent {
  state = {
    users: [],
    chatVisible: false,
    messages: [
      {
        owner: "others",
        messageType: "others",
        content: "你好"
      },
      {
        owner: "mine",
        messageType: "mine",
        content: "你好"
      }
    ],
    displayCount: 20,
    hasMore: false
  }

  fetchOnlineUsers() {
    request("/sysMessage/getOnlineUserInfos ", {}, false, true).then(data => {
      console.log(data)
      if (data && data.result) {
        this.setState({ users: data.result })
      }
    })
    // .catch(() => {})
  }

  sendMessage() {
    const { chatUser, messageContent } = this.props
    if (messageContent && chatUser) {
      this.props.dispatch({
        type: "message/sendMessage",
        payload: {
          toUser: chatUser,
          content: messageContent
        }
      })
    }
  }

  scrollToBottom() {
    if (this.messagePanel) {
      this.messagePanel.scrollTop = this.messagePanel.scrollHeight
    }
  }

  renderMessage() {
    setTimeout(() => {
      this.scrollToBottom()
    }, 200)
    const { displayCount, hasMore } = this.state
    const { messages } = this.props
    if (!messages) return null
    let start = messages.size - displayCount - 1
    start = start > 0 ? start : 0
    const list = []
    for (var index = start; index < messages.size; index++) {
      const currentMessage = messages.get(index)
      list.push(<OneMessage key={index} position={currentMessage.owner === "mine" ? "right" : "left"} content={currentMessage.content} />)
    }
    return (
      <InfiniteScroll
        pageStart={0}
        threshold={250}
        loadMore={page => {
          console.log(page)
          this.setState({ hasMore: true })
        }}
        isReverse={true}
        hasMore={hasMore}
        loader={
          <div className="loader" key={0}>
            Loading ...
          </div>
        }
        useWindow={false}
      >
        {list}
      </InfiniteScroll>
    )
  }

  render() {
    const { users, chatVisible, usersVisible } = this.state
    const { chatUser, messageContent, dispatch, unreadCount: totalUnread, messageState } = this.props
    return (
      <div>
        <Popover
          visible={usersVisible}
          placement="topRight"
          title={`在线用户${users.length}`}
          trigger="click"
          overlayClassName="messagePopup"
          overlayStyle={{ width: "300px" }}
          content={
            <List
              dataSource={users}
              renderItem={(item, index) => {
                if (item.id === currentUser.id) {
                  return <List.Item style={{ display: "none" }} />
                }
                const unreadCount = messageState.getIn(["message", item.id, "unreadCount"]) || 0
                return (
                  <List.Item
                    key={index}
                    className={chatUser.id === item.id ? "itemSelected" : ""}
                    onClick={() => {
                      this.setState({ chatVisible: true, usersVisible: false })
                      this.props.dispatch({
                        type: "system/changeChatUser",
                        payload: {
                          chatUser: item
                        }
                      })
                    }}
                  >
                    <List.Item.Meta
                      avatar={<Avatar src="https://zos.alipayobjects.com/rmsportal/ODTLcjxAfvqbxHnVXCYX.png" />}
                      title={item.name}
                      description="江苏增宇"
                    />
                    <div style={{ paddingRight: "10px" }}>
                      <Badge count={unreadCount}>
                        <a href="#" className="head-example" />
                      </Badge>
                    </div>
                  </List.Item>
                )
              }}
            ></List>
          }
          onVisibleChange={visible => {
            this.setState({ usersVisible: visible })
            visible && this.fetchOnlineUsers()
          }}
        >
          <div className="messageWrapper">
            <Badge count={totalUnread} className="badge">
              <a href="#" className="head-example" />
            </Badge>
          </div>
        </Popover>
        <Modal
          className="chatPanel"
          title="聊天框"
          width={800}
          footer={null}
          onCancel={() => {
            this.setState({ chatVisible: false })
            dispatch({
              type: "message/removeChatUser"
            })
          }}
          visible={chatVisible}
        >
          <div style={{ display: "flex" }}>
            <div className="left">
              <List
                dataSource={users}
                renderItem={(item, index) => {
                  if (item.id === currentUser.id) {
                    return <List.Item style={{ display: "none" }} />
                  }
                  const unreadCount = messageState.getIn(["message", item.id, "unreadCount"]) || 0
                  return (
                    <List.Item
                      key={index}
                      className={chatUser.id === item.id ? "itemSelected" : ""}
                      onClick={() => {
                        this.props.dispatch({
                          type: "system/changeChatUser",
                          payload: {
                            chatUser: item
                          }
                        })
                      }}
                    >
                      <List.Item.Meta
                        avatar={<Avatar src="https://zos.alipayobjects.com/rmsportal/ODTLcjxAfvqbxHnVXCYX.png" />}
                        title={item.name}
                        description="江苏增宇"
                      />
                      <div style={{ paddingRight: "10px" }}>
                        <Badge count={unreadCount}>
                          <a href="#" className="head-example" />
                        </Badge>
                      </div>
                    </List.Item>
                  )
                }}
              ></List>
            </div>
            <div className="right">
              <h3 className="title">聊天内容</h3>
              <div className="messagesPanel" ref={messagePanel => (this.messagePanel = messagePanel)}>
                {this.renderMessage()}
              </div>
              <div className="sendMessagePanel">
                <div className="textAreaWrapper">
                  <Input.TextArea
                    autosize={{ minRows: 3, maxRows: 3 }}
                    className="textArea"
                    value={messageContent}
                    onChange={e =>
                      dispatch({
                        type: "message/changeUserContentMessage",
                        payload: {
                          toUser: chatUser,
                          content: e.target.value
                        }
                      })
                    }
                  ></Input.TextArea>
                </div>
                <Button
                  size="small"
                  className="sendButton"
                  onKeyPress={e => e.which == 13 && this.sendMessage()}
                  onClick={() => this.sendMessage()}
                >
                  发送
                </Button>
              </div>
            </div>
          </div>
        </Modal>
      </div>
    )
  }
}
