/**
 * @Name : 提醒组件
 * @Description : 从ant design中拷贝
 * @Author : 王友生
 * @Creation Date : 2018/3/10 0010 10:51
 * @ModificationHistory
 */

import React, { PureComponent } from "react"
import { Popover, Icon, Tabs, Badge, Spin } from "antd"
import classNames from "classnames"
import List from "./NoticeList"
import styles from "./index.less"
import NoNoticeIcon from "../../assets/noNotice.svg"

const { TabPane } = Tabs

export default class NoticeIcon extends PureComponent {
  static defaultProps = {
    // onItemClick: () => {},
    // onPopupVisibleChange: () => {},
    // onTabChange: () => {},
    // onClear: () => {},
    loading: false,
    locale: {
      emptyText: "暂无数据",
      clear: "清空"
    },
    emptyImage: NoNoticeIcon
  }
  static Tab = TabPane

  constructor(props) {
    super(props)
    this.state = {}
    if (props.children && props.children[0]) {
      this.state.tabType = props.children[0].props.title
    }
  }

  onItemClick = (item, tabProps) => {
    const { onItemClick } = this.props
    onItemClick(item, tabProps)
  }
  onTabChange = tabType => {
    this.setState({ tabType })
    this.props.onTabChange(tabType)
  }

  getNotificationBox() {
    const { loading, locale } = this.props
    let { children } = this.props
    // console.log('children===>', children);
    if (!children) {
      return null
    }

    if (!(children instanceof Array)) {
      children = [children]
    }
    const panes = children.map(child => {
      const title =
        child.props.list && child.props.list.length > 0 ? `${child.props.title} (${child.props.list.length})` : child.props.title
      return (
        <TabPane tab={title} key={child.props.title}>
          <List
            {...child.props}
            data={child.props.list}
            onClick={item => this.onItemClick(item, child.props)}
            onClear={() => this.props.onClear(child.props.title)}
            title={child.props.title}
            locale={locale}
          />
        </TabPane>
      )
    })
    return (
      <Spin spinning={loading} delay={0}>
        <Tabs className="tabs" onChange={this.onTabChange}>
          {panes}
        </Tabs>
      </Spin>
    )
  }

  render() {
    const { className, count, popupAlign, onPopupVisibleChange, icon, extra } = this.props
    const noticeButtonClass = classNames(className, "noticeButton")
    const notificationBox = this.getNotificationBox()
    const trigger = (
      <span className={noticeButtonClass}>
        <Badge count={count} className="badge">
          {icon ? icon : <Icon type="bell" className="icon" />}
        </Badge>
      </span>
    )
    if (!notificationBox) {
      return trigger
    }
    const popoverProps = {}
    if ("popupVisible" in this.props) {
      popoverProps.visible = this.props.popupVisible
    }

    return (
      <Popover
        placement="bottomRight"
        content={
          <React.Fragment>
            {extra ? extra : ""}
            {notificationBox}
          </React.Fragment>
        }
        popupClassName="noticePopover"
        arrowPointAtCenter
        popupAlign={popupAlign}
        onVisibleChange={onPopupVisibleChange}
        {...popoverProps}
      >
        {trigger}
      </Popover>
    )
  }
}
