/*
 * @Author: 陈云翔
 * @Date: 2022-05-11 16:53:37
 * @Last Modified by: 陈云翔
 * @Last Modified time: 2023-01-17 10:56:51
 */

import React, { PureComponent, Fragment } from "react"
import { message, Button } from "antd"
import { request } from "../../utils/request"
import "./index.less"
import { convertListToTreeSource } from "../../utils"
import DraggableModal from "../DraggableModal"
import Organization from "../Organization"

class OrganizationModal extends PureComponent {
  state = {}

  constructor(props) {
    super(props)
  }

  render() {
    const { url, dispatch, id } = this.props
    return (
      <Fragment>
        <i
          className="iconfont icon-bumen1"
          style={{ color: "#1890ff" }}
          onClick={() => {
            request(url, { id: id }).then(res => {
              if (res && res.result.length > 0) {
                let tree = convertListToTreeSource(res.result, { id: "id", name: "name", pid: "pid" })
                this.setState({ organizationTree: tree, organization: true })
              } else {
                message.warning("无上下级关系")
              }
            })
          }}
        ></i>
        <DraggableModal
          visible={this.state.organization}
          title={"组织"}
          okText={"确定"}
          footer={[
            <Button onClick={() => this.setState({ organization: false })} size="small">取消</Button>
          ]}
          width={"80vw"}
          bodyStyle={{ padding: "12px", height: "70vh", overflow: "auto" }}
          cancelText={"取消"}
          // onOk={(e) => {}}
          onCancel={e => {
            this.setState({ organization: false })
          }}
        >
          <div className="organizationTree">
            <Organization
              click={item => {
                this.setState({ organization: false })
                dispatch({
                  type: "system/jumpToDetailPage",
                  payload: {
                    functionCode: item.funCode,
                    id: item.id
                  }
                })
              }}
              id={id}
              tree={this.state.organizationTree}
            />
          </div>
        </DraggableModal>
      </Fragment>
    )
  }
}

export default OrganizationModal
