/**
 * @Name : 页签组件
 * @Description : 页签导航
 * @Author : 王友生
 * @Creation Date : 2018/3/10 0010 10:51
 * @ModificationHistory
 * @props: tabs 数据源
 * @props： tabActiveKey 当前选中的tab
 * @props：onRemoveTab 移除tab的回调函数
 * @props: onTabChange tab切换时的回调函数
 * @props: onDeleteTabs 删除tab的回调函数
 */

import './index.less';
import classNames from 'classnames';
import React, { PureComponent } from 'react';
import Popover from 'antd/lib/popover';
import Icon from 'antd/lib/icon';
import Checkbox from 'antd/lib/checkbox';
import Button from 'antd/lib/button';
import Divider from 'antd/lib/divider';
// import HeaderSearch from "../HeaderSearch";
import { functionCodeType } from '../../common/const';
import { Dropdown, Menu as AntMenu } from "antd";

class PageTabs extends PureComponent {

  state = {
    ulLeft: 0,
    minLeftPosition: 0,
    deletableTabs: []
  }

  componentWillReceiveProps(newProps) {
    // 在添加或者删除tab时，调整tab的位置
    if (newProps.tabs.size !== this.props.tabs.size) {
      this.setState({ deletableTabs: newProps.tabs.filter(item => !item.hideClose).toArray() || [] })
      this.calculateTabsPosition();
    } else if (newProps.activeTab !== this.props.activeTab) {
      this.moveToActiveTab(newProps.activeTab);
    }
  }

  onTabClick(item) {
    const { onTabChange } = this.props;
    onTabChange && onTabChange(item);
  }

  onRemoveTab(e, item) {
    e.stopPropagation();
    const { onRemoveTab } = this.props;
    onRemoveTab && onRemoveTab(item)
  }

  calculateTabsPosition() {
    var actualWidth = 0;
    let children = this.tabsUl.children;
    for (var i = 0; i < this.tabsUl.children.length; i++) {
      actualWidth += children[i].clientWidth;
    }
    // 子的li长度大于ul的长度是，调整ul的左边距
    if (actualWidth > this.tabsUl.clientWidth) {
      const minLeftPosition = this.tabsUl.clientWidth - actualWidth - 30 - children[children.length - 1].clientWidth;
      this.setState({ ulLeft: minLeftPosition, minLeftPosition: minLeftPosition });
    } else {
      this.setState({ ulLeft: 0 });
    }
  }

  // 当tab被选中时，将tab移到可看到的位置
  moveToActiveTab(activeTab) {
    const { tabs } = this.props;
    const { ulLeft, minLeftPosition } = this.state;
    const filtersTabs = tabs.filter((item) => item === activeTab);
    if (filtersTabs && filtersTabs.size > 0) {
      const selectedTab = filtersTabs.get(0);
      const selectedTabIndex = tabs.indexOf(selectedTab);
      var selectedTabLeftPosition = 0;
      for (var i = 0; i < selectedTabIndex; i++) {
        selectedTabLeftPosition += this.tabsUl.children[i].clientWidth;
      }
      let selectedTabRightPosition = selectedTabLeftPosition + this.tabsUl.children[selectedTabIndex].clientWidth;
      if (selectedTabLeftPosition < - ulLeft) {
        this.setState({ ulLeft: this.tabsUl.clientWidth > selectedTabRightPosition ? 0 : - selectedTabLeftPosition });
      } else if (selectedTabRightPosition - this.tabsUl.clientWidth > ulLeft) {
        this.setState({ ulLeft: selectedTabRightPosition - this.tabsUl.clientWidth < minLeftPosition ? selectedTabRightPosition - this.tabsUl.clientWidth : minLeftPosition });
      }
    }
  }

  onTabLeftButtonClick(e) {
    e.stopPropagation();
    let { ulLeft } = this.state;
    ulLeft += this.tabsUl.clientWidth;
    this.setState({ ulLeft: ulLeft < 0 ? ulLeft : 0 });
  }

  onTabRightButtonClick(e) {
    e.stopPropagation();
    let { ulLeft, minLeftPosition } = this.state;
    const newUlLeft = ulLeft - this.tabsUl.clientWidth;
    this.setState({ ulLeft: newUlLeft < minLeftPosition ? minLeftPosition : newUlLeft });
  }

  // 当tab被选中时，记录它的索引
  onTabsChecked(item, index) {
    let { deletableTabs } = this.state;
    item.checked = !item.checked;
    deletableTabs[index] = item;
    this.setState({ deletableTabs: [...deletableTabs] })
  }

  onAllTabsChecked() {
    let { deletableTabs } = this.state;
    let allChecked = deletableTabs.filter(item => !item.checked).length <= 0;
    deletableTabs.map((item) => {
      item.checked = !allChecked;
    });
    this.setState({ deletableTabs: [...deletableTabs] })
  }

  onCloseCheckedTabs() {
    const { deletableTabs } = this.state;
    const { onDeleteTabs } = this.props;
    const checkedTabs = deletableTabs.filter(item => item.checked);
    if (onDeleteTabs && checkedTabs.length > 0) {
      checkedTabs.map(item => delete item.checked);
      onDeleteTabs(checkedTabs);
    }
  }

  renderDeletableTabs() {
    const { deletableTabs } = this.state;
    const { lang } = this.props;
    let allChecked = deletableTabs.filter(item => !item.checked).length <= 0;
    return <div className="taskManager">
      <div className="tabsWappers">
        {
          deletableTabs.map((item, index) => {
            if (item.hideClose) {
              return null;
            }
            return <div key={index}>
              <Checkbox checked={item.checked} onChange={() => this.onTabsChecked(item, index)} /><span>{lang === 'en' ? item.nameEN : item.name}</span>
              <Divider style={{ margin: '10px 0', width: '260px' }} />
            </div>
          })
        }
      </div>
      <div className="controlWrapper">
        <Divider style={{ margin: '0 -15px 10px', width: '280px' }} />
        <Checkbox checked={deletableTabs.length > 0 && allChecked} onChange={(e) => this.onAllTabsChecked()} />
        <Button size="small" onClick={() => this.onCloseCheckedTabs()}>关闭选中</Button>
      </div>
    </div>;
  }

  renderAdd() {
    const { quickMenus, onQuickItemClick, lang } = this.props;
    const menu = (
      <AntMenu>
        {
          quickMenus && quickMenus.map((item, index) => {
            if (item.items && item.items.length > 0) {
              return null
            } else if (item.funType === functionCodeType.list.value) {
              return <AntMenu.Item>
                <a rel="noopener noreferrer" href="#" onClick={() => onQuickItemClick && onQuickItemClick(item)}>新建{lang === 'en' ? item.nameEN : item.name}</a>
              </AntMenu.Item>
            }
            return null;
          })
        }
      </AntMenu>
    );
    return (
      <Dropdown placement="bottomRight" overlay={menu}>
        <div>
          <Icon type="plus" style={{ fontSize: '13px' }} />
        </div>
      </Dropdown>
    )
  }

  render() {
    const { tabs, activeTab, style, lang } = this.props;
    const { ulLeft } = this.state;

    const ulStyle = {
      left: ulLeft + 'px'
    };

    return (
      <div className="pageTabs" style={style}>
        <div className={classNames("control", "pre")} onClick={(e) => this.onTabLeftButtonClick(e)}>
          <Icon type="double-left" />
        </div>
        <div className={classNames("control", "next")} onClick={(e) => this.onTabRightButtonClick(e)}>
          <Icon type="double-right" />
        </div>
        <Popover content={this.renderDeletableTabs()} placement="bottomRight" title="任务管理器">
          <div className={classNames("control", "down")} onClick={(e) => this.onTabRightButtonClick(e)}>
            <Icon type="down" />
          </div>
        </Popover>
        {/* <div className={classNames("control", "add")}>
          {this.renderAdd()}
        </div> */}
        {/*<div className={classNames("control", "search")}>
          <HeaderSearch/>
        </div>*/}
        <div className="tabs">
          <ul style={ulStyle} className="title" ref={(ul) => this.tabsUl = ul}>
            {
              tabs && tabs.map((item, index) => {
                let liClassNames = '';
                if (window.navigationMultiFunction === '1') {
                  liClassNames = classNames(item.hideClose ? "noClose" : '', item.functionCodeKey === activeTab.functionCodeKey ? "selected" : '', item.id === 'dashboard_0001' ? "mainTab" : '');
                } else {
                  liClassNames = classNames(item.hideClose ? "noClose" : '', item === activeTab ? "selected" : '', item.id === 'dashboard_0001' ? "mainTab" : '');
                }

                return <li key={index} className={liClassNames} onClick={() => this.onTabClick(item)}>
                  <span>{index == 0 && <Icon type="home" theme="filled" style={{marginRight: '2px' }} />}{lang === 'en' ? item.nameEN : item.menuName}{item.showTitleName ? `-${item.showTitleName}` : ''}</span>
                  {!item.hideClose &&
                    <Icon type="close" className="close" onClick={(e) => this.onRemoveTab(e, item)} />}
                </li>
              })
            }
          </ul>
        </div>
      </div>
    )
  }
}

export default PageTabs;
