/**
 * Created by 王友生 on 2018-08-01.
 */

import styles from "./index.less"
import React, { Component } from "react"
import { Upload, Icon, Modal, message } from "antd"
import { serverUrl, fileServerUrl } from "../../common/config"
import { systemCommonFileFunctionCode, supportImage } from "../../common/const"
import classnames from "classnames"

export default class PicturesWall extends Component {
  constructor(props) {
    super(props)

    const { defaultFileList } = props
    if (defaultFileList) {
      const originalFileList = []
      defaultFileList.map((fileUrl, index) => {
        originalFileList.push({
          uid: index,
          url: fileServerUrl + fileUrl
        })
        return
      })
      this.state = { fileList: originalFileList, previewVisible: false, fullScreen: false }
    } else {
      this.state = { fileList: [], previewVisible: false, fullScreen: false }
    }
  }

  componentWillReceiveProps(newProps) {
    const { defaultFileList, disabled } = newProps
    if (disabled && defaultFileList && defaultFileList !== this.props.defaultFileList) {
      const originalFileList = []
      defaultFileList.map((fileUrl, index) => {
        originalFileList.push({
          uid: index,
          url: fileServerUrl + fileUrl
        })
        return
      })
      this.setState({ fileList: originalFileList, previewVisible: false })
    }
  }

  beforeUpload(file) {
    if (supportImage.indexOf(file.type) < 0) {
      message.error("请上传jpg，jpeg或者png的图片")
      return false
    }
    return true
  }

  handlePreview(file) {
    this.setState({
      previewImage: file.url || file.thumbUrl,
      previewVisible: true
    })
  }

  render() {
    const { previewVisible, previewImage, fileList, fullScreen } = this.state
    const { disabled } = this.props
    const hideClass = classnames(disabled ? "hideDeleteAction" : "")
    const uploadProps = {
      name: "file",
      disabled,
      onRemove: true,
      defaultFileList: this.state.fileList,
      headers: { authorization: localStorage.getItem("token") || "", accessType: "backend" },
      listType: "picture-card",
      beforeUpload: newFile => this.beforeUpload(newFile),
      action: `${serverUrl}/attach/uploadFiles?funCode=${this.props.functionCode || systemCommonFileFunctionCode}`,
      onPreview: file => this.handlePreview(file),
      onChange: info => {
        const status = info.file.status
        if (!info.file.status) {
          const deletedIndex = info.fileList.indexOf(info.file)
          info.fileList.splice(deletedIndex)
          this.setState({ fileList: info.fileList })
        }
        // if (status !== "uploading") {
        // }
        if (status === "done") {
          if (info.file.response && info.file.response.code === "0") {
            message.success(`${info.file.name} 上传成功！`)
            if (info.file.response.data && info.file.response.data.result && info.file.response.data.result.length > 0) {
              const fileInfo = info.file.response.data.result[0]
              this.props.uploadSuccess && this.props.uploadSuccess(fileInfo.fileUrl + fileInfo.newFileName)
            }
          } else {
            message.error(`${info.file.name} 上传失败！`)
            const deletedIndex = info.fileList.indexOf(info.file)
            info.fileList.splice(deletedIndex)
            this.setState({ fileList: info.fileList })
          }
        } else if (status === "error") {
          message.error(`${info.file.name} 上传失败！`)
        } else if (status === "removed") {
          const deletedIndex = this.state.fileList.indexOf(info.file)
          this.setState({ fileList: info.fileList })
          this.props.deleteFile && this.props.deleteFile(deletedIndex)
        }
      }
    }
    if (disabled) {
      uploadProps.fileList = this.state.fileList
    }
    const uploadButton = (
      <div>
        <Icon type="plus" />
        <div className="ant-upload-text">上传</div>
      </div>
    )
    return (
      <div className={hideClass}>
        <Upload {...uploadProps}>{fileList.length >= (this.props.max || 18) ? null : !disabled && uploadButton}</Upload>
        <Modal
          visible={previewVisible}
          bodyStyle={
            fullScreen
              ? {
                  width: "80vw",
                  height: "80vh",
                  overflow: "auto"
                }
              : {}
          }
          width={fullScreen ? "80vw" : "520px"}
          title={
            <div
              style={{
                height: "34px",
                lineHeight: "34px",
                padding: "0 10px"
              }}
            >
              预览
              <Icon
                style={{
                  cursor: "pointer",
                  float: "right",
                  marginRight: "40px",
                  marginTop: "10px"
                }}
                type={fullScreen ? "shrink" : "arrows-alt"}
                onClick={() => this.setState({ fullScreen: !fullScreen })}
                size="small"
              />
            </div>
          }
          style={fullScreen ? { width: "80vw", height: "80vh" } : {}}
          footer={null}
          onCancel={() => this.setState({ previewVisible: false })}
        >
          <img alt="example" style={{ width: "100%" }} src={previewImage} />
        </Modal>
      </div>
    )
  }
}
