import React, { PureComponent } from 'react';
import { InputNumber, message } from 'antd';

class PriceRange extends PureComponent {
  constructor(props) {
    super(props);

    this.state = {
      start: undefined,
      end: undefined,
    }
  }

  componentDidMount() {
    const { value } = this.props;
    this.handleChangeState(value);
  }

  componentWillReceiveProps(nextProps) {
    const { value } = nextProps;

    if (value
      && this.props.value
      && (value[0] !== this.props.value[0] || value[1] !== this.props.value[1])
    ) {
      this.handleChangeState(value);
    }
  }

  handleChangeState = (value) => {
    this.setState({
      start: value && value[0] ? value[0] : undefined,
      end: value && value[1] ? value[1] : undefined,
    })
  }

  handleValueChange = (start, end) => {
    this.props.onChange && this.props.onChange(JSON.stringify([start, end]));
  }

  render() {
    const { size, disabled, placeholder = [] } = this.props;
    return (
      <div className="custom-price-range">
        <InputNumber
          disabled={disabled}
          size={size}
          value={this.state.start}
          precision={Number(window.numberFixedConst) || 2}
          onBlur={() => {
            if (this.state.start > this.state.end) {
              this.setState({
                end: this.state.start
              })
            }
          }}
          onChange={value => {
            let end = this.state.end;
            if (value && !end) {
              end = value;
            }
            this.setState({
              start: value,
              end,
            });

            this.handleValueChange(value, this.state.end);
          }}
          placeholder={placeholder[0] || '￥'}
        />
        <span> ~ </span>
        <InputNumber
          disabled={disabled}
          size={size}
          value={this.state.end}
          precision={Number(window.numberFixedConst) || 2}
          onBlur={() => {
            if (this.state.end < this.state.start) {
              this.setState({
                start: this.state.end
              })
            }
          }}
          onChange={value => {
            let start = this.state.start;
            if (value && !start) {
              start = value;
            }

            this.setState({
              start,
              end: value,
            });

            this.handleValueChange(this.state.start, value);
          }}
          placeholder={placeholder[1] || '请输入结束金额'}
        />
      </div>
    )
  }
}

export default PriceRange;