/**
 * Created by 王友生 on 2018-04-03.
 */

import styles from './index.less';
import React, { PureComponent } from 'react';
import { Row, Col, Modal, Icon } from 'antd';
import QueueAnim from 'rc-queue-anim';
import CustomTree from '../CustomTree';
import classnames from 'classnames';
import EllipsisText from '../EllipsisText';
import DraggableModal from "../DraggableModal";

export default class QuickPanel extends PureComponent {

  state = {
    showQuickPanel: false,
    editVisible: false,
    checkedMenuItems: [],
    draggedIndex: - 1,
    overIndex: - 1,
    checkedKeys: []
  }

  componentWillReceiveProps(newProps) {
    if (newProps.config !== this.props.config) {
      const checkedKeys = Array.from(newProps.config, (item) => item.id);
      this.setState({ checkedMenuItems: newProps.config, checkedKeys });
    }
  }

  render() {
    const { menuList, originalMenu, saveQuickPanel } = this.props;
    const { checkedMenuItems, draggedIndex, overIndex, editVisible, checkedKeys, showQuickPanel } = this.state;
    return <div>
      {/*<div className={styles.quickContainer}*/}
           {/*onMouseEnter={() => this.setState({ showQuickPanel: true })}*/}
           {/*onMouseLeave={() => this.setState({ showQuickPanel: false })}>*/}

      <div className="quickContainer">
        {
          // showQuickPanel && <div className={styles.quickPanel}>
          <div className="quickPanel">
            <Row gutter={12}>
              <QueueAnim>
                {
                  this.props.config && this.props.config.map((item, index) =>{
                    if (item.items && item.items.length > 0) {
                      return null;
                    }
                    return <Col lg={8}
                         title={item.menuName}
                         key={index}
                         onClick={() => {
                           this.setState({ showQuickPanel: false });
                           this.props.onItemClick && this.props.onItemClick(item)
                         }
                         }>
                      <div className="itemWrapper">
                        {/*<img src={item.icon} />*/}
                        { item.icon ? <div><i className={"icon iconfont " + item.icon }></i></div> : <div><i className={"icon iconfont icon-dingdan" }></i></div> }
                        <EllipsisText>{item.menuName}</EllipsisText>
                      </div>
                    </Col>})
                }
                <Col lg={24}
                     title="添加更多"
                     key='setting'
                     onClick={() => {
                       this.setState({ editVisible: true })
                     }}>
                  <div className="itemWrapper">
                    {/*<img src={item.icon} />*/}
                    <div ><i className={"icon iconfont icon-xinzeng"} style={{border:'1px dashed #427DCE',color:'#427DCE'}}></i></div>
                    <EllipsisText>添加更多</EllipsisText>
                  </div>
                </Col>
              </QueueAnim>
            </Row>
            {/*<Icon className="setting" type="setting" onClick={() => this.setState({ editVisible: true })} />*/}
          </div>
        }
      </div>
      <DraggableModal title="常用功能配置"
             visible={editVisible}
             className="popUp"
             onOk={() => {
               saveQuickPanel && saveQuickPanel(checkedMenuItems);
               this.setState({ editVisible: false });
             }}
             onCancel={() => this.setState({ editVisible: false })}
             width={800}
             bodyStyle={{ padding: '0 8px', height: '400px', overflow: 'auto' }}>
        <div className="settingPanel">
          <div className="treeWrapper">
            <CustomTree
              allowSearch
              filterFromAll
              checkedKeys={checkedKeys}
              onCheck={(checkedMenuItems) => {
                this.setState({ checkedMenuItems })
              }}
              data={menuList || []}
              treeField={{ id: 'id', name: 'menuName', pid: 'pid' }} />
          </div>
          <div className="settingContent">
            <Row gutter={6}>
              {
                checkedMenuItems.map((item, index) => {
                  return <Col
                    lg={6}
                    key={index}
                    draggable
                    onDragStart={(e) => {
                      e.stopPropagation();
                      this.setState({ draggedIndex: index });
                    }}
                    onDragOver={(e) => {
                      e.stopPropagation();
                      draggedIndex !== index && overIndex !== index && this.setState({ overIndex: index });
                    }}
                    onDragEnd={(e) => {
                      e.stopPropagation();
                      const dragged = checkedMenuItems[draggedIndex];
                      checkedMenuItems.splice(draggedIndex, 1);
                      checkedMenuItems.splice(overIndex, 0, dragged);
                      this.setState({ overIndex: - 1, draggedIndex: - 1 });
                    }}
                  >
                    <div className={classnames("checkedItemWrapper", index === overIndex ? "over" : '')}>
                      <span>{item.menuName}</span>
                    </div>
                  </Col>
                })
              }
            </Row>
          </div>
        </div>
      </DraggableModal>
    </div>
  }
}
