/**
 * Created by 王友生 on 2018-06-21.
 */
import styles from './index.less';

import React, { PureComponent } from 'react';
import { Tabs, Button, Select, Cascader, Icon, Divider } from 'antd';
import { List, Map } from 'immutable';
import CustomTreeSelect from '../../components/CustomTreeSelect';
import CustomSelect from '../../components/CustomSelect';
import { referenceStyleType, referenceStyleFormStyleMapping } from '../../common/const';

const TabPane = Tabs.TabPane;

export default class ReferenceConfig extends PureComponent {

  state = {
    options: []
  }

  componentDidMount() {
    const options = [];
    const { tables, tableProps } = this.props;
    if (tables && tableProps) {
      tables.map(item => {
        options.push({
          value: item.tableName,
          label: item.tableDesc,
          children: Array.from(tableProps[item.tableName], x => {
            return {
              label: x.fieldDesc || x.title,
              value: x.field || x.dataIndex
            }
          })
        });
      });
      this.setState({ options });
    }
  }

  renderReferenceRelation(condition, conditionIndex, referenceSource, conditionKey) {
    const { item, onReferenceListChange } = this.props;
    const { options } = this.state;
    let source = [];
    if (referenceSource) {
      if (referenceSource.refType === referenceStyleType.select.value || referenceSource.refType === referenceStyleType.treeSelect.value || referenceSource.refType === referenceStyleType.popupTreeSelect.value) {
        source = referenceSource.source || [];
      } else if (referenceSource.refType === referenceStyleType.popTable.value || referenceSource.refType === referenceStyleType.popTreeTable.value || referenceSource.refType === referenceStyleType.business.value) {
        source = referenceSource.nodes || [];
      }
    }
    return <div className="relationWrapper" key={conditionIndex}>
      <div>
        <CustomSelect
          selectProps={{
            placeholder: "请选择参照字段",
            value: condition.get('referenceField')
          }}
          source={source}
          onChange={(value) =>
            onReferenceListChange(item.setIn([conditionKey, conditionIndex, 'referenceField'], value))
          } />
        <Icon type="swap" />
        <Cascader
          value={List.isList(condition.get('formField')) ? condition.get('formField').toJSON() : condition.get('formField')}
          style={{ width: '250px' }}
          expandTrigger="hover"
          placeholder="请选择表字段"
          options={options}
          onChange={(value) => onReferenceListChange(item.setIn([conditionKey, conditionIndex, 'formField'], value))} />
      </div>
      <Button shape='circle' type="danger" icon='delete'
        onClick={() => onReferenceListChange(item.deleteIn([conditionKey, conditionIndex]))} />
    </div>
  }

  renderReferenceWrapper() {
    const { references, item, onReferenceListChange } = this.props;
    const referenceSource = this.getReference(item.get('referenceKey'));
    return <div className="referenceWrapper">
      <div className="relationWrapper">
        <CustomTreeSelect
          needSelectItem
          antProps={{
            style: {
              minWidth: '250px'
            }
          }}
          placeholder="请选择参照"
          value={item.get('referenceKey')}
          data={references}
          treeField={{
            id: 'id',
            name: 'refName',
            pid: 'pid'
          }}
          onChange={(value, selectItem) => onReferenceListChange(item.set('referenceKey', value)
            .set('referenceField', '')
            .set('type', referenceStyleFormStyleMapping[selectItem.refType])
            .set('triggerReference', (referenceStyleType.select.value === selectItem.refType || referenceStyleType.treeSelect.value === selectItem.refType) ? '1' : '0'))} />
        <CustomSelect
          needSelectItem
          selectProps={{
            placeholder: "请选择参照字段",
            value: item.get('referenceField'),
            disabled: !referenceSource || referenceSource['nodes'].length === 0
          }}
          source={referenceSource && referenceSource['nodes'] || []}
          onChange={(value, selectItem) =>
            onReferenceListChange(item.set('referenceField', value))
          } />
      </div>
      <Divider orientation="left">
        参照条件
        <Button icon="plus" shape="circle" style={{ marginLeft: '5px' }} size="small"
          onClick={() => {
            onReferenceListChange(item.update('referenceConditions', conditions => {
              conditions = conditions || List([]);
              return conditions.push(Map({}))
            }))
          }}>
        </Button>
      </Divider>
      {
        item.get('referenceConditions') && item.get('referenceConditions').map((condition, conditionIndex) => {
          return this.renderReferenceRelation(condition, conditionIndex, referenceSource, 'referenceConditions');
        })
      }
      <Divider orientation="left">影响字段<Button icon="plus" style={{ marginLeft: '5px' }} shape="circle" size="small"
        onClick={() => {
          onReferenceListChange(item.update('affectFields', affectFields => {
            affectFields = affectFields || List([]);
            return affectFields.push(Map({}))
          }))
        }}></Button>
      </Divider>
      {
        item.get('affectFields') && item.get('affectFields').map((affectField, affectFieldIndex) => {
          return this.renderReferenceRelation(affectField, affectFieldIndex, referenceSource, 'affectFields');
        })
      }
    </div>
  }

  getReference(referenceKey) {
    let { references, referencesObj, fetchReference } = this.props;
    const referenceSource = referencesObj[referenceKey];
    if (referenceSource) {
      return referenceSource;
    } else {
      const filteredReference = references.filter(x => x.id === referenceKey) || [];
      if (filteredReference.length > 0) {
        fetchReference(referenceKey);
      }
    }
  }

  render() {
    return <div>
      {
        this.renderReferenceWrapper()
      }
    </div>
  }
}
