/**
 * Created by 王友生 on 2018-07-26.
 */

import React, { PureComponent } from "react"
import { Button, Modal, Tabs, Select, Form, Input, Divider } from "antd"
import { Map, fromJS } from "immutable"
import CustomTreeSelect from "../CustomTreeSelect"
import { request } from "../../utils"
import { tableTypeSourceKey, tableNickLabel } from "../../common/const"
import AutoGenerationForm from "../AutoGenerationForm"
import DraggableModal from "../DraggableModal"

const FormItem = Form.Item

const basicInfoConfig = [
  [
    {
      title: "弹窗",
      dataIndex: "shortFormId",
      type: "treeSelect",
      field: {
        value: "id",
        label: "winName",
        pid: "pid"
      },
      width: 12,
      source: []
    },
    {
      title: "数据源api",
      dataIndex: "dataSourceApi",
      type: "text",
      width: 12,
      help: "当弹窗打开时调用的api，调用之后覆盖弹窗中的数据,此api会传当前单据id"
    },
    {
      title: "保存api",
      dataIndex: "saveApi",
      type: "text",
      width: 12,
      help: "不填则调用表单的保存,填写'0'则不调取接口"
    }
  ]
]

export default class ShortFormConfig extends PureComponent {
  state = {
    visible: false,
    update: true,
    activeKey: "0"
  }

  openSettingPop() {
    const options = []
    const { tables, tableProps, mode } = this.props
    // if(mode == "free"){

    // } else {
    if (mode != "free") {
      tables.map(item => {
        if (item.tableType !== tableTypeSourceKey.childTableKey) {
          tableProps[item.tableName].map(filed => {
            options.push({
              value: item.tableName + tableNickLabel + filed.field,
              label: item.tableDesc + "/" + filed.fieldDesc
            })
            return true
          })
        }
        return true
      })
    }

    // }
    this.setState({ options, visible: true })
  }

  renderTabPane(itemConfig, index) {
    const { shortFormList, onConfigChange, config } = this.props
    return (
      <div>
        <div>
          <AutoGenerationForm
            config={basicInfoConfig}
            source={itemConfig}
            extraConfig={{
              shortFormId: {
                source: shortFormList
              }
            }}
            onItemValueChange={(value, dataIndex) => {
              if (dataIndex === "shortFormId") {
                const filterShortForms = shortFormList.filter(x => x.id === value)
                if (filterShortForms && filterShortForms.length > 0) {
                  const shortForm = filterShortForms[0]
                  if (shortForm.pid) {
                    this.onShortFormChanged(shortForm, index, value, itemConfig.get('shortFormId'))
                  }
                }
              } else {
                let { onConfigChange, config } = this.props
                onConfigChange(config.setIn([index, dataIndex], value))
              }
            }}
          />
        </div>
        {itemConfig.get("scenario") && (
          <div>
            <Divider style={{ margin: "8px 0 " }}>对应关系</Divider>
            {this.renderForm(itemConfig, index)}
          </div>
        )}
      </div>
    )
  }
  // 对应关系
  renderForm(itemConfig, index) {
    /*if(itemConfig.get('scenario').size == 0) {
      return;
    }*/
    const extraConfig = {}
    itemConfig.get("scenario").map(item => {
      extraConfig[item.dataIndex] = { source: this.state.options }
      return true
    })
    return (
      <AutoGenerationForm
        getItemValue={item => item["fromOption"]}
        onItemValueChange={(value, dataIndex) => {
          const currentItem = itemConfig.get("scenario").filter(x => x.dataIndex === dataIndex)[0]
          currentItem.fromOption = value
          this.setState({ update: !this.state.update })
        }}
        config={[itemConfig.get("scenario")]}
        extraConfig={extraConfig}
        source={Map({})}
      />
    )
  }

  onShortFormChanged(shortForm, index, equal) {
    let { onConfigChange, config } = this.props
    request("/generate/getSysSimpleWindow", { id: shortForm.id }).then(data => {
      if (data && data.result) {
        const { content } = data.result
        const shortFormConfig = JSON.parse(content)
        if (shortFormConfig && shortFormConfig["scenario"]) {
          let originalScenario = config.getIn([index, 'scenario'])
          let scenario = shortFormConfig["scenario"]
          if (equal&&originalScenario) {
            scenario.map(item => {
              let matching = originalScenario.filter(oitem => item.dataIndex == oitem.dataIndex && oitem.fromOption)
              if (matching.length > 0) {
                item.fromOption = matching[0].fromOption
              }
            })
          }
          shortFormConfig["scenario"].map(item => {
            item.originalType = item.type
            item.originalField = item.field
            item.originalReferenceKey = item.referenceKey
            item.origianlSourceKey = item.sourceKey
            item.originalDefaultValue = item.defaultValue
            item.originalDisabled = item.disabled
            item.disabled = "0"
            item.type = "select"
            item.filed = undefined
            item.referenceKey = undefined
            item.defaultValue = undefined
            return true
          })
          onConfigChange(
            config
              .setIn([index, "shortFormId"], shortForm.id)
              .setIn([index, "title"], shortForm["winName"])
              .setIn([index, "scenario"], shortFormConfig["scenario"])
          )
        }
      }
    })
  }

  editTabs(targetKey, action) {
    this[action](targetKey)
  }

  add() {
    let { config, onConfigChange } = this.props
    config = config.push(Map({ title: "新的窗口" }))
    onConfigChange(config)
    this.setState({ activeKey: `${config.size - 1}` })
  }

  remove(targetKey) {
    let { config, onConfigChange } = this.props
    config = config.delete(targetKey)
    if (targetKey === this.state.activeKey) {
      this.setState({ activeKey: "0" })
    } else if (targetKey < this.state.activeKey) {
      this.setState({ activeKey: `${this.state.activeKey - 1}` })
    }
    onConfigChange(config)
  }

  render() {
    const { visible, activeKey } = this.state
    const { config, onSaveShortFormConfig } = this.props
    return (
      <span>
        <Button size="small" type="primary" onClick={() => this.openSettingPop()}>
          弹窗配置
        </Button>
        <DraggableModal
          visible={visible}
          title="弹窗配置"
          width={800}
          bodyStyle={{ padding: "12px", height: "400px", overflow: "auto" }}
          onCancel={() => this.setState({ visible: false })}
          onOk={() => {
            this.setState({ visible: false })
            onSaveShortFormConfig && onSaveShortFormConfig()
          }}
        >
          {config && (
            <Tabs
              type="editable-card"
              size="small"
              activeKey={activeKey}
              onChange={activeKey => this.setState({ activeKey })}
              onEdit={(targetKey, action) => this.editTabs(targetKey, action)}
            >
              {config.map((itemConfig, index) => (
                <Tabs.TabPane key={index} tab={itemConfig.get("title")}>
                  {this.renderTabPane(itemConfig, index)}
                </Tabs.TabPane>
              ))}
            </Tabs>
          )}
        </DraggableModal>
      </span>
    )
  }
}
