import React from 'react';
import { Upload, message } from 'antd';
import { serverUrl } from '../../common/config';
import { systemCommonFileFunctionCode, supportImage } from '../../common/const';
import { fileServerUrl } from '../../common/config';


const UploadImageOrVideo = ({ id, onChange, onUploadStatusChange,functionCode }) => {
  const uploadUrl = `${serverUrl}/attach/uploadFiles?funCode=${functionCode || systemCommonFileFunctionCode}`

  const beforeUpload = (file) => {
    if (supportImage.indexOf(file.type) < 0 && file.type.indexOf('video') < 0) {
      message.error('请上传图片或视频')
      return false;
    }

    return true
  };

  const handleChange = (info) => {
    const status = info.file.status;
    if (onUploadStatusChange) {
      onUploadStatusChange(status);
    }

    if (status === 'done') {
      if (info.file.response && info.file.response.code === '0') {
        if (info.file.response.data && info.file.response.data.result && info.file.response.data.result.length > 0) {
          const fileInfo = info.file.response.data.result[0];
          //onChange && onChange(fileInfo.fileUrl + fileInfo.newFileName);
          if (fileServerUrl === '/') {
            onChange && onChange(`/${fileInfo.fileUrl}${fileInfo.newFileName}`);
            //console.log('服务器模式!')
          }else{
            onChange && onChange(`${fileServerUrl}${fileInfo.fileUrl}${fileInfo.newFileName}`);
            //console.log('开发模式!')
          }
        }
      }
    } else if (status === 'error') {
      message.error(`${info.file.name} 上传失败！`);
    }
  };

  return (
    <div id={id}>
      <Upload
        listType="picture-card"
        showUploadList={false}
        action={uploadUrl}
        headers={{
          authorization: localStorage.getItem('token') || '',
          accessType: 'backend'
        }}
        beforeUpload={beforeUpload}
        onChange={handleChange}
      />
    </div>
  );
};

export default UploadImageOrVideo;
