import React from 'react';
import { message, Spin } from 'antd';
import { Editor } from '@tinymce/tinymce-react';
import Upload from './Upload';
import isVideo from 'is-video';
import { fileServerUrl } from '../../common/config';

export default class TinyMCE extends React.PureComponent {

  constructor(props) {
    super(props);

    const { value } = props;
    this.state = {
      value: value || '',
      loading: false,
    };
  }

  handleEditorChange = (content) => {
    this.setState({
      value: content
    })

    const { onChange } = this.props;
    if (onChange) {
      onChange(content);
    }
  }

  handleUploadChange = (fileUrl) => {
    if (fileUrl) {
      let insertUrl = fileUrl;
      let text = '';
      if (isVideo(insertUrl)) {
        text = `<video src="${insertUrl}">您的浏览器不支持 video 标签。</video>`;
      } else {
        text = `<img src="${insertUrl}" alt=""></img>`;
      }
      this.editor.insertContent(text);
    } else {
      message.error('上传出错，请稍后重试');
    }
  };

  handleUploadStatusChange = (status) => {
    this.setState({
      loading: status === 'uploading'
    });
  };

  componentWillReceiveProps(nextProps) {
    const { value } = nextProps;

    this.setState({
      value
    })
  }

  render() {
    const { value, loading } = this.state;
    return (
      <Spin spinning={loading}>
        <Editor
          value={value}
          init={{
            height: 500,
            theme: 'modern',
            branding: false,
            relative_urls: false,
            remove_script_host: true,
            fontsize_formats: '12px 14px 16px 18px 20px 22px 24px 26px 39px 34px 38px 42px 48px',
            plugins: 'autoresize print preview searchreplace autolink directionality visualblocks visualchars fullscreen image link media template codesample table charmap hr pagebreak nonbreaking anchor toc insertdatetime advlist lists textcolor wordcount imagetools contextmenu colorpicker textpattern code paste',
            menubar: 'edit insert view format table tool', // remove tool
            toolbar1: 'fontselect fontsizeselect | formatselect | bold italic strikethrough forecolor backcolor | link image media anchor | alignleft aligncenter alignright alignjustify | numlist bullist outdent indent | removeformat image-upload styleselect fullscreen',
            image_advtab: true,
            language: 'zh_CN',
            paste_auto_cleanup_on_paste: true,
            paste_remove_styles: true,
            paste_remove_styles_if_webkit: true,
            paste_strip_class_attributes: true,
            autoresize_max_height: 500,
            autoresize_min_height: 100,
            autoresize_on_init: true,
            autoresize_bottom_margin: 10,
            autoresize_overflow_padding: 10,
            setup: (editor) => {
              this.editor = editor;
              editor.addButton('image-upload', {
                text: '',
                tooltip: '上传本地图片、视频',
                icon: 'mce-ico mce-i-image',
                onclick: () => {
                  const uploadEl = document.querySelector('#upload [type=file]');
                  if (!uploadEl) {
                    console.error('上传组件不存在');
                    return;
                  }

                  uploadEl.click();
                },
              });
            },
          }}
          onEditorChange={this.handleEditorChange}
        />

        <div style={{ display: 'none' }}>
          <Upload
            id="upload"
            functionCode={this.props.functionCode}
            onChange={this.handleUploadChange}
            onUploadStatusChange={this.handleUploadStatusChange} />
        </div>
      </Spin>
    );
  }
}