/**
 * Created by 王友生 on 2018-08-01.
 */

import React, { Component } from 'react';
import { newGuid } from '../../utils';
import { serverUrl } from '../../common/config';

const config = {
  serverUrl: serverUrl + '/SysExtend/ueditor',
  autoHeightEnabled: true,
  autoFloatEnabled: true,
  enableAutoSave: false,
  maximumWords: 100000,
  toolbars: [[
    'fullscreen', 'source', '|', 'undo', 'redo', '|',
    'bold', 'italic', 'underline', 'fontborder', 'strikethrough', 'superscript', 'subscript', 'removeformat', 'formatmatch', 'autotypeset', 'blockquote', 'pasteplain', '|', 'forecolor', 'backcolor', 'insertorderedlist', 'insertunorderedlist', 'selectall', 'cleardoc', '|',
    'rowspacingtop', 'rowspacingbottom', 'lineheight', '|',
    'customstyle', 'paragraph', 'fontfamily', 'fontsize', '|',
    'directionalityltr', 'directionalityrtl', 'indent', '|',
    'justifyleft', 'justifycenter', 'justifyright', 'justifyjustify', '|', 'touppercase', 'tolowercase', '|',
    'link', 'unlink', '|', 'imagenone', 'imageleft', 'imageright', 'imagecenter', '|',
    'simpleupload', 'emotion', 'scrawl', 'pagebreak', 'template', 'background', '|',
    'horizontal', 'date', 'time', 'spechars', 'snapscreen', 'wordimage', '|',
    'inserttable', 'deletetable', 'insertparagraphbeforetable', 'insertrow', 'insertvideo', 'deleterow', 'insertcol', 'deletecol', 'mergecells', 'mergeright', 'mergedown', 'splittocells', 'splittorows', 'splittocols', '|', 'preview', 'searchreplace', 'drafts'
  ]]
};

export default class UEditor extends Component {
  ueEditor = undefined;
  constructor(props) {
    super(props);
    this.state = {
      id: newGuid(),
      content: ''
    };
  }

  componentDidMount() {
    this.initEditor();
  }

  componentWillUnmount() {
    // 组件卸载后，清除放入库的id
    // window.UE.delEditor(this.state.id);
  }

  componentWillReceiveProps(newProps) {
    const { id } = this.state;
    if (id && newProps.content !== this.state.content) {
      const ueEditor = window.UE.getEditor(id, config);
      ueEditor.addListener('ready', (ueditor) => {
        if (!ueditor) {
          window.UE.delEditor(id);
          this.initEditor();
        } else {
          const content = newProps.content;
          ueEditor && ueEditor.setContent(content);
        }
      })
    }
  }

  initEditor() {
    const { id } = this.state;
    const { onChange } = this.props;
    const ueEditor = window.UE.getEditor(id, config);
    const self = this;

    ueEditor.ready((ueditor) => {
      if (!ueditor) {
        // window.UE.delEditor(id);
        self.initEditor();
      } else {
        const content = self.getContent();
        content && ueEditor.setContent(content);
        ueEditor.addListener('contentChange', () => {
          const newContent = ueEditor.getContent();
          this.setState({
            content: newContent
          })
          onChange(newContent);
        });
        this.ueEditor = ueditor;
      }
    });
  }

  getContent() {
    return this.props.content;
  }

  render() {
    return <div id={this.state.id} style={{ width: '100%', height: '500px' }} name="content" type="text/plain" />
  }
}

