/**
 * Created by 王友生 on 2018-07-04.
 */

import React, { PureComponent } from "react"
import { Upload, message, Modal, Icon, Slider } from "antd"
import AvatarEditor from "react-avatar-editor"
import { serverUrl, fileServerUrl } from "../../common/config"
import { systemCommonFileFunctionCode, supportImage } from "../../common/const"

export default class UploadThumb extends PureComponent {
  state = {
    previewVisible: false
  }

  editImage(file, fileList) {
    if (supportImage.indexOf(file.type) < 0) {
      message.error("请上传jpg，jpeg或者png的图片")
      return true
    }
    this.setState({ imageEdited: true, editedImage: file })
    return true
  }

  cancelEditImage() {
    this.setState({ imageEdited: false, editedImage: null })
  }

  beforeUpload(file) {
    if (supportImage.indexOf(file.type) < 0) {
      message.error("请上传jpg，jpeg或者png的图片")
      return false
    }
    return true
  }

  editImageCompleted() {
    const { editedImage, values } = this.state
    if (this.editor) {
      const canvas = this.editor.getImage()
      const form = new FormData()
      const file = {
        name: "file"
      }
      form.append("file", file)
      // request('')
      this.setState({
        imageEdited: false,
        editedImage: null
      })
    }
  }
  render() {
    const { value, onChange, disabled } = this.props
    const { previewVisible } = this.state
    const dragProps = {
      name: "file",
      disabled,
      showUploadList: false,
      listType: "picture-card",
      beforeUpload: file => this.beforeUpload(file),
      headers: { authorization: localStorage.getItem("token") || "", accessType: "backend" },
      action: `${serverUrl}/attach/uploadFiles?funCode=${this.props.functionCode || systemCommonFileFunctionCode}`,
      onChange: info => {
        const status = info.file.status
        // if (status !== "uploading") {
        // }
        if (status === "done") {
          if (info.file.response && info.file.response.code === "0") {
            // message.success(`${info.file.name} 上传成功！`);
            if (info.file.response.data && info.file.response.data.result && info.file.response.data.result.length > 0) {
              const fileInfo = info.file.response.data.result[0]
              onChange && onChange(fileInfo.fileUrl + fileInfo.newFileName)
            }
          }
        } else if (status === "error") {
          message.error(`${info.file.name} 上传失败！`)
        }
      }
    }
    return (
      <div>
        <Upload {...dragProps}>
          {value ? (
            <img
              src={fileServerUrl + value}
              style={{ width: "100px", height: "100px" }}
              alt="file"
              onClick={() => disabled && this.setState({ previewVisible: true })}
            />
          ) : (
            !disabled && (
              <div>
                <Icon type="plus" />
                <div className="ant-upload-text">上传</div>
              </div>
            )
          )}
        </Upload>
        <Modal visible={this.state.imageEdited} onCancel={() => this.cancelEditImage()} onOk={() => this.editImageCompleted()}>
          <div style={{ marginTop: "24px" }}>
            {this.state.editedImage && (
              <AvatarEditor
                ref={editor => (this.editor = editor)}
                image={this.state.editedImage}
                width={250}
                height={250}
                border={[110, 50, 110, 50]}
                color={[0, 0, 0, 0.6]} // RGBA
                rotate={0}
                scale={this.state.imageScaled}
              />
            )}
            <div className="icon-wrapper">
              <Icon
                type="minus"
                onClick={() => this.setState({ imageScaled: this.state.imageScaled - 0.1 > 0.2 ? this.state.imageScaled - 0.1 : 0.2 })}
              />
              <Slider
                defaultValue={1}
                min={0.2}
                max={2}
                step={0.1}
                value={this.state.imageScaled}
                onChange={value => this.setState({ imageScaled: value })}
              />
              <Icon
                type="plus"
                onClick={() => this.setState({ imageScaled: this.state.imageScaled + 0.1 < 2 ? this.state.imageScaled + 0.1 : 2 })}
              />
            </div>
          </div>
        </Modal>
        <Modal visible={previewVisible} footer={null} onCancel={() => this.setState({ previewVisible: false })}>
          <img alt="img" style={{ width: "100%" }} src={fileServerUrl + value} />
        </Modal>
      </div>
    )
  }
}
