import React, { PureComponent, Fragment } from "react"
import ReactDOM from "react-dom"
import _ from "lodash"
import BaseTable from "./BaseTable"

class VirtualTable extends PureComponent {
  // 填充节点
  static FillNode({ height, node, marginTop, marginBottom }) {
    if (node) {
      marginTop = marginTop || 0
      marginBottom = marginBottom || 0
      height = height || 0
      return ReactDOM.createPortal(
        <div style={{ height: `${height}px`, marginTop: `${marginTop}px`, marginBottom: `${marginBottom}px` }} />,
        node
      )
    }
    return null
  }

  constructor(props) {
    super(props)
    this.state = {
      startIndex: 0,
      visibleRowCount: 0,
      thresholdCount: 40,
      rowHeight: 0,
      topBlankHeight: 0,
      bottomBlankHeight: 0,
      maxTotalHeight: 15000000,
      top: true
    }
  }

  componentDidMount() {
    // 普通table布局
    // eslint-disable-next-line
    this.refScroll = ReactDOM.findDOMNode(this).getElementsByClassName("ant-table-body")[0]
    // 固定列的布局
    // eslint-disable-next-line
    const fixedEles = ReactDOM.findDOMNode(this).getElementsByClassName("ant-table-body-inner")
    this.refFixedLeftScroll = fixedEles && fixedEles.length ? fixedEles[0] : null
    this.refFixedRightScroll = fixedEles && fixedEles.length > 1 ? fixedEles[1] : null

    this.listenEvent = _.throttle(this.handleScrollEvent, 100)
    if (this.refScroll) {
      this.refScroll.addEventListener("scroll", this.listenEvent)
    }
    this.createTopFillNode()
    this.createBottomFillNode()
    // 初始化设置滚动条
    this.setRowHeight()
    this.handleScrollEvent()
  }

  componentWillReceiveProps(newProps) {
    const { dataSource, functionCode, rowSelection } = newProps
    const { dataSource: tdataSource, setScroll, functionCode: pFunctionCode, rowSelection: prowSelection } = this.props
    // 记录位置
    // if (pFunctionCode == functionCode && this.refScroll.scrollTop != newProps.y && this.refScroll.scrollTop != 0) {
    //   this.setScroll()
    //   return
    // }

    if (pFunctionCode != functionCode) {
      this.setScroll()
      this.refScroll.scrollTop = newProps.y ? newProps.y : 0
    }
    if (setScroll) {
      //兼容onSelect
      if (rowSelection && prowSelection && rowSelection.selectedRowKeys != prowSelection.selectedRowKeys) {
        return
      }
      if (newProps.y !== 0) {
        this.refScroll.scrollTop = newProps.y ? newProps.y : 0;//render时回到顶部bug
      }
      // this.refScroll.scrollTop = newProps.y ? newProps.y : 0
      this.setState({ top: false })
    }
    if (dataSource && dataSource.length != tdataSource.length) {
      this.refScroll.scrollTop = 0
      this.handleScroll(dataSource.length)
    }
  }

  componentWillUnmount() {
    this.setScroll()
    if (this.refScroll) {
      this.refScroll.removeEventListener("scroll", this.listenEvent)
      // const scrollTop = this.refScroll.scrollTop // 滑动的距离
      // this.props.setScroll && this.props.setScroll(scrollTop)
    }
  }
  setScroll() {
    const scrollTop = this.refScroll.scrollTop // 滑动的距离
    this.props.setScroll && this.props.setScroll(scrollTop)
  }
  createTopFillNode() {
    if (this.refScroll) {
      const ele = document.createElement("div")
      this.refScroll.insertBefore(ele, this.refScroll.firstChild)
      this.refTopNode = ele
    }
    if (this.refFixedLeftScroll) {
      const ele = document.createElement("div")
      this.refFixedLeftScroll.insertBefore(ele, this.refFixedLeftScroll.firstChild)
      this.refFixedLeftTopNode = ele
    }
    if (this.refFixedRightScroll) {
      const ele = document.createElement("div")
      this.refFixedRightScroll.insertBefore(ele, this.refFixedRightScroll.firstChild)
      this.refFixedRightTopNode = ele
    }
  }

  createBottomFillNode() {
    if (this.refScroll) {
      const ele = document.createElement("div")
      this.refScroll.appendChild(ele)
      this.refBottomNode = ele
    }
    if (this.refFixedLeftScroll) {
      const ele = document.createElement("div")
      this.refFixedLeftScroll.appendChild(ele)
      this.refFixedLeftBottomNode = ele
    }
    if (this.refFixedRightScroll) {
      const ele = document.createElement("div")
      this.refFixedRightScroll.appendChild(ele)
      this.refFixedRightBottomNode = ele
    }
  }

  setRowHeight() {
    this.refTable = this.refScroll.getElementsByTagName("table")[0]
    // this.refFixedLeftTable = this.refFixedLeftScroll.getElementsByTagName('table')[0]
    if (this.refTable) {
      const tr = this.refTable.getElementsByTagName("tr")[0]
      const rowHeight = (tr && tr.clientHeight) || 0
      this.setState({
        rowHeight: rowHeight
      })
    }
  }

  handleScrollEvent = e => {
    const { dataSource } = this.props
    if (this.props.pagination.pageSize >= 500) {
      this.handleScroll((dataSource || []).length)
    }
  }

  handleScroll = length => {
    //20220920 length替换 this.props.dataSource.length 解决视图bug
    const { rowHeight, maxTotalHeight } = this.state
    if (rowHeight && length) {
      const visibleHeight = Math.min(this.props.scroll.y, length * rowHeight) //this.refScroll.clientHeight // 显示的高度
      const scrollTop = this.refScroll.scrollTop // 滑动的距离
      // this.props.setScroll && this.props.setScroll(scrollTop)
      this.handleBlankHeight(length, rowHeight, maxTotalHeight, visibleHeight, scrollTop)
    } else {
      this.setRowHeight()
    }
  }

  getIndexByScrollTop(rowHeight, scrollTop) {
    const index = (scrollTop - (scrollTop % rowHeight)) / rowHeight
    return index
  }

  handleBlankHeight(length, rowHeight, maxTotalHeight, visibleHeight, scrollTop) {
    let oriRowHeight = rowHeight
    let totalHeight = length * rowHeight
    let isBigData = false
    const that = this
    if (totalHeight > maxTotalHeight) {
      isBigData = true
      totalHeight = maxTotalHeight
      rowHeight = totalHeight / length
      scrollTop = scrollTop > maxTotalHeight ? maxTotalHeight : scrollTop
    }
    if (length >= 10000) {
      isBigData = true
    }
    let topBlankHeight, bottomBlankHeight, startIndex, visibleRowCount
    startIndex = this.getIndexByScrollTop(rowHeight, scrollTop) //开始位置
    visibleRowCount = Math.ceil(visibleHeight / oriRowHeight) //显示数量
    topBlankHeight = rowHeight * startIndex
    topBlankHeight = this.getValidValue(topBlankHeight, 0, totalHeight) //顶部高度
    bottomBlankHeight = totalHeight - topBlankHeight - visibleHeight
    bottomBlankHeight = bottomBlankHeight > 0 ? bottomBlankHeight : 0 //底部高度

    const slideUpHeight = Math.abs(topBlankHeight - this.state.topBlankHeight)
    const slideDownHeight = Math.abs(bottomBlankHeight - this.state.bottomBlankHeight)
    if (!this.lastSlideUpHeight) {
      this.sameSlideHeightCount = 0
      this.lastSlideUpHeight = slideUpHeight //最后滑动高度
    } else if (this.lastSlideUpHeight === slideUpHeight) {
      this.sameSlideHeightCount++
    } else {
      this.lastSlideUpHeight = slideUpHeight
      this.sameSlideHeightCount = 0
    }
    let isValid = slideUpHeight >= rowHeight
    isValid = isValid || slideDownHeight >= rowHeight
    isValid = isValid || startIndex === 0
    // 真实滑动
    if (isValid) {
      startIndex = startIndex - 5
      visibleRowCount = visibleRowCount + 5
      if (this.sameSlideHeightCount >= 1) {
        return
      }
      if (topBlankHeight - this.state.topBlankHeight < rowHeight * visibleRowCount && topBlankHeight - this.state.topBlankHeight > 0) {
        if (scrollTop < 200) {
          return
        }
        this.setState(
          {
            topBlankHeight,
            bottomBlankHeight
          },
          () => {
            setTimeout(() => {
              this.refScroll.scrollTop = Number(scrollTop)
              this.sameSlideHeightCount = 0
              this.setState({
                startIndex,
                visibleRowCount
              })
            }, 100)
          }
        )
      } else {
        this.setState({
          topBlankHeight,
          bottomBlankHeight,
          startIndex,
          visibleRowCount
        })
      }

      if (isBigData && this.sameSlideHeightCount >= 1) {
        // 防止大数据持续滚动期间出现空白的问题
        this.refScroll.scrollTop = scrollTop
        this.sameSlideHeightCount = 0
      }
    }
  }

  getValidValue(val, min = 0, max = 40) {
    if (val < min) {
      return min
    } else if (val > max) {
      return max
    }
    return val
  }

  render() {
    const { dataSource, columns, ...rest } = this.props
    let { topBlankHeight, bottomBlankHeight, startIndex, visibleRowCount, rowHeight, thresholdCount } = this.state
    const { length } = dataSource || []
    const pageSize = this.props.pagination.pageSize
    let startCount = length - visibleRowCount
    startCount = startCount > 0 ? startCount : length
    let startIn = this.getValidValue(startIndex, 0, startCount)
    let endIn = startIndex + visibleRowCount
    if (!endIn) {
      // 初始化渲染数据
      endIn = length > thresholdCount ? thresholdCount : length
    }
    endIn = this.getValidValue(endIn, startIndex, length)
    const renderSource = (dataSource || []).slice(startIn, endIn)
    
    columns.map(item=>{
      if(item.type === "amount"){
      item.render = (text, record) => {
          return  text&&parseFloat(text).toFixed(2).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')
        }
      }
      
    })
    return (
      <Fragment>
        {pageSize >= 500 && <VirtualTable.FillNode height={topBlankHeight} node={this.refTopNode} />}
        <BaseTable {...rest} columns={columns} dataSource={dataSource} renderSource={pageSize >= 500 ? renderSource : dataSource} />
        {pageSize >= 500 && <VirtualTable.FillNode height={bottomBlankHeight} node={this.refBottomNode} />}

        {
          /**fixed 针对固定列的*/
          pageSize >= 500 && (
            <Fragment>
              <VirtualTable.FillNode height={topBlankHeight} node={this.refFixedLeftTopNode} />
              <VirtualTable.FillNode height={bottomBlankHeight} node={this.refFixedLeftBottomNode} />
              <VirtualTable.FillNode height={topBlankHeight} node={this.refFixedRightTopNode} />
              <VirtualTable.FillNode height={bottomBlankHeight} node={this.refFixedRightBottomNode} />
            </Fragment>
          )
        }
      </Fragment>
    )
  }
}

export default VirtualTable
