/**
 * Created by Administrator on 2019-01-16.
 */
import './index.less'

import React, { PureComponent } from 'react';
import DraggableModal from '../../DraggableModal';
import { connect } from 'react-redux';
import { Input } from 'antd';
import WorkflowEditor from '../Editor';
import { dealWithResultStatusConst } from '../../../common/const';
import { getStatusMappingLabelByValue } from '../../../utils';
import { Timeline, Icon } from 'antd';
import _ from 'lodash';
import {FormattedMessage} from 'react-intl';

const { TextArea } = Input;

@connect((state) => {
  const cache = state.get('workflow');
  const functionCode = cache.get('currentFunctionCode');
  const currentRecordId = cache.get('currentRecordId');
  const visibleWorkflow = cache.get('visibleWorkflow');
  if (! functionCode) {
    return {};
  }
  return {
    functionCode,
    currentRecordId,
    workflowTemplate: cache.get(functionCode),
    verifiedVisible: cache.get('verifiedVisible'),
    verifiedMessage: cache.get('verifiedMessage'),
    visibleWorkflow,
    workflowInfo: cache.get(functionCode + currentRecordId)
  }
})
export default class WorkflowActions extends PureComponent {

  renderHandlerRecord(record, index) {
    return <div key={index} className="workflowhandlerRecord">
        <div>
          <span className="workflowhandlerRecordName">{record.userName}</span>
          <span className="workflowhandlerRecordStatus">{getStatusMappingLabelByValue(dealWithResultStatusConst, record.dealWithResult)}</span>
          <span className="workflowhandlerRecordDateTime">{record.dateTime}</span>
        </div>
        {
          record.message && <div className="workflowhandlerRecordMessage">
            {record.message}
          </div>
        }
      </div>
  }

  // renderTimeLineItemProps(record) {
  //   if (record.dealWithResult === dealWithResultStatusConst.submitted.value || record.dealWithResult === dealWithResultStatusConst.verified.value)
  //     return {
  //       color: 'green',
  //     }

  // }

  render() {
    const { workflowTemplate, functionCode, verifiedVisible, dispatch, verifiedMessage, visibleWorkflow, workflowInfo, currentRecordId } = this.props;
    // workflowInfo && console.log('actons64',workflowInfo.toJSON());
    let handlerRecords = [];
    if (workflowInfo) {
      handlerRecords = workflowInfo.get('handlerRecords');
      // let liuchengList = _.cloneDeep(workflowInfo.get('handlerRecords'));
      // liuchengList.forEach(item => {
      //   item.timeChuo = new Date(item.dateTime);
      // });
    }
    if (! functionCode) {
      return null;
    }
    return <div>
      <DraggableModal
        visible={verifiedVisible}
        title={<FormattedMessage id={"work_flowCheck"}/>}
        okText={<FormattedMessage id={"work_flowCheck"}/>}
        bodyStyle={{ padding: '12px' }}
        cancelText={<FormattedMessage id={"work_reject"}/>}
        onOk={(e) => {
          dispatch({
            type: 'workflow/verfiedWorkflow',
          })
        }}
        onCancel={(e) => {
          if (e.target.value === undefined) {
            dispatch({
              type: 'workflow/cancelVerifiedWorkflow'
            })
          } else {
            dispatch({
              type: 'workflow/abandonWorkflow'
            });
          }
        }}>
        <TextArea
          line={4}
          value={verifiedMessage}
          onChange={e => dispatch({
            type: 'workflow/changeVerifiedMessage',
            payload: {
              functionCode:functionCode,
              verifiedMessage: e.target.value
            }
          })}>
        </TextArea>
      </DraggableModal>
      <DraggableModal
        visible={visibleWorkflow}
        title={<FormattedMessage id={"work_info"}/>}
        bodyStyle={{ padding: 0, height: '600px' }}
        width={1000}
        onCancel={() => {
          dispatch({
            type: 'workflow/hideWorkflowView'
          });
        }}
        onOk={() => {
          dispatch({
            type: 'workflow/hideWorkflowView'
          });
        }}>
        <div className="workflowActionsInfo">
          <div className="workflowPartWrapper">
            {
              workflowInfo && <WorkflowEditor
                dataSource={{ dataSource: workflowInfo.get('workflow') }}
                saveWorkFlow={({ dataSource }) => dispatch({
                  type: 'workflow/addNodeToWorkflow',
                  payload: {
                    functionCode,
                    recordId: currentRecordId,
                    workflow: dataSource
                  }
                })} />
            }
          </div>
          <div className="handlerRecordPartWrapper">
            <Timeline>
              {_.reverse(handlerRecords).map((record, index) => <Timeline.Item key={index}
                color={record.dealWithResult === dealWithResultStatusConst.submitted.value || record.dealWithResult === dealWithResultStatusConst.verified.value ? 'green' : 'red'}>
                {this.renderHandlerRecord(record, index)}
              </Timeline.Item>)}
            </Timeline>
          </div>
        </div>
      </DraggableModal>
    </div>
  }
}
