/**
 * Created by 王友生 on 2018-04-08.
 * 两种状态，编辑状态和非编辑状态，非编辑状态下可以根据某个节点进行加签
 */

import './index.less';
import React, { PureComponent } from 'react';
import classnames from 'classnames';
import Node, { statusIconMapping } from '../Node';
import NodeEditor from '../NodeEditor';
import { fromJS, List, Map } from 'immutable';
import {
  workFlowSelectorType,
  workFlowNodeAttrModeConst,
  workflowNodeType,
  workFlowStatusMapping
} from '../../../common/const';
import competeIcon from '../../../assets/compete.png';
import cooperateIcon from '../../../assets/cooperate.png';
import EmployeeSelector from '../../EmployeeSelector';
import { Tooltip, Button, Checkbox, Input } from 'antd';
import { FormattedMessage } from 'react-intl';

import { newGuid } from '../../../utils';
import connect from "react-redux/es/connect/connect";

@connect((state, route) => {
  const lang = state.get('system').get('lang');
  return {
    lang,
  }
})
export default class Workflow extends PureComponent {

  constructor(props) {
    super(props);
    this.state = {
      selectorVisible: false,
      currentSelectedKeyPath: '',
      nodeEditorVisible: false,
      editedNode: undefined,
      needSave: false,
      saveConfirm: props.dataSource.saveConfirm,
      needMessage: props.dataSource.needMessage || 0,
      editIgnore: props.dataSource.editIgnore || 0,
      immClose: props.dataSource.immClose || 0,
      yEnable: props.dataSource.yEnable || 0,
      name: props.dataSource.name || '',
      dataSource: Map.isMap(props.dataSource.dataSource) ? props.dataSource.dataSource : fromJS(props.dataSource.dataSource)
    }
  }

  componentWillReceiveProps(newProps) {
    if (newProps.dataSource && newProps.dataSource !== this.props.dataSource) {
      this.setState({
        dataSource: Map.isMap(newProps.dataSource.dataSource) ? newProps.dataSource.dataSource : fromJS(newProps.dataSource.dataSource),
        saveConfirm: newProps.dataSource.saveConfirm,
        needMessage: newProps.dataSource.needMessage || 0,
        editIgnore: newProps.dataSource.editIgnore || 0,
        immClose: newProps.dataSource.immClose || 0,
        name: newProps.dataSource.name || '',
        yEnable: newProps.dataSource.yEnable || 0,
      });
    }
  }

  //新增节点
  addNode(keyPath) {
    this.setState({ selectorVisible: true, currentSelectedKeyPath: keyPath, needSave: true });
  }

  // 删除节点时，若此节点上一级节点只有这一个节点，需要删除上一级节点，删除上一级节点之后，查看上一级节点的上一级节点，若次节点只剩下一条分支，合并分支。
  deleteNode(keyPath) {
    const { dataSource } = this.state;
    let newDataSource = dataSource.deleteIn(keyPath);
    keyPath.pop();
    const parentNode = newDataSource.getIn(keyPath);
    if (parentNode.size === 0) {
      newDataSource = newDataSource.deleteIn(keyPath);
      keyPath.pop();
      const grandParentNode = newDataSource.getIn(keyPath);
      if (grandParentNode.size === 1) {
        keyPath.pop();
        let lastKey = keyPath.pop();
        const editedNode = newDataSource.getIn(keyPath);
        newDataSource = newDataSource.mergeIn(keyPath, editedNode.splice(lastKey, 1, ...grandParentNode.get(0)));
      }
    }
    this.setState({ dataSource: newDataSource });
  }

  editNode(keyPath, node) {
    this.setState({ nodeEditorVisible: true, currentSelectedKeyPath: keyPath, editedNode: node, needSave: true });
  }

  // 根据节点的keyPath， 插入新的流程节点
  addNodeSuccess(type, nodes) {
    const { currentSelectedKeyPath, dataSource } = this.state;
    if (nodes.length <= 0) {
      this.setState({ selectorVisible: false });
      return;
    }
    // 并发
    if (type === workFlowSelectorType.supervene.key && nodes.length > 1) {
      const lastKey = currentSelectedKeyPath.pop();
      const editedNode = dataSource.getIn(currentSelectedKeyPath);
      const superveneNode = [];
      nodes.map(item => {
        superveneNode.push([item]);
        return true;
      });
      const newNode = {
        id: '',
        title: <FormattedMessage id={'work_meet'} />,
        new: true,
        isSupervene: true,
        children: superveneNode
      }
      this.setState({
        dataSource: dataSource.mergeIn(currentSelectedKeyPath, editedNode.splice(lastKey + 1, 0, fromJS(newNode))),
        selectorVisible: false
      });
      return;
      // 会签
    } else if (type === workFlowSelectorType.jointly.key) {
      const currentNode = dataSource.getIn(currentSelectedKeyPath);
      // 初始节点不能会签, 加签不能添加会签
      if ((!currentNode.get('id') || currentNode.get('id') !== this.props.currentNodeId)) {
        // 会签分为三种，1 首节点会签,2 会签节点与其他节点串发 3, 会签节点与其他节点并发
        let concatNode = [];
        nodes.map(item => {
          concatNode.push([item]);
          return true;
        });
        const jointlyNode = List([]).push(List([currentNode])).concat(fromJS(concatNode));
        const lastKey = currentSelectedKeyPath.pop();
        const parentNode = dataSource.getIn(currentSelectedKeyPath);
        if (currentNode.get('first')) {//首节点
          if (dataSource.size > 2) {
            let cacheList = dataSource.pop()//去除头尾后的主体节点
            cacheList = cacheList.shift()
            let firstNode = dataSource.getIn([0])//首节点
            let lastNode = dataSource.getIn([dataSource.size - 1])//结束节点
            let mergeNode = List([]).push(cacheList).concat(fromJS(concatNode))//新节点与主体节点合并
            const newNode = Map({
              id: '',
              title: <FormattedMessage id={'work_meet'} />,
              new: true,
              isSupervene: true,
              children: mergeNode
            });
            let allNode = List([firstNode, newNode, lastNode])//从新拼接流程
            this.setState({
              dataSource: allNode,
              selectorVisible: false
            });
          } else {
            message.info('首节点不得会签')
            this.setState({
              selectorVisible: false
            });
          }
          return
        } else if (parentNode.size === 1) { // 如果父节点中只有一个元素，代表此节点时和别的节点并发
          const parentKey = currentSelectedKeyPath.pop();//父级key
          const grandParent = dataSource.getIn(currentSelectedKeyPath);//父级节点
          this.setState({
            dataSource: dataSource.mergeIn(currentSelectedKeyPath, grandParent.splice(parentKey, 1, ...jointlyNode)),
            selectorVisible: false
          });
        } else {
          const newNode = Map({
            id: '',
            title: <FormattedMessage id={'work_meet'} />,
            new: true,
            isSupervene: true,
            children: jointlyNode
          });
          this.setState({
            dataSource: dataSource.mergeIn(currentSelectedKeyPath, parentNode.splice(lastKey, 1, newNode)),
            selectorVisible: false
          });
        }
        return;
      }
    }

    // 串发
    const lastKey = currentSelectedKeyPath.pop();
    const editedNode = dataSource.getIn(currentSelectedKeyPath);
    this.setState({
      dataSource: dataSource.mergeIn(currentSelectedKeyPath, editedNode.splice(lastKey + 1, 0, ...fromJS(nodes))),
      selectorVisible: false
    });
  }

  // 父节点修改并发节点的方式，竞争或者协同
  changeSuperveneType(keyPath) {
    const { dataSource } = this.state;
    keyPath.push('isSupervene');
    const isSupervene = dataSource.getIn(keyPath);
    this.setState({ dataSource: dataSource.setIn(keyPath, !isSupervene) });
  }

  getNodeMode(node, keyPath) {
    if (node.get('new') === '1') return 'edit';
    if (node.get('first')) return this.props.editing ? 'edit' : 'view';
    if (node.get('last')) return 'view';
    if (node.get('add') === '1') return 'add';
    return this.props.editing ? 'edit' : 'view';
  }

  // 创建流程节点，并且记录每个节点的keyPath
  renderItems(items, keyPath) {
    return items.map((item, itemIndex) => {

      //父节点
      if (List.isList(item.get('children'))) {
        const nodeKeyPath = [...keyPath];
        nodeKeyPath.pop();
        nodeKeyPath.push(itemIndex);
        const joinedKeyPath = [...nodeKeyPath]
        joinedKeyPath.push('children');
        const currentKeyPath = [...joinedKeyPath];
        currentKeyPath.push(0);
        return <div key={itemIndex} className="nodeWrapper">
          <Tooltip title={item.get('isSupervene') ? <FormattedMessage id={'work_team'} /> :
            <FormattedMessage id={'work_compete'} />}>
            <div className="cooperateAction" style={{ zIndex: item.size + 3 }}
              onClick={() => (item.get('new') || this.props.editing) && this.changeSuperveneType(nodeKeyPath)}>
              <img src={item.get('isSupervene') ? cooperateIcon : competeIcon} alt="" />
            </div>
          </Tooltip>
          {/*循环子节点*/}
          <div className="childrenContent">
            {
              item.get('children').map((nodeItems, nodeItemsIndex) => {
                const nodeItemKeyPath = [...currentKeyPath]
                nodeItemKeyPath.pop();
                nodeItemKeyPath.push(nodeItemsIndex);
                nodeItemKeyPath.push(0);
                return <div key={nodeItemsIndex} className={classnames("nodeWrapper", "childrenWrapper")}
                  style={{ zIndex: item.get('children').size + 1 - nodeItemsIndex }}>
                  <span className="title"></span>
                  <div className="orientationLine"></div>
                  {this.renderItems(nodeItems, nodeItemKeyPath)}
                </div>
              })
            }
          </div>
          {(this.getNodeMode(item) === 'edit') &&
            <div className="collect" onClick={() => this.addNode(nodeKeyPath)}></div>}
          <div className="orientationLine"></div>
        </div>;
      } else {
        //无父级
        keyPath.pop();
        keyPath.push(itemIndex);
        const currentKeyPath = [...keyPath];
        return <div key={itemIndex} className="nodeWrapper">
          {(item.get('enterConTitle') || item.get('ignoreConTitle')) &&
            <Tooltip title={<div>
              {item.get('enterConTitle') &&
                <div><FormattedMessage id={'work_entryCondition'} />：{item.get('enterConTitle')}</div>}
              {item.get('ignoreConTitle') &&
                <div><FormattedMessage id={'work_SkipConditions'} />：{item.get('ignoreConTitle')}</div>}
            </div>}>
              <div className="nodeFunction">ƒ</div>
            </Tooltip>}
          <span className={`title ${item.get('type') === 'end' ? 'lastNodeTitle' : ''}`}
            style={{ zIndex: items.size + 1 - itemIndex }}>
            {/*hover节点组件*/}
            <Node node={item}
              mode={this.getNodeMode(item)}
              addNode={() => this.addNode(currentKeyPath)}
              deleteNode={() => this.deleteNode(currentKeyPath)}
              editNode={() => this.editNode(currentKeyPath, item)} />
          </span>
          <div className="orientationLine"></div>
        </div>;
      }
    });
  }

  convertDataSourceFromServer(dataSource) {
    const loop = (workflow) => {
      const newChildren = [];
      workflow.children.map((item, index) => {
        if (item.children && item.children.length > 0) {
          const itemChildren = [];
          item.children.map(childItem => {
            itemChildren.push(loop(childItem));
          });
          item.children = itemChildren;
          newChildren.push(item);
        } else {
          newChildren.push(item);
        }
        return true;
      });
      return newChildren;
    };
    dataSource.children.pop();
    const newDataSource = loop(dataSource);
    newDataSource.push({
      type: 'end',
      last: true,
      pid: 0,
      nodeModeName: '协同',
      nodeType: 0,
      nodeAttribute: 1,
      nodeMode: 1,
    });
    return newDataSource;
  }

  saveWorkFlow() {
    const { dataSource, saveConfirm, needMessage, editIgnore, immClose, yEnable, name } = this.state;
    let newDataSource = dataSource;
    const loop = (source, keyPath) => {
      source.map((item, itemIndex) => {
        const nodeKeyPath = [...keyPath];
        nodeKeyPath.pop();
        nodeKeyPath.push(itemIndex);
        newDataSource = newDataSource.deleteIn([...nodeKeyPath, 'new']);
        newDataSource = newDataSource.deleteIn([...nodeKeyPath, 'status']);
        newDataSource = newDataSource.deleteIn([...nodeKeyPath, 'users']);
        newDataSource = newDataSource.deleteIn([...nodeKeyPath, 'childHandlerRecords']);
        newDataSource = newDataSource.deleteIn([...nodeKeyPath, 'dealWithResult']);
        newDataSource = newDataSource.deleteIn([...nodeKeyPath, 'add']);
        if (List.isList(item.get('children'))) {
          item.get('children').map((childeItem, index) => {
            loop(childeItem, [...nodeKeyPath, 'children', index, 0]);
          });
        }
        return true;
      })
    }
    loop(dataSource, [0]);
    this.props.saveWorkFlow && this.props.saveWorkFlow({
      dataSource: newDataSource.toJSON(),
      saveConfirm,
      needMessage,
      yEnable,
      editIgnore,
      immClose,
      name
    });
  }

  renderWorkflowStatus() {
    const { lang } = this.props
    return Object.keys(workFlowStatusMapping).map(key => {
      return <span key={key}>
        {
          statusIconMapping[workFlowStatusMapping[key].key]
        }
        {
          lang == 'en' ? workFlowStatusMapping[key].titleEN : workFlowStatusMapping[key].title
        }
      </span>
    })
  }

  render() {
    const { selectorVisible, nodeEditorVisible, editedNode, dataSource, needSave, saveConfirm, needMessage, editIgnore, immClose, yEnable, currentSelectedKeyPath, name } = this.state;
    const { wfCons, wfNodes } = this.props;
    if (!dataSource) return null;
    return <div className="workflowWrapper">
      {/*详情时上方彩色图例*/}
      <div className="actionPanel">
        {
          !this.props.editing && this.renderWorkflowStatus()
        }
        {/*修改后或修改页保存*/}
        {(needSave || this.props.editing) && <Button size="small" onClick={() => this.saveWorkFlow()}>保存</Button>}
      </div>
      {/*修改页流程控制1-0切换*/}
      {
        this.props.editing && <div>
          <Checkbox checked={yEnable === '1'}
            onChange={(e) => this.setState({
              yEnable: `${1 - parseInt(yEnable)}`,
              needSave: true
            })}>启用</Checkbox>
          <Checkbox checked={saveConfirm === '1'} onChange={(e) => this.setState({
            saveConfirm: `${1 - parseInt(saveConfirm)}`,
            needSave: true
          })}>保存提醒</Checkbox>
          <Checkbox checked={needMessage === '1'} onChange={(e) => this.setState({
            needMessage: `${1 - parseInt(needMessage)}`,
            needSave: true
          })}>审核必填</Checkbox>
          <Checkbox checked={editIgnore === '1'} onChange={(e) => this.setState({
            editIgnore: `${1 - parseInt(editIgnore)}`,
            needSave: true
          })}>修改删除不验证</Checkbox>
          <Checkbox checked={immClose === '1'} onChange={(e) => this.setState({
            immClose: `${1 - parseInt(immClose)}`,
            needSave: true
          })}>审核后关闭页面</Checkbox>
          <div style={{ display: 'flex', marginTop: '5px' }}>
            <label style={{ width: '60px', lineHeight: '30px' }}>流程名称</label>
            <Input value={name} placeholder="请使用{字段名}，例如：{userName}有新的订单"
              onChange={(e) => this.setState({ name: e.target.value, needSave: true })} />
          </div>
        </div>
      }
      <div className="editorContentWrapper">
        <div className="editorWrapper">
          {/*循环流程节点*/}
          {
            this.renderItems(dataSource, [0])
          }
          {/*弹出节点新建选择*/}
          <EmployeeSelector
            workFlowTypeShow
            visible={selectorVisible}
            itemExtra={() => {
              return {
                nodeModeName: '协同',
                nodeAttr: workFlowNodeAttrModeConst.supervene.value,
                unionid: newGuid(),
                new: '1'
              }
            }}
            extraRoleItems={wfNodes ? Array.from(wfNodes, wfNode => {
              return { id: wfNode.dataIndex, name: wfNode.title, type: workflowNodeType.formPerson.value }
            }) : []}
            onClose={() => this.setState({ selectorVisible: false })}
            onSelectEnd={(type, nodes) => this.addNodeSuccess(type, nodes)} />
          {/*节点属性*/}
          <NodeEditor
            visible={nodeEditorVisible}
            keyPath={currentSelectedKeyPath}
            node={editedNode}
            wfCons={wfCons || []}
            saveNode={(node) => this.setState({
              dataSource: dataSource.setIn(currentSelectedKeyPath, node),
              nodeEditorVisible: false
            })}
            onClose={() => this.setState({ nodeEditorVisible: false })} />
        </div>
      </div>
    </div>
  }
}
