/**
 * Created by 王友生 on 2018-04-08.
 * 节点有三种状态，edit编辑的，add 加签， view 显示
 */
import './index.less';
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import personIcon from '../../../assets/person.png';
import groupIcon from '../../../assets/group.png';
import departmentIcon from '../../../assets/department.png';
import managerIcon from '../../../assets/manager.png';
import endIcon from '../../../assets/end.png';
import { Map } from 'immutable';
import { Icon, Popover } from 'antd';
import { workflowNodeType, wfHandleType, workFlowStatusMapping } from '../../../common/const';
import { map } from 'lodash';

const roleIconMapping = {
  [workflowNodeType.person.value]: personIcon,
  [workflowNodeType.department.value]: departmentIcon,
  [workflowNodeType.group.value]: groupIcon,
  [workflowNodeType.role.value]: managerIcon,
  [workflowNodeType.end.value]: endIcon
};

const statusIconMappingTemp = { '': null };

statusIconMappingTemp[workFlowStatusMapping.completed.key] = <Icon style={{ backgroundColor: '#1fa11f' }} type="check-circle-o" />;
statusIconMappingTemp[workFlowStatusMapping.process.key] = <Icon style={{ backgroundColor: '#d97a16' }} type="exclamation-circle-o" />;
statusIconMappingTemp[workFlowStatusMapping.notArrived.key] = <Icon style={{ backgroundColor: '#f21414' }} type="minus-circle-o" />;
statusIconMappingTemp[workFlowStatusMapping.unread.key] = <Icon style={{ backgroundColor: '#d97a16', fontSize: '12px', padding: '2px' }} type="read" />;
statusIconMappingTemp[workFlowStatusMapping.read.key] = <Icon style={{ backgroundColor: '#1fa11f', fontSize: '12px', padding: '2px' }} type="read" />;

export const statusIconMapping = statusIconMappingTemp;

export default class Node extends PureComponent {
  static propTypes = {
    node: PropTypes.object,
  }

  static defaultProps = {
    node: Map({})
  }

  renderPopover() {
    const { addNode, deleteNode, editNode, mode, node } = this.props;
    // if (mode === 'edit') {
      return <ul className="popover">
        <li onClick={() => addNode && addNode()}>增加节点</li>
        <li onClick={() => editNode && editNode()}>节点属性</li>
        {!node.get('first') && <li onClick={() => deleteNode && deleteNode()}>删除节点</li>}
      </ul>
    // }
    // return null;
  }

  renderHandledState() {
    const { node } = this.props;
    console.log("node111", node.toJS());
    let handledUsers = [];
    let unhandledUsers = [];
    node.get('users').map(user => {
      if (user.handled) {
        handledUsers.push(user.username);
      } else {
        unhandledUsers.push(user.username);
      }
    })
    return <div className="handlerStateWrapper">
      {/* 判断是否是审批 */}
      {/* {
        node.get('handleType') === wfHandleType.handle.value&&<div className="handBox">
          {handledUsers.length>0&&<div className="handled">
            <h4>已办</h4>
            {handledUsers.toString()}
          </div>}
          {!(handledUsers.length>0)&&<div className="unhandled">
            <h4>未办</h4>
            {unhandledUsers.toString()}
          </div>}
          
        </div>
      } */}
      {/* 判断是否是知会 */}
      {/* {
        node.get('handleType') === wfHandleType.notice.value&&<div className="handBox">
          {handledUsers.length>0&&<div className="handled">
            <h4>已读</h4>
            {handledUsers.toString()}
          </div>}
          {!(handledUsers.length>0)&&<div className="unhandled">
            <h4>未读</h4>
            {unhandledUsers.toString()}
          </div>}
          
        </div>
      } */}
      <div className="unhandled">
        <h4>{node.get('handleType') === wfHandleType.notice.value ? '未阅' : '未办'}</h4>
        {unhandledUsers.toString()}
      </div>
      {
        handledUsers.length > 0 && <div className="handled">
          <h4>{node.get('handleType') === wfHandleType.notice.value ? '已阅' : '已办'}</h4>
          {handledUsers.toString()}
        </div>
      }
    </div>
  }

  renderHandledState1(){
    const { node, mode } = this.props;
    let plist =[];
    node.get('childHandlerRecords')&&node.get('childHandlerRecords').size>0&&node.get('childHandlerRecords').map(item=>{
      plist.push(item.userName);
    })
    if(mode=="edit"){
      return;
    }
    return <div className="handlerStateWrapper">
        {plist.length>0&&<h4 style={{color:"#0ba717"}}>已审批</h4>}
        {plist.length==0&&<h4 style={{color:"#f21414"}}>未到达</h4>}
        <p>{plist[0]}</p>
    </div>
  }

  renderNodeStatusIcon() {
    const { node, mode } = this.props;
    if (mode === 'view' && !node.get('last')) {
      return statusIconMappingTemp[node.get('status')] || <Icon style={{ backgroundColor: '#f21414' }} type="minus-circle-o" />;
    }
    if (mode === 'edit') {
      if (node.get('handleType') === wfHandleType.notice.value) {
        return <Icon style={{ backgroundColor: '#1fa11f', fontSize: '12px', padding: '2px' }} type="read" />;
      }
    }
    return statusIconMappingTemp[node.get('status')];
  }

  render() {
    const { node, addNode, mode } = this.props;
    const props = {};
    if (mode === 'add') props.onClick = () => addNode && addNode();
    if (mode !== 'edit' && node.get('users') && node.get('type') !== workflowNodeType.end.value) {
      return <Popover overlayStyle={{ zIndex: 9999 }} overlayClassName="handlerStatePopup" title="处理状态" content={this.renderHandledState()}>
        <div className="oneNodeWrapper" {...props}>
          <div>{node.get('name') || ''}</div>
          <div className="nodeIconWrapper">
            <img src={roleIconMapping[node.get('type')] || personIcon} alt="" />
            {this.renderNodeStatusIcon()}
          </div>
          <div>{node.get('nodeModeName') || ''}</div>
        </div>
      </Popover>
    }
    if(mode=="view"&&node.get("type")!="end"){
      return<Popover overlayStyle={{ zIndex: 9999 }} overlayClassName="handlerStatePopup" title={mode=="view"?"处理状态":null} content={this.renderHandledState1()}><div className="oneNodeWrapper" {...props}>
      <div>{node.get('name') || ''}</div>
      <div className="nodeIconWrapper">
        <img src={roleIconMapping[node.get('type')] || personIcon} alt="" />
        {this.renderNodeStatusIcon()}
      </div>
      <div>{node.get('nodeModeName') || ''}</div>
      {/* {node.get('type') !== workflowNodeType.end.value && this.renderPopover()} */}
      {this.renderPopover()}
    </div>
    </Popover>
    }
    return<div className="oneNodeWrapper" {...props}>
      <div>{node.get('name') || ''}</div>
      <div className="nodeIconWrapper">
        <img src={roleIconMapping[node.get('type')] || personIcon} alt="" />
        {this.renderNodeStatusIcon()}
      </div>
      <div>{node.get('nodeModeName') || ''}</div>
      {/* {node.get('type') !== workflowNodeType.end.value && this.renderPopover()} */}
      {this.renderPopover()}
    </div>
  }
}