/**
 * Created by 王友生 on 2018-04-12.
 */
import './index.less';
import React, { PureComponent } from 'react';
import { Tabs, Button, InputNumber, Input, Icon } from 'antd';
import PropTypes from 'prop-types';
import { Map } from 'immutable';
import {
  workFlowNodeAttrModeConst,
  wfRelationConst,
  wfConditionConst,
  selectedModeConst,
  wfHandleType,
  dealWithResultStatusConstArr
} from '../../../common/const';
import { translateObjectToList, strToHexCharCode, getItemSourceByKey, newGuid, fastDate } from '../../../utils';
import DraggableModal from '../../../components/DraggableModal';
import AutoGenerationForm from '../../../components/AutoGenerationForm';
import DragSortTable from '../../DragSortTable';
import CustomSelect from '../../CustomSelect';
import CustomTreeSelect from '../../CustomTreeSelect';
import _ from 'lodash';
import { connect } from 'react-redux';

const TabPane = Tabs.TabPane;

const formConfig = [[{
  dataIndex: 'unionid',
  title: '节点码',
  type: 'text',
  sourceKey: '',
}, {
  dataIndex: 'name',
  title: '节点名称',
  type: 'text',
  sourceKey: '',
}, {
  dataIndex: 'isEdit',
  title: '是否修改',
  type: 'checkbox',
  sourceKey: '',
}, {
  dataIndex: 'handleType',
  title: '处理类型',
  type: 'radio',
  sourceKey: '',
  source: translateObjectToList(wfHandleType)
}, {
  dataIndex: 'nodeAttr',
  title: '节点属性',
  type: 'radio',
  sourceKey: '',
  source: translateObjectToList(workFlowNodeAttrModeConst)
}, {
  dataIndex: 'enterCon',
  title: '节点条件',
  width: 24,
  type: 'text',
}, {
  dataIndex: 'ignoreCon',
  title: '跳过条件',
  width: 24,
  type: 'text',
}, {
  dataIndex: 'callBack',
  title: '保存回写',
  width: 24,
  type: 'text',
}]]

@connect((state, route) => {
  const reference = state.get('reference');
  const dictionary = state.get('dictionary');
  return {
    reference,
    dictionary,
    selectedRows: reference.get('selectedRows'),
    selectedSuccess: reference.get('selectedSuccess'),
    currentItem: reference.get('currentItem'),
    objectHash: reference.get('objectHash')
  }
}, null, null, { withRef: true })

export default class NodeEditor extends PureComponent {
  static propTypes = {
    node: PropTypes.object,
  }

  static defaultProps = {
    node: Map({}),
  }

  state = {
    node: Map({}),
    conType: '',
    update: false,
    extraConfig: {
      enterCon: {
        renderFunc: (commonProps, item, onChange, itemSource) => {
          return <div>{this.state.node.get('enterConTitle')}
            <Button size="small" onClick={() => this.setState({ conVisible: 'enter' })}>设置</Button>
          </div>
        }
      },
      ignoreCon: {
        renderFunc: (commonProps, item, onChange, itemSource) => {
          return <div>{this.state.node.get('ignoreConTitle')}
            <Button size="small" onClick={() => this.setState({ conVisible: 'ignore' })}>设置</Button>
          </div>
        }
      },
      callBack: {
        renderFunc: (commonProps, item, onChange, itemSource) => {
          return <div>{this.state.node.get('callBackTitle')}
            <Button size="small" onClick={() => this.setState({ callBackVisible: true })}>设置</Button>
          </div>
        }
      }
    },
    enterConCache: [],
    ignorConCache: [],
    callBackCache: []
  }

  componentWillReceiveProps(newProps) {
    if (newProps.visible !== this.props.visible && newProps.visible) {
      this.setState({
        node: newProps.node,
        enterConCache: newProps.node.get('enterCon') && JSON.parse(newProps.node.get('enterCon')) || [],
        ignorConCache: newProps.node.get('ignoreCon') && JSON.parse(newProps.node.get('ignoreCon')) || [],
        callBackCache: newProps.node.get('callBackCache') && JSON.parse(newProps.node.get('callBackCache')) || [],
      });
    }
  }

  onItemValueChange(value, dataIndex) {
    const { node } = this.state;
    this.setState({ node: node.set(dataIndex, value) });
  }

  renderBasicInfoForm() {
    const { node, extraConfig } = this.state;

    return <div style={{ height: '450px' }}>
      <AutoGenerationForm
        config={formConfig}
        extraConfig={extraConfig}
        source={node}
        onItemValueChange={(value, dataIndex) => this.onItemValueChange(value, dataIndex)}
        layout={{ md: 12 }} />
    </div>
  }

  getConDisplayName(con) {
    if (con.type === 'treeSelect' || con.type === 'treeSelectForbidParent' || con.type === 'select') {
      return con.value && con.value.label || '';
    } else if (con.type === 'popup') {
      return con.value && con.value.value && con.value.value.length > 0 && con.value.value[0].label || ''
    }
    return con.value;
  }

  render() {
    const { onClose, visible, saveNode, wfCons, reference, dictionary } = this.props;
    let wfConsN = _.cloneDeep(wfCons);

    // 流程设置保存回写将禁用属性全部改写成取消禁用
    wfConsN.forEach(item=>{
      if(item.disabled){
        item.disabled = false;
      }
    })
    const { conVisible, ignorConCache, enterConCache, callBackVisible, callBackCache } = this.state;
    const tableColumns = [{
      dataIndex: 'group',
      title: '组',
      width: 50,
      render: (text, record) => <InputNumber style={{ width: '50px' }} defaultValue={text}
        onChange={(value) => record.group = value} />
    }, {
      dataIndex: 'relation',
      title: '关系',
      width: 50,
      render: (text, record) => <CustomSelect style={{ minWidth: '50px' }} source={translateObjectToList(wfRelationConst)}
        defaultValue={text} onChange={(value) => record.relation = value} />
    }, {
      dataIndex: 'dataIndex',
      title: '字段',
      width: 100,
      render: (text, record) => <CustomSelect
        needSelectItem
        source={wfConsN}
        field={{ value: 'dataIndex', label: 'title' }}
        defaultValue={text}
        onChange={(value, selectedItem) => {
          record.dataIndex = value;
          record.dataName = selectedItem.title;
          record.type = selectedItem.type;
          record.value = null;
          this.setState({ update: !this.state.update })
        }} />
    }, {
      dataIndex: 'condition',
      title: '条件',
      width: 100,
      render: (text, record) => <CustomSelect style={{ minWidth: '80px' }}
        source={translateObjectToList(wfConditionConst)} defaultValue={text}
        onChange={(value) => record.condition = value} />
    }, {
      dataIndex: 'value',
      title: '值',
      width: 350,
      render: (text, record) => {
        let item = _.find(wfCons, { dataIndex: record['dataIndex'] });
        if (item) {
          if (item.type === 'select') {
            let { itemSource, config } = getItemSourceByKey(item, this.props.dispatch, reference, dictionary);
            return <CustomSelect
              selectProps={{ value: text && text.value }}
              needSelectItem
              source={itemSource}
              field={config && config.field}
              defaultValue={text && text.value}
              onChange={(value, selectedItem) => {
                record.value = { value, label: selectedItem[config && config.field && config.field.label || 'label'] };
                this.setState({ update: !this.state.update });
              }} />
          } else if (item.type === 'treeSelect' || item.type === 'treeSelectForbidParent') {
            let { itemSource, config } = getItemSourceByKey(item, this.props.dispatch, reference, dictionary);
            return <CustomTreeSelect
              needSelectItem
              data={itemSource || []}
              style={{ minWidth: '200px' }}
              treeField={config && config.field && {
                id: config.field.value,
                name: config.field.label,
                pid: config.field.pid
              }}
              onChange={(value, selectedItem) => {
                record.value = { value, label: selectedItem[config && config.field && config.field.label || 'label'] };
                this.setState({ update: !this.state.update });
              }}
              value={text && text.value} />
          } else if (item.type === 'popup') {
            const suffix = text && text.value ? <Icon type="close-circle" style={{ cursor: 'pointer' }} onClick={() => {
              record.value = null;
              this.setState({ update: !this.state.update });
            }} /> : null;
            return <Input value={text && text.value && text.value.length > 0 && text.value[0].label || ''}
              suffix={suffix}
              onFocus={(e) => {
                e.target.blur();
                this.props.dispatch({
                  type: 'reference/visibleReference',
                  payload: {
                    id: item.referenceKey,
                    item,
                    selectedMode: selectedModeConst.single,
                    objectHash: 'node-editor',
                    onSelectedSuccess: (selectedItems) => {
                      const mainField = _.find(item.affectFields, { referenceField: 'id' });
                      let value = [];
                      _.forEach(selectedItems, selectedItem => {
                        value.push({ value: selectedItem['id'], label: selectedItem[item.referenceField] });
                      })
                      let mainKey = mainField ? mainField.formField[1] : item[item.referenceField];
                      record.value = { value, mainKey };
                      this.setState({ update: !this.state.update });
                    }
                  }
                })
              }} />
          }
        }
        return <Input
          value={text}
          onChange={(e) => {
            record.value = e.target.value;
            this.setState({ update: !this.state.update });
          }} />
      }
    }, {
      dataIndex: 'delete',
      title: '删除',
      width: 50,
      render: (text, record) => <Button type="danger" size="small" onClick={() => {
        let conCache = conVisible === 'enter' ? enterConCache : ignorConCache;
        let deleteIndex = conCache.indexOf(record);
        conCache.splice(deleteIndex, 1);
        this.setState({ update: !this.state.update });
      }}>删除</Button>
    }]
    return <div>
      <DraggableModal title='请选择'
        visible={visible}
        bodyStyle={{ padding: '0 24px' }}
        className="NodeEditor"
        onCancel={() => onClose && onClose()}
        onOk={() => saveNode && saveNode(this.state.node)}
        width={800}>
        <Tabs defaultActiveKey="1"
          tabBarStyle={{ margin: 0 }}>
          <TabPane tab="基本信息" key="1">{this.renderBasicInfoForm()}</TabPane>
        </Tabs>
      </DraggableModal>
      <DraggableModal
        title="请设置条件"
        bodyStyle={{ height: '500px', overflow: 'auto', padding: '4px 8px' }}
        width={1000}
        visible={!!conVisible}
        onOk={() => {
          let displayName = '';
          let currentGroup = '';
          let cons = [];
          let conCache = conVisible === 'enter' ? enterConCache : ignorConCache;

          conCache.map(con => {
            if (con.group && con.dataIndex && con.relation && con.condition) {
              if (currentGroup !== con.group) {
                displayName = displayName + (currentGroup ? ') ' + wfRelationConst[con.relation].label + ' ' : '');
                displayName = displayName + '(' + con.dataName + wfConditionConst[con.condition].label + this.getConDisplayName(con);
              } else {
                displayName = displayName + ' ' + wfRelationConst[con.relation].label + ' ' + con.dataName + wfConditionConst[con.condition].label + this.getConDisplayName(con);
              }
              currentGroup = con.group;
              cons.push(con)
            }
          });
          displayName = displayName ? displayName + ')' : '';
          this.setState({
            conVisible: '',
            node: this.state.node.set(conVisible === 'enter' ? 'enterCon' : 'ignoreCon', JSON.stringify(cons))
              .set(conVisible === 'enter' ? 'enterConTitle' : 'ignoreConTitle', displayName)
          })
        }}
        onCancel={() => this.setState({ conVisible: false })}>
        <div>
          <Button size="small" onClick={() => this.setState({
            [conVisible === 'enter' ? 'enterConCache' : 'ignorConCache']: (conVisible === 'enter' ? enterConCache : ignorConCache).concat([{
              group: 1,
              key: newGuid()
            }])
          })}>添加</Button>
          <div>
            <DragSortTable
              tableProps={{ bordered: true, pagination: false, rowKey: (record) => record.key }}
              columns={tableColumns}
              onChange={newData => this.newTableConfig = newData}
              dataSource={conVisible === 'enter' ? enterConCache : ignorConCache} />
          </div>
        </div>
      </DraggableModal>
      <DraggableModal
        title="请设置回写"
        bodyStyle={{ height: '500px', overflow: 'auto', padding: '4px 8px' }}
        width={1000}
        visible={callBackVisible}
        onOk={() => {
          let displayName = '';
          let cons = [];
          console.log('callBackCache', callBackCache);

          callBackCache.map((item, idx) => {
            cons.push(item)
            displayName += `${item.dataName}${idx < callBackCache.length ? ',' : ''}`;
          })
          this.setState({
            callBackVisible: false,
            node: this.state.node.set('callBackCache', JSON.stringify(cons))
              .set('callBackTitle', displayName)
          })
        }}
        onCancel={() => this.setState({ callBackVisible: false })}>
        <div>
          <Button size="small" onClick={() => this.setState({
            callBackCache: callBackCache.concat([{
              key: newGuid()
            }])
          })}>添加</Button>
          <div>
            <DragSortTable
              tableProps={{ bordered: true, pagination: false, rowKey: (record) => record.key }}
              columns={[tableColumns[2], {
                dataIndex: 'state',
                title: '状态',
                width: 50,
                render: (text, record) => <CustomSelect
                  selectProps={{ mode: "multiple", value: text ? text.split(',') : [] }}
                  source={dealWithResultStatusConstArr}
                  fromPage={{type:"writeBack"}}
                  onChange={(value, selectItem) => {
                    record.state = value.join();
                    this.setState({ update: !this.state.update });
                  }
                  } />
              }, tableColumns[4], {
                dataIndex: 'delete',
                title: '删除',
                width: 50,
                render: (text, record) => <Button type="danger" size="small" onClick={() => {
                  let deleteIndex = callBackCache.indexOf(record);
                  callBackCache.splice(deleteIndex, 1);
                  this.setState({ update: !this.state.update });
                }}>删除</Button>
              }]}
              onChange={newData => this.newTableConfig = newData}
              dataSource={callBackCache} />
          </div>
        </div>
      </DraggableModal>
    </div>
  }
}
