import './index.less'
import React, {PureComponent} from 'react';
import {Menu as AntMenu, Dropdown} from 'antd';
import {connect} from 'react-redux';
import {request} from '../../../utils';
import _ from 'lodash';
import {FormattedMessage, injectIntl} from 'react-intl';


@connect((state, route) => {
  const basicInfo = state.getIn(['system', 'basicInfo']);
  const refreshCurrentWorkflow = state.getIn(['workflow', 'refreshCurrentWorkflow']);
  const lang = state.get('system').get('lang');
  return {
    lang,
    functionCode: route.functionCode,
    recordId: route.recordId,
    basicInfo,
    refreshCurrentWorkflow
  }
})
export default class WorkflowOperation extends PureComponent {
  constructor(props) {
    super(props);

    this.state = {
      bottom: props.bottom || 60,
      right: props.right || 60,
      visisble: false,
      highlight: false
    }
  }

  componentDidMount() {
    if (this.props.recordId && this.props.functionCode) {
      this.fetchWorkflowInfo(this.props.functionCode, this.props.recordId);
    }
  }

  componentWillReceiveProps(newProps) {
    if (newProps.refreshCurrentWorkflow !== this.props.refreshCurrentWorkflow) {
      this.fetchWorkflowInfo(newProps.functionCode, newProps.recordId);
    } else if (newProps.recordId && (this.props.recordId !== newProps.recordId || this.props.functionCode !== newProps.functionCode)) {
      this.fetchWorkflowInfo(newProps.functionCode, newProps.recordId);
    }
  }

  fetchWorkflowInfo(functionCode, recordId) {
    request('/sysWorkflow/getSysWorkflowInfo', {funCode: functionCode, fid: recordId})
      .then((data) => {
        if (data && data.result && data.result.wfContent) {
          const {currentNodes} = JSON.parse(data.result.wfContent);
          let handlerNode = undefined;
          if (currentNodes && currentNodes.length > 0) {
            handlerNode = _.find(currentNodes, node => {
              return _.find(node.users, {userId: this.props.basicInfo.id, handled: false});
            });
          }
          this.setState({visisble: true, highlight: !!handlerNode})
          this.props.dispatch({
            type: 'workflow/readWorkflow',
            payload: {
              result: data.result,
              functionCode,
              recordId
            }
          })
        }
      });
  }

  dragStart(e) {
    e.stopPropagation();
    let offsetX = e.nativeEvent.offsetX;
    let offsetY = e.nativeEvent.offsetY;
    this.setState({offsetX, offsetY});
  }

  dragOver(e) {
    const {offsetX, offsetY} = this.state;
    this.setState({
      bottom: document.body.clientHeight - e.clientY - offsetY,
      right: document.body.clientWidth - e.clientX - offsetX
    });
  }

  dragEnd(e) {
    const {offsetX, offsetY} = this.state;
    this.setState({
      bottom: document.body.clientHeight - e.clientY - offsetY,
      right: document.body.clientWidth - e.clientX - offsetX
    });
  }

  render() {
    const {bottom, right, visisble, highlight} = this.state;
    if (!visisble) {
      return null;
    }
    const {functionCode, recordId} = this.props;
    const menu = (
      <AntMenu>
        <AntMenu.Item>
          <a target="_blank" rel="noopener noreferrer"
             onClick={(e) => this.props.dispatch({
               type: 'workflow/submitWorkflow',
               payload: {
                 functionCode,
                 recordIds: [recordId],
               }
             })}><FormattedMessage id="work_submit"/></a>
        </AntMenu.Item>
        <AntMenu.Item>
          <a target="_blank" rel="noopener noreferrer"
             onClick={(e) => this.props.dispatch({
               type: 'workflow/withdrawWorkflow',
               payload: {
                 functionCode,
                 recordIds: [recordId],
               }
             })}><FormattedMessage id="work_recall"/></a>
        </AntMenu.Item>
        <AntMenu.Item>
          <a target="_blank" rel="noopener noreferrer"
             onClick={(e) => this.props.dispatch({
               type: 'workflow/beginVerifiedWorkflow',
               payload: {
                 functionCode,
                 recordIds: [recordId],
               }
             })}><FormattedMessage id="work_check"/></a>
        </AntMenu.Item>
        <AntMenu.Item>
          <a target="_blank" rel="noopener noreferrer"
             onClick={(e) => this.props.dispatch({
               type: 'workflow/recallWorkflow',
               payload: {
                 functionCode,
                 recordIds: [recordId],
               }
             })}><FormattedMessage id="work_abandonTrial"/></a>
        </AntMenu.Item>
        <AntMenu.Item>
          <a target="_blank" rel="noopener noreferrer" onClick={(e) => {
            e.preventDefault();
            this.props.dispatch({
              type: 'workflow/viewWorkflow',
              payload: {
                functionCode: functionCode,
                recordIds: [recordId]
              }
            })
          }}><FormattedMessage id="work_view"/></a>
        </AntMenu.Item>
      </AntMenu>
    );
    return <Dropdown overlay={menu} placement="topCenter" trigger="click" overlayClassName="workflowOperationMenu">
      <div
        className={`workflowOperation ` + (highlight ? 'workflowOperationHighlight' : '')}
        draggable
        style={{bottom: bottom + 'px', right: right + 'px'}}
        onDragStart={e => this.dragStart(e)}
        onDragOver={e => this.dragOver(e)}
        onDragEnd={e => this.dragEnd(e)}
      >
        <FormattedMessage id="work_flow"/>
      </div>
    </Dropdown>
  }
}
