import React from 'react';
import { Map } from 'immutable';
import ReactDOM from 'react-dom';
import rootSaga from './rootSagas';
import { Provider } from 'react-redux';
import rootReducer from './rootReducer';
import createSagaMiddleware from 'redux-saga';
import { createStore, applyMiddleware } from 'redux';
import createHistory from 'history/createBrowserHistory';
import { ConnectedRouter, routerMiddleware } from 'react-router-redux';
import './extension';
import './utils/event';
import * as _utils from 'utils'
import App from './app';
import _autoGenerationForm from 'components/AutoGenerationForm';
import _asyncComponent from 'components/AsyncComponent';
import _autoGenerationDetail from 'components/AutoGenerationDetail';
import _calculationFormula from 'components/CalculationFormula';
import _cardCollection from 'components/CardCollection';
import _checkableTags from 'components/CheckableTags';
import _cmMenu from 'components/CrmMenu';
import _customDraggerUpload from 'components/CustomDraggerUpload';
import _customList from 'components/CustomList';
import _customPopover from 'components/CustomPopover';
import _customSelect from 'components/CustomSelect';
import _customTable from 'components/CustomTable';
import _customToolTip from 'components/CustomToolTip';
import _customTree from 'components/CustomTree';
import _customTreeSelect from 'components/CustomTreeSelect';
import _descriptionList from 'components/DescriptionList';
import _draggableModal from 'components/DraggableModal';
import _dragSortTable from 'components/DragSortTable';
import _editableConfigNode from 'components/EditableConfigNode';
import _ellipsisText from 'components/EllipsisText';
import _footerToolbar from 'components/FooterToolbar';
import _globalLoading from 'components/GlobalLoading';
import _headerSearch from 'components/HeaderSearch';
import _keyValuePaired from 'components/KeyValuePaired';
import _mainMenu from 'components/MainMenu';
import _message from 'components/Message';
import _nodeProps from 'components/NodeProps';
import _noticeIcon from 'components/NoticeIcon';
import _pageTabs from 'components/PageTabs';
import _permission from 'components/Permission';
import _picturesWall from 'components/PicturesWall';
import _preview from 'components/Preview';
import _purchaseCount from 'components/PurchaseCount';
import _quickPanel from 'components/QuickPanel';
import _referenceConfig from 'components/ReferenceConfig';
import _resizeableTitle from 'components/ResizeableTitle';
import _searchScenarioConfig from 'components/SearchScenarioConfig';
import _selectButton from 'components/SelectButton';
import _shortFormConfig from 'components/ShortFormConfig';
import _uEditor from 'components/UEditor';
import _uploadButton from 'components/UploadButton';
import _uploadThumb from 'components/UploadThumb';
import _loginThumb from 'components/LoginThumb';
import _employeeSelector from 'components/EmployeeSelector';
import _customPopModal from 'components/CustomPopModal';
import _confirmDialog from 'components/ConfirmDialog';
import { VirtualTable as _VirtualTable } from 'components/VirtualTable';
import _calendar from 'components/Calendar';
import { WorkflowEditor as _workflowEditor, WorkflowOperation as _workflowOperation, WorkflowActions as _workflowActions } from 'components/Workflow';
import _freeCombinationForm from 'components/FreeCombinationForm';
import _customDateRange from 'components/CustomDateRange';

export const AutoGenerationForm = _autoGenerationForm;
export const AsyncComponent = _asyncComponent;
export const AutoGenerationDetail = _autoGenerationDetail;
export const CalculationFormula = _calculationFormula;
export const CardCollection = _cardCollection;
export const CheckableTags = _checkableTags;
export const CrmMenu = _cmMenu;
export const CustomDraggerUpload = _customDraggerUpload;
export const CustomList = _customList;
export const CustomPopover = _customPopover;
export const CustomSelect = _customSelect;
export const CustomTable = _customTable;
export const CustomToolTip = _customToolTip;
export const CustomTree = _customTree;
export const CustomTreeSelect = _customTreeSelect;
export const DescriptionList = _descriptionList;
export const DraggableModal = _draggableModal;
export const DragSortTable = _dragSortTable;
export const EditableConfigNode = _editableConfigNode;
export const EllipsisText = _ellipsisText;
export const FooterToolbar = _footerToolbar;
export const GlobalLoading = _globalLoading;
export const HeaderSearch = _headerSearch;
export const KeyValuePaired = _keyValuePaired;
export const MainMenu = _mainMenu;
export const Message = _message;
export const NodeProps = _nodeProps;
export const NoticeIcon = _noticeIcon;
export const PageTabs = _pageTabs;
export const Permission = _permission;
export const PicturesWall = _picturesWall;
export const Preview = _preview;
export const PurchaseCount = _purchaseCount;
export const QuickPanel = _quickPanel;
export const ReferenceConfig = _referenceConfig;
export const ResizeableTitle = _resizeableTitle;
export const SearchScenarioConfig = _searchScenarioConfig;
export const SelectButton = _selectButton;
export const ShortFormConfig = _shortFormConfig;
export const UEditor = _uEditor;
export const UploadButton = _uploadButton;
export const UploadThumb = _uploadThumb;
export const LoginThumb = _loginThumb;
export const WorkflowEditor = _workflowEditor;
export const WorkflowOperation = _workflowOperation;
export const WorkflowActions = _workflowActions;
export const utils = _utils;
export const EmployeeSelector = _employeeSelector;
export const CustomPopModal = _customPopModal;
export const ConfirmDialog = _confirmDialog;
export const VirtualTable = _VirtualTable;
export const Calendar = _calendar;
export const FreeCombinationForm = _freeCombinationForm
export const CustomDateRange = _customDateRange

const sagaMiddleware = createSagaMiddleware();
const history = createHistory();
const createStoreWithMiddleware = applyMiddleware(sagaMiddleware, routerMiddleware(history))(createStore);

var env = process.env.NODE_ENV || 'development';
let store;
if (env === 'development') {
  store = createStoreWithMiddleware(rootReducer, Map({}), window.__REDUX_DEVTOOLS_EXTENSION__ && window.__REDUX_DEVTOOLS_EXTENSION__());
} else {
  store = createStoreWithMiddleware(rootReducer, Map({}));
}
sagaMiddleware.run(rootSaga);
const innerRoutes = [];

export const Main = ({ routes }) => <div>
  <Provider store={store}>
    <GlobalLoading>
      <ConnectedRouter history={history}>
        <App routes={[...innerRoutes, ...routes]} />
      </ConnectedRouter>
    </GlobalLoading>
  </Provider>
</div>

