/**
 * Created by 王友生 on 2018-07-05.
 */

import {takeEvery, put, fork, call, select} from 'redux-saga/effects';
import {fromJS, Map} from 'immutable';
import {message} from 'antd';
import {post, url, download, downloadFromServer} from '../../service/api';
import {
  buttonPositionType,
  buttonType,
  printerTableStyle,
  editPagePositionConst,
  detailDisplayTypeConst, globalTranslationSuffix
} from '../../common/const';
import {pageInfo} from '../../common/config';
import {getItemActualValue, getItemDefaultValue, sortTableColumns, validateItemPermission} from '../../utils';
import _ from 'lodash';

export const watchFetchAdvanceListConfig = function* () {
  yield takeEvery('advanceList/initAdvanceList', initAdvanceList);
  yield takeEvery('advanceList/changeSelectedId', viewDetailPage);
  yield takeEvery('advanceList/changeCustomPreference', changeCustomPreference);
  yield takeEvery('advanceList/changeCustomTableConfig', changeCustomTableConfig);
}

function* initAdvanceList(action) {
  const cache = yield select(x => x.getIn(['advanceList', action.payload.functionCode]));
  if (!cache) {
    const data = yield call(fetchAdvanceListConfig, action);
    if (!data) {
      message.error('高级列表配置不完全！');
    } else {
      if (action.payload.initFromDashboard) {
        yield put({type: 'dashboard/fetchConfigForDashboard', payload: {functionCode: action.payload.functionCode}});
        return;
      } else {
        const {basicInfo, searchScenario, searchButton} = data;
        if (window.advanceListDefaultSearch) {
          yield fork(fetchAdvanceListData, {
            payload: {
              functionCode: action.payload.functionCode,
              searchScenario,
              searchButton,
              cacheScenario: action.payload.cacheScenario
            }
          })
        }
      }
    }
  } else {
    if (action.payload.initFromDashboard) {
      yield put({type: 'dashboard/fetchConfigForDashboard', payload: {functionCode: action.payload.functionCode}});
      return;
    }
    const basicInfo = cache.get('basicInfo');
    const searchScenario = cache.get('searchScenario');
    const searchButton = cache.get('searchButton');
    if (window.advanceListDefaultSearch) {
      yield fork(fetchAdvanceListData, {
        payload: {
          functionCode: action.payload.functionCode,
          searchScenario,
          searchButton,
          cacheScenario: action.payload.cacheScenario
        }
      })
    }
  }
}

function* changeCustomPreference(action) {
  const {functionCode, newSearchConfig} = action.payload;
  const basicInfo = yield select(x => x.getIn(['system', 'basicInfo']));
  newSearchConfig && localStorage.setItem(functionCode + basicInfo.id + 'advanceListSearchConfig', JSON.stringify(newSearchConfig));
  yield put({
    type: 'advancelist/changeSearchConfig',
    payload: {functionCode, newSearchConfig, searchRowCount: calculateSearchRowCount(newSearchConfig)}
  })
}

function* changeCustomTableConfig(action) {
  const {functionCode, newTableConfig} = action.payload;
  const cache = yield select(x => x.getIn(['advanceList', functionCode]));
  const userinfo = yield select(x => x.getIn(['system', 'basicInfo']));
  const basicInfo = cache.get('basicInfo');
  const supportBatch = basicInfo.batch === '1';
  const actionColumn = cache.get('actionColumn');
  let actualTableConfig = [];
  let totalWidth = 0;
  const calculateTotalWidth = (width) => {
    if (typeof width === 'string') {
      width = parseInt(width);
    }
    totalWidth += width;
  };
  newTableConfig.map(item => {
    if (item['hide'] === '1') {
      return true;
    }
    if (item.defaultHide === '1') {
      return true;
    }
    calculateTotalWidth(item.width || item.originalWidth);
    actualTableConfig.push({...item});
    return true;
  })
  if (actualTableConfig.length > 0) {
    let lastItem = _.findLast(actualTableConfig, (item) => item.fixed !== 'left' && item.fixed !== 'right');
    lastItem.originalWidth = lastItem['width'];
    lastItem && delete lastItem['width'];
  }

  if (supportBatch) {
    totalWidth = totalWidth + 35;
  }

  newTableConfig && localStorage.setItem(functionCode + userinfo.id + 'advancelistTableConfig', JSON.stringify(newTableConfig));

  if (actionColumn) {
    actualTableConfig.push(actionColumn);
    totalWidth = totalWidth + actionColumn.width * 2 + 50;
  }

  yield put({
    type: 'advancelist/changeCacheTableConfig', payload: {
      functionCode,
      totalWidth,
      fullTableConfig: newTableConfig,
      tableConfig: sortTableColumns(actualTableConfig)
    }
  })
}

function* fetchAdvanceListConfig(action) {
  const {functionCode} = action.payload;
  const data = yield call(post, url.getSysListConfig, {key: functionCode}, functionCode);

  /* if (! data && data.result) {
     return;
   }*/
  if (!data || !data.result) {
    return;
  }

  const permissions = yield select(x => x.getIn(['system', 'permissions'])) || [];

  const {content: contentString, id, buttons, prints} = data.result;
  if (!contentString) {
    return
  }

  const content = JSON.parse(contentString);
  const config = content['config'];
  const collect = content['collect'];
  const basicInfo = content['basicInfo'];
  const shortFormConfig = content['formShortFormConfig'];
  if (!config) {
    return;
  }
  let searchScenarioConfig = config['searchScenario'];
  let searchScenario = Map({});
  searchScenarioConfig.map(item => {
    searchScenario = searchScenario.set(item.dataIndex, getItemDefaultValue(item));
    return true
  });

  const userInfo = yield select(x => x.getIn(['system', 'basicInfo']));
  let cacheSearchConfigString = localStorage.getItem(action.payload.functionCode + userInfo.id + 'advanceListSearchConfig')
  if (cacheSearchConfigString) {
    let cacheSearchConfig = JSON.parse(cacheSearchConfigString);
    let actualSearchConfig = [];
    _.map(cacheSearchConfig, (configItem) => {
      const newItem = _.findLast(searchScenarioConfig, {dataIndex: configItem.dataIndex});
      newItem && actualSearchConfig.push({...newItem, defaultHide: configItem.defaultHide});
    })
    const extraSearchConfig = _.differenceWith(searchScenarioConfig, cacheSearchConfig, (item1, item2) => {
      return item1.dataIndex === item2.dataIndex;
    });
    searchScenarioConfig = [...actualSearchConfig, ...extraSearchConfig];
  }

  const fieldsArray = config['fields'] || [];
  let totalWidth = 0;
  const calculateTotalWidth = (width) => {
    if (typeof width === 'string') {
      width = parseInt(width);
    }
    totalWidth += width;
  };

  const listTopButtons = [];
  const listRecordButtons = [];
  const listMoreButtons = [];
  const listRelationFormButtons = [];
  const listRelationSearchButtons = [];
  const listToolButtons = [];
  let searchButton = undefined;
  let listPrintButtons = [];
  let listExportButtons = [];
  let listAuditingButton = undefined;
  let listAbandonButton = undefined;
  let listAddButton = undefined;
  let listBatchEditButton = undefined;
  let listBatchDeleteButton = undefined;
  let listBatchUpdateButton = undefined;
  let listSettingButton = undefined;

  const allotButtons = (button, list) => {
    if (button.btnType === buttonType.print.value) {
      listPrintButtons.push(button);
    } else if (button.btnType === buttonType.export.value) {
      listExportButtons.push(button);
    } else if (!listAuditingButton && button.btnType === buttonType.auditing.value) {
      listAuditingButton = button;
    } else if (!listAbandonButton && button.btnType === buttonType.abandon.value) {
      listAbandonButton = button;
    } else if (!listAddButton && button.btnType === buttonType.add.value) {
      listAddButton = button;
    } else if (!searchButton && button.btnType === buttonType.search.value) {
      searchButton = button;
    } else if (!listBatchEditButton && button.btnType === buttonType.batchEdit.value) {
      listBatchEditButton = button;
    } else if (!listBatchDeleteButton && button.btnType === buttonType.batchDelete.value) {
      listBatchDeleteButton = button;
    } else if (button.btnType === buttonType.batchUpdate.value) {
      listBatchUpdateButton = button;
    } else if (!listSettingButton && button.btnType === buttonType.setting.value) {
      listSettingButton = button;
    } else if (button.btnType === buttonType.addRelationForm.value || button.btnType === buttonType.addPopupRelationForm.value) {
      listRelationFormButtons.push(button);
    } else if (button.btnType === buttonType.tools.value) {
      listToolButtons.push(button);
    } else if (button.btnType === buttonType.relationSearch.value) {
      listRelationSearchButtons.push(button);
    } else {
      list.push(button);
    }
  }

  buttons.map(button => {
    if (button.btnPosition === buttonPositionType.listTop.value) {
      allotButtons(button, listTopButtons);
    } else if (button.btnPosition === buttonPositionType.listRecord.value) {
      listRecordButtons.push(button);
    } else if (button.btnPosition === buttonPositionType.listMore.value) {
      allotButtons(button, listMoreButtons);
    }
  });

  let fullTableConfig = fieldsArray;

  const userinfo = yield select(x => x.getIn(['system', 'basicInfo']));

  let cacheTableConfigString = localStorage.getItem(action.payload.functionCode + userinfo.id + 'advancelistTableConfig');
  if (cacheTableConfigString) {
    let cacheTableConfig = JSON.parse(cacheTableConfigString);
    let actualFullTableConfig = [];
    _.map(cacheTableConfig, (configItem) => {
      const newItem = _.findLast(fullTableConfig, {dataIndex: configItem.dataIndex});
      newItem && actualFullTableConfig.push({...newItem, defaultHide: configItem.defaultHide, fixed: configItem.fixed});
    })
    const extraTableConfig = _.differenceWith(fullTableConfig, cacheTableConfig, (item1, item2) => {
      return item1.dataIndex === item2.dataIndex;
    });
    fullTableConfig = [...actualFullTableConfig, ...extraTableConfig];
  }

  const newFieldsArray = [];

  fullTableConfig.map((item, index) => {
    if (item['hide'] === '1') {
      return true;
    }
    if (item.defaultHide === '1') {
      return true;
    }
    item.editDictionary = item.sourceKey && (!validateItemPermission(item.dicPermission, basicInfo)) ? '1' : 0;
    if (!validateItemPermission(item.showPermission, userInfo)) {
      return true;
    }
    calculateTotalWidth(item.width || item.originalWidth);
    newFieldsArray.push(item);
    return true;
  });

  if (newFieldsArray.length > 0) {
    let lastItem = _.findLast(newFieldsArray, (item) => item.fixed !== 'left' && item.fixed !== 'right');
    lastItem.originalWidth = lastItem['width'];
    lastItem && delete lastItem['width'];
  }
  let actionColumn = undefined;
  if (listRecordButtons.length > 0) {
    const columnWidth = listRecordButtons.length * 60 + 50
    actionColumn = {
      columnType: buttonPositionType.listRecord.value,
      buttons: listRecordButtons,
      width: columnWidth,
      fixed: 'right',
      title: '操作',
      exportIgnore: '1'
    }
    newFieldsArray.push(actionColumn);
    totalWidth += columnWidth;
  }

  const pageSize = localStorage.getItem(action.payload.functionCode + userInfo.id + 'pageSize') || window.defaultPageSize || pageInfo.pageSize;

  const payload = {
    searchScenarioConfig,
    collect: config['collect'],
    collectRowCount: calculateSearchRowCount(config['collect']),
    fullTableConfig: config['fields'],
    fields: sortTableColumns(newFieldsArray),
    functionCode: action.payload.functionCode,
    basicInfo: basicInfo,
    actionColumn,
    searchScenario,
    searchRowCount: calculateSearchRowCount(searchScenarioConfig),
    totalWidth,
    listTopButtons,
    listMoreButtons,
    searchButton,
    listPrintButtons,
    listExportButtons,
    listAuditingButton,
    listAbandonButton,
    listBatchEditButton,
    listBatchDeleteButton,
    listBatchUpdateButton,
    listRelationFormButtons,
    listToolButtons,
    listAddButton,
    shortFormConfig,
    listPrintTemplate: prints,
    pagination: {...pageInfo, pageSize}
  }
  yield put({
    type: 'advanceList/fetchConfigSuccess',
    payload,
  })
  return payload;
}

function calculateSearchRowCount(searchConfig) {
  if (!searchConfig) {
    return 0;
  }
  let searchRowCount = 0;
  let oneRow = 0;
  searchConfig.map(item => {
    if (item.defaultHide === '1' || item.hide === '1') {
      return true;
    }
    oneRow = oneRow + parseInt(item.width);
    if (oneRow >= 24) {
      searchRowCount++;
      oneRow = 0;
    }
    return true;
  });
  return searchRowCount;
}

export const watchFetchAdvanceListData = function* () {
  yield takeEvery('advanceList/fetchAdvanceListData', fetchAdvanceListData);
}

function* viewDetailPage(action) {
  const {selectedId, functionCode} = action.payload;
  const basicInfo = yield select(x => x.getIn(['advanceList', functionCode, 'basicInfo']));
  if (selectedId && window.detailDisplayType === detailDisplayTypeConst.tab.value) {
    yield put({
      type: 'system/jumpToDetailPage',
      payload: {
        id: selectedId,
        functionCode: basicInfo.funcCode
      }
    });
  } else {
    yield put({
      type: 'advanceList/changeCacheItem',
      payload: {
        value: selectedId,
        key: 'selectedId',
        functionCode: functionCode
      }
    });
  }
}

function* fetchAdvanceListData(action) {
  const {functionCode, cacheScenario} = action.payload;
  const cache = yield select(x => x.getIn(['advanceList', functionCode]));
  let searchScenario = cache.get('searchScenario');
  const defaultSearchScenario = cache.get('searchScenario');
  const basicInfo = cache.get('basicInfo');
  const searchKey = cache.get('searchKey');
  const searchButton = cache.get('searchButton');
  const pagination = cache.get('pagination');
  const sorter = cache.get('sorter');
  const expand = cache.get('expand');
  if (cacheScenario) {
    searchScenario = searchScenario || defaultSearchScenario;
    cacheScenario.map(item => {
      searchScenario = searchScenario.set(item.dataIndex, item.value);
    });
    yield put({type: 'advanceList/changeAllSearchScenario', payload: {searchScenario, functionCode}});
  }
  let params;
  if (basicInfo.blurry && !expand) {
    params[basicInfo.blurry] = searchKey;
  } else {
    params = searchScenario.toJSON();
  }
  params['pagination'] = pagination;
  params['sorter'] = sorter;
  if (searchButton) {
    const {result} = yield call(post, searchButton.btnUrl, params, functionCode, searchButton.id);
    pagination.total = result.total;
    yield put({type: 'advanceList/fetchAdvanceDataSuccess', payload: {functionCode, data: result, pagination}})
  } else {
    message.error('没有配置搜索按钮，或者您没有搜索条件')
  }
}

export const watchAdvanceListButtonClick = function* () {
  yield takeEvery('advanceList/listButtonClick', listButtonClick);
}

function* listButtonClick(action) {
  const {functionCode, button, params} = action.payload;

  if (button.btnType === buttonType.add.value) {
    const basicInfo = yield select(x => x.getIn(['advanceList', functionCode, 'basicInfo']));
    if (basicInfo && basicInfo.funcCode) {
      yield put({
        type: 'system/jumpToEditPage',
        payload: '-1',
        functionCode: basicInfo.funcCode,
        fromFunctionCode: functionCode
      });
    } else {
      message.error('未配置相关功能！');
    }
  } else if (button.btnType === buttonType.search.value) {
    let params = {functionCode}
    yield call(fetchAdvanceListData, {payload: params});
  } else if (button.btnType === buttonType.relationSearch.value) {
    if (button.btnUrl) {
      let response;
      if (button.btnPosition === buttonPositionType.listTop.value || button.btnPosition === buttonPositionType.listMore.value) {
        const selectedRowKeys = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRowKeys']));
        response = yield call(post, button.btnUrl, {selectedIds: selectedRowKeys}, functionCode, button.id);
      } else if (button.btnPosition === buttonPositionType.listRecord.value) {
        response = yield call(post, button.btnUrl, {id: params.id}, functionCode, button.id);
      }
      if (response && response.result) {
        yield put({
          type: 'system/jumpToFunctionWidthScenario',
          payload: {functionCode: button.relationCode, scenario: response.result}
        });
      }
    } else {
      yield put({type: 'system/jumpToFunctionWidthScenario', payload: {functionCode: button.relationCode}});
    }
  } else if (button.btnType === buttonType.addRelationForm.value) {
    const payload = {targetFunctionCode: button.relationCode};
    if (button.btnUrl) {
      if (button.btnPosition === buttonPositionType.listTop.value || button.btnPosition === buttonPositionType.listMore.value) {
        const selectedRowKeys = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRowKeys']));
        const response = yield call(post, button.btnUrl, {selectedIds: selectedRowKeys}, functionCode, button.id);
        payload['newRelationFormData'] = response.result;
        payload['attachments'] = response.attachments;
      } else if (button.btnPosition === buttonPositionType.listRecord.value) {
        const selectedRowKeys = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRowKeys']));
        const response = yield call(post, button.btnUrl, {id: params.id}, functionCode, button.id);
        payload['newRelationFormData'] = response.result;
        payload['attachments'] = response.attachments;
      }
    }
    yield put({
      type: 'form/generateRelationForm',
      payload
    })
  } else if (button.btnType === buttonType.custom.value) {
    if (button.btnUrl) {
      if (button.btnPosition === buttonPositionType.listTop.value || button.btnPosition === buttonPositionType.listMore.value) {
        const selectedRowKeys = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRowKeys']));
        yield call(post, button.btnUrl, {selectedIds: selectedRowKeys}, functionCode, button.id);
      } else if (button.btnPosition === buttonPositionType.listRecord.value) {
        const selectedRowKeys = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRowKeys']));
        yield call(post, button.btnUrl, {id: params.id}, functionCode, button.id);
      }
    }
    // yield put({ type: 'advanceList/fetchAdvanceDataSuccess', payload: { functionCode } });
  } else if (button.btnType === buttonType.print.value) {
    if (params && params.apiUrl && params.jsMethodName && window[params.jsMethodName]) {
      const cache = yield select(x => x.getIn(['advanceList', functionCode]));
      const searchScenario = cache.get('searchScenario');
      const basicInfo = cache.get('basicInfo');
      const searchKey = cache.get('searchKey');
      const expand = cache.get('expand');
      let requestParams = {};
      if (basicInfo.blurry && !expand) {
        requestParams[basicInfo.blurry] = searchKey;
      } else {
        requestParams = searchScenario.toJSON();
      }
      const data = yield call(params.apiUrl, requestParams);
      let LODOP = window.getLodop();
      window[params.jsMethodName](LODOP, data);
      LODOP.PREVIEW();
    } else {
      let LODOP = window.getLodop();
      LODOP.PRINT_INIT("打印");
      let htmlString = printerTableStyle;
      htmlString += document.getElementsByClassName('ant-table-body')[0].innerHTML;
      LODOP.ADD_PRINT_TABLE(1, 1, "100%", "100%", "<body style='margin:0'>" + htmlString + "</body>");
      LODOP.PREVIEW();
    }
  } else if (button.btnType === buttonType.export.value) {
    const tableConfig = yield select(x => x.getIn(['advanceList', functionCode, 'fieldsConfig']));
    const listData = yield select(x => x.getIn(['advanceList', functionCode, 'result']));
    const reference = yield select((x) => x.getIn(["reference"]));
    const dictionary = yield select((x) => x.getIn(["dictionary"]));
    const activeTab = yield select(x => x.getIn(['system', 'activeTab']));
    const fileName = activeTab.menuName || activeTab.name;
    let allData = [];
    // if (listData && listData.dataSource) {
    //   allData = [...listData.dataSource];
    //   if (listData.dataTotal) {
    //     allData.push(listData.dataTotal);
    //   }
    // }
    //2.7导出循环配置翻译
    if (listData && listData.dataSource && listData.dataSource.length > 0) {
      listData.dataSource.map((record) => {
        const tranRecord = {...record};
        for (const key in record) {
          const item = tableConfig.filter((x) => x.dataIndex === key);
          item[0] &&
          (tranRecord[key + globalTranslationSuffix] =
            getItemActualValue(
              item[0],
              record[key],
              put,
              reference,
              dictionary
            ) || "-");
        }
        allData.push(tranRecord);
      });
    }
    const result = yield call(downloadFromServer, button.btnUrl || url.exportToFile, {
      titles: tableConfig.filter(x => x.exportIgnore !== '1'),
      columns: allData,
      fileName: activeTab.menuName || activeTab.name,
      functionCode
    }, fileName, functionCode);
  } else if (button.btnType === buttonType.popForm.value) {
    const cache = yield select(x => x.getIn(['advanceList', functionCode]));
    const shortFormConfig = cache.get('shortFormConfig') || [];
    const currentShortFormConfigs = shortFormConfig.filter(x => button.relationCode === x.shortFormId);
    if (currentShortFormConfigs && currentShortFormConfigs.length > 0) {
      const currentShortFormConfig = currentShortFormConfigs[0];
      const selectedRowKeys = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRowKeys']));
      const selectedRows = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRow']));
      const searchScenario = yield select(x => x.getIn(['advanceList', functionCode, 'searchScenario']));
      if (currentShortFormConfig['scenario']) {
        yield put({
          type: 'shortForm/visibleShortForm',
          payload: {
            fromId: params ? params.id : undefined,
            selectedIds: selectedRowKeys,
            selectRows:selectedRows,
            searchScenario,
            width: button.popWidth,
            height: button.popHeight,
            config: currentShortFormConfig,
            fromFunctionCode: functionCode,
            refreshAction: 'advanceList/fetchAdvanceListData'
          }
        });
      }
    }
  } else if (button.btnType === buttonType.addRelationForm.value || button.btnType === buttonType.addPopupRelationForm.value) {
    const payload = {targetFunctionCode: button.relationCode};
    if (button.btnUrl) {
      if (button.btnPosition === buttonPositionType.listTop.value || button.btnPosition === buttonPositionType.listMore.value) {
        const selectedRowKeys = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRowKeys']));
        const response = yield call(post, button.btnUrl, {selectedIds: selectedRowKeys}, functionCode, button.id);
        payload['newRelationFormData'] = response.result;
      } else if (button.btnPosition === buttonPositionType.listRecord.value) {
        const response = yield call(post, button.btnUrl, {id: params.id}, functionCode, button.id);
        payload['newRelationFormData'] = response.result;
      }
    }
    payload.popup = button.btnType === buttonType.addPopupRelationForm.value ? editPagePositionConst.popup.value : editPagePositionConst.tab.value;
    payload.width = button.popWidth;
    payload.height = button.popHeight;
    yield put({
      type: 'form/generateRelationForm',
      payload,
    })
  } else if (button.btnType === buttonType.customPopModal.value) {
    let record = {};
    if (button.btnPosition === buttonPositionType.listTop.value || button.btnPosition === buttonPositionType.listMore.value) {
      record.selectedRowKeys = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRowKeys']));
    } else {
      const result = yield select(x => x.getIn(['advanceList', functionCode, 'result']));
      const listData = result.dataSource;
      record.selectedRow = _.findLast(listData, {id: params.id});
    }
    yield put({type: 'system/visibleCustomPopEdit', payload: {key: button.btnUrl, record: record, functionCode}});
  } else if (button.btnType === buttonType.grant.value) {
    let record = {};
    if (button.btnPosition === buttonPositionType.listTop.value || button.btnPosition === buttonPositionType.listMore.value) {
      record.selectedRowKeys = yield select(x => x.getIn(['advanceList', functionCode, 'selectedRowKeys']));
    } else {
      const result = yield select(x => x.getIn(['advanceList', functionCode, 'result']));
      const listData = result.dataSource;
      record.selectedRow = _.findLast(listData, {id: params.id});
    }
    yield put({type: 'system/visibleCustomPopEdit', payload: {key: 'GrantPromise', record: record, functionCode}});
  }
}
