/**
 * Created by 王友生 on 2018-06-12.
 */

import { takeEvery, put, fork, call, select } from 'redux-saga/effects';
import { fromJS, Map } from 'immutable';
import { post, url } from '../../service/api';
import { message } from 'antd';
import { buttonType, printerTableStyle } from '../../common/const';
import { getItemDefaultValue } from '../../utils';

export const watchFetchCustomReportConfig = function* () {
  yield takeEvery('customReports/initReport', initReport)
  yield takeEvery('customReports/initReportCharts', initReportCharts)
}

function* initReport(action) {
  const cache = yield select(x => x.getIn(['customReport', action.payload.functionCode]));
  if (!cache) {
    const data = yield call(fetchReportConfig, action);
    if (!data) {
      message.error('报表配置不完全！');
    }
  }
}
function* initReportCharts(action) {
  const cache = yield select(x => x.getIn(['customReport', action.payload.functionCode]));
  if (!cache) {
    const data = yield call(fetchReportChartsConfig, action);
    if (!data) {
      message.error('报表配置不完全！');
    }
  }
}

function* fetchReportConfig(action) {
  const { functionCode } = action.payload;
  const data = yield call(post, url.getCustomReportsConfig, { key: functionCode });
  if (!data || !data.result) {
    return;
  }
  const { content: contentString, id } = data.result;
  if (!contentString) {
    return
  }
  const content = JSON.parse(contentString);
  const seacherItem = fromJS(content['seacherItem']);
  const groupItem = content['groupItem'];
  const matrixItem = content['matrixItem'];
  let searchScenario = Map({});
  console.log('seacherItem', seacherItem);

  seacherItem &&
    seacherItem.map((item, index) => {
      searchScenario = searchScenario.update(item.get("dataIndex"), cache => Map({
        value: getItemDefaultValue(item.toJSON()),
        type: item.get("type"),
        wildcard: item.get("wildcard"),
      }));
      return true;
    });
  console.log('searchScenario', searchScenario);

  if (!content) {
    return;
  }

  const payload = {
    seacherItem,
    groupItem,
    matrixItem,
    functionCode,
    searchScenario,
    defaultSearchScenario: searchScenario,
  }
  yield put({
    type: 'customReports/fetchReportConfigSuccess',
    payload,
  })
  return payload;
}

function* fetchReportChartsConfig(action) {
  const { functionCode, searchParams } = action.payload;
  const data = yield call(post, url.getCustomReportsChartsConfig, { funCode: functionCode });
  if (!data || !data.result) {
    return;
  }
  const { reportCfg: contentString, id, reportName, reportType } = data.result;
  if (!contentString) {
    return
  }
  const content = JSON.parse(contentString);
  const seacherItem = fromJS(content['condtion']);
  let searchScenario = Map({});
  seacherItem &&
    seacherItem.map((item, index) => {
      searchScenario = searchScenario.update(item.get("dataIndex"), cache => Map({
        value: getItemDefaultValue(item.toJSON()),
        type: item.get("type"),
        wildcard: item.get("wildcard"),
      }));
      return true;
    });
  if (searchParams) {//兼容可视化传参
    for (let key in searchParams) {
      searchScenario = searchScenario.setIn([key, 'value'], searchParams[key]);
    }
  }
  if (!content) {
    return;
  }

  const payload = {
    seacherItem,
    functionCode,
    searchScenario,
    defaultSearchScenario: searchScenario,
    reportName,
    reportType,
    content
  }
  yield put({
    type: 'customReports/fetchReportConfigSuccess',
    payload,
  })

  if (Object.prototype.hasOwnProperty.call(content, "defaultQuery") ? content.defaultQuery : true) {
    yield call(fetchReportChartData, {
      payload: {
        apiUrl: '/sysReport/sysReportCommon',
        functionCode: functionCode,
        searchScenario,
        seacherItem
      },
    });
  }

  return payload;
}

export const watchFetchCustomReportData = function* () {
  yield takeEvery('customReports/fetchReportData', fetchReportData);
  yield takeEvery('customReports/fetchReportChartsData', fetchReportChartData);
}
// 报表数据
function* fetchReportData(action) {
  const { apiUrl, searchScenario, functionCode, xTitle, yTitle } = action.payload;
  const { result: data } = yield call(post, apiUrl, searchScenario.toJSON());
  let newChartData = [];

  if (data.chartData) {
    newChartData = Array.from(data.chartData, x => {
      const row = {};
      row[xTitle] = x.xData;
      row[yTitle] = x.yData;
      return row
    });
  }
  yield put({ type: 'report/fetchReportDataSuccess', payload: { functionCode, data, chartData: newChartData } })
}
// 图表数据
function* fetchReportChartData(action) {
  const { apiUrl, searchScenario, functionCode, seacherItem } = action.payload;
  let searchConfig = seacherItem && seacherItem.toJS();
  let condition = ''
  if (searchConfig) {
    let table = searchConfig[0].tableName
    condition = { [table]: searchScenario && searchScenario.toJS() || '' }
  }
  //报表参数修改
  const { result } = yield call(post, apiUrl, { funCode: functionCode, searchScenario: condition ? condition : {} });
  if (result && result.header && result.data) {
    let { header, data } = result
    let totalWidth = 0
    result.header = result.header.filter(column => column.hide !== '1')
    result.header.map(item => {
      item.width = item.width ? parseInt(item.width) : 100
      totalWidth += parseInt(item.width)
    })
    result.header.push({})
    yield put({
      type: 'customReports/fetchReportDataChartSuccess', payload: {
        functionCode, data, header, totalWidth, chartData: transitionArr(header, data)
      }
    })
  }
}
// 图表数据转换
function transitionArr(header, data) {
  let key = {}
  header.map(item => {
    key[item.dataIndex] = item.title
  })
  let arr = []
  data.map(item => {
    header.map((headerItem, idx) => {
      if (idx !== 0) {
        arr.push({
          [header[0].dataIndex]: item[header[0].dataIndex],
          row: key[headerItem.dataIndex],
          val: item[headerItem.dataIndex]
        })
      }
    })
  })
  return arr
}
