/**
 * Created by 王友生 on 2018-07-16.
 */

import { takeEvery, put, call, fork, select, takeLatest } from "redux-saga/effects"
import { delay } from "redux-saga"
import {
  functionCodeType,
  dashboardDefaultPageSizeConst,
  dashboardItemDisplayTypeConst,
  dealWithStatusConst,
  dealWithGradeConst
} from "../../common/const"
import { convertListToObject } from "../../utils"
import { post, url } from "../../service/api"
import { List, Map } from "immutable"

export const watchInitDashboardConfig = function* () {
  // yield takeEvery('dashboard/initDashboardConfig', initDashboardConfig);
  // yield call(initDashboardConfig);
  yield takeLatest("dashboard/initDashboardDealWithData", initDashboardDealWithData)
  yield takeLatest("dashboard/initDashboardDealWithData", initDashboardChart)
  yield takeLatest("dashboard/initDashboardDealWithData", initDashboardNoticeCount)
  yield takeLatest("dashboard/initDashboardDealWithData", initDashboardMailCount)
}

function* initDashboardConfig() {
  yield post("/sysCache/getCachePersonalWorkbench", { key: "1231" })
  const originalMenu = yield select(x => x.getIn(["system", "originalMenu"]))
  let sourceMenus = List([])
  const dashboardConfig = localStorage.getItem("dashboardConfig")
  let dashboardItems = []
  if (dashboardConfig) {
    dashboardItems = JSON.parse(dashboardConfig)["dashboardItems"]
  }
  Object.keys(originalMenu).map(key => {
    const menu = originalMenu[key]
    if (
      menu &&
      (menu.funType === functionCodeType.list.value ||
        menu.funType === functionCodeType.advanceList.value ||
        menu.funType === functionCodeType.report.value) &&
      (!menu.items || menu.items.length === 0)
    ) {
      sourceMenus = sourceMenus.push({
        ...menu,
        key: menu.funCode,
        title: menu.menuName
      })
    }
  })
  const dashboardItemConfig = convertListToObject(dashboardItems, "key")
  yield put({
    type: "dashboard/initDashboardConfigSuccess",
    payload: {
      configurableMenus: Map({ sourceMenus, leftPartMenus: List([]), rightPartMenus: List([]) }),
      dashboardItems,
      dashboardItemConfig
    }
  })
}

export const watchSaveDashboardConfig = function* () {
  yield takeEvery("dashboard/saveDashboardConfig", saveDashboardConfig)
}

function* saveDashboardConfig(action) {
  const { configurableMenus } = action.payload
  // localStorage.setItem('dashboardConfig', JSON.stringify({ dashboardItems }));
  // yield put({ type: 'dashboard/changeDashboardItem', payload: { key: 'configModalVisible', value: false } });
  const leftDashboardItems = configurableMenus.get("leftPartMenus").toJSON()
  const rightDashboardItems = configurableMenus.get("rightPartMenus").toJSON()
  const dashboardItemConfig = { ...convertListToObject(leftDashboardItems, "key"), ...convertListToObject(rightDashboardItems, "key") }
  yield put({
    type: "dashboard/saveDashboardConfigSuccess",
    payload: { dashboardItemConfig, leftDashboardItems, rightDashboardItems }
  })

  yield fork(initDashboardItemsConfig, { payload: { dashboardItems: leftDashboardItems } })
  yield fork(initDashboardItemsConfig, { payload: { dashboardItems: rightDashboardItems } })
}

function* initDashboardItemsConfig(action) {
  const { dashboardItems } = action.payload
  // yield fork(put, { type: 'list/fetchListConfigByFunctionCode', payload: item.funCode, initFromDashboard: true });
  for (var i = 0; i < dashboardItems.length; i++) {
    const item = dashboardItems[i]
    if (item.configed) {
      yield fork(initDashboardItemData, item)
    } else if (item.funType === functionCodeType.list.value) {
      yield put({ type: "list/fetchListConfigByFunctionCode", payload: item.funCode, initFromDashboard: true })
    } else if (item.funType === functionCodeType.advanceList.value) {
      yield put({
        type: "advanceList/initAdvanceList",
        payload: { functionCode: item.funCode, initFromDashboard: true }
      })
    } else if (item.funType === functionCodeType.report.value) {
      yield put({
        type: "report/initReport",
        payload: { functionCode: item.funCode, initFromDashboard: true }
      })
    }
  }
}

export const watchFetchConfigForDashboard = function* () {
  yield takeEvery("dashboard/fetchConfigForDashboard", initDashboardItemConfig)
  yield takeEvery("dashboard/saveDashboardItemConfig", saveDashboardItemConfig)
  yield takeLatest("dashboard/fetchConfigForDashboard", putDelaySaveConfigForDashboard)
  yield takeLatest("dashboard/saveDashboardItemConfig", putDelaySaveConfigForDashboard)
  yield takeLatest("dashboard/saveDashboardConfig", putDelaySaveConfigForDashboard)
  yield takeLatest("dashboard/delaySaveConfigForDashboard", delaySaveConfigForDashboard)
}

function* putDelaySaveConfigForDashboard() {
  yield put({ type: "dashboard/delaySaveConfigForDashboard" })
}

function* delaySaveConfigForDashboard() {
  yield delay(3000)
  const dashboard = yield select(x => x.get("dashboard"))
  const dashboardConfig = {
    dashboardItemConfig: dashboard.get("dashboardItemConfig"),
    leftDashboardItems: dashboard.get("leftDashboardItems"),
    rightDashboardItems: dashboard.get("rightDashboardItems")
  }
  const customConfig = yield select(x => x.getIn(["system", "customConfig"]))
  const id = dashboard.get("id")
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  if (!id) {
    yield post(url.saveDashboardConfig, {
      userId: basicInfo.id,
      pwbType: "001",
      pwbContent: JSON.stringify({ ...customConfig, dashboardConfig })
    })
  } else {
    yield post(url.modDashboardConfig, {
      id,
      userId: basicInfo.id,
      pwbType: "001",
      pwbContent: JSON.stringify({ ...customConfig, dashboardConfig })
    })
  }
}

function* initDashboardItemConfig(action) {
  const { functionCode } = action.payload
  const dashboardItemConfig = yield select(x => x.getIn(["dashboard", "dashboardItemConfig"]))
  const item = dashboardItemConfig[functionCode]
  item.configed = true
  if (item.funType === functionCodeType.list.value) {
    const cache = yield select(x => x.getIn(["list", functionCode]))
    const listTopButtons = cache.get("listTopButtons")
    const tableConfig = cache.getIn(["config", "tableConfig"])
    const originalConfig = []
    const allColumnsConfig = []
    const dataIndexArray = []
    for (var i = 0; i < tableConfig.length - 1; i++) {
      const newCellConfig = { ...tableConfig[i] }
      if (newCellConfig.dataIndex) {
        delete newCellConfig["render"]
        delete newCellConfig["width"]
        originalConfig.push(newCellConfig)
        dataIndexArray.push(newCellConfig.dataIndex)
        allColumnsConfig.push(newCellConfig)
      }
    }
    item.itemColumns = item.itemColumns || dataIndexArray.toString()
    item.originalConfig = originalConfig
    item.allColumnsConfig = allColumnsConfig
    item.itemPageSize = dashboardDefaultPageSizeConst
    item.style = item.style || dashboardItemDisplayTypeConst.table.value
    const searchButton = listTopButtons && listTopButtons.length > 0 ? listTopButtons[0] : {}
    item.searchUrl = url.getListPaging
    // item.searchUrl = searchButton.btnUrl ? searchButton.btnUrl : url.getListPaging;
    item.detailFuncCode = functionCode
  } else if (item.funType === functionCodeType.advanceList.value) {
    const cache = yield select(x => x.getIn(["advanceList", item.funCode]))
    const searchButton = cache.get("searchButton")
    const tableConfig = cache.get("fieldsConfig")
    const basicInfo = cache.get("basicInfo")
    const originalConfig = []
    const dataIndexArray = []
    const allColumnsConfig = []
    //2021-11-19 11:14:58 取消三条限制
    for (let i = 0; i < tableConfig.length; i++) {
      const newCellConfig = { ...tableConfig[i] }
      delete newCellConfig["render"]
      delete newCellConfig["width"]
      originalConfig.push(newCellConfig)
      dataIndexArray.push(newCellConfig.dataIndex)
      allColumnsConfig.push(newCellConfig)
    }
    item.originalConfig = originalConfig
    item.itemColumns = item.itemColumns || dataIndexArray.toString()
    item.originalConfig = originalConfig
    item.allColumnsConfig = allColumnsConfig
    item.itemPageSize = dashboardDefaultPageSizeConst
    item.style = item.style || dashboardItemDisplayTypeConst.table.value
    item.searchUrl = searchButton.btnUrl
    item.detailFuncCode = basicInfo.funcCode
  } else if (item.funType === functionCodeType.report.value) {
    const basicInfo = yield select(x => x.getIn(["report", functionCode, "basicInfo"]))
    item.searchUrl = basicInfo["apiUrl"]
    item.originalConfig = { ...basicInfo }
    item.chartHeight = basicInfo.chartHeight
  }
  yield fork(initDashboardItemData, item)
}

export const initDashboardItemData = function* (dashboardItem) {
  if (dashboardItem.funType === functionCodeType.list.value) {
    let queryView;
    if(dashboardItem.queryView){
      if(dashboardItem.queryView.paramKey){
        let value = {}
        value[dashboardItem.queryView.paramKey] = dashboardItem.queryView.paramValue
        queryView = JSON.stringify([{id:dashboardItem.queryView.id,value}])
      }else{
        queryView = JSON.stringify([{id:dashboardItem.queryView.id}])
      }
    }
    const { result } = yield call(
      post,
      dashboardItem.searchUrl,
      {
        funCode: dashboardItem.funCode,
        queryViewId: queryView,
        searchScenario: dashboardItem.searchScenario,
        pagination: {
          pageNo: dashboardItem.dataPageNo ? dashboardItem.dataPageNo : 1,
          pageSize: dashboardItem.itemPageSize
        }
      },
      dashboardItem.funCode
    )
    dashboardItem.dataSource = result["data1"]
    dashboardItem.dataPageNo = result["pageNo"]
    dashboardItem.dataTotal = result["total"]
  } else if (dashboardItem.funType === functionCodeType.advanceList.value) {
    let params = {
      pagination: {
        pageNo: dashboardItem.dataPageNo ? dashboardItem.dataPageNo : 1,
        pageSize: dashboardItem.itemPageSize
      }
    }
    if (dashboardItem.searchUrl) {
      const { result: data } = yield call(post, dashboardItem.searchUrl, params, dashboardItem.funCode)
      dashboardItem.dataSource = data.dataSource
      dashboardItem.dataPageNo = data.pagination && data.pagination["pageNo"] ? data.pagination["pageNo"] : params.pagination.pageNo
      dashboardItem.dataTotal = data["total"]
    }
  } else if (dashboardItem.funType === functionCodeType.report.value) {
    const { result: data } = yield call(post, dashboardItem.searchUrl, {}, dashboardItem.funCode)
    let newChartData = []
    if (data.chartData) {
      newChartData = Array.from(data.chartData, x => {
        const row = {}
        row[dashboardItem.originalConfig.xTitle] = x.xData
        row[dashboardItem.originalConfig.yTitle] = x.yData
        return row
      })
    }
    dashboardItem.dataSource = newChartData
  }
  yield put({ type: "dashboard/rerenderDashboard" })
}

function* saveDashboardItemConfig(action) {
  const { config, originalConfig } = action.payload
  if (config.itemPageSize) {
    originalConfig.itemPageSize = config.itemPageSize
  }
  if (config.itemTitle) {
    originalConfig.itemTitle = config.itemTitle
  }
  if (config.itemColumns) {
    originalConfig.itemColumns = config.itemColumns
  }
  if (config.style) {
    originalConfig.style = config.style
  }
  const newOriginalConfig = []
  const cardColumnsConfig = {}
  const listColumnsConfig = {}
  if (originalConfig.allColumnsConfig) {
    originalConfig.allColumnsConfig.map(column => {
      if ((config.itemColumns || originalConfig.itemColumns).indexOf(column.dataIndex) >= 0) {
        newOriginalConfig.push(column)
      }
      if (config.cardColumns) {
        if (column.dataIndex === config.cardColumns.title) {
          cardColumnsConfig.titleConfig = column
        }
        if (column.dataIndex === config.cardColumns.tag) {
          cardColumnsConfig.tagConfig = column
        }
        if (column.dataIndex === config.cardColumns.content) {
          cardColumnsConfig.contentConfig = column
        }
        if (column.dataIndex === config.cardColumns.desc) {
          cardColumnsConfig.descConfig = column
        }
        if (column.dataIndex === config.listColumns.title) {
          listColumnsConfig.titleConfig = column
        }
        if (column.dataIndex === config.listColumns.desc) {
          listColumnsConfig.descConfig = column
        }
      }
      return true
    })
    originalConfig.cardColumns = config.cardColumns ? config.cardColumns : {}
    originalConfig.listColumns = config.listColumns ? config.listColumns : {}
    originalConfig.cardColumnsConfig = cardColumnsConfig
    originalConfig.listColumnsConfig = listColumnsConfig
    originalConfig.originalConfig = newOriginalConfig
  }
  originalConfig.chartHeight = config.chartHeight
  if (config.itemQueryView) {
    originalConfig.queryView = config.itemQueryView
  } else if (config.itemQueryView === null) {
    //originalConfig.queryView = {};
    delete originalConfig.queryView
  }
  if (config.itemPageNo) {
    originalConfig.dataPageNo = config.itemPageNo
  }
  yield initDashboardItemData(originalConfig)
}

export const watchDashboardActions = function* () {
  yield takeEvery("dashboard/goToDashboardItemDetail", goToDashboardItemDetail)
  yield takeEvery("dashboard/goToDashboardItemMore", goToDashboardItemMore)
}

function* goToDashboardItemDetail(action) {
  const { dashboardItem, itemData } = action.payload
  if (itemData.id && dashboardItem.detailFuncCode) {
    yield put({
      type: "system/jumpToDetailPage",
      payload: { functionCode: dashboardItem.detailFuncCode, id: itemData.id }
    })
  }
}

function* goToDashboardItemMore(action) {
  const { dashboardItem } = action.payload
  yield put({ type: "system/addTab", payload: dashboardItem })
}

function* initDashboardDealWithData() {
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const { result } = yield post(url.getSysPendingItemsListPaging, {
    itemStatus: dealWithStatusConst.pedding.value,
    receiveUser: basicInfo.id,
    pagination: { pageNo: 0, pageSize: 5 }
  })
  yield put({ type: "dashboard/fetchDealWithSuccess", payload: { result } })
}

function* initDashboardChart() {
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const { result } = yield post(url.getSysPendingItemGradeCount, { receiveUser: basicInfo.id })
  const chartData = []
  let totalCount = 0
  result.map(item => {
    if (item["item_grade"] === dealWithGradeConst.normal.value) {
      chartData.push({ type: dealWithGradeConst.normal.label, percent: item["total"] })
      totalCount = totalCount + Number(item["total"])
    } else if (item["item_grade"] === dealWithGradeConst.important.value) {
      chartData.push({ type: dealWithGradeConst.important.label, percent: item["total"] })
      totalCount = totalCount + Number(item["total"])
    } else if (item["item_grade"] === dealWithGradeConst.critical.value) {
      chartData.push({ type: dealWithGradeConst.critical.label, percent: item["total"] })
      totalCount = totalCount + Number(item["total"])
    }
    return true
  })
  yield put({
    type: "dashboard/fetchDealWithChartSuccess",
    payload: {
      chartData,
      totalCount
    }
  })
}

function* initDashboardNoticeCount() {
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const { result } = yield post(url.getSysNoticeList, {
    receiveUser: basicInfo.id,
    pagination: { pageNo: 0, pageSize: 5 }
  })
  yield put({ type: "dashboard/fetchNoticeSuccess", payload: { result } })
}

function* initDashboardMailCount() {
  const disableFunction = localStorage.getItem("disableFunction") ? JSON.parse(localStorage.getItem("disableFunction")) : ""
  if (disableFunction && disableFunction.disableMail === "1") {
    return
  }
  const { result } = yield post(url.getSysPendingMailCount, {})
  yield put({ type: "dashboard/fetchMailSuccess", payload: { result } })
}
