import { takeEvery, put, call, select, fork, takeLatest } from "redux-saga/effects"
import { delay } from "redux-saga"
import { post, url, download, downloadFromServer, requestPrintServer, ajaxCors } from "../../service/api"
import { Map, List, fromJS } from "immutable"
import moment from "moment"

import {
  groupType,
  tableTypeSourceKey,
  sysFormulaEditTypeConst,
  buttonType,
  buttonPositionType,
  formulaType,
  operationType,
  functionType,
  tableNickLabel,
  formChildSelectedRowKeysConst,
  editPagePositionConst,
  classButton,
  detailDisplayTypeConst,
  wfConditionConst
} from "../../common/const"

import { initListPage } from "../list/sagas"
import { message } from "antd"
import _ from "lodash"
import { getItemDefaultValue, validateItemPermission, resetReferenceAndDictionary, ifTrue } from "../../utils"
import math from "mathjs"
import { checkRecordInWorkFlow } from "../workflow/sagas"
import tool from "../../utils/commonCalculation"

math.config({
  number: "BigNumber"
})
const mathParse = math.parser()

export const watchFetchFormConfigByFunctionCode = function* () {
  yield takeLatest("form/fetchFormConfigByFunctionCode", fetchFormConfigByFunctionCode)
}
let cacheChange = [];//计算公式缓存
function* fetchFormConfigByFunctionCode(action) {
  // if (Number(window.supportFormCache)) {//一个不被认可的属性
  try {

    cacheChange = []
    let functionCode = action.payload.functionCode;
    const cache = yield select(x => x.get("form").get(functionCode));
    if (cache && cache.get("config")) {
      if (cache.getIn(["config", "originalFormConfig"])) {
        yield put({ type: "form/setFormConfig", payload: { functionCode, formConfig: cache.getIn(["config", "originalFormConfig"]).toJS() } })
      }
      yield call(setFormData, action)
      return
    }
    // }
    const { result } = yield call(post, url.getSysFormConfig, { key: action.payload.functionCode }, action.payload.functionCode)
    if (!result || !result.content) {
      return
    }
    const originalMenu = yield select(x => x.get("system").get("originalMenu"))
    const permissions = yield select(x => x.get("system").get("permissions"))
    let configJSON = undefined;
    try {
      configJSON = JSON.parse(result.content)
    } catch (error) {
      message.error('配置解析出错,请检查表单配置')
    }
    if (!configJSON) {
      return
    }
    const relationship = configJSON["tables"]
    const config = configJSON["formConfig"]
    const referenceConfig = configJSON["referenceConfig"]
    const referenceList = configJSON["referenceList"] || []
    const formulaListJSON = configJSON["formulaList"]
    const shortFormConfig = configJSON["formShortFormConfig"]
    const formExtra = configJSON["formExtra"]
    const formulaList = (configJSON["formulaList"] && fromJS(configJSON["formulaList"])) || List([])
    const formulaTriggerMapping = (configJSON["formulaTriggerMapping"] && fromJS(configJSON["formulaTriggerMapping"])) || Map({})
    let tabs = []
    let mainTableName = "";
    let tabsM = {};
    const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
    relationship.map(table => {
      if (table.tableType === tableTypeSourceKey.mainTableKey) {//主表
        mainTableName = table.tableName
        tabsM = table
      }
      if (table.relFun) {//关联功能
        const menu = originalMenu[table.relFun]
        if (menu) {
          tabs.push({
            ...menu,
            ...table,//位置调换tab sortId与menu sortId key重复导致tab排序变成菜单排序
            mainTableName
          })
        }
      }
    })
    //排序
    tabs = _.sortBy(tabs, (o) => {
      return o.sortId ? Number(o.sortId) : 100
    });
    let titleItem = undefined
    let detailTopButtons = []
    let detailMoreButtons = []
    let detailRelationFormButtons = []
    let detailPopupRelationFormButtons = []
    let detailRelationSearchButtons = []
    let detailXuandanButtons = []
    let detailToolButtons = []
    let childTableButton = {}
    let childTableRecordButton = {}
    let detailPrintButton = undefined
    let detailAuditingButton = undefined
    let detailAbandonButton = undefined
    let detailEditButton = undefined
    let detailDeleteButton = undefined
    let formAddButton = undefined
    let detailAddButton = undefined
    let formSaveButton = undefined
    let formResetButton = undefined
    let formButtons = []
    const allotDetailButtons = (button, list) => {
      if (button.btnType === buttonType.print.value) {
        detailPrintButton = button
      } else if (!detailAuditingButton && button.btnType === buttonType.auditing.value) {
        detailAuditingButton = button
      } else if (!detailAbandonButton && button.btnType === buttonType.abandon.value) {
        detailAbandonButton = button
      } else if (!detailEditButton && button.btnType === buttonType.edit.value) {
        detailEditButton = button
      } else if (!detailDeleteButton && button.btnType === buttonType.delete.value) {
        detailDeleteButton = button
      } else if (button.btnGroup === classButton.more.value) {
        detailMoreButtons.push(button)
      } else if (
        button.btnType === buttonType.addRelationForm.value ||
        button.btnGroup === classButton.pushRelationForm.value ||
        button.btnType === buttonType.addPopupRelationForm.value
      ) {
        detailRelationFormButtons.push(button)
      } else if (button.btnType === buttonType.xuandan.value) {
        detailXuandanButtons.push(button)
      } else if (button.btnType === buttonType.relationSearch.value) {
        detailRelationSearchButtons.push(button)
      } else if (button.btnType === buttonType.tools.value) {
        detailToolButtons.push(button)
      } else if (!detailAddButton && button.btnType === buttonType.add.value) {
        detailAddButton = button
      } else {
        list.push(button)
      }
    }
    const allFormButtons = (button, list) => {
      if (button.btnType === buttonType.add.value) {
        formAddButton = button
      } else if (button.btnType === buttonType.save.value) {
        formSaveButton = button
      } else if (button.btnType === buttonType.reset.value) {
        formResetButton = button
      } else {
        list.push(button)
      }
    }
    result.buttons &&
      result.buttons.map(button => {
        if (button.btnPosition === buttonPositionType.formChildTable.value) {
          let formTableButtons = childTableButton[button.btnPositionName] || []
          formTableButtons.push(button)
          childTableButton[button.btnPositionName] = formTableButtons
        } else if (button.btnPosition === buttonPositionType.formChildTableRecord.value) {
          let formTableButtons = childTableRecordButton[button.btnPositionName] || []
          formTableButtons.push(button)
          childTableRecordButton[button.btnPositionName] = formTableButtons
        } else if (button.btnPosition === buttonPositionType.detailTop.value) {
          allotDetailButtons(button, detailTopButtons)
        } else if (button.btnPosition === buttonPositionType.detailMore.value) {
          allotDetailButtons(button, detailMoreButtons)
        } else if (button.btnPosition === buttonPositionType.formTop.value) {
          allFormButtons(button, formButtons)
        }
        return true
      })
    // formData
    let formData = Map({})
    const actualConfig = []
    for (var i = 0; i < config.length; i++) {
      const group = config[i]
      let newGroup = []
      if (group.groupType === groupType.part) { //主表
        const groupItem = []
        const children = group.children
        if (group.showTitle) {  // 块标题
          groupItem.push({ showTitle: group.showTitle })
        }
        for (var j = 0; j < group.children.length; j++) {
          //子级
          const item = group.children[j]
          if (!i && !j) {
            titleItem = item
          }
          //2021-10-21 16:08:10 反向清除关联
          if (item.referenceConditions && item.referenceConditions.length > 0) {
            try {
              let affects = []
              if (item.affectFields && item.affectFields.length > 0) {
                item.affectFields.map(affItem => {
                  affects.push(affItem.formField)
                })
              }
              const conditions = item.referenceConditions
              conditions.map(ct => {
                //影响
                children.map(cd => {
                  //子级
                  if (ct.formField[1] == cd.eventDataIndex) {
                    if (cd.reverseAffect && Array.isArray(cd.reverseAffect)) {
                      cd.reverseAffect.push([item.tableName, item.dataIndex], ...affects)
                    } else {
                      cd.reverseAffect = [[item.tableName, item.dataIndex], ...affects]
                    }
                  }
                })
              })
            } catch (e) {
              // console.log("item", item)
              message.error(`${item.title},影响字段为空`)
            }
          }
          formData = formData.update(item.tableName, part => {
            const itemData = part || Map({})
            return itemData.set(item.dataIndex, getItemDefaultValue(item))
          })
          item.forbidEdit = item.forbidEdit === "1" ? "1" : validateItemPermission(item.editPermission, basicInfo) ? "0" : "1"
          item.editDictionary = item.sourceKey && !validateItemPermission(item.dicPermission, basicInfo) ? "1" : 0
          if (validateItemPermission(item.showPermission, basicInfo)) {
            groupItem.push(item)
          }
        }
        actualConfig.push(groupItem)
      } else if (group.groupType === groupType.table) {//处理子表
        const children = group.left.concat(group.middle).concat(group.right)
        delete group.middle
        delete group.left
        delete group.right
        let totalWidth = 0
        let defaultObj = {}
        const newChildren = []
        let formulas = []//存储子表影响子表配置的配置
        formulaList.toJS().map(formula => {
          let result = formula['result']
          let tableName = result && result[0]
          if (formula.type == 'formConfig' && tableName == group.tableName) {
            formulas.push(formula)
          }
        })
        children.map(item => {
          defaultObj[item.dataIndex] = getItemDefaultValue(item)
          item.forbidEdit = item.forbidEdit === "1" ? "1" : validateItemPermission(item.editPermission, basicInfo) ? "0" : "1"
          item.editDictionary = item.sourceKey && !validateItemPermission(item.dicPermission, basicInfo) ? "1" : 0
          if (validateItemPermission(item.showPermission, basicInfo)) {
            if (item.hide !== "1") {
              totalWidth += item.width
            }
            newChildren.push(item)
          }
          return true
        })
        const recordButtons = childTableRecordButton[i + 1]
        if (recordButtons && recordButtons.length > 0) {
          totalWidth += recordButtons.length * 60
        }
        actualConfig.push({
          ...group,
          children: newChildren,
          totalWidth,
          defaultObj,
          buttons: childTableButton[i + 1],
          recordButtons,
          triggerReference: "1",
          formulas
        })
        // console.log('actualConfig', actualConfig);

        formData.set(group.tableName, List([]))
      }
    }
    // console.log('formData', formData);

    let allField = [];//全部字段
    actualConfig.map(item => {
      if (Array.isArray(item)) {
        item.map(itemChild => {
          if (itemChild.hide !== "1") {
            allField.push(itemChild)
          }
        })
      }
    })
    // console.log('allField', allField);
    yield put({
      type: "form/fetchFormConfigSuccess",
      payload: {
        functionCode: action.payload.functionCode,
        config: Map({
          formConfig: actualConfig,
          originalFormConfig: fromJS(actualConfig),
          formExtra: formExtra,
          allField
        }),
        tabs,
        tabsM,
        referenceConfig,
        relationship,
        titleItem,
        formButtons,
        detailTopButtons,
        detailMoreButtons,
        detailPrintButton,
        detailAuditingButton,
        detailAbandonButton,
        detailDeleteButton,
        detailEditButton,
        formAddButton,
        detailAddButton,
        detailRelationFormButtons,
        detailPopupRelationFormButtons,
        detailXuandanButtons,
        detailToolButtons,
        detailRelationSearchButtons,
        formSaveButton,
        formResetButton,
        buttons: result.buttons,
        formulaList,
        formulaTriggerMapping,
        mainTableName,
        shortFormConfig,
        formPrintTemplate: result.prints,
        formData: formData.set("id", "-1"),
        fieldMonitor: result.fieldMonitor
      }
    })
    yield call(setFormData, action)
  } catch (e) {
    console.error(335, e)
  }
}

function* setFormData(action) {
  if (!action.payload.id) {
    return
  }
  let cache = yield select(x => x.get("form").get(action.payload.functionCode))
  let result = cache.get("formDetail").get(action.payload.id)
  let originalFormData = cache.get("formData")
  let originalId = Map.isMap(originalFormData) ? originalFormData && originalFormData.get("id") : originalFormData && originalFormData.id;
  if (!originalFormData || originalId !== action.payload.id || action.payload.ynMenuForm === "1") {
    let attachments = cache.getIn(["formDetailAttachments", action.payload.id])
    if (!result || action.payload.ynMenuForm === "1") {
      result = yield call(fetchFormDetailData, action)
      const response = yield call(fetchFormDetailAttachments, action)
      attachments = response["result"]
    }
    yield put({
      type: "form/setFormData",
      payload: { formData: fromJS(result), ...action.payload, attachmentList: List(attachments) }
    })
  }
  if (!originalFormData && action.payload.id === "-1" && !action.payload.withoutDefault) {
    yield call(fetchFormDefault, action)
  }
}

function* resetFormData(action) {
  if (!action.payload.id) {
    return
  }
  if (action.payload.id === "-1") {
    yield call(fetchFormDefault, action)
    return
  }
  let cache = yield select(x => x.get("form").get(action.payload.functionCode))
  let result = cache.get("formDetail").get(action.payload.id)
  let attachments = cache.getIn(["formDetailAttachments", action.payload.id])
  if (!result) {
    result = yield call(fetchFormDetailData, action)
    const response = yield call(fetchFormDetailAttachments, action)
    attachments = response["result"]
  }
  yield put({
    type: "form/setFormData",
    payload: { formData: fromJS(result), ...action.payload, attachmentList: List(attachments) }
  })
}

function* fetchFormDefault(action) {
  let cache = yield select(x => x.get("form").get(action.payload.functionCode))
  if (action.payload.id === "-1") {
    const addButton = cache.get("formAddButton")
    if (addButton && addButton.btnUrl) {
      const formData = yield call(formCustomAction, {
        payload: {
          functionCode: action.payload.functionCode,
          btnUrl: addButton.btnUrl,
          editIndex: -1
        }
      })
      if (formData) {
        yield put({ type: "form/setFormData", payload: { functionCode: action.payload.functionCode, formData } })
        return formData
      }
    }
  }
}

export const watchRemoveTabSuccess = function* () {
  yield takeEvery("removeTabSuccess", removeTabSuccess)
}

function* removeTabSuccess(action) {
  const { functionCode, type } = action.payload
  if (type === "edit") {
    yield put({ type: "form/resetFormData", payload: { functionCode, id: "-1" } })
  }
}

export const watchFetchDetailData = function* () {
  yield takeEvery("form/fetchFormDetailData", initFormData)
}

function* initFormData(action) {
  yield fork(fetchFormDetailData, action)
  yield fork(fetchFormDetailAttachments, action)
  // yield fork(referenceTabs, action);
  // yield fork(fetchWorkFlowStatus, action);
  // yield fork(fetchInvoiceWorkflowConfig, action);
}

export const getFormDetailData = function* (action) {
  const { id, functionCode } = action.payload
  let detail = yield select(x => x.getIn(["form", functionCode, "formDetail", id]))
  if (!detail) {
    detail = yield fetchFormDetailData(action)
  }
  return detail
}

function* fetchFormDetailData(action) {
  let activeTab = yield select(x => x.getIn(["system", "activeTab"]))
  if (action.payload.id === "-1") {
    return
  }
  let { result } = yield call(
    post,
    url.fetchFormDetail,
    {
      funCode: action.payload.functionCode,
      id: action.payload.id
    },
    action.payload.functionCode
  )
  result.id = action.payload.id
  // 取显示字段
  try {
    if (activeTab.showTitle) {
      yield delay(200)
      let tabs = yield select(x => x.getIn(["system", "tabs"]))
      let showTitle = activeTab.showTitle.split(".")
      tabs.map((item, idx) => {
        if (item.cache && item.cache.id && item.cache.id == action.payload.id) {
          item.showTitleName = result[showTitle[0]][showTitle[1]]
          activeTab.showTitleName = result[showTitle[0]][showTitle[1]]
        }
      })
      yield put({ type: "system/changeTabs", payload: tabs })
      //2021-11-08 16:48:06 客户档案翻译id 导致tabs无法激活
      if (action.payload.showTitle) {
        yield put({ type: "system/tabChange", payload: activeTab })
      }
    }
  } catch (e) {
    console.log("取tab显示字段失败")
  }

  yield put({
    type: "form/FetchFormDetailDataSuccess",
    payload: {
      functionCode: action.payload.functionCode,
      id: action.payload.id,
      formDetail: result
    }
  })
  return result
}

function* fetchFormDetailAttachments(action) {
  if (action.payload.id === "-1") {
    return
  }
  let { result } = yield call(post, url.fetchAttachments, { fid: action.payload.id })
  yield put({
    type: "form/FetchFormDetailAttachmentsSuccess",
    payload: {
      functionCode: action.payload.functionCode,
      id: action.payload.id,
      formDetailAttachments: result
    }
  })
  return { result }
}

function* referenceTabs(action) {
  const { id, functionCode } = action.payload
  let cache = yield select(x => x.getIn(["form", functionCode]))
  if (cache && cache.get("tabs")) {
    const tabs = cache.get("tabs")
    for (var i = 0; i < tabs.length; i++) {
      const tab = tabs[i]
      if (tableTypeSourceKey.childTableKey === tab.tableType) {
        yield put({
          type: "list/fetchListDataByParentId",
          payload: tab.relFun,
          fetchByParentId: true,
          fromFunctionCode: functionCode,
          parentId: id
        })
      } else if (tableTypeSourceKey.relationTableKey === tab.tableType) {
        yield put({ type: "form/fetchFormDetailData", payload: { id, functionCode: tab.relFun } })
      }
    }
  }
}

function* fetchWorkFlowStatus(action) {
  const { result } = yield call(post, url.fetchWorkflowStatus, {
    funCode: action.payload.functionCode,
    invoice: action.payload.id
  })
  if (result.handleFlag !== "0") {
    // yield fork fetchInvoiceWorkFlowInfo()
  }
  // console.log(111111111111111111);
  // console.log(result);
  // yield put({
  //   type: 'form/FetchFormDetailDataSuccess',
  //   payload: {
  //     functionCode: action.payload.functionCode,
  //     id: action.payload.id,
  //     formDetail: result
  //   }
  // });
}

function* fetchInvoiceWorkflowConfig(action) {
  const { result } = yield call(post, url.fetchInvoiceWorkflowConfig, {
    funCode: action.payload.functionCode,
    invoice: action.payload.id
  })
  yield put({
    type: "form/fetchInvoiceWorkflowConfigSuccess",
    payload: {
      functionCode: action.payload.functionCode,
      id: action.payload.id,
      workflowConfig: result
    }
  })
  // console.log(result);
  // if (result.handleFlag !== '0') {
  // yield fork fetchInvoiceWorkFlowInfo()
  // }
}

export const watchSaveFormData = function* () {
  yield takeEvery("from/saveFormData", saveFormData)
  yield takeEvery("from/saveFormDataFromShortForm", saveFormDataFromShortForm)
  yield takeEvery("form/refreshFormFromShortForm", refreshFormFromShortForm)
}

function* refreshFormFromShortForm(aciton) {
  const { functionCode, shortFormResult: newRelationFormData } = aciton.payload
  if (!newRelationFormData) {
    return
  }
  const cache = yield select(x => x.getIn(["form", functionCode]))
  let newFormData = cache.get("formData")
  const relationship = cache.get("relationship")
  let mainTableName = cache.get("mainTableName")
  if (newRelationFormData) {
    const result = newRelationFormData
    relationship.map(table => {
      if (table.tableType !== tableTypeSourceKey.mainTableKey && result[table.tableName]) {
        newFormData = newFormData.set(table.tableName, fromJS(result[table.tableName]))
        //1.29回写选中行
        if (result[table.tableName + "_selectedRowKeys"]) {
          newFormData = newFormData.set(table.tableName + "_selectedRowKeys", fromJS(result[table.tableName + "_selectedRowKeys"]))
        }
        delete result[table.tableName]
      }
    })
    newFormData = newFormData.mergeIn([mainTableName], fromJS(result))
    yield put({ type: "form/setFormData", payload: { functionCode, formData: newFormData } })
    return
  }
}

function* saveFormDataFromShortForm(action) {
  const { id, functionCode, shortFormData, shortFormId } = action.payload
  const cache = yield select(x => x.getIn(["form", functionCode]))
  const attachments = cache.get("formAttachments")
  const shortFormConfig = cache.get("shortFormConfig") || []
  const currentShortFormConfig = _.find(shortFormConfig, { shortFormId })
  let formData = Map({})
  currentShortFormConfig &&
    currentShortFormConfig["scenario"].map(item => {
      if (item.fromOption) {
        const itemIndexInfo = item.fromOption.split(tableNickLabel)
        if (itemIndexInfo && itemIndexInfo.length > 1) {
          formData = formData.setIn(itemIndexInfo, shortFormData.get(item.dataIndex))
        }
      }
      return true
    })
  let params = formData.toJSON()
  const data = []
  Object.keys(params).map(key => {
    key !== "id" &&
      data.push({
        tableName: key,
        values: params[key]
      })
  })
  const requestParams = {
    funCode: functionCode,
    id,
    data,
    sysAttachments: attachments ? attachments.toJSON() : []
  }
  yield call(post, url.modFormData, requestParams, functionCode)
  yield put({ type: "form/fetchFormDetailData", payload: { id, functionCode } })
}

function* saveFormData(action) {
  // -1 代表创建
  const { id, functionCode } = action.payload
  const formExtra = yield select(x => x.getIn(["form", functionCode, "config", "formExtra"]))
  const formData = yield select(x => x.getIn(["form", functionCode, "formData"]))
  const attachments = yield select(x => x.getIn(["form", functionCode, "formAttachments"]))
  const mainTableName = yield select(x => x.getIn(["form", functionCode, "mainTableName"]))
  let mainTableId = formData.getIn([mainTableName, "id"])
  if (formExtra && formExtra.attachNotNull === 1 && (!attachments || attachments.size === 0)) {
    message.error("请上传附件")
    return false
  }
  const params = formData.toJSON()
  let newId = mainTableId || id
  const data = []
  Object.keys(params).map(key => {
    key !== "id" &&
      data.push({
        tableName: key,
        values: params[key]
      })
  })
  const requestParams = {
    funCode: functionCode,
    id: newId,
    data,
    sysAttachments: attachments ? attachments.toJSON() : []
  }
  const saveUrl = action.payload.saveUrl
  if (newId === "-1") {
    const data = yield call(post, saveUrl || url.addFormData, requestParams, functionCode)
    if (data && data.result) {
      newId = data.result.id
      let newbc = window.localStorage.getItem("newBack");
      if (newbc == "1") {
        window.localStorage.setItem("newPeoId", data.result.id);
      }
    }
  } else {
    yield call(post, saveUrl || url.modFormData, requestParams, functionCode)
  }
  return newId
}

export const watchDeleteFormData = function* () {
  yield takeEvery("form/deleteFormData", deleteFormData)
}

function* deleteFormData(action) {
  const { functionCode, id, deleteButton } = action.payload
  const cache = yield select(x => x.getIn(["form", functionCode]))
  const cacheDetail = cache.getIn(["formDetail", id])
  const mainTableName = cache.get("mainTableName")
  const data = yield call(
    post,
    deleteButton && deleteButton.btnUrl ? deleteButton.btnUrl : url.removeFormData,
    {
      funCode: functionCode,
      ...cacheDetail[mainTableName],
      id
    },
    functionCode
  )
}

export const watchGenerateRelationForm = function* () {
  yield takeEvery("form/generateRelationForm", generateRelationForm)
}

function* generateRelationForm(action) {
  const { targetFunctionCode, fromData, newRelationFormData, popup, width, height, attachments } = action.payload
  const systemReducer = yield select(x => x.get("system"))
  const originalMenu = systemReducer.get("originalMenu")

  try {
    yield put({
      type: "system/jumpToEditPage",
      payload: "-1",
      functionCode: targetFunctionCode,
      relation: true, //推单
      popup,
      withoutDefault: true,
      width,
      height
    })
    // 自定义推单
    if (originalMenu[targetFunctionCode] && originalMenu[targetFunctionCode].menuUrl) {
      yield put({ type: "form/setFormData", payload: { functionCode: targetFunctionCode, formData: newRelationFormData } })
      return
    }
    yield call(fetchFormConfigByFunctionCode, {
      payload: {
        functionCode: targetFunctionCode,
        id: "-1",
        withoutDefault: true
      }
    })
    // 获取list失败暂时解决方案
    yield put({ type: "list/fetchListConfigByFunctionCode", payload: targetFunctionCode })
    const cache = yield select(x => x.getIn(["form", targetFunctionCode]))
    if (!cache) {
      console.error("找不到该功能")
      // return
    }
    let formData = yield call(fetchFormDefault, { payload: { id: "-1", functionCode: targetFunctionCode } })
    let newFormData = formData || cache.get("formDetail").get("-1")
    const relationship = cache.get("relationship")
    let mainTableName = cache.get("mainTableName")
    // 来源数据
    if (newRelationFormData) {
      const result = newRelationFormData
      const filterTables = relationship.map(table => {
        if (table.tableType !== tableTypeSourceKey.mainTableKey && result[table.tableName]) {
          newFormData = newFormData.set(table.tableName, fromJS(result[table.tableName]))
          delete result[table.tableName]
        }
      })
      newFormData = newFormData.mergeIn([mainTableName], fromJS(result))
      newFormData = newFormData.set('relationship', true)//推单数据标记
      yield put({ type: "form/setFormData", payload: { functionCode: targetFunctionCode, formData: newFormData } })
      if (attachments && attachments.length > 0) {
        yield put({ type: "form/addFormAttachment", payload: { functionCode: targetFunctionCode, attachments } })
      }
      // yield call(fetchFormDefault, { payload: { id: '-1', functionCode: targetFunctionCode } });
      return
    }
    relationship.map(table => {
      const key = table["tableName"]
      if (fromData) {
        if (fromData[key]) {
          if (table.tableType === tableTypeSourceKey.mainTableKey) {
            newFormData = newFormData.mergeIn([key], fromJS(fromData[key]))
            newFormData = newFormData.setIn([key, table.primaryKey || "ID"], "")
          } else if (table.tableType === tableTypeSourceKey.relationTableKey) {
            newFormData = newFormData.mergeIn([key], fromJS(fromData[key]))
            if (table.primaryKey) {
              newFormData = newFormData.update(mainTableName, mainTable =>
                mainTable.set(table.primaryKey, fromData[key][table.relationField || "ID"])
              )
            }
          }
          // else if (table.tableType === tableTypeSourceKey.childTableKey) {
          // }
        }
      }
    })
    yield put({ type: "form/setFormData", payload: { functionCode: targetFunctionCode, formData: newFormData } })
    if (attachments && attachments.length > 0) {
      yield put({ type: "form/addFormAttachment", payload: { functionCode: targetFunctionCode, attachments } })
    }
  } catch (e) {
    console.log("701推单", e)
  }
}

export const watchFormButtonClick = function* () {
  yield takeEvery("form/formButtonClick", formButtonClick)
}

//按钮点击
function* formButtonClick(action) {
  const { functionCode, button, id, params, state, isDetail } = action.payload
  // 清空字典缓存
  if (functionCode == "0012023" && button.btnType == buttonType.custom.value) {
    yield put({ type: "dictionary/iniStateSaga" })
    yield put({ type: "reference/iniStateSaga" })
    resetReferenceAndDictionary()
  }
  //置顶
  const cache = yield select(x => x.getIn(["form", functionCode]))
  if (button.btnType === buttonType.up.value) {
    let formData
    if (state) {
      formData = state.getIn([functionCode, "formData"])
    } else {
      formData = cache.get("formData")
    }
    let formDataJson = formData.toJSON()
    for (let key in formDataJson) {
      let keySplit = key.split(formChildSelectedRowKeysConst) //拆分所有key

      if (keySplit.length > 1) {
        let arr = formDataJson[keySplit[0]]
        let selectedArr = []
        formDataJson[key].map(item => {
          arr[item].createTime = moment(Date.now()).format("YYYY-MM-DD HH:mm:ss")
          selectedArr.push(arr[item])
        })
        for (let i = 0; i < selectedArr.length; i++) {
          for (let n = arr.length - 1; n >= 0; n--) {
            if (arr[n].productId == selectedArr[i].productId) {
              arr.splice(n, 1)
            }
          }
        }
        formDataJson[keySplit[0]] = [...selectedArr, ...arr]
        formDataJson[key] = List([])
        yield put({ type: "form/setFormData", payload: { functionCode, formData: fromJS(formDataJson) } })
      }
    }
  }
  //新增
  if (button.btnType === buttonType.add.value) {
    yield put({ type: "form/resetFormData", payload: { functionCode } })
    if (button.btnPosition === buttonPositionType.formTop.value) {
      yield put({
        type: "system/changeActiveTabCache",
        payload: {
          id: "-1"
        }
      })
      if (button.btnUrl) {
        const formData = yield call(formCustomAction, {
          payload: {
            functionCode,
            btnUrl: button.btnUrl,
            editIndex: -1
          }
        })
        if (formData) {
          yield put({ type: "form/setFormData", payload: { functionCode, formData } })
        }
      }
    } else {
      let actionResult = true
      if (button.btnUrl) {
        actionResult = yield call(post, button.btnUrl, { funCode: functionCode }, functionCode, button.id)
      }
      if (actionResult) {
        yield put({
          type: "system/jumpToEditPage",
          payload: "-1"
        })
      }
    }
    //  修改
  } else if (button.btnType === buttonType.edit.value) {
    if (yield checkRecordInWorkFlow({ payload: { functionCode, recordId: id } })) {
      message.error("流程已提交，不能修改。")
      return
    }

    let actionResult = true
    if (button.btnUrl) {
      const cache = yield select(x => x.getIn(["form", functionCode]))
      const data = cache.getIn(["formDetail", id])
      const mainTableName = cache.get("mainTableName")
      actionResult = yield call(
        post,
        button.btnUrl,
        {
          funCode: functionCode,
          id,
          ...data[mainTableName]
        },
        functionCode,
        button.id
      )
    }
    let popup;
    if (button.popup) {
      popup = button.popup
    }
    if (actionResult) {
      yield put({
        type: "system/jumpToEditPage",
        payload: id,
        functionCode,
        popup: popup ? popup : ''
      })
    }
    if (window.saveBackList === "1") {
      const activeTab = yield select(x => x.getIn(["system", "activeTab"]))
      yield delay(200)
      if (window.detailDisplayType != "2") {
        yield put({ type: "system/tabRemove", payload: activeTab })
      }
    }
    //  强制修改
  } else if (button.btnType === buttonType.editForce.value) {
    let actionResult = true
    if (button.btnUrl) {
      const cache = yield select(x => x.getIn(["form", functionCode]))
      const data = cache.getIn(["formDetail", id])
      const mainTableName = cache.get("mainTableName")
      actionResult = yield call(
        post,
        button.btnUrl,
        {
          funCode: functionCode,
          id,
          ...data[mainTableName]
        },
        functionCode,
        button.id
      )
    }
    if (actionResult) {
      yield put({
        type: "system/jumpToEditPage",
        payload: id,
        functionCode
      })
    }
    //  删除
  } else if (button.btnType === buttonType.delete.value) {
    if (yield checkRecordInWorkFlow({ payload: { functionCode, recordId: id } })) {
      message.error("流程已提交，不能删除。")
      return
    }
    if (button.btnUrl) {
      const cache = yield select(x => x.getIn(["form", functionCode]))
      const data = cache.getIn(["formDetail", id])
      const mainTableName = cache.get("mainTableName")
      yield call(
        post,
        button.btnUrl,
        {
          funCode: functionCode,
          id,
          ...data[mainTableName]
        },
        functionCode,
        button.id
      )
      // yield fork(initListPage, { payload: action.payload.functionCode });
      // yield put({ type: 'list/changeSelectedId', payload: { selectedId: '', functionCode } });
      // yield put({ type: 'list/fetchListDataByFunctionCode', payload: { functionCode } });
    } else {
      yield call(deleteFormData, { payload: { functionCode, id } })
    }
    const activeTab = yield select(x => x.getIn(["system", "activeTab"]))
    if (activeTab.type !== "list") {
      yield put({
        type: "system/jumpToFunctionWidthScenario",
        payload: { functionCode: activeTab.fromFunctionCode || activeTab.functionCode }
      })
      yield delay(200)
      yield put({ type: "system/tabRemove", payload: activeTab })
    } else {
      yield fork(initListPage, { payload: action.payload.functionCode })
      yield put({ type: "list/changeSelectedId", payload: { selectedId: "", functionCode } })
    }
    yield put({ type: "list/removeDeletedSelectedId", payload: { functionCode, id } })
    if (window.saveBackListAutoRefresh === "1") {
      yield put({ type: "list/fetchListDataByFunctionCode", payload: { functionCode } })
    }
    //  保存
  } else if (button.btnType === buttonType.save.value) {
    let newId = id,
      flag = true
    if (!button.btnUrl) {
      newId = yield call(saveFormData, { payload: { functionCode, id, saveUrl: button.btnUrl, params } })
      if (newId === false) {
        return
      }
    } else {
      const savePayload = {
        payload: {
          id: id === "-1" ? "" : id,
          functionCode,
          btnUrl: button.btnUrl,
          editIndex: -1
        }
      }
      const formData = yield call(formCustomAction, savePayload)
      newId = savePayload.payload.id
    }
    // 流程
    if (id === "-1") {
      yield put({ type: "workflow/createWorkflow", payload: { functionCode, recordId: newId } })
    } else {
      yield put({ type: "workflow/checkWorkflow", payload: { functionCode, recordId: newId } })
    }

    const popEdit = yield select(x => x.getIn(["system", "popEdit"]))
    // 弹窗
    if (popEdit.functionCode) {
      //参照新增 从新弹出参照
      const callBackPop = yield select(x => x.getIn(["reference", "callBackPop"]))
      if (callBackPop && callBackPop.callBackFunCode == functionCode) {
        yield put({ type: "reference/visibleReferenceWithData", payload: callBackPop })
      }
      yield put({ type: "form/resetFormData", payload: { functionCode } });//清除保存的formData
      yield put({ type: "system/changePopEdit", payload: { functionCode: undefined, id: undefined } })
      yield put({ type: "system/refreshCurrentPageData", payload: {} })
    } else {
      const activeTab = yield select(x => x.getIn(["system", "activeTab"]))
      // 保存返回列表
      if (window.saveBackList === "1") {
        yield put({
          type: "system/jumpToFunctionWidthScenario",
          payload: { functionCode: activeTab.fromFunctionCode || activeTab.functionCode }
        })
        yield delay(200)
        if (window.saveBackListAutoRefresh === "1") {
          flag = false
          yield put({
            type: "list/listButtonClick",
            payload: {
              functionCode: activeTab.fromFunctionCode || activeTab.functionCode,
              button: {
                btnType: buttonType.search.value
              }
            }
          })
        }
        yield put({ type: "system/tabRemove", payload: activeTab, save: true })
      } else {
        yield put({ type: "system/tabRemove", payload: activeTab, save: true })
        //2021-10-26 13:43:41 修改后删除showTitleName导致多一项空白
        if (activeTab.showTitleName) {
          delete activeTab.showTitleName
          yield put({ type: "system/tabRemove", payload: activeTab, save: true })
        }
        yield delay(200)
        // 滑动模式不跳转详情
        if (window.detailDisplayType !== "2") {
          yield put({ type: "system/jumpToDetailPage", payload: { functionCode, id: newId } })
        }
      }
    }

    if (window.saveBackListAutoRefresh === "1" && flag) {
      yield put({ type: "list/fetchListDataByFunctionCode", payload: { functionCode } })
    }
    if (id === "-1" && window.saveBackListAutoRefresh === "1" && window.detailDisplayType === detailDisplayTypeConst.left.value) {
      yield put({ type: "list/changeSelectedId", payload: { functionCode, selectedId: newId } })
    }
    //  推单
  } else if (button.btnType === buttonType.addRelationForm.value || button.btnType === buttonType.addPopupRelationForm.value) {
    const payload = { targetFunctionCode: button.relationCode }
    if (button.btnUrl) {
      if (button.btnPosition === buttonPositionType.detailTop.value || button.btnPosition === buttonPositionType.detailMore.value) {
        const response = yield call(post, button.btnUrl, { id }, functionCode, button.id)
        payload["newRelationFormData"] = response.result
        payload["attachments"] = response.attachments
      } else if (button.btnPosition === buttonPositionType.formTop.value) {
        const response = yield call(formCustomAction, {
          payload: {
            id: id === "-1" ? "" : id,
            functionCode,
            btnUrl: button.btnUrl,
            editIndex: -1,
            notFormatter: true
          }
        })
        payload["newRelationFormData"] = response.result
      }
    }
    payload.popup =
      button.btnType === buttonType.addPopupRelationForm.value ? editPagePositionConst.popup.value : editPagePositionConst.tab.value
    payload.popup = button.popup ? button.popup : payload.popup
    yield put({
      type: "workflow/initWorkflow",
      payload: { functionCode: button.relationCode }
    })
    yield put({
      type: "form/generateRelationForm",
      payload
    })
    //  重置
  } else if (button.btnType === buttonType.reset.value) {
    yield resetFormData({ payload: { functionCode, id } })
    // yield put({ type: 'form/resetFormData', payload: { functionCode, id } });
    //  打印
  } else if (button.btnType === buttonType.print.value) {
    if (params && params.provider == 2) {
      requestPrintServer(params, id)
    } else {
      if (params && params.jsMethodName && window[params.jsMethodName]) {
        const detail = yield select(x => x.getIn(["form", functionCode, "formDetail", id]))
        const relationship = yield select(x => x.getIn(["form", functionCode, "relationship"]))
        let requestParams = {}
        relationship.map(table => {
          if (table.tableType === tableTypeSourceKey.mainTableKey) {
            requestParams = { ...detail[table.tableName], ...requestParams }
          } else if (table.tableType === tableTypeSourceKey.childTableKey) {
            if (detail[table.tableName]) {
              requestParams[table.tableName] = detail[table.tableName]
            }
          }
        })
        if (params.apiUrl) {
          const data = yield call(post, params.apiUrl, requestParams, functionCode, button.id)
          if (data) {
            requestParams = data.result
          }
        }
        let LODOP = window.getLodop()
        window[params.jsMethodName](LODOP, requestParams)
        LODOP.PREVIEW()
      } else {
        message.error("请配置模板！")
      }
    }
    //  弹窗
  } else if (button.btnType === buttonType.popForm.value) {
    const cache = yield select(x => x.getIn(["form", functionCode]))
    const shortFormConfig = cache.get("shortFormConfig") || []
    const currentShortFormConfigs = shortFormConfig.filter(x => button.relationCode === x.shortFormId)
    if (currentShortFormConfigs && currentShortFormConfigs.length > 0) {
      const currentShortFormConfig = currentShortFormConfigs[0]
      if (button.btnPosition === buttonPositionType.detailTop.value) {
        const formDetail = cache.getIn(["formDetail", id])
        if (currentShortFormConfig["scenario"]) {
          let shortFormData = Map({})
          currentShortFormConfig["scenario"].map(item => {
            item.type = item.originalType
            item.field = item.originalField
            item.disabled = item.origianlDisabled
            item.referenceKey = item.originalReferenceKey
            item.sourceKey = item.origianlSourceKey
            item.defaultValue = item.originalDefaultValue
            shortFormData = shortFormData.set(item.dataIndex, getItemDefaultValue(item))
            if (item.fromOption) {
              const itemIndexInfo = item.fromOption.split(tableNickLabel)
              if (itemIndexInfo && itemIndexInfo.length > 1) {
                shortFormData = shortFormData.set(item.dataIndex, formDetail[itemIndexInfo[0]][itemIndexInfo[1]])
              }
            }
            return true
          })
          yield put({
            type: "shortForm/visibleShortForm",
            payload: {
              data: shortFormData,
              config: currentShortFormConfig,
              width: button.popWidth,
              height: button.popHeight,
              fromId: id,
              fromFunctionCode: functionCode,
              refreshAction: "form/fetchFormDetailData",
              saveAction: "from/saveFormDataFromShortForm"
            }
          })
        }
      } else if (
        button.btnPosition === buttonPositionType.formTop.value ||
        button.btnPosition === buttonPositionType.formChildTable.value ||
        button.btnPosition === buttonPositionType.formChildTableRecord.value
      ) {
        const cache = yield select(x => x.getIn(["form", functionCode]))
        const relationship = cache.get("relationship")
        let formData = isDetail ? fromJS(cache.getIn(["formDetail", id])) : cache.get("formData")
        let extraParams = {}
        let mainTableName = ""
        let childTables = []
        let relationTables = []
        relationship.map(table => {
          if (table.tableType === tableTypeSourceKey.mainTableKey) {
            mainTableName = table.tableName
            extraParams = { ...formData.get(table.tableName).toJSON(), ...extraParams }
          } else if (table.tableType === tableTypeSourceKey.childTableKey) {
            childTables.push(table.tableName)
            if (formData.get(table.tableName)) {
              extraParams[table.tableName] = formData.get(table.tableName).toJSON()
              extraParams[table.tableName + formChildSelectedRowKeysConst] = formData.get(table.tableName + formChildSelectedRowKeysConst)
            }
          } else if (table.tableType === tableTypeSourceKey.relationTableKey) {
            relationTables.push(table.tableName)
            if (formData.get(table.tableName)) {
              extraParams[table.tableName] = formData.get(table.tableName).toJSON()
            }
          }
        })
        extraParams.rowSelectedKey = params ? params.selectedIndex : -1
        let shortFormData = Map({})
        currentShortFormConfig["scenario"].map(item => {
          item.type = item.originalType
          item.field = item.originalField
          item.referenceKey = item.originalReferenceKey
          item.sourceKey = item.origianlSourceKey
          item.disabled = item.origianlDisabled
          item.defaultValue = item.originalDefaultValue
          shortFormData = shortFormData.set(item.dataIndex, getItemDefaultValue(item))
          if (item.fromOption) {
            const itemIndexInfo = item.fromOption.split(tableNickLabel)
            if (itemIndexInfo && itemIndexInfo.length > 1) {
              shortFormData = shortFormData.set(item.dataIndex, formData.getIn([itemIndexInfo[0], itemIndexInfo[1]]))
            }
          }
          return true
        })
        extraParams = yield put({
          type: "shortForm/visibleShortForm",
          payload: {
            data: shortFormData,
            config: currentShortFormConfig,
            width: button.popWidth,
            height: button.popHeight,
            fromId: id,
            extraParams,
            fromFunctionCode: functionCode,
            refreshAction: "form/refreshFormFromShortForm"
          }
        })
      }
    }
    //  联查
  } else if (button.btnType === buttonType.relationSearch.value) {
    if (button.btnUrl) {
      let response = yield call(post, button.btnUrl, { id }, functionCode, button.id)
      if (response && response.result) {
        yield put({
          type: "system/jumpToFunctionWidthScenario",
          payload: { functionCode: button.relationCode, scenario: response.result }
        })
      }
    } else {
      yield put({ type: "system/jumpToFunctionWidthScenario", payload: { functionCode: button.relationCode } })
    }
    //  下载
  } else if (button.btnType === buttonType.download.value) {
    const activeTab = yield select(x => x.getIn(["system", "activeTab"]))
    const fileName = activeTab.menuName || activeTab.name
    if (button.btnUrl) {
      yield call(download, button.btnUrl, fileName)
    }
    //  自定义弹窗
  } else if (button.btnType === buttonType.customPopModal.value) {
    let record = {}
    const cache = yield select(x => x.getIn(["form", functionCode]))
    const data = cache.getIn(["formDetail", id])
    const mainTableName = cache.get("mainTableName")
    if (data && mainTableName) record = data[mainTableName]

    yield put({ type: "system/visibleCustomPopEdit", payload: { key: button.btnUrl, record: record, functionCode } })
    //  导出
  } else if (button.btnType === buttonType.export.value) {
    if (button.btnUrl) {
      yield call(downloadFromServer, button.btnUrl, { id, funCode: functionCode }, functionCode)
    }
    //  流程重置
  } else if (button.btnType === buttonType.recreateWorkflow.value) {
    yield put({ type: "workflow/recreateWorkflow", payload: { functionCode, recordId: id } })
    //自定义请求
  } else if (button.btnType === buttonType.customRequest.value) {
    let response = yield call(ajaxCors, button.btnUrl, { id, funCode: functionCode })
    response = response ? JSON.parse(response) : '';
    if(response && response.message){
      message.success(response.message)
    }
  } else if (button.btnType === buttonType.customRequestJump.value) {//暂时不用
    let urls = button.btnUrl.split(',')
    let response = yield call(post, urls[0], { id, funCode: functionCode })
    if (response) {
      window.open(urls[1] + `?aaa=666`)
    }
  } else {
    if (button.btnUrl) {
      if (
        button.btnPosition === buttonPositionType.formTop.value ||
        button.btnPosition === buttonPositionType.formChildTableRecord.value ||
        button.btnPosition === buttonPositionType.formChildTable.value
      ) {
        const formData = yield call(formCustomAction, {
          payload: {
            id,
            functionCode,
            btnUrl: button.btnUrl,
            editIndex: -1,
            rowSelectedKey: params ? params.selectedIndex : -1
          }
        })
        if (formData) {
          yield put({ type: "form/setFormData", payload: { functionCode, formData } })
        }
      } else {
        yield call(post, button.btnUrl, { id, funCode: functionCode }, functionCode, button.id)
        yield put({ type: "form/fetchFormDetailData", payload: { functionCode, id } })
      }
    }
  }
}

function* formCustomAction(action) {
  const { functionCode, btnUrl, editIndex, state, sysFormulaEditType, notFormatter, rowSelectedKey } = action.payload
  const cache = yield select(x => x.getIn(["form", functionCode]))
  const relationship = cache.get("relationship")
  let formData
  if (state) {
    formData = state.getIn([functionCode, "formData"])
  } else {
    formData = cache.get("formData")
  }
  let params = { sysFormulaIndex: editIndex, sysFormulaEditType }
  let mainTableName = ""
  let childTables = []
  let relationTables = []
  relationship.map(table => {
    if (table.tableType === tableTypeSourceKey.mainTableKey) {
      if (formData.get(table.tableName)) {
        mainTableName = table.tableName
        params = { ...formData.get(table.tableName).toJSON(), ...params }
      }
    } else if (table.tableType === tableTypeSourceKey.childTableKey) {
      childTables.push(table.tableName)
      if (formData.get(table.tableName)) {
        params[table.tableName] = formData.get(table.tableName).toJSON()
        params[table.tableName + formChildSelectedRowKeysConst] = formData.get(table.tableName + formChildSelectedRowKeysConst)
      }
    } else if (table.tableType === tableTypeSourceKey.relationTableKey) {
      relationTables.push(table.tableName)
      if (formData.get(table.tableName)) {
        params[table.tableName] = formData.get(table.tableName).toJSON()
      }
    }
  })
  params.rowSelectedKey = rowSelectedKey
  const data = yield call(post, btnUrl, params, functionCode)
  if (notFormatter) {
    return data
  }
  if (data && data.result) {
    const result = data.result
    childTables.map(tableName => {
      if (result[tableName]) {
        formData = formData.set(tableName, fromJS(result[tableName]))
        delete result[tableName]
      }
    })
    relationTables.map(tableName => {
      if (result[tableName]) {
        formData = formData.set(tableName, fromJS(result[tableName]))
        delete result[tableName]
      }
    })
    formData = formData.set(mainTableName, fromJS(result))
    // 回传id
    action.payload.id = result.id
    return formData
  }
}

export const watchChangeFormData = function* () {
  yield takeEvery("form/changeFormData", changeFormData)
}

function* changeFormData(action) {
  const { functionCode, item, value, referenceResult } = action.payload
  let state = yield select(x => x.get("form"))
  state = state.update(functionCode, cache =>
    cache.update("formData", formData => {
      // formData = formData.update(item.tableName, table => table.set(item.dataIndex, value))后置至清除关联之后
      try {
        if (item.affectFields && Object.keys(referenceResult).length > 0 || item.affectFields && !value) {//清空关联
          item.affectFields.map(affectitem => {
            if (formData.hasIn(affectitem.formField)) {
              formData = formData.setIn(affectitem.formField, referenceResult[affectitem.referenceField] || "")
            }
            return true
          })
        }
      } catch (e) {
        console.log('影响字段出错1312', e);
      }
      formData = formData.update(item.tableName, table => table.set(item.dataIndex, value))
      // 反向清除关联（暂时隐藏，影响有事件赋值，隐藏后，清空字段无影响，有问题再放开）
      // if (item.reverseAffect) {
      //   item.reverseAffect.map(affectitem => {
      //     if (formData.hasIn(affectitem)) {
      //       formData = formData.setIn(affectitem, "")
      //     }
      //     return true
      //   })
      // }
      return formData
    })
  )
  yield put({ type: "form/setForm", payload: state })
}

export const watchChangeTable = function* () {
  yield takeEvery("form/changeTable", changeTable)
}

function* changeTable(action) {
  const { item, functionCode, referenceResult, fromReference, newTriggerRef } = action.payload
  // item:子表配置,referenceResult:选项,
  // console.log(item, functionCode, referenceResult, fromReference, newTriggerRef)
  let state = yield select(x => x.get("form"))
  let startIndex = 0
  let addCount = 0
  let calcField = []
  if(item.addUrl&&Array.isArray(referenceResult)){
    referenceResult.map(item=>{
      item._firstAdd = "1";
    })
    let obg = {
      referenceField:"_firstAdd",
      formField:[item.tableName, "_firstAdd"]
    }
    item.affectFields.push(obg);
  }
  
  if (fromReference) {
    state = state.updateIn([functionCode, "formData", item.tableName], tableData => {
      tableData = tableData || List([])
      startIndex = tableData.size
      const newData = []
      _.forEach(referenceResult, (x, xIndex) => {
        // 循环选中项
        let value = { ...item.defaultObj }
        // 初始值
        item.affectFields &&
          item.affectFields.map(affectItem => {
            // 循环影响字段
            if (_.head(affectItem.formField) === item.tableName) {
              //影响字段表名等于子表名
              const tableField = _.last(affectItem.formField)
              //影响字段名(判断字段是否有默认值，如果有默认值，默认值没有该字段或者为空，取弹窗（或者面板功能）提交的列表数据)
              value[tableField] = item.defaultObj[tableField] && (item.defaultObj[tableField] != "" && item.defaultObj[tableField] != undefined && item.defaultObj[tableField] != null) ? item.defaultObj[tableField] : x[affectItem.referenceField];
              // value[tableField] = x[affectItem.referenceField];
              // 初始值
              xIndex === 0 && calcField.push(tableField)
              //计算字段
            }
            return true
          })
        newData.push(value)
        return value
      })

      // console.log(item.affectFields)
      // console.log(calcField)
      addCount = newData.length
      return tableData.concat(fromJS(newData))
    })
  } else {
    state = state.updateIn([functionCode, "formData", item.tableName], tableData => {
      tableData = tableData || List([])
      startIndex = tableData.size
      const newData = referenceResult
      if (referenceResult && referenceResult.length > 0) {
        calcField = Object.keys(referenceResult[0])
      }
      addCount = 1
      return tableData.concat(fromJS(newData))
    })
  }
  const formulaList = state.getIn([functionCode, "formulaList"])
  // console.log(formulaList.toJS())
  if (newTriggerRef) {
    for (let index = startIndex; index < startIndex + addCount; index++) {
      for (let calcFieldIndex = 0; calcFieldIndex < calcField.length; calcFieldIndex++) {
        // console.log(index, calcFieldIndex)
        state = yield call(
          calculateWithFormula,
          state,
          {
            tableName: item.tableName,
            dataIndex: calcField[calcFieldIndex]
          },
          functionCode,
          startIndex + addCount,
          false,
          sysFormulaEditTypeConst.add
        )
      }
    }
  }
  //2.1新增取值
  if (item.addUrl) {
    let { result } = yield call(post, item.addUrl, state.toJS()[functionCode]["formData"])
    result[item.tableName].map(item=>{
      delete item["_firstAdd"];
    })
    // newFormData = newFormData.mergeIn([mainTableName], fromJS(result));
    state = state.mergeIn([functionCode, "formData"], fromJS(result))
  }
  yield put({ type: "form/setForm", payload: state })
}

export const watchChangeFormTableData = function* () {
  yield takeEvery("form/changeFormTableData", changeFormTableData)
  yield takeEvery("form/changeSelectedRowKeys", changeSelectedRowKeys)
}

function* changeFormTableData(action) {
  const { value, item, functionCode, index, selectRow } = action.payload
  let state = yield select(x => x.get("form"))
  state = state.setIn([functionCode, "formData", item.tableName, index, item.dataIndex], value)
  try {
    if (item.affectFields) {
      const calcField = []
      state = state.updateIn([functionCode, "formData", item.tableName, index], tableData => {
        item.affectFields.map(affectItem => {
          if (_.head(affectItem.formField) === item.tableName) {
            const tableField = _.last(affectItem.formField)
            tableData = tableData.set(tableField, selectRow[affectItem.referenceField])
            calcField.push(tableField)
          }
          return true
        })
        return tableData
      })
      for (let calcFieldIndex = 0; calcFieldIndex < calcField.length; calcFieldIndex++) {
        state = yield call(
          calculateWithFormula,
          state,
          {
            tableName: item.tableName,
            dataIndex: calcField[calcFieldIndex]
          },
          functionCode,
          index,
          false,
          sysFormulaEditTypeConst.edit
        )
      }
    }
  } catch (e) {
    message.info(`${item.title}影响字段出错`)
  }
  // 子表排序
  // try {
  //   if (item.sortField) {
  //     // let listArr = state.getIn([functionCode, 'formData', item.tableName])
  //     state = state.updateIn([functionCode, "formData", item.tableName], tableData => {
  //       tableData = tableData.sortBy(
  //         (val, index, obj) => {
  //           return Number(val.get(item.dataIndex))
  //         },
  //         (a, b) => {
  //           if (a < b) {
  //             return -1
  //           }
  //           if (a > b) {
  //             return 1
  //           }
  //           if (a === b) {
  //             return 0
  //           }
  //         }
  //       )
  //       return tableData
  //     })
  //   }
  // } catch (e) {
  //   message.info(`${item.title}排序失败`)
  // }
  // console.log('changeFormTableData',);
  yield put({ type: "form/setForm", payload: state, fun: 'changeFormTableData' })
}

export const watchDeleteTable = function* () {
  yield takeEvery("form/deleteTable", deleteTable)
  yield takeEvery("form/deleteTableRecords", deleteTableRecords)
}

function* deleteTable(action) {
  console.log("删除表")
  const { functionCode, tableName, index } = action.payload
  let state = yield select(x => x.get("form"))
  const deleteTable = state.getIn([functionCode, "formData", tableName, index])
  state = state.deleteIn([functionCode, "formData", tableName, index])
  if (deleteTable) {
    const table = deleteTable.toJSON()
    const keys = Object.keys(table)
    for (let keyIndex = 0; keyIndex < keys.length; keyIndex++) {
      const key = keys[keyIndex]
      state = yield call(calculateWithFormula, state, { tableName, dataIndex: key }, functionCode, index == 0 ? 0 : index - 1, true)
    }
  }
  yield put({ type: "form/setForm", payload: state })
}

function* deleteTableRecords(action) {
  const { functionCode, tableName } = action.payload
  console.log("删除表记录")
  let state = yield select(x => x.get("form"))
  const deleteTable = state.getIn([functionCode, "formData", tableName, 0])
  state = state.setIn([functionCode, "formData", tableName], List([]))
  if (deleteTable) {
    const table = deleteTable.toJSON()
    const keys = Object.keys(table)
    for (let keyIndex = 0; keyIndex < keys.length; keyIndex++) {
      const key = keys[keyIndex]
      state = yield call(calculateWithFormula, state, { tableName, dataIndex: key }, functionCode, 0, true)
    }
  }
  yield put({ type: "form/setForm", payload: state })
}

export const watchCalculateWithFormula = function* () {
  yield takeLatest("form/changeFormData", delayExecFormulaByForm)
  yield takeEvery("form/pushOrderInitFormula", delayExecFormulaByForm)
  yield takeLatest("form/changeFormTableData", delayExecFormulaByTable)
}

function* delayExecFormulaByForm(action) {
  yield delay(200)
  const { functionCode, item, value } = action.payload
  if (value == null) {
    return
  }
  let state = yield select(x => x.get("form"))
  state = yield call(calculateWithFormula, state, item, functionCode)
  //console.log('计算公式form后', state.getIn([functionCode, "formData"]).toJS());
  yield put({ type: "form/setForm", payload: state })
}

function* delayExecFormulaByTable(action) {
  console.log("计算公式table", action)
  yield delay(200)
  const { value, item, functionCode, index } = action.payload
  let state = yield select(x => x.get("form"))
  state = yield call(calculateWithFormula, state, item, functionCode, index)
  yield put({ type: "form/setForm", payload: state, fun: 'delayExecFormulaByTable' })
}
function* calculateWithFormula(
  state,
  item,
  functionCode,
  childIndex,
  isDelete = false,
  sysFormulaEditType = sysFormulaEditTypeConst.edit,
  lastItems = []
) {
  // 计算公式引用下标
  const formulaTriggerMapping = state.getIn([functionCode, "formulaTriggerMapping"])
  if (!formulaTriggerMapping) {
    return state
  }
  const formulas = formulaTriggerMapping.getIn([item.tableName, item.dataIndex]) //计算公式下标
  if (!formulas || formulas.size <= 0) {
    return state
  }
  let formulaArr = []
  // 计算公式列表
  const formulaList = state.getIn([functionCode, "formulaList"])
  for (let formulaIndex = 0; formulaIndex < formulas.size; formulaIndex++) {
    let formData = state.getIn([functionCode, "formData"])//取最新数据
    let relationship = state.getIn([functionCode, "relationship"])//关系表
    let formConfig = yield select(x => x.getIn(["form", functionCode, "config", "formConfig"]))
    // 具体计算公式
    const formula = formulaList.get(formulas.get(formulaIndex))
    formulaArr.push(formula.toJS())
    if (formula.get("type") === formulaType.normal.value) {
      //普通
      let firstKeyPath = formula.get("first").toJSON()
      let secondKeyPath = formula.get("second").toJSON()
      let resultKeyPath = formula.get("result").toJSON()
      if (_.find(lastItems, { tableName: resultKeyPath[0], dataIndex: resultKeyPath[1] })) {
        continue
      }
      let newLastItems = [
        {
          tableName: firstKeyPath[0],
          dataIndex: firstKeyPath[1]
        },
        {
          tableName: secondKeyPath[0],
          dataIndex: secondKeyPath[1]
        }
      ]
      if (childIndex >= 0) {
        firstKeyPath.splice(1, 0, childIndex)
        secondKeyPath.splice(1, 0, childIndex)
        resultKeyPath.splice(1, 0, childIndex)
      }
      const operation = formula.get("operation")
      let first = formData.getIn(firstKeyPath)
      let second = formData.getIn(secondKeyPath)
      let result = undefined
      if (typeof first !== "number") {
        first = parseFloat(first)
      }
      if (typeof second !== "number") {
        second = parseFloat(second)
      }
      if ((first || first === 0) && (second || second === 0)) {
        if (operation === operationType["+"].value) {
          result = parseFloat(math.eval(`${first} + ${second}`).toString())
        } else if (operation === operationType["-"].value) {
          result = parseFloat(math.eval(`${first} - ${second}`).toString())
        } else if (operation === operationType["*"].value) {
          if (first === 0 || second === 0) {
            result = 0
          } else {
            result = parseFloat(math.eval(`${first} * ${second}`).toString())
          }
        } else if (operation === operationType["/"].value) {
          if (first === 0 || second === 0) {
            result = 0
          } else {
            result = parseFloat(math.eval(`${first} / ${second}`).toString())
            result = result.toFixed(window.numberFixedConst)
          }
        }
        state = state.setIn([functionCode, "formData", ...resultKeyPath], result)
        if (childIndex >= 0) {
          resultKeyPath.splice(1, 1)
        }
        state = yield call(
          calculateWithFormula,
          state,
          {
            tableName: resultKeyPath[0],
            dataIndex: resultKeyPath[1]
          },
          functionCode,
          childIndex,
          isDelete,
          sysFormulaEditType,
          newLastItems
        )
      }
    } else if (formula.get("type") === formulaType.expression.value) {
      // 代码注入
      console.log('formula', formula.toJS());
      let areaKeyPath = formula.get("area")
      areaKeyPath = '(function (){' + areaKeyPath + '})()'
      console.log('areaKeyPath', areaKeyPath);
      if (sysFormulaEditType == sysFormulaEditTypeConst.add) {
        childIndex -= 1
      }
      try {
        const cache = yield select(x => x.getIn(["form", functionCode]))
        const relationship = cache.get("relationship")
        let mainTableName = ""
        let childTables = []
        let params = { sysFormulaIndex: childIndex, sysFormulaEditType }
        relationship.map(table => {
          if (table.tableType === tableTypeSourceKey.mainTableKey) {
            mainTableName = table.tableName
            params = { ...formData.get(table.tableName).toJSON(), ...params }
          } else if (table.tableType === tableTypeSourceKey.childTableKey) {
            childTables.push(table.tableName)
            if (formData.get(table.tableName)) {
              params[table.tableName] = formData.get(table.tableName).toJSON()
            }
          }
          return true
        })
        params = _.cloneDeep(params)
        let T = tool;//工具类
        areaKeyPath = '(function (){' + areaKeyPath + '})()'
        console.log('params', params);
        eval(areaKeyPath);
        const result = params;
        childTables.map(tableName => {
          if (result[tableName]) {
            formData = formData.set(tableName, fromJS(result[tableName]))
            delete result[tableName]
          }
          return true
        })
        formData = formData.set(mainTableName, fromJS(result))
        state = state.setIn([functionCode, "formData"], formData)
      } catch (e) {
        console.log('err', e);
        message.info('无法解析的公式' + e.message)
        return
      }
    } else if (formula.get("type") === formulaType.inner.value) {
      //内置函数
      if (childIndex >= 0) {
        const funcName = formula.get("funcName")
        let firstKeyPath = formula.get("first").toJSON()
        let resultKeyPath = formula.get("result").toJSON()
        if (_.find(lastItems, { tableName: resultKeyPath[0], dataIndex: resultKeyPath[0] })) {
          continue
        }
        let newLastItems = [{ tableName: firstKeyPath[0], dataIndex: firstKeyPath[1] }]
        const fieldName = firstKeyPath.pop()
        const calcArray = formData.getIn(firstKeyPath)
        let result = 0
        calcArray.map(item => {
          let itemValue = item.get(fieldName)
          if (typeof itemValue !== "number") {
            itemValue = parseFloat(itemValue)
          }
          if (funcName === functionType.max.value) {
            result = Math.max(result, itemValue) || 0
          } else if (funcName === functionType.min.value) {
            result = Math.min(result, itemValue) || 0
          } else if (funcName === functionType.sum.value) {
            result += itemValue || 0
          }
          return true
        })
        result = result.toFixed(window.numberFixedConst)
        state = state.setIn([functionCode, "formData", ...resultKeyPath], result)
        state = yield call(
          calculateWithFormula,
          state,
          {
            tableName: resultKeyPath[0],
            dataIndex: resultKeyPath[1]
          },
          functionCode,
          -1,
          isDelete,
          sysFormulaEditType,
          newLastItems
        )
      }
      // else {
      // }
    } else if (formula.get("type") === formulaType.custom.value) {
      //自定义函数
      const funcName = formula.get("funcName")
      if (window[funcName] && typeof window[funcName] === "function") {
        try {
          const cache = yield select(x => x.getIn(["form", functionCode]))
          const relationship = cache.get("relationship")
          let mainTableName = ""
          let childTables = []
          let params = { sysFormulaIndex: childIndex, sysFormulaEditType }
          relationship.map(table => {
            if (table.tableType === tableTypeSourceKey.mainTableKey) {
              mainTableName = table.tableName
              params = { ...formData.get(table.tableName).toJSON(), ...params }
            } else if (table.tableType === tableTypeSourceKey.childTableKey) {
              childTables.push(table.tableName);
              if (formData.get(table.tableName)) {
                params[table.tableName] = formData.get(table.tableName).toJSON();
              }
            }
            return true
          })
          const result = window[funcName](params)
          childTables.map(tableName => {
            if (result[tableName]) {
              formData = formData.set(tableName, fromJS(result[tableName]));
              // 增加是否有当前表选择的已选字段，有的话将字段列表设置为[];
              if (formData.get(tableName + "_selectedRowKeys")) {
                formData = formData.set(tableName + "_selectedRowKeys", []);
              }
              delete result[tableName];
            }
            return true
          })
          formData = formData.set(mainTableName, fromJS(result))
          state = state.setIn([functionCode, "formData"], formData)
        } catch (e) {
          message.error(`计算公式${funcName}错误!`, e)
        }
      }
      // else {
      // }
    } else if (formula.get("type") === formulaType.api.value) {
      //api后台接口
      console.log('api后台接口这里走了吗?', formula.get("apiUrl"))
      const apiUrl = formula.get("apiUrl")
      if (apiUrl && !isDelete) {
        try {
          const formData = yield call(formCustomAction, {
            payload: {
              functionCode,
              btnUrl: apiUrl,
              editIndex: childIndex,
              sysFormulaEditType,
              state
            }
          })
          if (formData) {
            state = state.setIn([functionCode, "formData"], formData)
            let firstKeyPath = formula.get("calcFirst")
            let secondKeyPath = formula.get("calcSecond")
            let thirdKeyPath = formula.get("calcThird")
            let forthKeyPath = formula.get("calcForth")
            if (firstKeyPath && firstKeyPath.size > 0) {
              state = yield call(
                calculateWithFormula,
                state,
                {
                  tableName: firstKeyPath.get(0),
                  dataIndex: firstKeyPath.get(1)
                },
                functionCode,
                childIndex,
                isDelete,
                sysFormulaEditType,
                [item]
              )
            }
            if (secondKeyPath && secondKeyPath.size > 0) {
              state = yield call(
                calculateWithFormula,
                state,
                {
                  tableName: secondKeyPath.get(0),
                  dataIndex: secondKeyPath.get(1)
                },
                functionCode,
                childIndex
              )
            }
            if (thirdKeyPath && thirdKeyPath.size > 0) {
              state = yield call(
                calculateWithFormula,
                state,
                {
                  tableName: thirdKeyPath.get(0),
                  dataIndex: thirdKeyPath.get(1)
                },
                functionCode,
                childIndex,
                isDelete,
                sysFormulaEditType,
                [item]
              )
            }
            if (forthKeyPath && forthKeyPath.size > 0) {
              state = yield call(
                calculateWithFormula,
                state,
                {
                  tableName: forthKeyPath.get(0),
                  dataIndex: forthKeyPath.get(1)
                },
                functionCode,
                childIndex,
                isDelete,
                sysFormulaEditType,
                [item]
              )
            }
          }
        } catch (e) {
          console.log(1836, e)
        }
      }
    } else if (formula.get("type") === formulaType.formConfig.value) {
      //表单配置
      const condition = formula.get('condition');
      let conValue = formula.get('conValue');
      const result = formula.get('result');
      const newKeys = formula.get('newKeys').toJS();
      let tableType = relationship.filter(item => {
        return item.tableName == result.getIn([0])
      })[0].tableType;
      let value;
      if (tableType == '001') {//主表
        value = result && formData.getIn(result)
      } else if (tableType == '002') {//子表
        value = result && formData.getIn([result.getIn([0]), childIndex, result.getIn([1])])
      }
      if (List.isList(conValue)) {
        if (typeof conValue.toJS()[0] == 'string') {
          conValue = conValue.join(',')
        } else if (conValue.toJS()[0] && conValue.toJS()[0].label) {
          conValue = conValue.toJS()[0].label
        }
      }
      let conResult = wfConditionConst[condition].operation(value, conValue)
      let changeNum = 0;
      if (conResult) {//改变配置
        newKeys.map(key => {
          if (key.dataIndex[2] && key.dataIndex[2] == '002') {//影响子表
            // console.log('formula', formula.toJS(), formulas.get(formulaIndex));
            formConfig.map(block => {
              if (typeof block == 'object') {//子表
                if (block.tableName == key.dataIndex[0]) {//对应表名
                  if (tableType !== '002') {
                    block.children.map(item => {
                      if (item.dataIndex == key.dataIndex[1]) {//对应字段
                        // requiredSubTable,hideSubTable
                        if (Boolean(key.hide) != Boolean(Number(item.hide))) {
                          item.hide = key.hide
                          changeNum += 1
                        }
                        cacheChange.push(`${key.dataIndex[0]},${key.dataIndex[1]},hide,${key.hide},${key.dataIndex[2]}`)
                        if (Boolean(key.required) != Boolean(Number(item.required))) {
                          item.required = key.required
                          changeNum += 1
                        }
                        cacheChange.push(`${key.dataIndex[0]},${key.dataIndex[1]},required,${key.required},${key.dataIndex[2]}`)

                        if (Boolean(key.disabled) != Boolean(Number(item.disabled))) {
                          item.disabled = key.disabled
                          changeNum += 1
                        }
                        cacheChange.push(`${key.dataIndex[0]},${key.dataIndex[1]},disabled,${key.disabled},${key.dataIndex[2]}`)

                        if (Boolean(key.requiredSubTable) != Boolean(Number(block.requiredSubTable))) {
                          block.required = key.requiredSubTable
                          changeNum += 1
                        }
                        cacheChange.push(`${key.dataIndex[0]},requiredSubTable,${key.requiredSubTable},${key.dataIndex[2]}`)
                        if (Boolean(key.hideSubTable) != Boolean(Number(block.hideSubTable))) {
                          block.hideSubTable = key.hideSubTable
                          changeNum += 1
                        }
                        cacheChange.push(`${key.dataIndex[0]},hideSubTable,${key.hideSubTable},${key.dataIndex[2]}`)
                      }
                    })
                  }
                }
              }
            })
          } else {//主表
            formConfig.map(block => {
              if (Array.isArray(block)) {
                block.map(item => {
                  if (item.tableName == key.dataIndex[0] && item.dataIndex == key.dataIndex[1]) {
                    if (Boolean(key.hide) != Boolean(Number(item.hide))) {
                      item.hide = key.hide
                      changeNum += 1
                    }
                    cacheChange.push(`${key.dataIndex[0]},${key.dataIndex[1]},hide,${key.hide}`)
                    if (Boolean(key.required) != Boolean(Number(item.required))) {
                      item.required = key.required
                      changeNum += 1
                    }
                    cacheChange.push(`${key.dataIndex[0]},${key.dataIndex[1]},required,${key.required}`)
                    
                    if (Boolean(key.disabled) != Boolean(Number(item.disabled))) {
                      item.disabled = key.disabled
                      changeNum += 1
                    }
                    cacheChange.push(`${key.dataIndex[0]},${key.dataIndex[1]},disabled,${key.disabled}`)
                    console.log('item', item);
                  }
                })
              }
            })
          }
        })
      } else {
        if (tableType !== '002') {
          //移除缓存当前项
          newKeys.map(key => {
            let hide = cacheChange.indexOf(key.dataIndex[2] == '002' ? `${key.dataIndex[0]},${key.dataIndex[1]},hide,${key.hide},${key.dataIndex[2]}` : `${key.dataIndex[0]},${key.dataIndex[1]},hide,${key.hide}`)
            hide >= 0 && cacheChange.splice(hide, 1)
            let required = cacheChange.indexOf(key.dataIndex[2] == '002' ? `${key.dataIndex[0]},${key.dataIndex[1]},required,${key.required},${key.dataIndex[2]}` : `${key.dataIndex[0]},${key.dataIndex[1]},required,${key.required}`)
            required >= 0 && cacheChange.splice(required, 1)
            let hideSubTable = cacheChange.indexOf(`${key.dataIndex[0]},hideSubTable,${key.hideSubTable},${key.dataIndex[2]}`)
            hideSubTable >= 0 && cacheChange.splice(hideSubTable, 1)
            let requiredSubTable = cacheChange.indexOf(`${key.dataIndex[0]},requiredSubTable,${key.requiredSubTable},${key.dataIndex[2]}`)
            requiredSubTable >= 0 && cacheChange.splice(requiredSubTable, 1)
          })
          let trueChange = _.uniq(cacheChange)
          let originalFormConfig = state.getIn([functionCode, "config", "originalFormConfig"]).toJS()
          // 刷新原始数据
          // console.log('originalFormConfig', originalFormConfig);
          trueChange.map(change => {
            change = change.split(',')
            if (change[4] && change[4] == '002' || change[3] && change[3] == '002') {//子表
              originalFormConfig.map(block => {
                if (typeof block == 'object') {
                  if (block.tableName == change[0]) {
                    if (change[1] == 'hideSubTable') {
                      if (ifTrue(change[2]) != ifTrue(block.hideSubTable)) {
                        block.hideSubTable = ifTrue(change[2])
                      }
                    }
                    if (change[1] == 'requiredSubTable') {
                      if (ifTrue(change[2]) != ifTrue(block.requiredSubTable)) {
                        block.requiredSubTable = ifTrue(change[2])
                      }
                    }
                    block.children.map(item => {
                      if (item.dataIndex == change[1]) {
                        if (change[2] == 'hide') {
                          if (ifTrue(change[3]) != ifTrue(item.hide)) {
                            item.hide = ifTrue(change[3])
                          }
                        }
                        if (change[2] == 'required') {
                          if (ifTrue(change[3]) != ifTrue(item.required)) {
                            item.required = ifTrue(change[3])
                          }
                        }
                      }
                    })
                  }
                }
              })
            } else {
              originalFormConfig.map(block => {
                if (Array.isArray(block)) {
                  block.map(item => {
                    if (item.tableName == change[0] && item.dataIndex == change[1]) {
                      if (change[2] == 'hide') {
                        if (ifTrue(change[3]) != ifTrue(item.hide)) {
                          item.hide = ifTrue(change[3])
                        }
                      }
                      if (change[2] == 'required') {
                        if (ifTrue(change[3]) != ifTrue(item.required)) {
                          item.required = ifTrue(change[3]) 
                        }
                      }
                    }
                  })
                }
              })
            }
          })
          state = state.setIn([functionCode, "config", "formConfig"], _.cloneDeep(originalFormConfig))
        }
      }
      if (changeNum > 0) {
        state = state.setIn([functionCode, "config", "formConfig"], _.cloneDeep(formConfig))
      }
    }
  }
  return state
}

function* changeSelectedRowKeys(action) {
  console.log("changeSelectedRowKeys")
  let { functionCode, tableName, selectedRowKeys, isDetail, id } = action.payload
  let state = yield select(x => x.get("form"))
  if (isDetail) {
    state = state.update(functionCode, cache =>
      cache.updateIn(["formDetail", id], formData => {
        if (!Array.isArray(selectedRowKeys)) {
          selectedRowKeys = selectedRowKeys.toJS()
        }
        formData[tableName + formChildSelectedRowKeysConst] = selectedRowKeys
        return formData
      })
    )
  } else {
    state = state.update(functionCode, cache =>
      cache.update("formData", formData => {
        if (!Array.isArray(selectedRowKeys)) {
          selectedRowKeys = selectedRowKeys.toJS()
          console.log(selectedRowKeys)
        }
        formData = formData.set(tableName + formChildSelectedRowKeysConst, selectedRowKeys)
        return formData
      })
    )
  }
  yield put({ type: "form/setForm", payload: state })
}