import { Map } from 'immutable';
import { stat } from 'fs';

const initState = Map({});

export default (state = initState, action) => {
  switch (action.type) {
    case 'list/fetchListConfigSuccess':
      return state.update(action.payload.functionCode, cache => {
        const newCache = cache || Map({});
        return newCache.set('config', action.payload.config)
          .set('pagination', action.payload.pagination)
          .set('sorter', {})
          .set('listTopButtons', action.payload.listTopButtons)
          .set('listPrintTemplate', action.payload.listPrintTemplate)
          .set('listMoreButtons', action.payload.listMoreButtons)
          .set('listPrintButtons', action.payload.listPrintButtons)
          .set('listExportButtons', action.payload.listExportButtons)
          .set('listAuditingButton', action.payload.listAuditingButton)
          .set('listAbandonButton', action.payload.listAbandonButton)
          .set('listAuditButtons', action.payload.listAuditButtons)
          .set('listQueryViewButtons', action.payload.listQueryViewButtons)
          .set('listSearchButton', action.payload.listSearchButton)
          .set('listAddButton', action.payload.listAddButton)
          .set('listSettingButton', action.payload.listSettingButton)
          .set('listBatchUpdateButton', action.payload.listBatchUpdateButton)
          .set('listBatchEditButton', action.payload.listBatchEditButton)
          .set('listBatchDeleteButton', action.payload.listBatchDeleteButton)
          .set('listGrantButton', action.payload.listGrantButton)
          .set('listRelationFormButtons', action.payload.listRelationFormButtons)
          .set('listRelationSearchButtons', action.payload.listRelationSearchButtons)
          .set('listToolButtons', action.payload.listToolButtons)
          .set('forbidDetail', action.payload.forbidDetail)
          .set('checkSearchScenario', action.payload.checkSearchScenario)
          .set('supportIndex', action.payload.supportIndex)
          .set('supportBatch', action.payload.supportBatch)
          .set('switchLeft', action.payload.switchLeft);
      })
    case 'list/changeSearchConfig': {
      return state.setIn([action.payload.functionCode, 'config', 'searchConfig'], [action.payload.newSearchConfig])
    }
    case 'list/changeSearchScenario': {
      const { item, value, functionCode } = action.payload;
      return state.update(functionCode, cache =>
        cache.update('config', config =>
          config.update('searchScenario', searchScenario =>
            searchScenario.update(item.tableName, field => field.update(item.dataIndex, x => x.set('value', value)))
          )));
    }
    case 'list/changeSearchScenarioList': {
      const { functionCode, code, operator} = action.payload;
      return state.update(functionCode, cache =>
        cache.update('config', config =>
          config.set('searchScenarioList', Map(code)).set("operator",operator)
        ));
    }
    case 'list/changeCacheTableConfig': {
      let newState = state.mergeIn([action.payload.functionCode, 'config'], action.payload.config);
      return newState
    }
    case 'list/changeAllSearchScenario': {
      const { searchScenario, functionCode } = action.payload;
      return state.setIn([functionCode, 'config', 'searchScenario'], searchScenario);
    }
    case 'list/changePersonScope': {
      const { personScope, functionCode } = action.payload
      return state.setIn([functionCode, 'config', 'personScope'], personScope);
    }
    case 'list/changeCacheItem': {
      const { key, value, functionCode, sorter } = action.payload;
      if(sorter){
        return state.updateIn([functionCode, 'config'], cache => cache.set(key, value)).setIn([functionCode, 'sorter'], sorter);
      }
      return state.updateIn([functionCode, 'config'], cache => cache.set(key, value));
    }
    case 'list/changeExpandCache': {
      // 多视图
      const { functionCode, expandedKeys, idx } = action.payload;
      if (idx || idx == 0) {
        return state.setIn([functionCode, 'expandedKeys', idx], expandedKeys);
      }
      return state.setIn([functionCode, 'expandedKeys'], expandedKeys);
    }
    case 'list/changeCacheListData': {
      return state.update(action.payload.functionCode, cache => {
        const newCache = cache || Map({});
        const { pagination = { pageNo: 1, pageSize: 10 } } = action.payload;
        (action.payload.list || []).forEach((v, index) => {
          v._index = (pagination.pageNo - 1) * pagination.pageSize + index;
        });
        return newCache.set('listData', action.payload.list)
          .set('pagination', action.payload.pagination)
          .set('listTotal', action.payload.listTotal)
          .set('queryViewId', action.payload.queryViewId)
          .set('queryViewSql', action.payload.queryViewSql)
          .set('stageData', action.payload.stageData)
          .set('listType', action.payload.listType);
      });
    }
    case 'list/changeCacheListDataByParentId': {
      return state.update(action.payload.functionCode, cache => {
        const newCache = cache || Map({});
        return newCache.set('listDataByParentId', action.payload.list);
      });
    }
    case 'list/removeDeletedSelectedId': {
      let selectedRowKeys = state.getIn([action.payload.functionCode, 'selectedRowKeys']);
      let selectedIndex = selectedRowKeys.indexOf(action.payload.id);
      if (selectedIndex >= 0) {
        selectedRowKeys.splice(selectedIndex, 1);
      }
      return state;
    }
    case 'list/changeSelectedIdValue':
      return state.update(action.payload.functionCode, cache => cache && cache.set('selectedId', action.payload.selectedId))
    case 'list/changeSelectedRowKeys':
      return state.setIn([action.payload.functionCode, 'selectedRowKeys'], action.payload.selectedRowKeys);
    case 'list/setScroll':
      return state.setIn([action.payload.functionCode, 'y'], action.payload.y);
    case 'list/setQueryViewId':
      return state.setIn([action.payload.functionCode, 'queryViewId'], JSON.stringify(action.payload.queryViewId));
    case 'list/setStageId':
      return state.setIn([action.payload.functionCode, 'stageId'], action.payload.stageId)
        .setIn([action.payload.functionCode, 'config', 'favoriteId'], '')//清除收藏夹过滤
        .setIn([action.payload.functionCode, 'config', 'nowCheckedId'], '')//清除激活收藏夹
    case 'list/switchLeft':
      return state.setIn([action.payload.functionCode, 'switchLeft'], action.payload.switchLeft);
    case 'list/resetList':
      return initState;
    case 'system/logoutSuccess':
      return initState;
    case 'list/savePersonScopeState':
      return state.setIn([action.payload.functionCode, 'personScopeState'], action.payload.personScopeState)
    case 'list/customSwitchData':
      return state.set('customSwitchData', action.payload.customSwitchData)
    default:
      return state;
  }
}
