import { takeEvery, put, call, select, fork } from "redux-saga/effects";
import { post, url, download, downloadFromServer } from "../../service/api";
import { fromJS, Map } from "immutable";
import { pageInfo, sysTheme } from "../../common/config";
import {
  buttonType,
  classButton,
  buttonPositionType,
  printerTableStyle,
  formFieldType,
  editPagePositionConst,
  detailDisplayTypeConst,
  globalTranslationSuffix,
} from "../../common/const";
import {
  getItemDefaultValue,
  sortTableColumns,
  validateItemPermission,
  getItemActualValue,
  getItemSourceByKey
} from "../../utils";
import { message } from "antd";
import _ from "lodash";
import { checkRecordInWorkFlow } from "../workflow/sagas";

export const watchFetchListConfigByFunctionCode = function* () {
  yield takeEvery("list/fetchListConfigByFunctionCode", initListPage);
  yield takeEvery("list/changeCustomPreference", changeCustomPreference);
  yield takeEvery("list/changeCustomTableConfig", changeCustomTableConfig);
  yield takeEvery("list/changeQuerySetting", changeQuerySetting);
  yield takeEvery("list/onceFetchListConfigByFunctionCode", fetchListConfigByFunctionCode);//单独取列表配置无缓存判断
};

export const initListPage = function* (action) {
  const cache = yield select((x) =>
    x.getIn(["list", action.payload, "config"])
  );
  const menuProps = yield select((x) =>
    x.getIn(["system", 'menuProps'])
  );
  if (cache) {
    if (action.initFromDashboard) {
      yield put({
        type: "dashboard/fetchConfigForDashboard",
        payload: { functionCode: action.payload },
      });
      return;
    }
    // 菜单传参时不自动查询
    if (menuProps) {
      return
    }
    if (!action.fetchByParentId) {
      if (
        (window.defaultSearch && cache.checkSearchScenario === "0") ||
        (action.cacheScenario && action.cacheScenario.queryViewId)
      ) {
        yield fork(fetchListDataByFunctionCode, {
          payload: {
            functionCode: action.payload,
            cacheScenario: action.cacheScenario,
          },
        });
      }
    } else {
      yield call(fetchListDataByParentId, action);
    }
  } else {
    const config = yield call(fetchListConfigByFunctionCode, action);
    //console.log('70-config',config)
    //console.log('71-action',action)
    if (config) {
      if (action.initFromDashboard) {
        yield put({
          type: "dashboard/fetchConfigForDashboard",
          payload: { functionCode: action.payload, cacheScenario: action.cacheScenario },
        });
        return;
      }
      const listSearchButton = config.listSearchButton;
      const payload = {
        functionCode: action.payload,
        config,
        cacheScenario: action.cacheScenario,
      };
      if (listSearchButton && listSearchButton.btnUrl) {
        payload["searchUrl"] = listSearchButton.btnUrl;
      }
      if (menuProps) {
        return
      }
      if (!action.fetchByParentId) {
        if (
          (window.defaultSearch && config.checkSearchScenario === "0") ||
          (action.cacheScenario && action.cacheScenario.queryViewId)
        )
          yield call(fetchListDataByFunctionCode, {
            payload,
          });
      } else {
        yield call(fetchListDataByParentId, action);
      }
    }
  }
};

function* changeQuerySetting(action) {
  const { functionCode, queryScope } = action.payload;
  const basicInfo = yield select((x) => x.getIn(["system", "basicInfo"]));
  queryScope &&
    localStorage.setItem(
      functionCode + basicInfo.id + "listQueryScope",
      queryScope
    );
}

function* changeCustomPreference(action) {
  const { functionCode, newSearchConfig } = action.payload;
  const basicInfo = yield select((x) => x.getIn(["system", "basicInfo"]));
  newSearchConfig &&
    localStorage.setItem(
      functionCode + basicInfo.id + "listSearchConfig",
      JSON.stringify(newSearchConfig)
    );
  yield put({
    type: "list/changeSearchConfig",
    payload: {
      functionCode,
      newSearchConfig,
      searchRowCount: calculateSearchRowCount(newSearchConfig),
    },
  });
}

function* changeCustomTableConfig(action) {
  const { functionCode, newTableConfig } = action.payload;
  const cache = yield select((x) => x.getIn(["list", functionCode]));
  const basicInfo = yield select((x) => x.getIn(["system", "basicInfo"]));
  const supportBatch = cache.get("supportBatch");
  const actionColumn = cache.getIn(["config", "actionColumn"]);
  let actualTableConfig = [];
  let totalWidth = 0;
  const calculateTotalWidth = (width) => {
    if (typeof width === "string") {
      width = parseInt(width);
    }
    totalWidth += width;
  };
  newTableConfig.map((item) => {
    if (item.defaultHide === "1") {
      return false;
    }
    calculateTotalWidth(item.width);
    if (item.sorter) {
      item.width = item.width += 10
      calculateTotalWidth(10);
    }
    actualTableConfig.push({ ...item });
    return true;
  });
  if (actualTableConfig.length > 0) {
    let lastItem = _.findLast(
      actualTableConfig,
      (item) => item.fixed !== "left" && item.fixed !== "right"
    );
    lastItem.originalWidth = lastItem["width"];
    lastItem && delete lastItem["width"];
  }

  if (supportBatch) {
    totalWidth = totalWidth + 35;
  }

  newTableConfig &&
    localStorage.setItem(
      functionCode + basicInfo.id + "listTableConfig",
      JSON.stringify(newTableConfig)
    );

  if (actionColumn) {
    actualTableConfig.push(actionColumn);
    totalWidth = totalWidth + actionColumn.width * 2 + 50;
  }

  yield put({
    type: "list/changeCacheTableConfig",
    payload: {
      functionCode,
      config: Map({
        totalWidth,
        fullTableConfig: newTableConfig,
        tableConfig: sortTableColumns(actualTableConfig),
      }),
    },
  });
}

/**
 * method_name: fetchListConfigByFunctionCode
 * param: none
 * describe: 通过页面的功能码获取列表页的配置
 * create_user: 王友生
 * create_date: 2018/3/10 0010
 * create_time: 10:54
 **/
function* fetchListConfigByFunctionCode(action) {
  const { result } = yield call(
    post,
    url.getSysListConfig,
    { key: action.payload },
    action.payload
  );
  if (!result || !result.content) {
    return;
  }

  const listConfig = fromJS(JSON.parse(result.content));
  const buttons = result.buttons;
  let listTopButtons = [];
  let listRecordButtons = [];
  let listMoreButtons = [];
  let listPrintButtons = [];
  let listExportButtons = [];
  let listRelationFormButtons = [];
  let listToolButtons = [];
  let listRelationSearchButtons = [];
  let listAuditButtons = [];
  let listAuditingButton = undefined;
  let listAbandonButton = undefined;
  let listSearchButton = undefined;
  let listAddButton = undefined;
  let listSettingButton = undefined;
  let listBatchEditButton = undefined;
  let listBatchDeleteButton = undefined;
  let listGrantButton = listGrantButton;
  let listBatchUpdateButton = [];
  let listQueryViewButtons = [];
  const basicInfo = yield select((x) => x.getIn(["system", "basicInfo"]));
  const allotButtons = (button, list) => {
    if (button.btnType === buttonType.print.value) {//打印
      listPrintButtons.push(button);
    }
    else if (//导入导出
      button.btnType === buttonType.export.value ||
      button.btnType === buttonType.exportAll.value ||
      button.btnType === buttonType.import.value ||
      button.btnType === buttonType.download.value
    ) {
      listExportButtons.push(button);
    }
    // else if (//审核
    //   !listAuditingButton &&
    //   button.btnType === buttonType.auditing.value
    // ) {
    //   listAuditingButton = button;
    // }
    // else if (//弃审
    //   !listAbandonButton &&
    //   button.btnType === buttonType.abandon.value
    // ) {
    //   listAbandonButton = button;
    // }
    else if (//审核
      button.btnGroup === classButton.audit.value
    ) {
      listAuditButtons.push(button);
    }
    else if (//授权
      !listGrantButton &&
      button.btnType === buttonType.grant.value
    ) {
      listGrantButton = button;
    }

    else if (//新增
      !listAddButton && button.btnType === buttonType.add.value) {
      listAddButton = button;
    }
    else if (//批量修改
      !listBatchEditButton &&
      button.btnType === buttonType.batchEdit.value
    ) {
      listBatchEditButton = button;
    }
    else if (//批量删除
      !listBatchDeleteButton &&
      button.btnType === buttonType.batchDelete.value
    ) {
      listBatchDeleteButton = button;
    }
    else if (//设置
      !listSettingButton &&
      button.btnType === buttonType.setting.value
    ) {
      listSettingButton = button;
    } else if (//查询
      !listSearchButton &&
      button.btnType === buttonType.search.value
    ) {
      listSearchButton = button;
    } else if (//视图
      button.btnType === buttonType.queryView.value) {
      listQueryViewButtons.push(button);
    } else if (//更多
      button.btnGroup === classButton.more.value) {
      listMoreButtons.push(button)
    } else if (//推单
      button.btnType === buttonType.addRelationForm.value ||
      button.btnType === buttonType.addPopupRelationForm.value ||
      button.btnGroup === classButton.pushRelationForm.value
    ) {
      listRelationFormButtons.push(button);
    } else if (//联查
      button.btnType === buttonType.relationSearch.value) {
      listRelationSearchButtons.push(button);
    } else if (//数据批改
      button.btnType === buttonType.batchUpdate.value) {
      listBatchUpdateButton.push(button);
    } else if (//工具
      button.btnType === buttonType.tools.value) {
      listToolButtons.push(button);
    } else {
      list.push(button);
    }
  };

  buttons &&
    buttons.map((button, index) => {
      // 列表上按钮
      if (button.btnPosition === buttonPositionType.listTop.value) {
        // 风格暂时弃用
        if (sysTheme && sysTheme == "1") {
          if (button.btnType === buttonType.search.value) {
            listSearchButton = button;
          } else {
            listTopButtons.push(button);
          }
        } else {
          allotButtons(button, listTopButtons);
        }
      } else if (button.btnPosition === buttonPositionType.listRecord.value) {
        listRecordButtons.push(button);
      } else if (button.btnPosition === buttonPositionType.listMore.value ||
        button.btnGroup === classButton.more.value
      ) {
        if (sysTheme) {
          listMoreButtons.push(button);
          // console.log('listMoreButtons', listMoreButtons);

        } else {
          allotButtons(button, listMoreButtons);
        }
      } else {
        allotButtons(button, []);
      }
      return true;
    });

  let searchConfig = listConfig.getIn([0, "children"]);
  const tableConfig = listConfig.get(1);
  if (!tableConfig) {
    message.error("请将列表配置完整");
    return;
  }
  const supportBatch = listConfig.getIn([1, "supportBatch"]);
  const supportIndex = listConfig.getIn([1, "supportIndex"]);
  const forbidDetail = listConfig.getIn([1, "forbidDetail"]);
  const blurredSearch = [];
  let blurredTitle = "";
  listConfig.get(2) &&
    listConfig.get(2).map((item) => {
      const blurredSearchItem = listConfig.getIn(item.toJSON());
      blurredTitle = blurredTitle + blurredSearchItem.get("title") + "/";
      blurredSearch.push({
        tableName: blurredSearchItem.get("tableName"),
        field: blurredSearchItem.get("dataIndex"),
      });
    });
  let searchScenario = Map({});
  let searchScenarioList = Map({ list: [] });
  let mainScenario = [];
  let mainIndex = -1;
  let checkSearchScenario = "0";
  let getTableName = "";
  let searchConfigPeo = {}
  searchConfig && searchConfig.map((item, index) => {
    if (//视图
      item.get("mainScenario") === "1" &&
      item.get("type") === formFieldType.treeSelect.value || item.get("mainScenario") === "1" && item.get("type") === formFieldType.multipleTreeSelect.value
    ) {
      mainScenario.push(item.toJS());
      // mainIndex = index;
    }
    if (item.get("defaultHide") !== "1" && item.get("required") === "1") {
      checkSearchScenario = "1";
    }


    getTableName = item.get("tableName");
    searchConfigPeo = {
      [getTableName]: {
        charge: { value: "", type: "" },
        relevant: { value: "", type: "" },
      }
    };
    searchScenario = searchScenario.update(item.get("tableName"), (x) => {
      let newScenario = x || Map({});
      return newScenario.set(
        item.get("dataIndex"),
        Map({
          value: getItemDefaultValue(item.toJSON()),
          type: item.get("type"),
          wildcard: item.get("wildcard"),
        })
      );
    });
    return true;
  });
  // 删除搜索项
  // if (mainIndex >= 0) {
  //   searchConfig = searchConfig.delete(mainIndex);
  // }
  searchConfig = searchConfig.toJSON();
  searchConfig &&
    searchConfig.map((item) => {
      item.editDictionary =
        item.sourceKey && !validateItemPermission(item.dicPermission, basicInfo)
          ? "1"
          : 0;
    });
  let cacheSearchConfigString = localStorage.getItem(
    action.payload + basicInfo.id + "listSearchConfig"
  );
  if (cacheSearchConfigString) {
    let cacheSearchConfig = JSON.parse(cacheSearchConfigString);
    let actualSearchConfig = [];
    _.map(cacheSearchConfig, (configItem) => {
      const newItem = _.findLast(searchConfig, {
        tableName: configItem.tableName,
        dataIndex: configItem.dataIndex,
      });
      newItem &&
        actualSearchConfig.push({
          ...newItem,
          defaultHide: configItem.defaultHide,
        });
    });
    const extraSearchConfig = _.differenceWith(
      searchConfig,
      cacheSearchConfig,
      (item1, item2) => {
        return (
          item1.tableName === item2.tableName &&
          item1.dataIndex === item2.dataIndex
        );
      }
    );
    searchConfig = [...actualSearchConfig, ...extraSearchConfig];
  }

  const actualTableConfig = [];
  let totalWidth = 0;
  let fullTableConfig = [];

  const calculateTotalWidth = (width) => {
    if (typeof width === "string") {
      width = parseInt(width);
    }
    totalWidth += width;
  };
  const convertToTableConfig = (item) => {
    const newItem = item.toJSON();
    newItem.dataIndex = item.get("nickName");
    return newItem;
  };
  tableConfig.get("left") &&
    tableConfig.get("left").map((item) => {
      item.fixed = item.fixed === "none" ? "" : item.fixed;
      if (!item) {
        return true;
      }
      if (item.get("hide") === "1") {
        return true;
      }
      if (!validateItemPermission(item.toJSON().showPermission, basicInfo)) {
        return true;
      }
      fullTableConfig.push(convertToTableConfig(item));
      return true;
    });

  tableConfig.get("middle") &&
    tableConfig.get("middle").map((item, index) => {
      item.fixed = item.fixed === "none" ? "" : item.fixed;
      if (!item) {
        return true;
      }
      if (item.get("hide") === "1") {
        return true;
      }
      if (!validateItemPermission(item.toJSON().showPermission, basicInfo)) {
        return true;
      }
      fullTableConfig.push(convertToTableConfig(item));
      return true;
    });

  // actualTableConfig.push({});
  tableConfig.get("right") &&
    tableConfig.get("right").map((item) => {
      item.fixed = item.fixed === "none" ? "" : item.fixed;
      if (!item) {
        return true;
      }
      if (item.get("hide") === "1") {
        return true;
      }
      if (!validateItemPermission(item.toJSON().showPermission, basicInfo)) {
        return true;
      }
      fullTableConfig.push(convertToTableConfig(item));
      return true;
    });

  let cacheTableConfigString = localStorage.getItem(
    action.payload + basicInfo.id + "listTableConfig"
  );
  if (cacheTableConfigString) {
    let cacheTableConfig = JSON.parse(cacheTableConfigString);
    let actualFullTableConfig = [];
    _.map(cacheTableConfig, (configItem) => {
      const newItem = _.findLast(fullTableConfig, {
        tableName: configItem.tableName,
        dataIndex: configItem.dataIndex,
      });
      newItem &&
        actualFullTableConfig.push({
          ...newItem,
          defaultHide: configItem.defaultHide,
          fixed: configItem.fixed,
        });
    });
    const extraTableConfig = _.differenceWith(
      fullTableConfig,
      cacheTableConfig,
      (item1, item2) => {
        return (
          item1.tableName === item2.tableName &&
          item1.dataIndex === item2.dataIndex
        );
      }
    );
    fullTableConfig = [...actualFullTableConfig, ...extraTableConfig];
  }

  fullTableConfig.map((item) => {
    if (item.defaultHide === "1") {
      return false;
    }
    calculateTotalWidth(item.width);
    if (item.sorter) {
      calculateTotalWidth(10);
      item.width = item.width + 10
    }

    actualTableConfig.push({ ...item });
    return true;
  });
  if (actualTableConfig.length > 0) {
    let lastItem = _.findLast(
      actualTableConfig,
      (item) => item.fixed !== "left" && item.fixed !== "right"
    );
    lastItem.originalWidth = lastItem["width"];
    lastItem && delete lastItem["width"];
  }

  if (supportBatch) {
    totalWidth = totalWidth + 35;
  }
  let actionColumn = undefined;
  if (listRecordButtons.length > 0) {
    const columnWidth = listRecordButtons.length * 60 + 50;
    actionColumn = {
      columnType: buttonPositionType.listRecord.value,
      buttons: listRecordButtons,
      width: columnWidth,
      fixed: "right",
      title: "操作",
      exportIgnore: "1",
    };
    actualTableConfig.push(actionColumn);
    totalWidth = totalWidth + columnWidth * 2 + 50;
  }

  // NOTE:20220714  合同产品点击发货完成报错，因为禁止详情无法请求getfomr,没有拿到form内容（暂时禁止）
  // if (!forbidDetail) {
  yield put({
    type: "form/fetchFormConfigByFunctionCode",
    payload: { functionCode: action.payload },
  });
  // }

  let correctings = listConfig.getIn([0, "correcting"]),
    correctingsConfig = [];
  correctings &&
    correctings.map((item) => {
      correctingsConfig.push({
        width: 2,
        type: "checkbox",
        title: "",
        sourceKey: "",
        hide: item.get("hide"),                        //20220706 多选框和字段一样是否隐藏
        dataIndex: "cbx_" + item.get("dataIndex"),
      });
      item = item.toJSON()
      //2021-10-18 15:25:00 批修改初始值
      item.defaultValue = item.defaultValue && getItemDefaultValue(item)
      correctingsConfig.push({ ...item, width: 10 });
      return true;
    });
  const pageSize =
    localStorage.getItem(action.payload + basicInfo.id + "pageSize") ||
    window.defaultPageSize ||
    pageInfo.pageSize;

  const payload = {
    functionCode: action.payload,
    config: Map({
      searchConfig: [searchConfig],
      peoSearchConfig: searchConfigPeo,
      tableName: getTableName,
      correctingsConfig: [correctingsConfig],
      tableConfig: sortTableColumns(actualTableConfig),
      mainScenario: mainScenario,
      activeKey: [1],
      blurredSearch,
      searchScenario,
      searchScenarioList,
      defaultSearchScenario: searchScenario,
      totalWidth,
      fullTableConfig,
      actionColumn,
      searchRowCount: calculateSearchRowCount(searchConfig),
      blurredTitle:
        blurredTitle && blurredTitle.substring(0, blurredTitle.length - 1),
    }),
    listTopButtons,
    listMoreButtons,
    listPrintButtons,
    listExportButtons,
    listAuditingButton,
    listAbandonButton,
    listAuditButtons,
    listSearchButton,
    listQueryViewButtons,
    listAddButton,
    listBatchDeleteButton,
    listGrantButton,
    listBatchEditButton,
    listBatchUpdateButton,
    listSettingButton,
    listRelationFormButtons,
    listRelationSearchButtons,
    listToolButtons,
    supportBatch,
    supportIndex,
    checkSearchScenario,
    forbidDetail,
    listPrintTemplate: result.prints,
    pagination: { ...pageInfo, pageSize },
    switchLeft: true,
  };
  yield put({
    type: "list/fetchListConfigSuccess",
    payload,
  });
  return payload;
}

function calculateSearchRowCount(searchConfig) {
  if (!searchConfig) {
    return 0;
  }
  let searchRowCount = 0;
  let oneRow = 0;
  searchConfig.map((item) => {
    if (item.defaultHide === "1" || item.hide === "1") {
      return true;
    }
    oneRow = oneRow + parseInt(item.width);
    if (oneRow >= 24) {
      searchRowCount++;
      oneRow = 0;
    }
    return true;
  });
  return searchRowCount;
}

/**
 * method_name: fetchListConfigByFunctionCode
 * param: none
 * describe: 通过页面的功能和查询条件获取数据
 * create_user: 王友生
 * create_date: 2018/3/10 0010
 * create_time: 10:54
 **/

export const watchFetchListDataByFunctionCode = function* () {
  yield takeEvery(
    "list/fetchListDataByFunctionCode",
    fetchListDataByFunctionCode
  );
  yield takeEvery(
    "list/fetchListDataWithMainScenario",
    fetchListDataWithMainScenario
  );
  yield takeEvery("list/changeSelectedId", viewDetailPage);
};

function* viewDetailPage(action) {
  const { selectedId, functionCode } = action.payload;
  if (
    selectedId &&
    window.detailDisplayType === detailDisplayTypeConst.tab.value
  ) {
    yield put({
      type: "system/jumpToDetailPage",//页签
      payload: {
        id: selectedId,
        functionCode,
      },
    });
  } else {
    // const switchLeft = yield select((x) => x.get("list").get(functionCode).get('switchLeft'))
    yield put({
      type: "list/changeSelectedIdValue",//滑动
      payload: {
        selectedId,
        functionCode,
      },
    });
    // 取消自动滑动
    // yield put({
    //   type: "list/switchLeft",
    //   payload: {
    //     functionCode,
    //     switchLeft: selectedId ? false : true
    //   }
    // })
  }
}

function* fetchListDataByFunctionCode(action) {
  let {
    functionCode,
    config,
    cacheScenario,
    queryViewId,
    queryViewSql,
    advanceSearch,
    pageNo,
    favoriteId
  } = action.payload;
  const cache = config
    ? Map(config)
    : yield select((x) => x.get("list").get(functionCode));
  if (!cache) {
    return;
  }
  if (!cache.get("config")) {
    return;
  }
  // 收藏夹
  if (cache.getIn(['config', 'nowCheckedId'])) {
    const { result } = yield call(
      post,
      '/favorite/getSysFavoriteItemList',
      { ...cache.get('pagination'), favoriteId: cache.getIn(['config', 'nowCheckedId']), funCode: functionCode },
      functionCode,
    );
    yield put({
      type: "list/changeCacheListData",
      payload: {
        functionCode,
        pagination: { pageNo: result.pageNo, pageSize: result.pageSize, total: result.total },
        list: result.data1,
        listTotal: result.listTotal,
        queryViewId: "",
        listType: "favorite"
      }
    })
    return
  }
  const oldQueryViewId =
    typeof queryViewId === "undefined" ? cache.get("queryViewId") : queryViewId;
  const oldQueryViewSql =
    typeof queryViewSql === "undefined" ? '' : queryViewSql;
  const searchButton = cache.get("listSearchButton") || {};
  let searchScenario = cache.getIn(["config", "searchScenario"]),
    personScope = cache.getIn(["config", "personScope"]),
    searchScenarioList = cache.getIn(["config", "searchScenarioList"]),
    defaultSearchScenario = cache.getIn(["config", "defaultSearchScenario"]),
    pagination = cache.get("pagination"),
    sorter = cache.get("sorter"),
    stageId = cache.get("stageId"),
    searchKey = cache.getIn(["config", "searchKey"]),
    blurredSearch = cache.getIn(["config", "blurredSearch"]),
    searchConfig = cache.get("config", "searchConfig"),
    expand = cache.getIn(["config", "expand"]),
    operator = cache.getIn(["config", "operator"])
  let needBlurredSearch = blurredSearch && blurredSearch.length > 0;

  if (pageNo) {
    pagination.pageNo = pageNo;
  }


  let params = {
    funCode: functionCode,
    pagination: pagination,
    sorter,
    favoriteId,
    queryViewId: typeof oldQueryViewId == 'string' ? oldQueryViewId : JSON.stringify(oldQueryViewId),
    queryViewSql: oldQueryViewSql
  };

  const params1 = {
    funCode: functionCode,
    pagination: pagination,
    sorter,
    operator: operator

  };
  if (cacheScenario) {
    if (cacheScenario instanceof Array) {
      searchScenario = searchScenario || defaultSearchScenario;
      cacheScenario.map((item) => {
        searchScenario = searchScenario.update(item.tableName, (field) =>
          field.update(item.dataIndex, (x) => x.set("value", item.value))
        );
      });
    } else {
      Object.keys(cacheScenario).map((key) => {
        params[key] = cacheScenario[key];
      });
    }
    yield put({
      type: "list/changeAllSearchScenario",
      payload: { searchScenario, functionCode },
    });
    pagination.pageNo = 1;
    params.searchScenario = searchScenario.toJSON();
  } else {
    // 展开高级搜索和不展开高级搜索两种情境
    if (searchScenario) {
      // params.searchScenario = searchScenario.toJSON();
      params.searchScenario = _.cloneDeep(searchScenario.toJS());

      if (!advanceSearch) {
        // if (needBlurredSearch && !expand) {
        //   for (let key in params.searchScenario) {
        //     for (let childKey in params.searchScenario[key]) {
        //       if (params.searchScenario[key][childKey] && /dateRange/i.test(params.searchScenario[key][childKey].type)) {
        //         // params.searchScenario[key][childKey].value = "";
        //       }
        //     }
        //   }
        // } 
        if(!needBlurredSearch || expand){
          for (let key in params.searchScenario) {
            let tableConfig = [];
            if (searchConfig) {
              const config = searchConfig.toJSON();
              tableConfig = (
                (config.searchConfig && config.searchConfig[0]) ||
                []
              ).filter((v) => v.tableName === key);
            }

            for (let childKey in params.searchScenario[key]) {
              for (let config of tableConfig) {
                if (
                  childKey === config.dataIndex &&
                  config.defaultHide == "1"
                ) {
                  params.searchScenario[key][childKey].value = "";
                }
              }
            }
          }
        }
      }
    }

    if (Object.keys(searchScenarioList.toJSON().list).length > 0) {
      let newData = searchScenarioList.toJSON().list;
      let tName = (searchConfig.toJSON().blurredSearch && searchConfig.toJSON().blurredSearch[0] && searchConfig.toJSON().blurredSearch[0].tableName) || null;
      if (!tName) {
        tName = (searchConfig.toJSON().searchConfig && searchConfig.toJSON().searchConfig[0] && searchConfig.toJSON().searchConfig[0][0] && searchConfig.toJSON().searchConfig[0][0].tableName) || '';
      }
      let tableNames = searchConfig.toJSON().searchConfig[0].map(el => el.tableName);          //筛选出搜索配置的所有表名
      let tableNames1 = [...new Set(tableNames)];
      let searchScenarioR = {};
      tableNames1.forEach(tname => {
        // searchScenarioR[tname]
        let nist = newData.filter(el => el.tableName == tname);
        searchScenarioR[tname] = nist
      })
      // params1.searchScenario = { [tName]: newData };
      params1.searchScenario = searchScenarioR;
      // TODO:步骤条修改
      // newData.length>0&&newData.forEach(item => {
      //   if(item.indexName==="stage"){
      //     yield put({
      //       type: "list/changeAllSearchScenario",
      //       payload: {stageId:item.value, functionCode}
      //     });
      //   }
      // });
    }

    params.fuzzySearchField = blurredSearch;
    params.fuzzy = searchKey;
  }
  // 阶段
  let showStage = undefined;
  searchConfig.get('searchConfig').map(item => {
    if (Array.isArray(item)) {
      item.map(itemC => {
        if (itemC.showStage) {
          showStage = itemC
        }
      })
    }
  })
  if (showStage) {
    params.key = showStage.dataIndex;
    params.value = stageId;
    params1.key = showStage.dataIndex;
    params1.value = stageId;
  }
  //2021-11-09 15:51:12 关闭视图状态
  yield put({
    type: "system/warningList",
    payload: false,
  });

  if (personScope) {
    params.personScope = personScope
  }

  const { result } = yield call(
    post,
    searchButton.btnUrl ? searchButton.btnUrl : searchScenarioList.toJSON().list.length > 0 ? url.advancedListPaging : url.getListPaging,
    searchScenarioList.toJSON().list.length > 0 ? params1 : params,
    // searchButton.btnUrl ? searchButton.btnUrl : url.getListPaging,
    // params,
    functionCode,
    searchButton.id
  );
  //缓存搜索条件
  yield put({
    type: 'list/changeCacheItem',
    payload: {
      key: 'cacheParams', value: searchScenarioList.toJSON().list.length > 0 ? params1 : params, functionCode
    }
  })

  yield put({
    type: "list/changeCacheListData",
    payload: {
      queryViewId: params.queryViewId,
      queryViewSql: params.queryViewSql,
      functionCode,
      list: result.data1,
      listTotal: result.listTotal,
      stageData: result.stageData,
      pagination: {
        ...pagination,
        total: result.total,
      },
    },
  });

}

function* fetchSearchParamsOrListData(action, ynFetch) {
  let {
    functionCode,
    config,
    cacheScenario,
    queryViewId,
    queryViewSql,
    advanceSearch,
  } = action.payload;
  const cache = config
    ? Map(config)
    : yield select((x) => x.get("list").get(functionCode));
  if (!cache) {
    return;
  }
  const oldQueryViewId =
    typeof queryViewId === "undefined" ? cache.get("queryViewId") : queryViewId;
  const oldQueryViewSql =
    typeof queryViewSql === "undefined" ? '' : queryViewSql;
  const searchButton = cache.get("listSearchButton") || {};
  let searchScenario = cache.getIn(["config", "searchScenario"]),
    searchScenarioList = cache.getIn(["config", "searchScenarioList"]),
    defaultSearchScenario = cache.getIn(["config", "defaultSearchScenario"]),
    pagination = cache.get("pagination"),
    sorter = cache.get("sorter"),
    searchKey = cache.getIn(["config", "searchKey"]),
    blurredSearch = cache.getIn(["config", "blurredSearch"]),
    searchConfig = cache.get("config", "searchConfig"),
    expand = cache.getIn(["config", "expand"]);
  let needBlurredSearch = blurredSearch && blurredSearch.length > 0;

  const params = {
    funCode: functionCode,
    pagination: pagination,
    sorter,
    queryViewId: oldQueryViewId,
    queryViewSql: oldQueryViewSql
  };


  if (cacheScenario) {
    if (cacheScenario instanceof Array) {
      searchScenario = searchScenario || defaultSearchScenario;
      cacheScenario.map((item) => {
        searchScenario = searchScenario.update(item.tableName, (field) =>
          field.update(item.dataIndex, (x) => x.set("value", item.value))
        );
      });
    } else {
      Object.keys(cacheScenario).map((key) => {
        params[key] = cacheScenario[key];
      });
    }
    yield put({
      type: "list/changeAllSearchScenario",
      payload: { searchScenario, functionCode },
    });
    pagination.pageNo = 1;
    params.searchScenario = searchScenario.toJSON();
  } else {
    // 展开高级搜索和不展开高级搜索两种情境
    if (searchScenario) {
      params.searchScenario = searchScenario.toJSON();

      if (!advanceSearch) {
        if (needBlurredSearch && !expand) {
          for (let key in params.searchScenario) {
            for (let childKey in params.searchScenario[key]) {
              if (
                params.searchScenario[key][childKey] &&
                /dateRange/i.test(params.searchScenario[key][childKey].type)
              ) {
                params.searchScenario[key][childKey].value = "";
              }
            }
          }
        } else {
          for (let key in params.searchScenario) {
            let tableConfig = [];
            if (searchConfig) {
              const config = searchConfig.toJSON();
              tableConfig = (
                (config.searchConfig && config.searchConfig[0]) ||
                []
              ).filter((v) => v.tableName === key);
            }

            for (let childKey in params.searchScenario[key]) {
              for (let config of tableConfig) {
                if (
                  childKey === config.dataIndex &&
                  config.defaultHide == "1"
                ) {
                  params.searchScenario[key][childKey].value = "";
                }
              }
            }
          }
        }
      }
    }


    params.fuzzySearchField = blurredSearch;
    params.fuzzy = searchKey;
  }

  if (ynFetch) {
    const { result } = yield call(
      post,
      searchButton.btnUrl ? searchButton.btnUrl : url.getListPagingNo,
      params,
      functionCode,
      searchButton.id
    );
    return result;
  }
  return params;
}

function* fetchListDataWithMainScenario(action) {
  const { selectKey, functionCode, idx } = action.payload;
  let mainScenario = yield select((x) =>
    x.getIn(["list", functionCode, "config", "mainScenario"])
  );
  // 多视图
  if (idx || idx == 0) {
    mainScenario = mainScenario[idx]
  }

  yield fetchListDataByFunctionCode({
    payload: {
      functionCode,
      cacheScenario: [{ ...mainScenario, value: selectKey }],
    },
  });
}

export const watchFetchListDataByParentId = function* () {
  yield takeEvery("list/fetchListDataByParentId", fetchListDataByParentId);
};

function* fetchListDataByParentId(action) {
  const { payload: functionCode, parentId, fromFunctionCode, config, id } = action;
  if (!parentId && parentId !== 0) {
    yield put({
      type: "list/changeCacheListDataByParentId",
      payload: {
        functionCode,
        list: [],
      },
    });
    // message.error('找不到关联字段');
    return;
  }
  const cache = config
    ? Map(config)
    : yield select((x) => x.get("list").get(functionCode));
  if (!cache) {
    return;
  }
  const sorter = cache.get("sorter");

  const params = {
    sourceFunCode: fromFunctionCode,
    targetFunCode: functionCode,
    sorter,
    value: parentId,
    id
  };
  const { result } = yield call(
    post,
    url.fetchChildrenByParentId,
    params,
    functionCode,
  );
  yield put({
    type: "list/changeCacheListDataByParentId",
    payload: {
      functionCode,
      list: result.data1,
    },
  });
}

export const watchListButtonClick = function* () {
  yield takeEvery("list/listButtonClick", listButtonClick);
};

function* listButtonClick(action) {
  const { functionCode, button, params, advanceSearch, queryViewSql, favoriteId, pageNo } = action.payload;
  let response;

  if (!button) {
    message.error("搜索无权限或没有配置，请到角色中授权，或者到功能建模中配置")
    return;
  }

  if (button.btnType === buttonType.add.value) {
    yield put({ type: 'form/resetFormData', payload: { functionCode } })
    yield put({
      type: "system/jumpToEditPage",
      payload: "-1",
    });
  } else if (button.btnType === buttonType.search.value) {//搜索
    let params = { functionCode, queryViewSql, favoriteId };
    if (button.btnUrl) {
      params["searchUrl"] = button.btnUrl;
    }
    if (pageNo) {//第一页
      params["pageNo"] = pageNo;
    }
    const listCache = yield select((x) =>
      x.getIn(["list", functionCode,])
    );
    if (listCache && !favoriteId) {//收藏夹过滤不走阶段
      yield put({
        type: "list/setStageId",
        payload: {
          stageId: '',
          functionCode,
        },
      });
    }
    // 收藏夹
    if (listCache && listCache.getIn(['config', 'nowCheckedId'])) {
      const { result } = yield call(
        post,
        '/favorite/getSysFavoriteItemList',
        { ...listCache.get('pagination'), favoriteId: listCache.getIn(['config', 'nowCheckedId']), funCode: functionCode },
        functionCode,
        button.id
      );
      yield put({
        type: "list/changeCacheListData",
        payload: {
          functionCode,
          pagination: { pageNo: result.pageNo, pageSize: result.pageSize, total: result.total },
          list: result.data1,
          listTotal: result.listTotal,
          queryViewId: "",
          listType: "favorite"
        },
      })
    } else {
      yield call(fetchListDataByFunctionCode, {
        payload: { advanceSearch, ...params },
      });
    }

    // 查询视图
  } else if (button.btnType === buttonType.queryView.value) {
    let queryViewId = yield select((x) =>
      x.getIn(["list", functionCode, "queryViewId"])
    ) || []
    let paramsAll = { functionCode, pageNo: 1 };
    queryViewId = queryViewId ? typeof queryViewId === 'string' ? JSON.parse(queryViewId) : queryViewId : []
    // 重置
    if (button.queryType && button.queryType == 'reset') {
      yield put({
        type: "list/setQueryViewId",
        payload: { functionCode, queryViewId: undefined }
      });
      paramsAll["queryViewId"] = undefined;
      yield call(fetchListDataByFunctionCode, { payload: paramsAll });
      return
    }
    let value = params
    const toParse = () => {
      if (queryViewId && typeof queryViewId == 'string') {//类型转换
        queryViewId = JSON.parse(queryViewId)
        toParse(queryViewId)
      }
    }
    toParse(queryViewId)
    queryViewId = queryViewId ? queryViewId : []
    let filterId = queryViewId.filter(item => {
      return item.id == button.id
    })
    if (filterId.length > 0) {
      queryViewId = queryViewId.filter(item => {
        return item.id !== button.id
      })
      if (value) {
        queryViewId.push({ id: button.id, value: { [button.paramKey]: value } })
      }
    } else {
      queryViewId.push(value ? { id: button.id, value: { [button.paramKey]: value } } : { id: button.id })
    }
    yield put({
      type: "list/setQueryViewId",
      payload: { functionCode, queryViewId }
    });

    yield put({
      type: 'list/changeSelectedRowKeys',
      payload: {
        functionCode,
        selectedRowKeys: []
      }
    });
    // 赋值请求列表
    paramsAll["queryViewId"] = queryViewId;
    yield call(fetchListDataByFunctionCode, { payload: paramsAll });
  } else if (button.btnType === buttonType.edit.value) {
    if (!params) {
      message.error("不能批量修改。");
      return;
    }
    if (
      yield checkRecordInWorkFlow({
        payload: { functionCode, recordId: params.id },
      })
    ) {
      message.error("流程已提交，不能修改。");
      return;
    }
    if (button.btnUrl) {
      yield call(
        post,
        button.btnUrl,
        {
          funCode: functionCode,
          id: params.id,
        },
        functionCode,
        button.id
      );
    }
    yield put({
      type: "system/jumpToEditPage",
      payload: params.id,
      functionCode,
    });
  } else if (button.btnType === buttonType.editForce.value) {
    if (!params) {
      message.error("不能批量修改。");
      return;
    }
    if (button.btnUrl) {
      yield call(
        post,
        button.btnUrl,
        {
          funCode: functionCode,
          id: params.id,
        },
        functionCode,
        button.id
      );
    }
    yield put({
      type: "system/jumpToEditPage",
      payload: params.id,
      functionCode,
    });
  } else if (button.btnType === buttonType.relationSearch.value) {
    if (button.btnUrl) {
      let response;
      if (
        button.btnPosition === buttonPositionType.listTop.value ||
        button.btnPosition === buttonPositionType.listMore.value
      ) {
        const selectedRowKeys = yield select((x) =>
          x.getIn(["list", functionCode, "selectedRowKeys"])
        );
        response = yield call(
          post,
          button.btnUrl,
          { selectedIds: selectedRowKeys },
          functionCode,
          button.id
        );
      } else if (button.btnPosition === buttonPositionType.listRecord.value) {
        response = yield call(
          post,
          button.btnUrl,
          { id: params.id },
          functionCode,
          button.id
        );
      }
      if (response && response.result) {
        yield put({
          type: "system/jumpToFunctionWidthScenario",
          payload: {
            functionCode: button.relationCode,
            scenario: response.result,
          },
        });
      }
    } else {
      yield put({
        type: "system/jumpToFunctionWidthScenario",
        payload: { functionCode: button.relationCode },
      });
    }
  } else if (
    //推单弹窗推单
    button.btnType === buttonType.addRelationForm.value ||
    button.btnType === buttonType.addPopupRelationForm.value
  ) {
    const payload = { targetFunctionCode: button.relationCode };
    // 自带连接
    if (button.btnUrl) {
      // 列表上
      if (
        button.btnPosition === buttonPositionType.listTop.value ||
        button.btnPosition === buttonPositionType.listMore.value
      ) {
        const selectedRowKeys = action.payload.selectedRowKeys ? action.payload.selectedRowKeys : yield select((x) =>
          x.getIn(["list", functionCode, "selectedRowKeys"])
        );
        // 取值
        const response = yield call(
          post,
          button.btnUrl,
          { selectedIds: selectedRowKeys },
          functionCode,
          button.id
        );
        payload["newRelationFormData"] = response.result;
        payload["attachments"] = response.attachments;
      } else if (button.btnPosition === buttonPositionType.listRecord.value) {
        const selectedRowKeys = yield select((x) =>
          x.getIn(["advanceList", functionCode, "selectedRowKeys"])
        );
        const response = yield call(
          post,
          button.btnUrl,
          { id: params.id },
          functionCode,
          button.id
        );
        payload["newRelationFormData"] = response.result;
        payload["attachments"] = response.attachments;
      }
    }
    payload.popup =
      button.btnType === buttonType.addPopupRelationForm.value
        ? editPagePositionConst.popup.value
        : editPagePositionConst.tab.value;
    payload.width = button.popWidth;
    payload.height = button.popHeight;
    yield put({
      type: "form/generateRelationForm",
      payload,
    });
  } else if (button.btnType === buttonType.custom.value) {
    if (
      button.btnPosition === buttonPositionType.listTop.value ||
      button.btnPosition === buttonPositionType.listMore.value
    ) {
      const selectedRowKeys = yield select((x) =>
        x.getIn(["list", functionCode, "selectedRowKeys"])
      );

      const newList = _.cloneDeep(selectedRowKeys);  //20220824 解决选择列表上的按钮点击后 取消选中（如有问题可删除）-》 将redux中的值复制出来，再清空redux

      yield put({
        type: 'list/changeSelectedRowKeys',
        payload: {
          functionCode,
          selectedRowKeys: []
        }
      });

      yield call(
        post,
        button.btnUrl,
        { selectedIds: newList },
        functionCode,
        button.id
      );







    } else if (button.btnPosition === buttonPositionType.listRecord.value) {
      const selectedRowKeys = yield select((x) =>
        x.getIn(["advanceList", functionCode, "selectedRowKeys"])
      );
      yield call(
        post,
        button.btnUrl,
        { id: params.id },
        functionCode,
        button.id
      );
    }
    yield put({
      type: "list/fetchListDataByFunctionCode",
      payload: { functionCode },
    });
  } else if (button.btnType === buttonType.print.value) {
    if (
      params &&
      params.apiUrl &&
      params.jsMethodName &&
      window[params.jsMethodName]
    ) {
      const cache = yield select((x) => x.get("list").get(functionCode));
      const searchScenario = cache.getIn(["config", "searchScenario"]),
        searchKey = cache.getIn(["config", "searchKey"]),
        blurredSearch = cache.getIn(["config", "blurredSearch"]),
        expand = cache.get("expand");
      let needBlurredSearch = blurredSearch && blurredSearch.length > 0;

      const requestParams = {};
      if (!needBlurredSearch || expand) {
        if (searchScenario) params.searchScenario = searchScenario.toJSON();
      } else {
        requestParams.fuzzySearchField = blurredSearch;
        requestParams.fuzzy = searchKey;
      }
      const data = yield call(post, params.apiUrl, requestParams);
      let LODOP = window.getLodop();
      window[params.jsMethodName](LODOP, data);
      LODOP.PREVIEW();
    } else {
      let LODOP = window.getLodop();
      LODOP.PRINT_INIT("打印");
      let htmlString = printerTableStyle;
      htmlString += document.getElementsByClassName("ant-table-body")[0]
        .innerHTML;
      LODOP.ADD_PRINT_TABLE(
        1,
        1,
        "100%",
        "100%",
        "<body style='margin:0'>" + htmlString + "</body>"
      );
      LODOP.PREVIEW();
    }
  } else if (button.btnType === buttonType.export.value) {
    const tableConfig = yield select((x) => x.getIn(["list", functionCode, "config", "tableConfig"]));
    const listData = yield select((x) => x.getIn(["list", functionCode, "listData"]));
    const reference = yield select((x) => x.getIn(["reference"]));
    const dictionary = yield select((x) => x.getIn(["dictionary"]));
    const activeTab = yield select((x) => x.getIn(["system", "activeTab"]));
    const fileName = activeTab.menuName || activeTab.name;
    let tranArray = [];
    if (listData && listData.length > 0) {
      listData.map((record) => {
        const tranRecord = { ...record };
        for (const key in record) {
          const item = tableConfig.filter((x) => x.dataIndex === key);
          item[0] &&
            (tranRecord[key + globalTranslationSuffix] =
              getItemActualValue(
                item[0],
                record[key],
                put,
                reference,
                dictionary
              ) || "-");
        }
        tranArray.push(tranRecord);
      });
    }

    const result = yield call(
      downloadFromServer,
      button.btnUrl || url.exportToFile,
      {
        titles: tableConfig.filter((x) => x.exportIgnore !== "1"),
        columns: tranArray,
        fileName,
        functionCode,
      },
      `${fileName}.xls`,
      functionCode
    );
  } else if (button.btnType === buttonType.exportAll.value) {
    const tableConfig = yield select((x) =>
      x.getIn(["list", functionCode, "config", "tableConfig"])
    );
    //const listData = yield select(x => x.getIn(['list', functionCode, 'listData']));
    //TODO  全部导出根据 查询条件
    const activeTab = yield select((x) => x.getIn(["system", "activeTab"]));
    const fileName = activeTab.menuName || activeTab.name;
    const listData = yield select((x) =>
      x.getIn(["list", functionCode, "listData"])
    );
    console.log("function*listButtonClick -> listData", listData);
    if (!listData || listData.length === 0) {
      message.warn("无数据");
      return true;
    }
    let params = { functionCode };
    /* 预留 后期优化列表时，可以减少传输中字节数
    if (button.btnUrl) {
      params['searchUrl'] = button.btnUrl;
    }
    let searchParams = yield call(fetchSearchParamsOrListData, { payload: { advanceSearch, ...params } });
    searchParams.downloadParams = { titles: tableConfig.filter(x => x.exportIgnore !== '1'), columns: [], fileName, functionCode };
    const result = yield call(downloadFromServer, button.btnUrl || url.exportToFileAll,searchParams, `${fileName}.xls`,functionCode); */

    const reference = yield select((x) => x.getIn(["reference"]));
    const dictionary = yield select((x) => x.getIn(["dictionary"]));

    let searchData = yield call(
      fetchSearchParamsOrListData,
      { payload: { advanceSearch, ...params } },
      true
    );
    let searchResult = searchData && searchData.data1,
      tranArray = [];

    if (searchResult && searchResult.length > 0) {
      searchResult.map((record) => {
        const tranRecord = {};
        for (const key in record) {
          const item = tableConfig.filter((x) => x.dataIndex === key);
          item[0] &&
            (tranRecord[key + globalTranslationSuffix] =
              getItemActualValue(
                item[0],
                record[key],
                put,
                reference,
                dictionary
              ) || "-");
        }
        tranArray.push(tranRecord);
      });
    }
    console.log("function*listButtonClick -> tranArray", tranArray);
    yield call(
      downloadFromServer,
      button.btnUrl || url.exportToFile,
      {
        titles: tableConfig.filter((x) => x.exportIgnore !== "1"),
        columns: tranArray,
        fileName,
        functionCode,
      },
      `${fileName}.xls`,
      functionCode
    );
  } else if (button.btnType === buttonType.delete.value) {
    if (
      yield checkRecordInWorkFlow({
        payload: { functionCode, recordId: params.id },
      })
    ) {
      message.error("流程已提交，不能删除。");
      return;
    }
    if (button.btnUrl) {
      const cache = yield select((x) => x.getIn(["form", functionCode]));
      const mainTableName = cache.get("mainTableName");
      yield call(
        post,
        button.btnUrl,
        {
          funCode: functionCode,
          id: params.id,
        },
        functionCode,
        button.id
      );
    } else {
      yield deleteFormData(button, params.id, functionCode);
    }
    yield fetchListDataByFunctionCode({ payload: { functionCode } });
    yield put({ type: "list/changeSelectedId", payload: { selectedId: "" } });
  } else if (button.btnType === buttonType.popForm.value) {
    const cache = yield select((x) => x.getIn(["form", functionCode]));
    console.log("cache11111", cache);
    const shortFormConfig = action.payload.shortFormConfig ? action.payload.shortFormConfig : cache.get("shortFormConfig") || [];
    console.log("cache11111", shortFormConfig);
    const currentShortFormConfigs = shortFormConfig.filter(
      (x) => button.relationCode === x.shortFormId
    );
    if (currentShortFormConfigs && currentShortFormConfigs.length > 0) {
      const currentShortFormConfig = currentShortFormConfigs[0];
      let selectedRowKeys;
      if (action.payload.selectedRowKeys) {
        selectedRowKeys = action.payload.selectedRowKeys
      } else {
        selectedRowKeys = yield select((x) =>
          x.getIn(["list", functionCode, "selectedRowKeys"]))
      }
      if (currentShortFormConfig["scenario"]) {
        yield put({
          type: "shortForm/visibleShortForm",
          payload: {
            fromId: params ? params.id : undefined,
            width: button.popWidth,
            height: button.popHeight,
            selectedIds: selectedRowKeys,
            config: currentShortFormConfig,
            fromFunctionCode: functionCode,
            refreshAction: "list/fetchListDataByFunctionCode",
          },
        });
      }
    }
  } else if (button.btnType === buttonType.customPopModal.value) {
    let record = {};
    if (
      button.btnPosition === buttonPositionType.listTop.value ||
      button.btnPosition === buttonPositionType.listMore.value
    ) {
      record.selectedRowKeys = yield select((x) =>
        x.getIn(["list", functionCode, "selectedRowKeys"])
      );
    } else {
      const listData = yield select((x) =>
        x.getIn(["list", functionCode, "listData"])
      );
      record.selectedRow = _.findLast(listData, { id: params.id });
    }
    yield put({
      type: "system/visibleCustomPopEdit",
      payload: { key: button.btnUrl, record: record, functionCode },
    });
  } else if (button.btnType === buttonType.grant.value) {
    let record = {};
    if (
      button.btnPosition === buttonPositionType.listTop.value ||
      button.btnPosition === buttonPositionType.listMore.value
    ) {
      record.selectedRowKeys = yield select((x) =>
        x.getIn(["list", functionCode, "selectedRowKeys"])
      );
    } else {
      const listData = yield select((x) =>
        x.getIn(["list", functionCode, "listData"])
      );
      record.selectedRow = _.findLast(listData, { id: params.id });
    }
    yield put({
      type: "system/visibleCustomPopEdit",
      payload: { key: "GrantPromise", record: record, functionCode },
    });
  } else if (button.btnType === buttonType.download.value) {
    const activeTab = yield select((x) => x.getIn(["system", "activeTab"]));
    const fileName = activeTab.menuName || activeTab.name;
    if (button.btnUrl) {
      yield call(download, button.btnUrl, {}, fileName);
    }
  } else {
    if (
      button.btnPosition === buttonPositionType.listTop.value ||
      button.btnPosition === buttonPositionType.listMore.value
    ) {
      const selectedRowKeys = yield select((x) =>
        x.getIn(["list", functionCode, "selectedRowKeys"])
      );
      yield call(
        post,
        button.btnUrl,
        { selectedIds: selectedRowKeys },
        functionCode,
        button.id
      );
    } else if (button.btnPosition === buttonPositionType.listRecord.value) {
      const selectedRowKeys = yield select((x) =>
        x.getIn(["advanceList", functionCode, "selectedRowKeys"])
      );
      yield call(
        post,
        button.btnUrl,
        { id: params.id },
        functionCode,
        button.id
      );
    }
    yield put({
      type: "list/fetchListDataByFunctionCode",
      payload: { functionCode },
    });
  }
}


function* deleteFormData(deleteButton, id, functionCode) {
  const data = yield call(
    post,
    deleteButton && deleteButton.btnUrl
      ? deleteButton.btnUrl
      : url.removeFormData,
    {
      funCode: functionCode,
      id,
    },
    functionCode,
    deleteButton.id
  );
}
