/**
 * Created by 王友生 on 2018-06-29.
 */

import { Map, List } from 'immutable';

const initState = Map({
  message: Map({}),
  unreadCount: 0,
});

export default (state = initState, action) => {
  switch (action.type) {
    case 'message/receiveNewMessageSuccess':
      const { fromUser, content, isUnreadMessage } = action.payload;
      state = state.updateIn(['message', fromUser.id], user => {
        user = user || Map({ ...fromUser, unreadCount: 0 });
        if (isUnreadMessage) {
          user = user.set('unreadCount', user.get('unreadCount') + 1);
        }
        user = user.update('messages', messages => {
          messages = messages || List([]);
          messages = messages.push({
            owner: 'other',
            messageType: 'message',
            content
          });
          return messages;
        })
        return user;
      });
      if (isUnreadMessage) {
        state = state.set('unreadCount', state.get('unreadCount') + 1);
      }
      return state;
    case 'system/changeChatUser':
      state = state.set('chatUser', action.payload.chatUser);
      const chatUserUnreadCount = state.getIn(['message', action.payload.chatUser.id, 'unreadCount']) || 0;
      if (chatUserUnreadCount !== 0) {
        state = state.setIn(['message', action.payload.chatUser.id, 'unreadCount'], 0);
        const totalUnreadCount = state.get('unreadCount');
        state = state.set('unreadCount', totalUnreadCount - chatUserUnreadCount);
      }
      return state;
    case 'message/sendMessageToOne': {
      const { toUser, content } = action.payload;
      return state.updateIn(['message', toUser.id], user => {
        user = user || Map(toUser);
        user = user.set('messageContent', '')
          .update('messages', messages => {
            messages = messages || List([]);
            messages = messages.push({
              owner: 'mine',
              messageType: 'message',
              content
            });
            return messages;
          })
        return user;
      });
    }
    case 'message/removeChatUser':
      return state.set('chatUser', undefined);
    case 'message/changeUserContentMessage': {
      const { toUser, content } = action.payload;
      return state.updateIn(['message', toUser.id], user => {
        user = user || Map(toUser);
        user = user.set('messageContent', content)
        return user;
      });
    }
    default:
      return state;
  }
}

