import { takeEvery, put, call, select, fork, takeLatest, cancel, take } from 'redux-saga/effects';
import { post, url } from '../../service/api';
import { referenceStyleType } from '../../common/const';
import { pageInfo } from '../../common/config';
let referenceIds = [];

export const watchFetchReferenceConfig = function* () {
  while (true) {
    const action = yield take('reference/initReference');
    // console.log(this, referenceIds)
    if (referenceIds.indexOf(action.payload) < 0) {
      referenceIds.push(action.payload);
      yield fork(initReference, action);
    }
    yield takeEvery('reference/iniStateSaga', iniStateSaga);
  }
}

function* iniStateSaga() {
  referenceIds = [];
  yield put({ type: 'reference/iniState' });
}

//12.3无感参照
function* initReference(action) {
  const data = yield call(post, url.getReferenceConfig, { key: action.payload }, '', '', false);
  if (data && data.result) {
    //2022-06-28 16:59:44 参照新增
    const { refName, refType, content, relationCode, ynAdd } = data.result;
    const contentJSON = content ? JSON.parse(content) : {};
    const config = {
      field: {
        value: contentJSON.value,
        label: contentJSON.label,
        pid: contentJSON.pid,
        id: contentJSON.id,
        foreignKey: contentJSON.foreignKey
      },
      refName,
      refType,
      relationCode,
      ynAdd
    };
    let treeData = undefined;
    if (refType !== referenceStyleType.popTable.value) {
      const data = yield call(post, url.fetchReferenceTreeData, { id: action.payload }, '', '', false);
      treeData = data ? data.result : [];
    }
    const nodes = contentJSON['nodes'];
    if (nodes) {
      const searchField = [];
      const displayedColumns = [];
      let searchPlaceHolder = '';
      let totalWidth = 0;
      nodes.map(item => {
        if (item.search === '1') {
          searchPlaceHolder = searchPlaceHolder + item['title'] + '/';
          searchField.push(item['dataIndex']);
        }
        if (item.display === '1') {
          displayedColumns.push(item);
          totalWidth += item.width;
        }
        return true;
      });
      config.searchPlaceHolder = searchPlaceHolder && searchPlaceHolder.substring(0, searchPlaceHolder.length - 1);
      config.searchField = searchField;
      config.tableConfig = {
        totalWidth,
        displayedColumns,
      };
    }
    yield put({
      type: 'reference/loadReferenceConfigSuccess',
      payload: {
        id: action.payload,
        config,
        treeData,
      }
    });
    return config
  }
}

export const watchFetchReferenceTableData = function* () {
  yield takeEvery('reference/fetchReferenceTableData', fetchReferenceTableData);
  yield takeEvery('reference/refreshReference', initReference);
  yield takeEvery('reference/showReferenceWithFuzzy', showReferenceWithFuzzy);
}

function* fetchReferenceTableData(action) {
  const { id, pagination, fuzzy, fuzzySearchField, searchScenario, condition, loading } = action.payload;
  const filterTableRecord = yield select(x => x.getIn(['reference', 'filterTableRecord']));
  const activeTab = yield select(x => x.getIn(['system', 'activeTab']));
  const data = yield call(post, url.fetchReferenceTableData, {
    id,
    pagination,
    fuzzy,
    fuzzySearchField,
    searchScenario,
    condition,
    filterTableRecord
  }, activeTab ? activeTab.functionCode : '', '', loading);
  yield put({
    type: 'reference/fetchReferenceTableDataSuccess',
    payload: { id: action.payload.id, tableData: data.result || [] }
  });
}

function* showReferenceWithFuzzy(action) {
  // {
  //       id: referenceKey,
  //       item,
  //       selectedMode: selectedModeConst.single,
  //       extraSearchScenario: sourceScenario,
  //       objectHash: props.objectHash
  //     }
  yield put({ type: 'reference/cachePop', payload: action.payload })//缓存参照
  const { id, item, selectedMode, extraSearchScenario, objectHash, fuzzy, onSelectedSuccess } = action.payload;
  const activeTab = yield select(x => x.getIn(['system', 'activeTab']));
  let config = yield select(x => x.getIn(['reference', 'config']));
  if (!config) {
    config = yield initReference({ payload: id });
  }
  let treeData = yield select(x => x.getIn(['reference', id, "treeData"]));
  const data = yield call(post, url.fetchReferenceTableData, {
    id,
    pagination: { ...pageInfo, pageSize: window.defaultPageSize || pageInfo.pageSize },
    fuzzy: fuzzy,
    fuzzySearchField: config.searchField,
    condition: extraSearchScenario
  }, activeTab ? activeTab.functionCode : '');
  if (data.result && data.result.data1 && id != '910b3e7220e74baf9c67cdf0e4c33f2a' && data.result.data1.length === 1) {
    // 单条回写兼容树表暂时用id区别
    if (typeof onSelectedSuccess === 'function') {
      onSelectedSuccess(data.result.data1);
      yield put({ type: 'reference/hideReference' });
    } else {
      yield put({
        type: 'reference/selectReferenceSuccess',
        payload: data.result.data1,
        objectHash,
        currentItem: item
      });
    }
    // yield put({
    //   type: 'reference/visibleReferenceWithData',
    //   payload: {
    //     ...action.payload,
    //     fuzzy,
    //     tableData: data.result || []
    //   }
    // })
  } else {
    yield put({
      type: 'reference/visibleReferenceWithData',
      payload: {
        ...action.payload,
        fuzzy,
        tableData: data.result || []
      }
    })
  }
}
