/**
 * Created by 王友生 on 2018-06-12.
 */

import { takeEvery, put, fork, call, select } from 'redux-saga/effects';
import { fromJS, Map } from 'immutable';
import { post, url } from '../../service/api';
import { message } from 'antd';
import { buttonType, printerTableStyle } from '../../common/const';
import { getItemDefaultValue } from '../../utils';

export const watchFetchReportConfig = function* () {
  yield takeEvery('report/initReport', initReport)
}

function* initReport(action) {
  const cache = yield select(x => x.getIn(['report', action.payload.functionCode]));
  if (!cache) {
    const data = yield call(fetchReportConfig, action);
    if (!data) {
      message.error('报表配置不完全！');
    } else {
      const { basicInfo, searchScenario } = data;
      if (action.payload.initFromDashboard) {
        yield put({ type: 'dashboard/fetchConfigForDashboard', payload: { functionCode: action.payload.functionCode } });
        return;
      } else {
        if (window.reportDefaultSearch) {
          yield fork(fetchReportData, {
            payload: {
              functionCode: action.payload.functionCode,
              searchScenario,
              apiUrl: basicInfo['apiUrl'],
              xTitle: basicInfo['xTitle'],
              yTitle: basicInfo['yTitle']
            }
          })
        }
      }
    }
  } else {
    const basicInfo = cache.get('basicInfo');
    const searchScenario = cache.get('searchScenario');
    if (action.payload.initFromDashboard) {
      yield put({ type: 'dashboard/fetchConfigForDashboard', payload: { functionCode: action.payload.functionCode } });
      return;
    } else {
      if (window.reportDefaultSearch) {
        yield fork(fetchReportData, {
          payload: {
            functionCode: action.payload.functionCode,
            searchScenario,
            apiUrl: basicInfo['apiUrl'],
            xTitle: basicInfo['xTitle'],
            yTitle: basicInfo['yTitle']
          }
        })
      }
    }
  }
}

function* fetchReportConfig(action) {
  const { functionCode } = action.payload;
  const data = yield call(post, url.getSysListConfig, { key: functionCode });
  if (!data || !data.result) {
    return;
  }

  const { content: contentString, id } = data.result;
  if (!contentString) {
    return
  }
  const content = JSON.parse(contentString);

  const config = content['config'];
  const columns = content['columns'];
  const basicInfo = content['basicInfo'];

  const baseAuth = yield call(post, "/sysGrantAuth/getBasePromiseSql", { funCode: basicInfo.refAuthFun });
  if (baseAuth && baseAuth.result) {
    basicInfo.basePromise = baseAuth.result;
  }

  if (!config) {
    return;
  }
  const searchScenarioConfig = config['searchScenario'];
  let searchScenario = Map({});
  searchScenarioConfig.map(item => {
    searchScenario = searchScenario.set(item.dataIndex, getItemDefaultValue(item));
  });
  const payload = {
    searchScenarioConfig: config['searchScenario'],
    collect: config['collect'],
    fields: config['fields'],
    columns: fromJS(columns),
    functionCode: action.payload.functionCode,
    basicInfo: basicInfo,
    searchScenario,
    reportPrintTemplate: data.result.prints,
  }
  yield put({
    type: 'report/fetchReportConfigSuccess',
    payload,
  })
  return payload;
}

export const watchFetchReportData = function* () {
  yield takeEvery('report/fetchReportData', fetchReportData);
}

function* fetchReportData(action) {
  const { apiUrl, searchScenario, functionCode, xTitle, yTitle } = action.payload;
  const { result: data } = yield call(post, apiUrl, searchScenario.toJSON());
  let newChartData = [];

  if (data.chartData) {
    newChartData = Array.from(data.chartData, x => {
      const row = {};
      row[xTitle] = x.xData;
      row[yTitle] = x.yData;
      return row
    });
  }
  yield put({ type: 'report/fetchReportDataSuccess', payload: { functionCode, data, chartData: newChartData } })
}

export const watchReportButtonClick = function* () {
  yield takeEvery('report/reportButtonClick', reportButtonClick);
}

function* reportButtonClick(action) {
  const { functionCode, button, params } = action.payload;
  let response;
  if (button.btnType === buttonType.print.value) {
    if (params && params.apiUrl && params.jsMethodName && window[params.jsMethodName]) {
      const cache = yield select(x => x.get('report').get(functionCode));
      const searchScenario = cache.getIn(['searchScenario']);
      const requestParams = searchScenario.toJSON();
      const data = yield call(post, params.apiUrl, requestParams);
      let LODOP = window.getLodop();
      window[params.jsMethodName](LODOP, data);
      LODOP.PREVIEW();
    } else {
      let LODOP = window.getLodop();
      LODOP.PRINT_INIT("打印");
      let htmlString = printerTableStyle;
      htmlString += document.getElementsByClassName('ant-table-body')[0].innerHTML;
      LODOP.ADD_PRINT_TABLE(1, 1, "100%", "100%", "<body style='margin:0'>" + htmlString + "</body>");
      LODOP.PREVIEW();
    }
  }
  // else if (button.btnType === buttonType.export.value) {
  //   const tableConfig = yield select(x => x.getIn(['list', functionCode, 'config', 'tableConfig']));
  //   const listData = yield select(x => x.getIn(['list', functionCode, 'listData']));
  //   const activeTab = yield select(x => x.getIn(['system', 'activeTab']));
  //   const fileName = activeTab.menuName || activeTab.name;
  //   const result = yield call(download, button.btnUrl || url.exportToFile, { titles: tableConfig, columns: listData, fileName, functionCode }, fileName);
  // }
}
