/**
 * Created by 王友生 on 2018-07-27.
 */
import { delay } from 'redux-saga';
import { takeEvery, put, fork, call, select, takeLatest } from 'redux-saga/effects';
import { post, url } from '../../service/api';
import { fromJS, Map, List } from 'immutable';
import { getItemDefaultValue } from '../../utils';
import _ from 'lodash';
import { sysFormulaEditTypeConst, formulaType, operationType, functionType } from '../../common/const';
import math from 'mathjs';
import { message } from 'antd';

math.config({
  number: 'BigNumber'
});
const mathParse = math.parser();

export const watchWatchShortFormAction = function* () {
  yield takeEvery('shortForm/visibleShortForm', visibleShortForm);
  yield takeEvery('shortForm/saveShortForm', saveShortForm);
  yield takeEvery('shortForm/changeFormData', changeFormData);
  yield takeEvery('shortForm/changeFormTableData', changeFormTableData);
  yield takeLatest('shortForm/changeFormData', delayExecFormulaByForm);
  yield takeLatest('shortForm/changeFormTableData', delayExecFormulaByTable);
  yield takeEvery('shortForm/changeTable', changeTable)
  yield takeEvery('shortForm/deleteTable', deleteTable)
  yield takeEvery('shortForm/deleteTableRecords', deleteTableRecords);
}
function* deleteTableRecords(action) {
  const { tableName } = action.payload;
  let state = yield select(x => x.get('shortForm'));
  state = state.setIn(['data', tableName], List([]));
  yield put({ type: 'shortForm/setForm', payload: state });
}

function* deleteTable(action) {
  const { tableName, index } = action.payload;
  let state = yield select(x => x.get('shortForm'));
  const deleteTable = state.getIn(['data', tableName, index]);
  state = state.deleteIn(['data', tableName, index]);
  if (deleteTable) {
    const table = deleteTable.toJSON();
    const keys = Object.keys(table);
    for (let keyIndex = 0; keyIndex < keys.length; keyIndex++) {
      const key = keys[keyIndex];
      state = yield call(calculateWithFormula, state, { tableName, dataIndex: key }, state.get('shortFormId'), index, true);
    }
  }
  yield put({ type: 'shortForm/setForm', payload: state });
}


function* changeTable(action) {
  const { item, referenceResult, fromReference, newTriggerRef } = action.payload;
  // item:子表配置,referenceResult:选项,
  // console.log(item, functionCode, referenceResult, fromReference, newTriggerRef)
  let state = yield select(x => x.get('shortForm'));
  let startIndex = 0;
  let addCount = 0;
  let calcField = [];
  if (fromReference) {
    state = state.updateIn(['data', 'collect'], tableData => {
      tableData = tableData || List([]);
      startIndex = tableData.size;
      const newData = [];
      _.forEach(referenceResult, (x, xIndex) => {
        // 循环选中项
        let value = { ...item.defaultObj };
        // 初始值
        item.affectFields && item.affectFields.map((affectItem) => {
          // 循环影响字段
          if (_.head(affectItem.formField) === item.tableName) {
            //影响字段表名等于子表名
            const tableField = _.last(affectItem.formField);
            //影响字段名
            value[tableField] = x[affectItem.referenceField];
            // 初始值
            xIndex === 0 && calcField.push(tableField);
            //计算字段
          }
          return true;
        });
        newData.push(value);
        return value;
      });
      addCount = newData.length;
      return tableData.concat(fromJS(newData));
    });
  } else {
    state = state.updateIn(['data', 'collect'], tableData => {
      tableData = tableData || List([]);
      startIndex = tableData.size;
      const newData = referenceResult;
      if (referenceResult && referenceResult.length > 0) {
        calcField = Object.keys(referenceResult[0]);
      }
      addCount = 1;
      console.log('tableData', tableData);

      console.log('tableData.concat(fromJS(newData));', tableData.concat(fromJS(newData)));

      return tableData.concat(fromJS(newData));
    });
  }

  yield put({ type: 'shortForm/setForm', payload: state });
}



function* visibleShortForm(action) {
  let { config, fromId, fromFunctionCode, data, refreshAction, saveAction, selectedIds, selectRows, searchScenario, extraParams, width, height } = action.payload;
  let cacheConfig = yield select(x => x.getIn(['shortForm', 'config', config.shortFormId]));
  if (!cacheConfig) {
    const response = yield call(post, url.fetchShortFormConfig, { key: config.shortFormId });
    if (response && response.result) {
      const { content: contentString, winName, title } = response.result;
      let formConfig = [];
      let defaultFormData = Map({});
      if (contentString) {
        const { scenario, tableField, collect, formulaTriggerMapping, formulaList, supportBatch, isDelete, isNew, } = JSON.parse(contentString);
        if (scenario && scenario.length > 0) {
          defaultFormData = defaultFormData.set('scenario', Map({}));
          const scenarioConfig = [];
          scenario.map((item) => {
            defaultFormData = defaultFormData.setIn(['scenario', item.dataIndex], getItemDefaultValue(item));
            scenarioConfig.push({ ...item, tableName: 'scenario' });
            return true;
          });
          formConfig.push(scenarioConfig);
        }
        if (tableField && collect && collect.length > 0) {
          if (!tableField) {
            message.error('请配置表体字段');
          }
          let totalWidth = 0;
          let defaultObj = {};
          collect.map((item) => {
            item.tableName = 'collect';
            totalWidth = totalWidth + (parseInt(item.width) || 0)
            defaultObj[item.dataIndex] = getItemDefaultValue(item);
            return true;
          });
          formConfig.push({
            tableName: 'collect',
            children: collect,
            title,
            supportBatch,
            totalWidth,
            defaultObj,
            hideEmpty: (!isNew || isNew == 0) ? '1' : 0,
            hideActions: (!isNew || isNew == 0) ? '1' : 0,
            hideDeleteAction: (!isDelete || isDelete == 0) ? '1' : 0,
            mutableDelete: isDelete,
          })
        }
        cacheConfig = Map({
          defaultFormData,
          formConfig,
          winName,
          tableField
        })
        yield put({ type: 'shortForm/initShortFormConfig', payload: { width, height, supportBatch, formConfig, winName, shortFormId: config.shortFormId, tableField, defaultFormData, formulaList: fromJS(formulaList), formulaTriggerMapping: fromJS(formulaTriggerMapping) || Map({}) } })
      } else {
        console.log('获取弹窗配置失败！');
        return;
      }
    } else {
      console.log('获取弹窗配置失败！');
      return;
    }
  }
  let actualData = cacheConfig.get('defaultFormData').mergeIn(['scenario'], data);
  if (config && config.dataSourceApi) {
    const response = yield call(post, config.dataSourceApi, { id: fromId, selectedIds, extraParams, functionCode: fromFunctionCode, selectRows, searchScenario });
    if (response && response.result) {
      const tableField = cacheConfig.get('tableField');
      if (tableField && response.result[tableField]) {
        actualData = actualData.set('collect', fromJS(response.result[tableField]))
        delete response.result[tableField];
      }
      actualData = actualData.mergeIn(['scenario'], fromJS(response.result));
    }
  }
  yield put({ type: 'shortForm/initShortForm', payload: { data: actualData, extraParams, fromId, selectedIds, shortFormId: config.shortFormId, fromFunctionCode, refreshAction, saveAction, saveApi: config.saveApi } });
}

function* saveShortForm(action) {
  const cache = yield select(x => x.get('shortForm'));
  const shortFormId = cache.get('shortFormId');
  const config = cache.getIn(['config', shortFormId]);
  const tableField = config.get('tableField');
  const cacheData = cache.get('data');
  const extraParams = cache.get('extraParams');
  const selectedRowKeys = cache.getIn(['selectedRowKeys', 'collect']) || [];
  const functionCode = cache.get('fromFunctionCode');
  const data = cacheData.get('scenario') ? cacheData.get('scenario').toJSON() : {};
  if (tableField) {
    data[tableField] = cacheData.get('collect').toJSON();
  }
  if (cache.get('saveApi')) {
    if(cache.get('saveApi')=="0"){
      yield put({ type: 'list/changeSelectedRowKeys', payload: { functionCode, selectedRowKeys: [] } });
      yield put({ type: 'shortForm/closeShortForm' });
      return;
    }
    const shortFormResult = yield call(post, cache.get('saveApi'), { ...data, extraParams, id: cache.get('fromId'), selectedIds: cache.get('selectedIds'), selectedRowKeys, functionCode: cache.get('fromFunctionCode') });
    //2021-10-14 16:35:41 弹窗保存刷新详情页面
    if (cache.get('refreshAction')) {
      yield put({ type: cache.get('refreshAction'), payload: { id: cache.get('fromId'), functionCode: cache.get('fromFunctionCode'), shortFormResult: shortFormResult ? shortFormResult.result : undefined } });
    }
  } else if (cache.get('saveAction')) {
    yield put({ type: cache.get('saveAction'), payload: { id: cache.get('fromId'), functionCode: cache.get('fromFunctionCode'), shortFormData: data, shortFormId: cache.get('shortFormId') } })
  }
  yield put({ type: 'list/changeSelectedRowKeys', payload: { functionCode, selectedRowKeys: [] } });
  yield put({ type: 'shortForm/closeShortForm' });
}

function* changeFormData(action) {
  const { item, value, referenceResult } = action.payload;
  let state = yield select(x => x.get('shortForm'));
  state = state.update('data', formData => {
    formData = formData.update(item.tableName, table => table.set(item.dataIndex, value));
    if (item.affectFields) {
      item.affectFields.map(affectitem => {
        if (formData.hasIn(affectitem.formField)) {
          formData = formData.setIn(affectitem.formField, referenceResult[affectitem.referenceField]);
        }
        return true;
      })
    }
    return formData;
  });
  yield put({ type: 'shortForm/setForm', payload: state });
}

function* changeFormTableData(action) {
  const { value, item, index, selectRow } = action.payload;
  let state = yield select(x => x.get('shortForm'));
  state = state.setIn(['data', item.tableName, index, item.dataIndex], value);
  if (item.affectFields) {
    const calcField = [];
    state = state.updateIn(['data', item.tableName, index], tableData => {
      item.affectFields.map((affectItem) => {
        if (_.head(affectItem.formField) === item.tableName) {
          const tableField = _.last(affectItem.formField);
          tableData = tableData.set(tableField, selectRow[affectItem.referenceField]);
          calcField.push(tableField);
        }
        return true;
      })
      return tableData;
    })
    for (let calcFieldIndex = 0; calcFieldIndex < calcField.length; calcFieldIndex++) {
      state = yield call(calculateWithFormula, state, {
        tableName: item.tableName,
        dataIndex: calcField[calcFieldIndex]
      }, state.get('shortFormId'), index, false, sysFormulaEditTypeConst.edit);
    }
  }
  yield put({ type: 'shortForm/setForm', payload: state });
}

function* delayExecFormulaByForm(action) {
  yield delay(500);
  const { item, value } = action.payload;
  let state = yield select(x => x.get('shortForm'));
  state = yield call(calculateWithFormula, state, item, state.get('shortFormId'));
  yield put({ type: 'shortForm/setForm', payload: state });
}

function* delayExecFormulaByTable(action) {
  yield delay(500);
  const { value, item, index } = action.payload;
  let state = yield select(x => x.get('shortForm'));
  state = yield call(calculateWithFormula, state, item, state.get('shortFormId'), index);
  yield put({ type: 'shortForm/setForm', payload: state });
}

function* calculateWithFormula(state, item, shortFormId, childIndex, isDelete = false, sysFormulaEditType = sysFormulaEditTypeConst.edit) {
  const formulaTriggerMapping = state.getIn(['config', shortFormId, 'formulaTriggerMapping']);
  if (!formulaTriggerMapping) {
    return state;
  }
  const formulas = formulaTriggerMapping.getIn([item.tableName, item.dataIndex]);
  if (!formulas || formulas.size <= 0) {
    return state;
  }

  const formulaList = state.getIn(['config', shortFormId, 'formulaList']);
  let formData = state.get('data');
  for (let formulaIndex = 0; formulaIndex < formulas.size; formulaIndex++) {
    const formula = formulaList.get(formulas.get(formulaIndex));
    if (formula.get('type') === formulaType.normal.value) {
      let firstKeyPath = formula.get('first').toJSON();
      let secondKeyPath = formula.get('second').toJSON();
      let resultKeyPath = formula.get('result').toJSON();
      if (childIndex >= 0) {
        firstKeyPath.splice(1, 0, childIndex);
        secondKeyPath.splice(1, 0, childIndex);
        resultKeyPath.splice(1, 0, childIndex);
      }
      const operation = formula.get('operation');
      let first = formData.getIn(firstKeyPath);
      let second = formData.getIn(secondKeyPath);
      let result = undefined;
      if (typeof first !== 'number') {
        first = parseFloat(first);
      }
      if (typeof second !== 'number') {
        second = parseFloat(second);
      }
      if ((first || first === 0) && (second || second === 0)) {
        if (operation === operationType['+'].value) {
          result = parseFloat(math.eval(`${first} + ${second}`).toString());
        } else if (operation === operationType['-'].value) {
          result = parseFloat(math.eval(`${first} - ${second}`).toString());
        } else if (operation === operationType['*'].value) {
          if (first === 0 || second === 0) {
            result = 0
          } else {
            result = parseFloat(math.eval(`${first} * ${second}`).toString());
          }
        } else if (operation === operationType['/'].value) {
          if (first === 0 || second === 0) {
            result = 0
          } else {
            result = parseFloat(math.eval(`${first} / ${second}`).toString());
            result = result.toFixed(window.numberFixedConst);
          }
        }
        state = state.setIn(['data', ...resultKeyPath], result);
        if (childIndex >= 0) {
          resultKeyPath.splice(1, 1);
        }
        state = yield call(calculateWithFormula, state, {
          tableName: resultKeyPath[0],
          dataIndex: resultKeyPath[1]
        }, shortFormId, childIndex);
      }
    } else if (formula.get('type') === formulaType.inner.value) {
      if (childIndex >= 0) {
        const funcName = formula.get('funcName');
        let firstKeyPath = formula.get('first').toJSON();
        let resultKeyPath = formula.get('result').toJSON();
        const fieldName = firstKeyPath.pop();
        const calcArray = formData.getIn(firstKeyPath);
        let result = 0;
        calcArray.map(item => {
          let itemValue = item.get(fieldName);
          if (typeof itemValue !== 'number') {
            itemValue = parseFloat(itemValue);
          }
          if (funcName === functionType.max.value) {
            result = Math.max(result, itemValue) || 0;
          } else if (funcName === functionType.min.value) {
            result = Math.min(result, itemValue) || 0;
          } else if (funcName === functionType.sum.value) {
            result += (itemValue || 0);
          }
          return true;
        });
        state = state.setIn(['data', ...resultKeyPath], result);
        state = yield call(calculateWithFormula, state, {
          tableName: resultKeyPath[0],
          dataIndex: resultKeyPath[1]
        }, shortFormId);
      } else {
        console.log(`内置函数只适用于子表`);
      }
    } else if (formula.get('type') === formulaType.custom.value) {
      const funcName = formula.get('funcName');
      if (window[funcName] && typeof window[funcName] === 'function') {
        const config = yield select(x => x.getIn(['shortForm', 'config', shortFormId]));
        const tableField = config.get('tableField');
        let params = { sysFormulaIndex: childIndex, sysFormulaEditType, ...formData.get('scenario').toJSON() };
        if (tableField && formData.get(tableField)) {
          params[tableField] = formData.get(tableField).toJSON();
        }
        const result = window[funcName](params);
        if (tableField) {
          formData = formData.set(tableField, result[tableField]);
          delete result[tableField];
        }
        formData = formData.set('scenario', fromJS(result));
        state = state.set('data', formData);
      } else {
        console.log(`找不到${funcName}方法`);
      }
    } else if (formula.get('type') === formulaType.api.value) {
      const apiUrl = formula.get('apiUrl');
      if (apiUrl && !isDelete) {
        try {
          const config = yield select(x => x.getIn(['shortForm', 'config', shortFormId]));
          const tableField = config.get('tableField');
          let params = { ...formData.get('scenario').toJSON(), sysFormulaIndex: childIndex, sysFormulaEditType };
          if (tableField && formData.get(tableField)) {
            params[tableField] = formData.get(tableField).toJSON();
          }
          const response = yield call(post, apiUrl, params);
          if (response && response.result) {
            if (tableField && response.result[tableField]) {
              state = state.setIn(['data', 'collect'], fromJS(response.result[tableField]));
              delete response.result[tableField];
            }
            state = state.mergeIn(['data', 'scenario'], fromJS(response.result));
            let firstKeyPath = formula.get('calcFirst');
            let secondKeyPath = formula.get('calcSecond');
            let thirdKeyPath = formula.get('calcThird');
            let forthKeyPath = formula.get('calcForth');
            if (firstKeyPath && firstKeyPath.size > 0) {
              state = yield call(calculateWithFormula, state, {
                tableName: firstKeyPath.get(0),
                dataIndex: firstKeyPath.get(1),
              }, shortFormId, childIndex);
            }
            if (secondKeyPath && secondKeyPath.size > 0) {
              state = yield call(calculateWithFormula, state, {
                tableName: secondKeyPath.get(0),
                dataIndex: secondKeyPath.get(1),
              }, shortFormId, childIndex);
            }
            if (thirdKeyPath && thirdKeyPath.size > 0) {
              state = yield call(calculateWithFormula, state, {
                tableName: thirdKeyPath.get(0),
                dataIndex: thirdKeyPath.get(1),
              }, shortFormId, childIndex);
            }
            if (forthKeyPath && forthKeyPath.size > 0) {
              state = yield call(calculateWithFormula, state, {
                tableName: forthKeyPath.get(0),
                dataIndex: forthKeyPath.get(1),
              }, shortFormId, childIndex);
            }
          }
        } catch (e) {
          console.log(433,e);
        }
      }
    }
  }
  return state;
}
