import { Map, List } from 'immutable';

const initState = Map({
  loading: false,
  menu: undefined,
  menuProps: undefined,
  lang: localStorage.getItem('lang') ? localStorage.getItem('lang') : 'zh',
  tabs: List([
    {
      id: 'dashboard_0001',
      menuName: '我的桌面',
      nameEN: 'Home',
      hideClose: true,
      type: 'static',
      menuUrl: '/dashboard/dashboard'
    }
  ]),
  activeTab: {},
  fromTab: {},
  defaultOpenKeys: [],
  permissions: [],
  basicInfo: {},
  unDealMessageCount: 0,
  warningMessages: [],
  dealWithMessages: [],
  noticeMessages: [],
  menuList: [],
  customConfig: {},
  popEdit: {},
  warningConfig: [],
  customPopEdit: Map({}),
  warningList: false,
  singleDetail: {},
  calendarState: {},
  dictionaryAndFunctionalModeling:{},
  showMore: false
});

export default (state = initState, action) => {
  switch (action.type) {
    case 'system/fetchMenuSuccess':
      return state.set('menu', action.payload.menus)
        .set('defaultOpenKeys', action.payload.defaultOpenKeys)
        .set('originalMenu', action.payload.originalMenu)
        .set('menuList', action.payload.menuList)
        .set('menuLast', action.payload.menuLast)
        .set('reportMenus', action.payload.reportMenus)
        .set('quickMenus', action.payload.quickMenus);
    case 'system/defaultOpenMenus': {
      const { defaultOpenMenus } = action.payload;
      let tabs = state.get('tabs');
      tabs = tabs.concat(List(defaultOpenMenus));
      return state.set('tabs', tabs);
    }
    case 'system/changeQuickMenus':
      return state.set('quickMenus', action.payload.quickMenus);
    case 'system/changeCustomConfig': {
      if (action.payload.content.quickMenuItems) {
        state = state.set('quickMenus', action.payload.content.quickMenuItems);
      }
      return state.set('customConfig', action.payload.content)
    }
    case 'system/changeActiveTab':
      return state.set('activeTab', action.payload);
    // 保存上一个选中的Tab;
    case 'system/fromTab':
      return state.set('fromTab', action.payload);
    case 'system/changeTabs':
      return state.set('tabs', action.payload);
    case 'system/changeLoadingStatus':
      return state.set('loading', action.payload);
    case 'system/setPermissions':
      return state.set('permissions', action.payload);
    case 'system/setBasicInfo':
      return state.set('basicInfo', action.payload);
    case 'system/updateMessages': {
      const { unDealMessageCount, warningMessages, dealWithMessages, noticeMessages } = action.payload;
      return state.set('unDealMessageCount', unDealMessageCount)
        .set('warningMessages', warningMessages)
        .set('dealWithMessages', dealWithMessages)
        .set('noticeMessages', noticeMessages);
    }
    case 'system/increaseUnreadMessage': {
      let unDealMessageCount = state.get('unDealMessageCount');
      return state.set('unDealMessageCount', unDealMessageCount++)
    }
    case 'system/logoutSuccess':
      return initState;
    case 'system/changePopEdit':
      return state.set('popEdit', {
        functionCode: action.payload.functionCode,
        id: action.payload.id,
        width: action.payload.width,
        height: action.payload.height,
        type: action.payload.type
      });
    case 'system/showCustomPop': {
      const { name, functionCode, callBackAction } = action.payload;
      return state.set(name, Map({ visible: true, functionCode, callBackAction }));
    }
    case 'system/hideCustomPop': {
      const { name } = action.payload;
      return state.set(name, { visible: false });
    }
    case 'system/visibleCustomPopEdit': {
      const { key, record, functionCode } = action.payload;
      return state.setIn(['customPopEdit', 'activeKey'], key)
        .setIn(['customPopEdit', 'record'], record)
        .setIn(['customPopEdit', 'functionCode'], functionCode);
    }
    case 'system/hideCustomPopEdit': {
      return state.set('customPopEdit', Map({}))
    }
    case 'system/changeHomeTab': {
      return state.setIn(['tabs', 0], action.payload.tab);
    }
    case 'system/showPreview': {
      if (action.payload.id && action.payload.functionCode) {
        return state.set('previewFunctionCode', action.payload.functionCode).set('previewDetailId', action.payload.id);
      }
      return state
    }
    case 'system/closePreviewPage': {
      return state.set('previewFunctionCode', '').set('previewDetailId', '');
    }
    case 'system/setUserWarningConfig': {
      return state.set('warningConfig', action.payload.warningConfig)
        .set('allWarningCount', action.payload.allWarningCount);
    }
    case 'system/switchLanguage': {
      const lang = state.get('lang');
      localStorage.setItem('lang', lang === 'zh' ? 'en' : 'zh')
      return state.set('lang', lang === 'zh' ? 'en' : 'zh');
    }
    case 'system/setMenuProps': {
      return state.set('menuProps', action.payload);
    }
    case 'system/warningList': {
      return state.set('warningList', action.payload);
    }
    case 'system/setSingleDetail': {
      return state.set('singleDetail', action.payload);
    }
    case 'system/SaveDictionaryAndFunctionalModeling': {
      return state.set('dictionaryAndFunctionalModeling', action.payload);
    }
    // 保存日历状态
    case 'system/calendarState': {
      return state.set('calendarState', action.payload);
    }
    case 'system/switchShowMore': {
      return state.set('showMore', action.payload);
    }
    default:
      return state;
  }
}
