import { takeEvery, put, call, select, fork } from "redux-saga/effects"
import { delay } from "redux-saga"
import { push } from "react-router-redux"
import { routePrefix } from "../../common/config"
import { functionTypeMapping, functionCodeType, messageTypeConst, editPagePositionConst } from "../../common/const"
import { message } from "antd"
import { convertListToObject, ConfirmModal } from "../../utils"
import { post, url } from "../../service/api"
import { List, Map } from "immutable"
import _ from "lodash"
import { initDashboardItemData } from "../dashboard/sagas"
import en from "../../locale/en_US"
import zh from "../../locale/zh_CN"

/**
 * method_name: fetchMenu
 * param: none
 * describe: 获取页面上menu的数据源
 * create_user: 王友生
 * create_date: 2018/3/10 0010
 * create_time: 10:54
 **/

const getRouteByTab = tab => {
  let route = `${routePrefix}/main${tab.menuUrl}`
  if (tab.type === "static" && tab.pageType) {
    route = route + "/" + tab.pageType
  } else if (tab.type === "list") {
    route = route + "/list/" + tab.functionCode
  } else if (tab.type === "edit") {
    route = route + "/edit/" + tab.functionCode
  } else if (tab.type === "detail") {
    route = route + "/detail/" + tab.functionCode
  } else if (tab.type === "report") {
    route = route + "/report/" + tab.functionCode
  } else if (tab.type === "advanceList") {
    route = route + "/advance-list/" + tab.functionCode
  } else if (tab.type === "treeForm") {
    route = route + "/tree-form/" + tab.functionCode
  } else if (tab.type === "customReports") {
    route = route + "/customReports/" + tab.functionCode
  } else if (tab.type === "customReportCharts") {
    route = route + "/customReportCharts/" + tab.functionCode
  }
  if (tab.dataId) {
    route = route + "/" + tab.dataId
  }
  return route
}

export const watchFetchMenu = function* () {
  yield takeEvery("system/initMenu", init)
  yield takeEvery("system/initSysConfig", initSysConfig)
  yield takeEvery("system/saveQuickMenu", saveQuickMenu)
  yield takeEvery("system/goToWarningDetail", goToWarningDetail)
  yield takeEvery("system/initWarningConfig", initWarningConfig)
}

function* init(action) {
  yield fork(initBasicInfo)
  yield fork(initPermission)
  yield fork(initMenu, action)
  yield fork(initSysConfig)
  yield fork(fetchNoticeList)
  yield fork(initWarningConfig)
}

export const watchFetchNoticeList = function* () {
  yield takeEvery("system/fetchNoticeList", fetchNoticeList)
}

function* fetchNoticeList() {
  const basicInfoString = localStorage.getItem("basicInfo")
  if (!basicInfoString) {
    return
  }
  const basicInfo = JSON.parse(basicInfoString)
  const response = yield call(post, url.getSysMessageList, { channelId: basicInfo.id }, "", "", false)
  if (response && response.result) {
    const warningMessages = []
    const dealWithMessages = []
    const alertMessages = []
    const noticeMessages = []
    response.result.map(item => {
      if (item.msgType === messageTypeConst.dealWith.value) {
        dealWithMessages.push(item)
      } else if (item.msgType === messageTypeConst.warning.value) {
        warningMessages.push(item)
      } else if (item.msgType === messageTypeConst.notice.value) {
        noticeMessages.push(item)
      } else if (item.msgType === messageTypeConst.alert.value) {
        alertMessages.push(item)
      }
      return true
    })
    yield put({
      type: "system/updateMessages",
      payload: {
        unDealMessageCount: warningMessages.length + dealWithMessages.length + noticeMessages.length + alertMessages.length,
        warningMessages,
        dealWithMessages,
        alertMessages,
        noticeMessages
      }
    })
  }
}

function* initWarningConfig() {
  const basicInfoString = localStorage.getItem("basicInfo")
  if (!basicInfoString) {
    return
  }
  const basicInfo = JSON.parse(basicInfoString)
  const warningConfigRes = yield post("/sysCache/getCachePersonalWarningResult", { key: basicInfo.id })
  const warningConfigResNonZero = []
  let allWarningCount = 0
  if (warningConfigRes.result) {
    warningConfigRes.result.map(item => {
      if (item.num != 0) {
        allWarningCount += item.num
        warningConfigResNonZero.push(item)
      }
    })
  }
  yield put({
    type: "system/setUserWarningConfig",
    payload: { warningConfig: warningConfigResNonZero, allWarningCount }
  })
}

function* goToWarningDetail(action) {
  const { item } = action.payload
  const activeTab = yield select(x => x.getIn(["system", "activeTab"]))
  // 当前功能直接请求列表
  if (activeTab.functionCode === item.funCode) {
    yield put({
      type: "list/fetchListDataByFunctionCode",
      payload: { functionCode: item.funCode, queryViewId: [{ id: item.btnId }] } //兼容老模式
    })
  } else {
    // 跳转
    yield put({
      type: "system/jumpToFunctionWidthScenario",
      payload: {
        functionCode: item.funCode,
        scenario: { queryViewId: JSON.stringify([{ id: item.btnId }]) } //兼容老模式
      }
    })
    yield put({
      type: "system/warningList",
      payload: true
    })
  }
}

function* initBasicInfo() {
  const basicInfoString = localStorage.getItem("basicInfo")
  if (!basicInfoString) {
    return
  }

  const basicInfo = JSON.parse(basicInfoString)
  yield put({ type: "system/setBasicInfo", payload: basicInfo })
}

function* initPermission() {
  const permissionsString = localStorage.getItem("permissions")
  if (!permissionsString) {
    return
  }

  const permissions = JSON.parse(permissionsString)
  yield put({ type: "system/setPermissions", payload: permissions })
}

function* initMenu(action) {
  // const response = yield call(fetchMenuRequest);
  const menuString = localStorage.getItem("menus")
  if (!menuString) {
    message.error("您没有可用的菜单！")
    yield put(push("/login"))
    return
  }
  const menuList = JSON.parse(menuString)
  const menus = []
  let selectItem
  let defaultItem
  let defaultOpenKeys = []
  let quickMenus = []
  let reportMenus = []
  let defaultOpenMenus = []
  menuList.map(item => {
    item.menuUrl = item.funUrl || ""
    item.menuName = item.name
    item.functionCode = item.funCode
    // 配置的功能
    const mapping = functionTypeMapping[item.funType]
    if (mapping) {
      item.type = mapping.type
    }
    let hasParent = false
    let reportItem = []
    menuList.map(childItem => {
      item.items = item.items || []
      if (childItem.pid === item.id) {
        item.items.push(childItem)
        if (childItem.ynReport === "1") {
          reportItem.push(childItem)
        }
      }
      if (childItem.id === item.pid) {
        hasParent = true
      }
      return true
    })
    if (!hasParent) {
      menus.push(item)
    }

    // if (item.ynExpand) {
    //   defaultOpenKeys.push(item.id);
    // }

    const url = getRouteByTab(item)
    if (action.pathname.indexOf(url) >= 0 && url !== "/main") {
      selectItem = item
    }
    if (item.ynQuick === "1") {
      if (item.funType === functionCodeType.list.value) {
        quickMenus.push({ ...item, type: "edit" })
      } else {
        quickMenus.push({ ...item })
      }
    }
    if (item.ynStart === "1") {
      defaultItem = item
    }
    if (item.ynPage === "1") {
      defaultOpenMenus.push({ ...item })
    }
    if (reportItem.length > 0 && !hasParent) {
      reportMenus.push({ ...item, items: reportItem })
    }
    return true
  })
  const originalMenu = convertListToObject(menuList, "functionCode")
  let allotMenu = (allotMenus, allotKeyPath) => {
    allotMenus.map((allotItem, allotIndex) => {
      const currentKeyPath = [...allotKeyPath]
      currentKeyPath.splice(0, 0, allotItem["id"])
      const hasChildren = allotItem["items"] && allotItem["items"].length > 0
      if (allotItem.ynSpread === "1" && hasChildren) {
        defaultOpenKeys.push(JSON.stringify({ hasChildren, currentKeyPath }))
      }
      hasChildren && allotMenu(allotItem["items"], currentKeyPath)
    })
  }
  yield put({ type: "system/defaultOpenMenus", payload: { defaultOpenMenus } })
  yield delay(200)
  allotMenu(menus, [])
  // 菜单末级
  let menuLast = []
  function recursionMenu(menus, pid) {
    try {
      menus.map(item => {
        if (item.items.length < 1 && item.ynHidden !== 1) {
          menuLast.push(item)
        } else if (item.items.length > 0) {
          if (item.pid != -1) {
            item.items.map(itemChild => {
              itemChild.pid = itemChild.pid + "," + item.pid
            })
          }

          recursionMenu(item.items)
        }
      })
    } catch (err) {
      console.log("292", menus)
    }
  }
  recursionMenu(menus)

  yield put({
    type: "system/fetchMenuSuccess",
    payload: {
      menus,
      menuLast,
      defaultOpenKeys,
      originalMenu,
      quickMenus,
      menuList,
      reportMenus
    }
  })
  yield fork(initSystemConfig, originalMenu)
  if (defaultItem) {
    defaultItem.hideClose = true
    yield put({ type: "system/changeHomeTab", payload: { tab: defaultItem } })
  }
  if (selectItem) {
    yield call(addTabs, selectItem)
  } else {
    if (defaultItem) {
      yield call(addTabs, defaultItem)
    } else {
      yield call(tabChange, {
        payload: {
          id: "dashboard_0001",
          type: "static",
          menuUrl: "/dashboard/dashboard"
        }
      })
    }
  }
}
// 首页报表
function* initSystemConfig(originalMenu) {
  const basicInfoString = localStorage.getItem("basicInfo")
  if (!basicInfoString) {
    return
  }

  const basicInfo = JSON.parse(basicInfoString)
  const data = yield post("/sysCache/getCachePersonalWorkbench", {
    key: basicInfo.id
  })
  let leftDashboardItems = []
  let rightDashboardItems = []
  let dashboardItemConfig = {}
  if (data && data.result && data.result.id) {
    const { pwbContent } = data.result
    if (pwbContent) {
      const content = JSON.parse(pwbContent)
      if (content.dashboardConfig) {
        leftDashboardItems = content.dashboardConfig.leftDashboardItems
        rightDashboardItems = content.dashboardConfig.rightDashboardItems
        dashboardItemConfig = content.dashboardConfig.dashboardItemConfig
      }
      yield put({ type: "system/changeCustomConfig", payload: { content } })
    }
  }
  let sourceMenus = List([])
  const dashboardConfig = localStorage.getItem("dashboardConfig")
  let dashboardItems = []
  if (dashboardConfig) {
    dashboardItems = JSON.parse(dashboardConfig)["dashboardItems"]
  }
  Object.keys(originalMenu).map(key => {
    const menu = originalMenu[key]
    if (
      menu &&
      (menu.funType === functionCodeType.list.value ||
        menu.funType === functionCodeType.advanceList.value ||
        menu.funType === functionCodeType.report.value) &&
      (!menu.items || menu.items.length === 0)
    ) {
      if (!!_.find(leftDashboardItems, { id: menu.id }) || !!_.find(leftDashboardItems, { id: menu.id })) {
        return true
      }
      sourceMenus = sourceMenus.push({
        ...menu,
        key: menu.funCode,
        title: menu.menuName
      })
    }
    return true
  })
  for (let i = 0; i < leftDashboardItems.length; i++) {
    yield fork(initDashboardItemData, leftDashboardItems[i])
  }
  for (let i = 0; i < rightDashboardItems.length; i++) {
    yield fork(initDashboardItemData, rightDashboardItems[i])
  }
  const newDashboardItemConfig = {
    ...convertListToObject(dashboardItems, "key"),
    ...dashboardItemConfig
  }

  yield put({
    type: "dashboard/initDashboardConfigSuccess",
    payload: {
      id: data.result.id,
      configurableMenus: Map({
        sourceMenus,
        leftPartMenus: List(leftDashboardItems),
        rightPartMenus: List(rightDashboardItems)
      }),
      dashboardItems,
      leftDashboardItems,
      rightDashboardItems,
      dashboardItemConfig: newDashboardItemConfig
    }
  })
}

function* saveQuickMenu(action) {
  const { quickMenuItems } = action.payload
  const customConfig = yield select(x => x.getIn(["system", "customConfig"]))

  const dashboard = yield select(x => x.get("dashboard"))
  const id = dashboard.get("id")
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  if (!id) {
    yield post(url.saveDashboardConfig, {
      userId: basicInfo.id,
      pwbType: "001",
      pwbContent: JSON.stringify({ ...customConfig, quickMenuItems })
    })
  } else {
    yield post(url.modDashboardConfig, {
      id,
      userId: basicInfo.id,
      pwbType: "001",
      pwbContent: JSON.stringify({ ...customConfig, quickMenuItems })
    })
  }
  yield put({
    type: "system/changeQuickMenus",
    payload: { quickMenus: quickMenuItems }
  })
}

export const watchTabChange = function* () {
  yield takeEvery("system/tabChange", tabChange)
}

/**
 * method_name: tabChange
 * param: tab的key
 * describe: 切换tab
 * create_user: 王友生
 * create_date: 2018/3/10 0010
 * create_time: 10:54
 **/

function* tabChange(action) {
  if (action.payload.functionCode) {
    yield put({
      type: "workflow/initWorkflow",
      payload: { functionCode: action.payload.functionCode }
    })
  }
  yield put({ type: "system/changeActiveTab", payload: action.payload })
  yield put(push(getRouteByTab(action.payload)))
}

export const watchMenuClick = function* () {
  yield takeEvery("system/menuClick", menuClick)
}

/**
 * method_name: menuClick
 * param: menu的keypath
 * describe: 点击操作menu，检查page导航是否存在此页签，不存在则添加，存在则宣章
 * create_user: 王友生
 * create_date: 2018/3/10 0010
 * create_time: 10:54
 **/

function* menuClick(action) {
  const systemReducer = yield select(x => x.get("system"))
  const menu = systemReducer.get("menu")
  let tabs = systemReducer.get("tabs")
  let keyPath = undefined
  var selectItem = {}
  var currentLevelMenu = menu
  if (Array.isArray(action.payload)) {
    keyPath = action.payload
  } else {
    keyPath = action.payload.keyPath
    yield put({ type: "system/setMenuProps", payload: action.payload.param })
  }
  try {
    for (var i = keyPath.length - 1; i >= 0; i--) {
      let key = keyPath[i]
      if (i === 0) {
        selectItem = currentLevelMenu.filter(item => item.id === key)[0]
      } else {
        currentLevelMenu = currentLevelMenu.filter(item => item.id === key)[0].items
      }
    }
  } catch (err) {
    return
  }

  if (!selectItem || selectItem == null) {
    message.error("账号无权限请联系管理员配置!")
    yield put({
      type: "system/changeActiveTab",
      payload: tabs.get(0)
    })
    return;
  }

  if (selectItem.menuUrl && selectItem.menuUrl.indexOf("http") >= 0) {
    if (selectItem.menuUrl.indexOf("{cfIp}")) {
      window.open(selectItem.menuUrl.replace("{cfIp}", window.location.hostname + ":" + window.location.port), "_blank")
    } else {
      window.open(selectItem.menuUrl, "_blank")
    }
    return
  }
  if (selectItem.ynForm === "1") {
    const data = yield post("/senior/getDataIdByFunCodeAndCondition", {
      funCode: selectItem.functionCode,
      formCondition: selectItem.formCondition
    })
    if (data.result && data.result.id && data.result.id.length > 0) {
      yield put({
        type: "system/jumpToEditPage",
        payload: data.result.id,
        functionCode: selectItem.functionCode,
        ynMenuForm: selectItem.ynForm
      })
    } else {
      message.error("单据编码不存在,不能直接进入表单")
    }
  } else {
    //tabs增加带参
    if (action.payload && action.payload.selectItemItems) {
      selectItem = { ...selectItem, ...action.payload.selectItemItems }
    }
    yield call(addTabs, selectItem)
  }
}

export const watchAddTab = function* () {
  yield takeEvery("system/addTab", addOneTab)
}

function* addOneTab(action) {
  yield call(addTabs, action.payload)
}

/**
 * method_name: addTabs
 * param: tab
 * describe: 向tab的数据源中插入一条数据
 * create_user: 王友生
 * create_date: 2018/3/10 0010
 * create_time: 10:54
 **/

export const addTabs = function* (tab) {
  const systemReducer = yield select(x => x.get("system"))
  let activeTab = systemReducer.get("activeTab")
  let tabs = systemReducer.get("tabs")
  //if (window.navigationMultiFunction === "1" && false) {
  // 多行导航
  //} else {
  if (tab.cache && tab.cache.id) {
    delete tab.showTitleName
  }
  const filterItem = tabs.filter(x => getRouteByTab(tab) === getRouteByTab(x))
  // console.log('filterItem', filterItem.toJSON());
  //1.30详情多页签
  const isId = tabs.filter(item => {
    if (item.cache && tab.cache && item.cache.id == tab.cache.id) {
      if (item.showTitleName) {
        tab.showTitleName = item.showTitleName
      }
      return item
    }
    // return item.cache && tab.cache && item.cache.id == tab.cache.id
  })
  // console.log('tab', tab);
  // console.log('tabs', tabs);
  // console.log('isId', isId.toJSON());
  // console.log('tabindex', tabs.indexOf(tab))
  // console.log('filterItem', filterItem.toJSON());
  if (filterItem.size === 0 || (tab.type === "detail" && isId.size < 1)) {
    const listIndex = tab.type === "edit" && tabs.findIndex(v => v.functionCode === tab.functionCode && v.type === "list")
    let editIndex = false
    if (tab.type === "edit") {
      if (filterItem.length > 0) {
        let index = tabs.indexOf(filterItem[0])
        tabs = tabs.delete(index)
      }
      if (tab.cache.id != "-1") {
        // 修改
        editIndex = tabs.findIndex(v => v.functionCode === tab.functionCode && v.cache && v.cache.id === tab.cache.id)
      } else {
        editIndex = tabs.findIndex(
          // 新建
          v => v.functionCode === tab.functionCode && v.type === "detail"
        )
      }
    }
    let detailIndex = false
    if (tab.type === "detail") {
      // 进入的是详情页面保存一下来源页面;
      yield put({ type: "system/fromTab", payload: activeTab })
      // 如果有详情插入最后一个详情后
      if (tabs.findIndex(v => v.functionCode === tab.functionCode && v.type === "detail") > 0) {
        detailIndex = tabs.findLastIndex(v => v.functionCode === tab.functionCode && v.type === "detail")
      } else {
        // 没有插入列表
        detailIndex = tabs.findIndex(v => v.functionCode === tab.functionCode && v.type === "list")
      }
    }

    if (detailIndex && detailIndex !== -1) {
      tabs = List.of(...tabs.slice(0, detailIndex + 1), tab, ...tabs.slice(detailIndex + 1, tabs.length))
    } else if (editIndex && editIndex !== -1) {
      tabs = List.of(...tabs.slice(0, editIndex + 1), tab, ...tabs.slice(editIndex + 1, tabs.length))
    } else if (listIndex && listIndex !== -1) {
      tabs = List.of(...tabs.slice(0, listIndex + 1), tab, ...tabs.slice(listIndex + 1, tabs.length))
    } else {
      // 如果进的是请的是详情获取当前active的位置索引，详情插在其后面
      if (activeTab.menuUrl != "/dashboard/dashboard" && tab.type == "detail") {
        let suoyon
        tabs.forEach((item, index) => {
          if (item.menuUrl == activeTab.menuUrl) {
            suoyon = index
          }
        })
        tabs = tabs.splice(suoyon + 1, 0, tab)
      } else {
        // 不是详情就继续插在最后面
        tabs = tabs.push(tab)
      }
    }
  } else {
    let index = tabs.indexOf(filterItem.get(0))
    //2021-11-25 15:57:07 多详情二次跳转
    if (tab.type === "detail" && tab.cache) {
      tabs.map((item, idx) => {
        if (item.cache && item.cache.id == tab.cache.id) {
          index = idx
        }
      })
    }
    // if (tab.type === 'edit') {
    //   tabs = tabs.delete(index);
    //   if (tab.cache.id != '-1') {
    //     // 修改
    //     index = tabs.findIndex(
    //       (v) => v.functionCode === tab.functionCode && v.cache && v.cache.id === tab.cache.id
    //     );
    //   } else {
    //     index = tabs.findIndex(
    //       // 新建
    //       (v) => v.functionCode === tab.functionCode && v.type === "detail"
    //     );
    //   }
    // }

    tabs = tabs.set(index, tab)
  }
  // }

  if (tab.functionCode) {
    yield put({
      type: "workflow/initWorkflow",
      payload: { functionCode: tab.functionCode }
    })
  }
  if (tab.funType === functionCodeType.advanceForm.value) {
    yield put({
      type: "form/fetchFormConfigByFunctionCode",
      payload: {
        id: "-1",
        functionCode: tab.functionCode,
        withoutDefault: true
      }
    })
  }
  yield put({ type: "system/changeTabs", payload: tabs })
  yield put({ type: "system/changeActiveTab", payload: tab })
  yield put(push(getRouteByTab(tab)))
}

export const watchAddNewTabWithoutMenu = function* () {
  yield takeEvery("system/addNewTabWithoutMenu", addNewTabWithoutMenu)
}

/**
 * method_name: addTabs
 * param: tab
 * describe: 添加一个没有在menu上直接显示的tab页签
 * create_user: 王友生
 * create_date: 2018/3/10 0010
 * create_time: 10:54
 **/

export const addNewTabWithoutMenu = function* (action) {
  const { menuName, menuUrl, cache, nameEN } = action.payload
  const systemReducer = yield select(x => x.get("system"))
  const newTab = {
    menuUrl,
    menuName,
    cache,
    nameEN
  }
  yield call(addTabs, newTab)
}

export const watchTabRemove = function* () {
  yield takeEvery("system/tabRemove", removeTab)
}

export const watchRemoveActiveTab = function* () {
  yield takeEvery("system/removeActiveTab", removeActiveTab)
}

function* removeActiveTab() {
  const systemReducer = yield select(x => x.get("system"))
  let activeTab = systemReducer.get("activeTab")
  yield call(removeTab, { payload: activeTab })
}

/**
 * method_name: removeTab
 * describe: 移除tab，并设置一个新的选中的tab
 * create_user: 王友生
 * create_date: 2018/3/10 0010
 * create_time: 10:54
 **/

function* removeTab(action) {
  const systemReducer = yield select(x => x.get("system"))
  let tabs = systemReducer.get("tabs")
  let activeTab = systemReducer.get("activeTab")
  let fromTab = systemReducer.get("fromTab")
  let lang = systemReducer.get("lang")
  if (
    (action.payload.funType === functionCodeType.advanceList.value || action.payload.funType === functionCodeType.list.value) &&
    action.payload.type === "edit" &&
    action.checkEdit
  ) {
    const result = yield ConfirmModal({
      title: lang == "en" ? en.locale_promptTitle : zh.locale_promptTitle,
      content: lang == "en" ? en.locale_promptContent : zh.locale_promptContent
    })
    if (!result) {
      return
    }
  }
  const index = tabs.indexOf(action.payload)
  //2021-11-08 15:58:06 推单不返回上一级,增加推单判断
  if (action.payload === activeTab) {
    if (!action.payload.relation || !action.save) {
      let selectItem = tabs.get(index - 1)

      // 删除的tab不是详情,删除的是来源的tab需要清空 来源状态
      if (action.payload.menuUrl === fromTab.menuUrl) {
        let obg = {}
        yield put({ type: "system/fromTab", payload: obg })
      }

      // 判断去除的tab是不是详情页面,如果是详情页面,active变成来源页面
      if (Object.keys(fromTab).length != 0 && action.payload.type == "detail") {
        tabs.toJS().forEach(item => {
          if (item.menuName == fromTab.menuName) {
            selectItem = fromTab
          }
        })
      }
      yield put({ type: "system/changeActiveTab", payload: selectItem })
      yield put(push(getRouteByTab(selectItem)))
    }
  }
  if (index !== undefined) {
    // 删除的tab不是详情,删除的是来源的tab需要清空 来源状态
    if (action.payload.menuUrl === fromTab.menuUrl) {
      let obg = {}
      yield put({ type: "system/fromTab", payload: obg })
    }
    tabs = tabs.delete(index)
  }
  yield put({ type: "system/changeTabs", payload: tabs })
  yield put({ type: "removeTabSuccess", payload: action.payload })
}

export const watchTabsRemove = function* () {
  yield takeEvery("system/tabsRemove", removeTabs)
}

/**
 * method_name: removeTabs
 * describe: 批量移除tab, 进入home页面
 * create_user: 王友生
 * create_date: 2018/3/10 0010
 * create_time: 10:54
 **/

function* removeTabs(action) {
  const systemReducer = yield select(x => x.get("system"))
  let tabs = systemReducer.get("tabs")
  let activeTab = systemReducer.get("activeTab")
  let deleteSelectedTab = false
  action.payload.map(item => {
    const index = tabs.indexOf(item)
    if (index !== undefined) {
      tabs = tabs.delete(index)
    }
    if (item === activeTab) {
      deleteSelectedTab = true
    }
    return true
  })
  if (deleteSelectedTab) {
    const selectItem = tabs.get(0)
    yield put({ type: "system/changeActiveTab", payload: selectItem })
    yield put(push(getRouteByTab(selectItem)))
  }
  yield put({ type: "system/changeTabs", payload: tabs })
}

export const watchSaveCurrentTabCache = function* () {
  yield takeEvery("system/saveCurrentTabCache", saveCurrentTabCache)
}

function* saveCurrentTabCache(action) {
  const systemReducer = yield select(x => x.get("system"))
  let tabs = systemReducer.get("tabs")
  let activeTab = action.activeTab
  const index = tabs.indexOf(activeTab)
  if (index && index != -1) {
    tabs = tabs.update(index, tab => {
      tab.cache = action.payload
      return tab
    })
  }
  yield put({ type: "system/changeTabs", payload: tabs })
}

export const watchJumpToDetailPage = function* () {
  yield takeEvery("system/jumpToDetailPage", jumpToDetailPage)
  yield takeEvery("system/refreshCurrentPageData", refreshCurrentPageData)
}

function* jumpToDetailPage(action) {
  const { functionCode, id, params } = action.payload
  if (!id) {
    return
  }
  if (!functionCode) {
    return
  }
  try {
    const originalMenu = yield select(x => x.get("system").get("originalMenu"))
    const menu = originalMenu[functionCode]
    if (!menu) {
      message.error("无权限访问或页面不存在!")
      return
    }
    let newActiveTab = { ...menu, hideClose: false }
    newActiveTab.type = "detail"
    newActiveTab.menuName = newActiveTab.menuName + "详情"
    newActiveTab.cache = { id }
    if (params) {
      newActiveTab.params = params
    }
    yield call(addTabs, newActiveTab)
  } catch (e) {
    message.error("无权限访问或页面不存在!")
    console.error(857, e);
  }
}

export const watchJumpToEditPage = function* () {
  yield takeEvery("system/jumpToEditPage", jumpToEditPage)
}

function* jumpToEditPage(action) {
  const systemReducer = yield select(x => x.get("system"))
  // 弹窗跳转
  if (window.editPagePosition === editPagePositionConst.popup.value || action.popup === editPagePositionConst.popup.value) {
    let functionCode = action.functionCode
    if (!functionCode) {
      let newActiveTab = systemReducer.get("activeTab")
      functionCode = newActiveTab.functionCode
    }
    // if (newActiveTab.funType === functionCodeType.list.value) {
    yield put({
      type: "form/fetchFormConfigByFunctionCode",
      payload: {
        id: action.payload,
        functionCode: functionCode,
        withoutDefault: action.withoutDefault
      }
    })
    // }
    // dispatch();
    yield put({
      type: "system/changePopEdit",
      payload: {
        functionCode: functionCode,
        id: action.payload,
        width: action.width,
        height: action.height,
        type: "edit"
      }
    })
    return
  }
  if (action.functionCode) {
    //有编码
    const originalMenu = systemReducer.get("originalMenu")
    // 有menuUrl代表用的是自定义界面
    let menu = originalMenu[action.functionCode]
    if (!menu.menuUrl) {
      yield put({
        type: "form/fetchFormConfigByFunctionCode",
        payload: {
          id: action.payload,
          functionCode: action.functionCode,
          withoutDefault: action.withoutDefault,
          ynMenuForm: action.ynMenuForm
        }
      })
    }
    if (!menu) {
      menu = { ...systemReducer.get("activeTab") }
    }
    let newActiveTab = { ...menu, hideClose: false }
    if (menu.menuUrl) {
      console.log('menu.menuUrl', menu.menuUrl);
    } else {
      newActiveTab.type = "edit"
      newActiveTab.menuName = "编辑" + newActiveTab.menuName
    }
    // newActiveTab.type = "edit"
    // newActiveTab.menuName = "编辑" + newActiveTab.menuName
    newActiveTab.cache = { id: action.payload }
    newActiveTab.relation = action.relation //推单
    yield call(addTabs, newActiveTab)
  } else {
    let newActiveTab = { ...systemReducer.get("activeTab"), hideClose: false }
    if (!newActiveTab.menuUrl) {
      yield put({
        type: "form/fetchFormConfigByFunctionCode",
        payload: {
          id: action.payload,
          functionCode: newActiveTab.functionCode,
          withoutDefault: action.withoutDefault
        }
      })
    }
    newActiveTab.type = "edit"
    newActiveTab.menuName = "编辑" + newActiveTab.menuName
    newActiveTab.cache = { id: action.payload }
    newActiveTab.relation = action.relation //推单
    if (action.fromFunctionCode) {
      newActiveTab.fromFunctionCode = action.fromFunctionCode
    }
    yield call(addTabs, newActiveTab)
  }
}

export const watchChangeActiveTabCache = function* () {
  yield takeEvery("system/changeActiveTabCache", changeActiveTabCache)
}

function* changeActiveTabCache(action) {
  const activeTab = yield select(x => x.get("system").get("activeTab"))
  activeTab.cache = {
    ...activeTab.cache,
    ...action.payload
  }
  yield put({ type: "system/changeActiveTab", payload: activeTab })
  yield put(push(getRouteByTab(activeTab)))
  // yield call(addTabs, activeTab);
}

export const watchNoticeItemClick = function* () {
  yield takeEvery("system/noticeItemClick", noticeItemClick)
  yield takeEvery("system/noticeItemClick", readNotice)
  yield takeEvery("system/dealWithClick", dealWithClick)
}

function* readNotice(action) {
  const { item } = action.payload
  if (item.id) {
    yield post("/sysMessage/modSysMessageStatusRead", { id: item.id })
    yield fetchNoticeList()
  }
}

function* dealWithClick(action) {
  yield fork(noticeItemClick, action)
  const { item } = action.payload
  yield put({
    type: "workflow/saveNoticeMes",
    payload:{
      item:item
    }
  })
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  yield post("/sysMessage/modSysMessageStatusRead", {
    channelId: basicInfo.id,
    fid: item.fid,
    funCode: item.funCode
  })
  yield fetchNoticeList()
}

function* noticeItemClick(action) {
  const { item } = action.payload
  if (item) {
    if (item.msgType === messageTypeConst.notice.value) {
      if (item.fid) {
        yield addTabs({
          type: "static",
          menuUrl: `/sys/notice/detail/${item.fid}`,
          menuName: "公告详情"
        })
      } else {
        message.error("没有可用的公告！")
      }
    } else {
      const { funCode, fid } = item
      if (funCode) {
        const originalMenu = yield select(x => x.getIn(["system", "originalMenu"]))
        const functionMenu = originalMenu[funCode]
        if (!functionMenu) {
          message.error("没有可用的菜单!")
        } else {
          // 如果是高级表单，会跳入高级表单中，没有fid进入列表，有fid进入详情中
          if (functionMenu.funType === functionCodeType.advanceForm.value) {
            yield jumpToEditPage({
              payload: { functionCode: funCode, id: fid }
            })
          } else if (functionMenu.funType === functionCodeType.list.value) {
            if (fid) {
              yield jumpToDetailPage({
                payload: { functionCode: funCode, id: fid }
              })
            } else {
              yield addTabs(functionMenu)
            }
          } else if (functionMenu.funType === functionCodeType.custom.value) {
            if (fid) {
              console.log('functionMenu', functionMenu);
              yield addTabs({ ...functionMenu, historyId: fid })
            }
          } else {
            yield addTabs(functionMenu)
          }
        }
      } else {
        message.error("没有可用的功能!")
      }
    }
  }
}

export const watchTurnCorpLogin = function* () {
  yield takeEvery("system/turnCorpLogin", turnCorpLogin)
  yield takeEvery("system/jumpToFunctionWidthScenario", jumpToFunctionWidthScenario)
  // yield takeEvery("system/jumpToListWithScenario", jumpToListWithScenario)
}

function* turnCorpLogin(action) {
  const data = yield call(post, url.turnCorpLogin, action.payload.corp)
  const basicInfoString = localStorage.getItem("basicInfo")
  if (basicInfoString) {
    const basicInfo = {
      ...JSON.parse(basicInfoString),
      ...JSON.parse(data.result)
    }
    localStorage.setItem("basicInfo", JSON.stringify(basicInfo))
    window.location.reload()
  }
}

function* initSysConfig() {
  let cfgContent = localStorage.getItem("cfgContent")
  const refreshSysConfig = sessionStorage.getItem("refreshSysConfig")
  sessionStorage.removeItem("refreshSysConfig")
  if (!cfgContent || refreshSysConfig) {
    const data = yield post(url.fetchSysSetting, { id: 6 })
    if (data && data.result && data.result.cfgContent) {
      cfgContent = data.result.cfgContent
      localStorage.setItem("cfgContent", data.result.cfgContent)
      localStorage.setItem("productDev", data.result.memo == "productDev")
    }
  }
  if (cfgContent) {
    const config = JSON.parse(cfgContent)
    if (config) {
      if (config.precision || config.precision === 0) {
        window.numberFixedConst = config.precision || 0
      }
      Object.entries(config).forEach(([key, value]) => {
        window[key] = value
      })
      window.supportWorkflow = config.supportWorkflow === "1"
      window.defaultSearch = config.defaultSearch === "1"
      window.advanceListDefaultSearch = config.advanceListDefaultSearch === "1"
      window.reportDefaultSearch = config.reportDefaultSearch === "1"
      window.editPagePosition = config.editPagePosition || editPagePositionConst.tab.value
      window.websiteTitle = config.websiteTitle
      window.saveBackList = config.saveBackList
      window.navbarFontColor = config.navbarFontColor
      window.mainBackgroundColor = config.mainBackgroundColor
      window.detailDisplayType = config.detailDisplayType
      window.queryScope = config.queryScope
      window.navigationMultiFunction = config.navigationMultiFunction
      window.thousandthsDisplay = config.thousandthsDisplay === '1'
      document.title = window.websiteTitle || "CRM"
    }
  }
}

function* jumpToFunctionWidthScenario(action) {
  const { scenario, functionCode } = action.payload
  const systemReducer = yield select(x => x.get("system"))
  const originalMenu = systemReducer.get("originalMenu")
  const tabs = systemReducer.get("tabs");
  let menu = originalMenu[functionCode]
  const newMenu = { ...menu, cache: { scenario } }
  console.log("newMenu", newMenu);
  console.log("newMenu", newMenu.ynForm);
  if (!newMenu.id) {
    message.error("没有菜单配置请至菜单中配置!")
    yield put({
      type: "system/changeActiveTab",
      payload: tabs.get(0)
    })
    return;
  }
  if (newMenu.ynForm !== "1") {
    yield addTabs(newMenu)
  } else {
    message.error("菜单为表单模式!")
  }
}

function* refreshCurrentPageData() {
  const activeTab = yield select(x => x.getIn(["system", "activeTab"]))
  if (activeTab.type === "list") {
    yield put({
      type: "list/fetchListDataByFunctionCode",
      payload: { functionCode: activeTab.functionCode }
    })
  } else if (activeTab.type === "advanceList") {
    yield put({
      type: "advanceList/fetchAdvanceListData",
      payload: { functionCode: activeTab.functionCode }
    })
  }
}

// function* jumpToListWithScenario(action) {
// }
