import { takeEvery, put, fork, call, select } from 'redux-saga/effects';
import { message } from 'antd';
import { post, url } from '../../service/api';
import { Map, fromJS, List } from 'immutable';
import { getItemDefaultValue, resetReferenceAndDictionary } from '../../utils';
import { groupType } from '../../common/const';
import _ from 'lodash';

export const watchTreeFormAction = function* () {
  yield takeEvery('treeForm/initTreeForm', initTreeForm);
  yield takeEvery('treeForm/fetchTreeFormDetail', fetchTreeFormSelectDetail);
  yield takeEvery('treeForm/saveTreeForm', saveTreeFormDetail);
  yield takeEvery('treeForm/deleteTreeForm', deleteTreeFormDetail);
  yield takeEvery('treeForm/changeTable', changeTable);
  yield takeEvery('treeForm/deleteTable', deleteTable);
  yield takeEvery('treeForm/changeFormTableData', changeFormTableData);
}

function* initTreeForm(action) {
  const cache = yield select(x => x.getIn(['treeForm', action.payload.functionCode]));
  if (!cache) {
    const data = yield call(fetchTreeFormConfig, action);
  }
}

function* fetchTreeFormConfig(action) {
  const { functionCode } = action.payload;
  const data = yield call(post, url.getSysListConfig, { key: functionCode }, functionCode);

  if (!data && data.result) {
    return;
  }

  const { content: contentString } = data.result;
  if (!contentString) {
    return
  }
  const content = JSON.parse(contentString);
  const config = content['config'];
  const basicInfo = content['basicInfo'];
  if (!config) {
    return;
  }
  const searchScenarioConfig = config['scenario'];

  let searchScenario = Map({});
  searchScenarioConfig && searchScenarioConfig.map(item => {
    searchScenario = searchScenario.set(item.dataIndex, getItemDefaultValue(item));
    return true
  });

  const formConfig = [searchScenarioConfig];
  const fields = config['children'];
  delete config['scenario'];
  if (basicInfo.childTable && fields && fields.length > 0) {
    let tableDefault = {};
    fields && fields.map(item => {
      tableDefault[item.dataIndex] = getItemDefaultValue(item);
      // searchScenario = searchScenario.set(item.dataIndex, getItemDefaultValue(item));
      return true
    });
    config.tableName = basicInfo.childTable;
    config.defaultObj = tableDefault;
    config.groupType = groupType.table;
    formConfig.push(config);
  }

  const payload = {
    searchScenarioConfig: formConfig,
    functionCode: action.payload.functionCode,
    basicInfo: basicInfo,
    searchScenario,
  }
  yield put({
    type: 'treeForm/fetchConfigSuccess',
    payload,
  })
  return payload;
}

function* fetchTreeFormSelectDetail(action) {
  const { selectId, functionCode } = action.payload;
  const basicInfo = yield select(x => x.getIn(['treeForm', functionCode, 'basicInfo']));
  const response = yield call(post, url.fetchFormDetail, {
    funCode: functionCode,
    id: selectId
  }, action.payload.functionCode);

  if (response && response.result) {
    const detail = response.result[basicInfo.mainTable];
    yield put({
      type: 'treeForm/fetchSelectTreeFormDetailDataSuccess',
      payload: {
        functionCode: functionCode,
        selectId: selectId,
        searchScenario: fromJS(detail)
      }
    });
  }
}

function* saveTreeFormDetail(action) {
  const functionCode = action.payload.functionCode;
  const cache = yield select(x => x.getIn(['treeForm', functionCode]));
  const detail = cache.get('searchScenario');
  const basicInfo = cache.get('basicInfo');
  const data = [{
    tableName: basicInfo.mainTable,
    values: detail.toJSON()
  }];
  if (basicInfo.childTable && detail.get(basicInfo.childTable)) {
    data.push({
      tableName: basicInfo.childTable,
      values: detail.get(basicInfo.childTable).toJSON()
    })
  }
  const requestParams = {
    funCode: functionCode,
    id: detail.get('id'),
    data,
  };
  yield put({ type: 'dictionary/iniStateSaga' });
  yield put({ type: 'reference/iniStateSaga' });
  resetReferenceAndDictionary()
  if (!requestParams.id) {
    const response = yield call(post, url.addFormData, requestParams, functionCode);
    yield fork(fetchTreeFormSelectDetail, { payload: { selectId: response.result.id, functionCode } });
  } else {
    const response = yield call(post, url.modFormData, requestParams, functionCode);
  }

  yield put({ type: 'reference/refreshReference', payload: basicInfo.referenceKey });
}

function* deleteTreeFormDetail(action) {
  const { selectId, functionCode } = action.payload;
  const basicInfo = yield select(x => x.getIn(['treeForm', functionCode, 'basicInfo']));
  yield call(post, url.removeFormData, {
    funCode: functionCode,
    id: selectId
  }, functionCode, '', true, true);
  yield put({ type: 'reference/refreshReference', payload: basicInfo.referenceKey });
}

function* changeTable(action) {
  const { item, functionCode, referenceResult, fromReference, newTriggerRef } = action.payload;
  let state = yield select(x => x.get('treeForm'));
  let calcField = [];
  if (fromReference) {
    state = state.updateIn([functionCode, 'searchScenario', item.tableName], tableData => {
      tableData = tableData || List([]);
      const newData = [];
      _.forEach(referenceResult, (x, xIndex) => {
        let value = { ...item.defaultObj };
        item.affectFields && item.affectFields.map((affectItem) => {
          if (_.head(affectItem.formField) === item.tableName) {
            const tableField = _.last(affectItem.formField);
            value[tableField] = x[affectItem.referenceField];
            xIndex === 0 && calcField.push(tableField);
          }
          return true;
        });
        newData.push(value);
        return value;
      });
      return tableData.concat(fromJS(newData));
    });
  } else {
    state = state.updateIn([functionCode, 'searchScenario', item.tableName], tableData => {
      tableData = tableData || List([]);
      const newData = referenceResult;
      if (referenceResult && referenceResult.length > 0) {
        calcField = Object.keys(referenceResult[0]);
      }
      return tableData.concat(fromJS(newData));
    });
  }
  yield put({ type: 'treeForm/setForm', payload: state });
}

function* deleteTable(action) {
  const { functionCode, tableName, index } = action.payload;
  let state = yield select(x => x.get('treeForm'));
  state = state.deleteIn([functionCode, 'searchScenario', tableName, index]);
  yield put({ type: 'treeForm/setForm', payload: state });
}

function* changeFormTableData(action) {
  const { value, item, functionCode, index, selectRow } = action.payload;
  let state = yield select(x => x.get('treeForm'));
  state = state.setIn([functionCode, 'searchScenario', item.tableName, index, item.dataIndex], value);
  if (item.affectFields) {
    const calcField = [];
    state = state.updateIn([functionCode, 'searchScenario', item.tableName, index], tableData => {
      item.affectFields.map((affectItem) => {
        if (_.head(affectItem.formField) === item.tableName) {
          const tableField = _.last(affectItem.formField);
          tableData = tableData.set(tableField, selectRow[affectItem.referenceField]);
          calcField.push(tableField);
        }
        return true;
      })
      return tableData;
    })
  }
  yield put({ type: 'treeForm/setForm', payload: state });
}
