/**
 * Created by Administrator on 2019-01-16.
 */

import { fork, put, select, take, takeEvery } from "redux-saga/effects"
import { request, strToHexCharCode } from "../../utils"
import { message, Modal } from "antd"
import { fromJS, List, Map } from "immutable"
import {
  workflowStatusConst,
  workflowNodeType,
  dealWithResultStatusConst,
  workFlowStatusMapping,
  dateTimeFormatConst,
  workFlowNodeAttrModeConst,
  wfConditionConst,
  wfRelationConst,
  wfHandleType
} from "../../common/const"
import _ from "lodash"
import { getFormDetailData } from "../form/sagas"
import moment from "moment"
import ConfirmDialog from "../../components/ConfirmDialog"
import en from "../../locale/en_US"
import zh from "../../locale/zh_CN"

let notHandlerPersion,
  withdrawSuccess,
  precessForbitAbandon,
  notSubmit,
  handlerSuccess,
  notRecord,
  noneWorkflow,
  recallVerifiedWorkflow,
  recallSuccess,
  unRejectWorkflow,
  abandonSuccess,
  submitSuccess,
  work_flowPromptTitle,
  work_flowPromptContent,
  work_inputPrompt,
  work_rejectedToAsk,
  work_beforeNode,
  work_firstNode,
  work_err,
  work_addSuccess
let lang = localStorage.getItem("lang")
switchLang(lang)

// const notHandlerPersion = '当前流程不属于您操作！';
// const withdrawSuccess = '流程撤销成功！';
// const precessForbitAbandon = '进行中流程不允许撤回！';
// const notSubmit = '当前流程未提交！';
// const handlerSuccess = '处理成功！';
// const notRecord = '请勾选记录！';
// const noneWorkflow = '暂无流程';
// const recallVerifiedWorkflow = '流程已被处理，不能弃审';
// const recallSuccess = '弃审成功！';
// const unRejectWorkflow = '流程未被处理！';
// const abandonSuccess = '驳回成功！';
// const submitSuccess = '流程提交成功！'

// work_flowPromptTitle '流程提醒?'
// work_flowPromptContent: '此单据存在流程，是否提交流程'
function switchLang(lang) {
  let language = lang == "en" ? en : zh
  notHandlerPersion = language.work_notHandlerPersion
  withdrawSuccess = language.work_withdrawSuccess
  precessForbitAbandon = language.work_precessForbitAbandon
  notSubmit = language.work_notSubmit
  handlerSuccess = language.work_handlerSuccess
  notRecord = language.work_notRecord
  noneWorkflow = language.work_noneWorkflow
  recallVerifiedWorkflow = language.work_recallVerifiedWorkflow
  recallSuccess = language.work_recallSuccess
  unRejectWorkflow = language.work_unRejectWorkflow
  abandonSuccess = language.work_abandonSuccess
  submitSuccess = language.work_submitSuccess
  work_flowPromptTitle = language.work_flowPromptTitle
  work_flowPromptContent = language.work_flowPromptContent
  work_inputPrompt = language.work_inputPrompt
  work_rejectedToAsk = language.work_rejectedToAsk
  work_beforeNode = language.work_beforeNode
  work_firstNode = language.work_firstNode
  work_err = language.work_err
  work_addSuccess = language.work_addSuccess
}

export const watchWorkflowAction = function* () {
  yield fork(watchInitWorkflow)
  yield takeEvery("system/switchLanguage", switchLanguage)
  yield takeEvery("workflow/checkWorkflow", checkWorkflow)//确认提交
  yield takeEvery("workflow/createWorkflow", createWorkflow) //创建
  yield takeEvery("workflow/handlerWorkflow", handlerWorkflow) //处理
  yield takeEvery("workflow/withdrawWorkflow", withdrawWorkflow) //撤回
  yield takeEvery("workflow/submitWorkflow", submitWorkflow) //提交
  yield takeEvery("workflow/verfiedWorkflow", verfiedWorkflow) //审核
  yield takeEvery("workflow/abandonWorkflow", abandonWorkflow) //驳回
  yield takeEvery("workflow/readWorkflow", readWorkflow) //读取已读
  yield takeEvery("workflow/recallWorkflow", recallWorkflow) //弃审
  yield takeEvery("workflow/viewWorkflow", viewWorkflow)
  yield takeEvery("workflow/recreateWorkflow", recreateWorkflow) //重置
  yield takeEvery("workflow/addNodeToWorkflow", addNodeToWorkflow)
}

function* switchLanguage() {
  const system = yield select(x => x.get("system"))
  switchLang(system.get("lang"))
}

function* watchInitWorkflow() {
  let functionCodes = []
  while (true) {
    const action = yield take("workflow/initWorkflow")
    if (functionCodes.indexOf(action.payload.functionCode) < 0) {
      functionCodes.push(action.payload.functionCode)
      yield fork(initWorkflowTemplate, action)
    }
  }
}

function* submitWorkflow(action) {
  const { functionCode, recordIds } = action.payload
  if (!recordIds || recordIds.length === 0) {
    message.error(notRecord)
    return
  }
  yield submitOneWorkflow({ payload: { functionCode, recordId: recordIds[0] } })
}

function* submitOneWorkflow(action) {
  const { functionCode, recordId } = action.payload
  const data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
  if (data && data.result && data.result.wfContent) {
    if (data.result.wfState === workflowStatusConst.new.value) {
      yield handlerWorkflow({ payload: { functionCode, recordId, workflowResult: data, successMessage: submitSuccess } })
    } else {
      message.error(notHandlerPersion)
    }
  } else {
    message.error(notHandlerPersion)
  }
}

function* initWorkflowTemplate(action) {
  const { functionCode } = action.payload
  yield put({ type: "workflow/changeCurrentFunctionCode", payload: { currentFunctionCode: functionCode } })
  const data = yield request("/sysWorkflow/getSysWorkflowTemplateList", { funCode: functionCode })
  if (data && data.result && data.result && data.result.length > 0) {
    if (!data.result[0].wfContent) {
      return
    }
    const { yEnable } = JSON.parse(data.result[0].wfContent)
    if (yEnable !== "1") {
      return
    }
    yield put({
      type: "workflow/fetchWorkflowTemplateSuccess",
      payload: { workflowTemplate: data.result[0].wfContent, functionCode }
    })
  }
}

function* checkWorkflow(action) {
  const { functionCode, recordId } = action.payload
  const workflowTemplate = yield select(x => x.getIn(["workflow", functionCode]))
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
  if (workflowTemplate && workflowTemplate.get("saveConfirm") === "1") {
    if (data && data.result && data.result.wfContent && data.result.wfState === workflowStatusConst.new.value) {
      let wfContent = JSON.parse(data.result.wfContent)
      let { currentNodes } = wfContent
      const firstNode = _.find(currentNodes, { keyPath: [0] })
      if (!firstNode) {
        return
      }
      const user = _.find(firstNode.users, { userId: basicInfo.id })
      if (user) {
        const result = yield ConfirmModal({
          title: work_flowPromptTitle,
          content: work_flowPromptContent
        })
        //确认提交
        if (result) {
          yield handlerWorkflow({ payload: { functionCode, recordId, successMessage: submitSuccess } })
        }
      }
    }
  }
  yield put({ type: "workflow/refreshCurrentWorkflow" , payload : {functionCode:functionCode,id:recordId} })
}

function ConfirmModal(config) {
  return new Promise(resolve => {
    Modal.confirm({
      ...config,
      onOk() {
        resolve(true)
      },
      onCancel() {
        resolve(false)
      }
    })
  })
}

function* createWorkflow(action) {
  const { functionCode, recordId } = action.payload
  let workflowTemplate = yield select(x => x.getIn(["workflow", functionCode]))
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const mainTableName = yield select(x => x.getIn(["form", functionCode, "mainTableName"]))
  if (workflowTemplate && workflowTemplate.get("dataSource")) {
    let detailInfo = yield getFormDetailData({ payload: { id: recordId, functionCode } })
    // 设置表名通知名
    let name = workflowTemplate.get("name")
    if (workflowTemplate.get("name")) {
      try {
        name = workflowTemplate.get("name").replace(/\{(\w+)\}/gi, (a, b) => {
          //增加关系表在流程中使用
          if (b.indexOf("0") > 0) {
            let ss = b.split("0")
            return detailInfo[ss[0]][ss[1]] || ""
          } else {
            return detailInfo[mainTableName][b] || ""
          }
        })
      } catch (e) {
        message.error(`${workflowTemplate.get("name")}配置错误`)
      }
    }
    //当前节点
    const currentNodes = [{ keyPath: [0], users: [{ userId: basicInfo.id, username: basicInfo.name, handled: false }] }]
    //创建流程
    yield request("/sysWorkflow/saveSysWorkflowInfo", {
      funCode: functionCode,
      fid: recordId,
      wfState: workflowStatusConst.new.value,
      wfName: name,
      wfContent: JSON.stringify({
        workflow: workflowTemplate.get("dataSource"),
        currentNodes
      })
    })
    //是否提交
    yield checkWorkflow({ payload: action.payload })
  }
}

function* recreateWorkflow(action) {
  const { functionCode, recordId } = action.payload
  let workflowTemplate = yield select(x => x.getIn(["workflow", functionCode]))
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const mainTableName = yield select(x => x.getIn(["form", functionCode, "mainTableName"]))
  if (workflowTemplate && workflowTemplate.get("dataSource")) {
    let detailInfo = yield getFormDetailData({ payload: { id: recordId, functionCode } })
    let name = workflowTemplate.get("name")
    if (workflowTemplate.get("name")) {
      name = workflowTemplate.get("name").replace(/\{(\w+)\}/gi, (a, b) => {
        return detailInfo[mainTableName][b] || ""
      })
    }
    const data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
    if (data && data.result && data.result.wfContent) {
      let { currentNodes: dataCurrentNodes, handlerRecords: dataHandlerRecords } = JSON.parse(data.result.wfContent)
      let currentNodes = [{ keyPath: [0], users: [{ userId: basicInfo.id, username: basicInfo.name, handled: false }] }]
      if (dataHandlerRecords) {
        currentNodes = [
          { keyPath: [0], users: [{ userId: dataHandlerRecords[0].userId, username: dataHandlerRecords[0].userName, handled: false }] }
        ]
      } else {
        currentNodes = [
          {
            keyPath: [0],
            users: [{ userId: dataCurrentNodes[0]["users"][0].userId, username: dataCurrentNodes[0]["users"][0].username, handled: false }]
          }
        ]
      }
      yield request("/sysWorkflow/sysWorkflowNodeController", {
        id: recordId,
        wfState: workflowStatusConst.new.value,
        nodeState: dealWithResultStatusConst.recreate.value,
        sysWorkflowInfo: {
          ...data.result,
          wfState: workflowStatusConst.new.value,
          wfContent: JSON.stringify({
            workflow: workflowTemplate.get("dataSource"),
            currentNodes
          })
        }
      })
      yield checkWorkflow({ payload: action.payload })
    }
  }
}

function* verfiedWorkflow(action) {
  const workflowCache = yield select(x => x.get("workflow"))
  const currentRecordIds = action.payload && action.payload.recordIds ? action.payload.recordIds : workflowCache.get("currentRecordIds")
  const verifiedMessage = workflowCache.getIn(["codeVerifiedMessage", action.payload.functionCode,currentRecordIds[0], "verifiedMessage"]) ? workflowCache.getIn(["codeVerifiedMessage", action.payload.functionCode,currentRecordIds[0], "verifiedMessage"]) : workflowCache.get("verifiedMessage");
  const currentFunctionCode =
    action.payload && action.payload.functionCode ? action.payload.functionCode : workflowCache.get("currentFunctionCode")
  if (!currentRecordIds || currentRecordIds.length === 0) {
    message.error(notRecord)
    return
  }
  const needMessage = yield select(x => x.getIn(["workflow", currentFunctionCode, "needMessage"]))
  if (needMessage === "1" && !verifiedMessage) {
    message.error(work_inputPrompt)
    return
  }
  yield handlerWorkflow({ payload: { functionCode: currentFunctionCode, recordId: currentRecordIds[0], verifiedMessage } })
  yield put({ type: "workflow/cancelVerifiedWorkflow" })
  yield put({ type: "workflow/refreshCurrentWorkflow" , payload : {functionCode:currentFunctionCode,id:currentRecordIds[0]} }) //刷新
  yield closeDetailTab({ payload: { functionCode: currentFunctionCode, recordId: currentRecordIds[0] } })
}

function* recallWorkflow(action) {
  const { functionCode, recordIds } = action.payload
  if (!recordIds || recordIds.length === 0) {
    message.error(notRecord)
    return
  }
  yield recallOneWorkflow({ payload: { functionCode, recordId: recordIds[0] } })
}

function* recallOneWorkflow(action) {
  const { functionCode, recordId } = action.payload
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
  const detailInfo = yield getFormDetailData({ payload: { id: recordId, functionCode } })
  const mainTableName = yield select(x => x.getIn(["form", functionCode, "mainTableName"]))
  if (data && data.result && data.result.wfContent) {
    let wfContent = JSON.parse(data.result.wfContent)
    let { workflow: workflowJSON, currentNodes, handlerRecords, name } = wfContent
    let workflow = fromJS(workflowJSON)
    const handlerRecord = _.findLast(handlerRecords, { userId: basicInfo.id })
    if (!handlerRecord) {
      message.error(notHandlerPersion)
      return
    }
    if (handlerRecord.dealWithResult !== dealWithResultStatusConst.verified.value) {
      message.error(unRejectWorkflow)
      return
    }
    let handlerNode = _.find(currentNodes || [], node => {
      return _.find(node.users, { userId: basicInfo.id, handled: true })
    })
    let removePendingItems = []
    if (handlerNode) {
      _.map(handlerNode.users, user => {
        if (user.userId === basicInfo.id) {
          user.handled = false
          removePendingItems.push(user.userId)
        }
      })
    } else {
      const { nextNodes } = yield calculateNextNodes(workflow, handlerRecord.keyPath, currentNodes, detailInfo[mainTableName])
      const afterHandlerRecords = handlerRecords.slice(handlerRecords.indexOf(handlerRecord) + 1)
      _.reverse(afterHandlerRecords)
      // 流程未结束，撤回当前节点后面的所有节点
      if (nextNodes && nextNodes.length > 0) {
        const checkedNodes = Array.from(nextNodes, keyPath => {
          keyPath
        })
        const checkHandlerRecords = _.uniqBy(afterHandlerRecords, checkedNodes, "keyPath")
        let verifiedCount = 0
        let recallCount = 0
        _.map(checkHandlerRecords, checkRecord => {
          if (checkRecord.dealWithResult === dealWithResultStatusConst.verified.value) {
            verifiedCount++
          }
          if (checkRecord.dealWithResult === dealWithResultStatusConst.recall.value) {
            recallCount++
          }
        })

        // 审核的记录大于弃审的记录，代表已经被审核
        if (verifiedCount > recallCount) {
          message.error(recallVerifiedWorkflow)
          return
        }
        for (let nextNodeIndex = 0; nextNodeIndex < nextNodes.length; nextNodeIndex++) {
          let nextNode = nextNodes[nextNodeIndex]
          const nextNodeUsers = yield getWorkflowNodeUsers({
            payload: {
              workflow,
              keyPath: nextNode,
              recordId,
              functionCode
            }
          })
          removePendingItems = removePendingItems.concat(Array.from(nextNodeUsers, user => user.userId))
          currentNodes = _.filter(currentNodes, node => {
            return !node.keyPath.equals(nextNode)
          })
        }
      }
      const users = yield getWorkflowNodeUsers({
        payload: {
          workflow,
          keyPath: handlerRecord.keyPath,
          handled: true,
          recordId,
          functionCode
        }
      })
      users.map(user => {
        if (user.userId === basicInfo.id) {
          user.handled = false
        }
        return true
      })
      currentNodes.push({ keyPath: handlerRecord.keyPath, preKeyPath: handlerRecord.preKeyPath, users })
    }
    // return;
    try {
      handlerRecords.push({
        keyPath: handlerRecord.keyPath,
        preKeyPath: handlerRecord.preKeyPath,
        userId: basicInfo.id,
        userName: basicInfo.name,
        dealWithResult: dealWithResultStatusConst.recall.value
        // dateTime: moment().format(dateTimeFormatConst)后台取值
      })
      yield request("/sysWorkflow/sysWorkflowNodeController", {
        id: recordId,
        nodeCode: workflow.getIn([...handlerRecord.keyPath, "unionid"]),
        callBack: workflow.getIn([...handlerRecord.keyPath, "callBackCache"]),
        nodeState: dealWithResultStatusConst.recall.value,
        wfState: workflowStatusConst.process.value,
        removePendingItems,
        addPendingItems: [basicInfo.id],
        sysWorkflowInfo: {
          ...data.result,
          wfContent: JSON.stringify({
            ...wfContent,
            workflow: workflowJSON,
            currentNodes: currentNodes,
            handlerRecords
          })
        }
      })
      message.success(recallSuccess)
      yield put({ type: "workflow/refreshCurrentWorkflow" , payload : {functionCode:functionCode,id:recordId} }) //刷新
    } catch (e) {
      console.log(466,e)
    }
  } else {
    message.error(notHandlerPersion)
  }
}

function* abandonWorkflow(action) {
  const workflowCache = yield select(x => x.get("workflow"))
  const currentRecordIds = action.payload && action.payload.recordIds ? action.payload.recordIds : workflowCache.get("currentRecordIds")
  // const verifiedMessage = workflowCache.get("verifiedMessage")
  const verifiedMessage = workflowCache.getIn(["codeVerifiedMessage", action.payload.functionCode,currentRecordIds[0], "verifiedMessage"]) ? workflowCache.getIn(["codeVerifiedMessage", action.payload.functionCode,currentRecordIds[0], "verifiedMessage"]) : workflowCache.get("verifiedMessage");
  const currentFunctionCode =
    action.payload && action.payload.functionCode ? action.payload.functionCode : workflowCache.get("currentFunctionCode")
  if (!currentRecordIds || currentRecordIds.length === 0) {
    message.error(notRecord)
    return
  }
  const needMessage = yield select(x => x.getIn(["workflow", currentFunctionCode, "needMessage"]))
  if (needMessage === "1" && !verifiedMessage) {
    message.error(work_inputPrompt)
    return
  }
  const toTopNode = yield ConfirmFunc({
    title: work_flowPromptTitle,
    content: work_rejectedToAsk,
    cancelText: work_beforeNode,
    okText: work_firstNode
  })
  yield abandonOneWorkflow({
    payload: {
      functionCode: currentFunctionCode,
      recordId: currentRecordIds[0],
      verifiedMessage,
      toTopNode
    }
  })
  yield put({ type: "workflow/cancelVerifiedWorkflow" })
  yield put({ type: "workflow/refreshCurrentWorkflow" , payload : {functionCode:currentFunctionCode,id:currentRecordIds[0]} }) //刷新数据
  yield closeDetailTab({ payload: { functionCode: currentFunctionCode, recordId: currentRecordIds[0] } })
}

function ConfirmFunc(config) {
  return new Promise(resolve => {
    ConfirmDialog({
      ...config,
      onOk() {
        resolve(true)
      },
      onCancel() {
        resolve(false)
      }
    })
  })
}
// 弃审
function* abandonOneWorkflow(action) {
  const { functionCode, recordId, verifiedMessage, toTopNode } = action.payload
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
  const detailInfo = yield getFormDetailData({ payload: { id: recordId, functionCode } })
  const mainTableName = yield select(x => x.getIn(["form", functionCode, "mainTableName"]))
  if (data && data.result && data.result.wfContent) {
    let wfContent = JSON.parse(data.result.wfContent)
    let { workflow: workflowJSON, currentNodes, handlerRecords } = wfContent
    let workflow = fromJS(workflowJSON)
    handlerRecords = handlerRecords || []
    const addPendingItems = []
    let handlerNode = _.find(currentNodes, node => {
      return _.find(node.users, { userId: basicInfo.id })
    })
    if (!handlerNode) {
      message.error(notHandlerPersion)
      return
    }
    // 退回首节点
    let wfState = data.result.wfState
    let removePendingItems = []
    if (toTopNode || (handlerNode.preKeyPath && handlerNode.preKeyPath.equals([0]))) {
      handlerRecords.push({
        keyPath: handlerNode.keyPath,
        preKeyPath: handlerNode.preKeyPath,
        userId: basicInfo.id,
        userName: basicInfo.name,
        dealWithResult: dealWithResultStatusConst.reject.value,
        // dateTime: moment().format(dateTimeFormatConst),后台取值
        message: verifiedMessage
      })
      const firstHandlerRecord = _.head(handlerRecords)
      _.map(currentNodes, currentNode => {
        _.map(currentNode.users, user => {
          if (!user.handled) {
            removePendingItems.push(user.userId)
          }
        })
      })
      currentNodes = [
        {
          keyPath: [0],
          users: [{ userId: firstHandlerRecord.userId, userName: firstHandlerRecord.userName, handled: false }]
        }
      ]
      wfState = workflowStatusConst.new.value
      addPendingItems.push(firstHandlerRecord.userId)
      // 撤回之后作废已处理记录
      handlerRecords = Array.from(handlerRecords, handlerRecord => {
        // removePendingItems.push(handlerNode.userId)
        return { ...handlerRecord, invalid: true }
      })
    } else {
      currentNodes.splice(currentNodes.indexOf(handlerNode), 1)
      // 3.2回退const {nextNodes} = yield calculateNextNodes(workflow, handlerNode.preKeyPath, detailInfo[mainTableName]);
      const { nextNodes } = yield calculateNextNodes(workflow, handlerNode.preKeyPath, currentNodes)

      if (nextNodes) {
        _.map(handlerNode.users, user => {
          if (!user.handled) {
            removePendingItems.push(user.userId)
          }
        })
        _.map(nextNodes, nextKeyPath => {
          const checkedKeyPath = nextKeyPath.slice(0, nextKeyPath.length - 1)
          currentNodes = _.filter(currentNodes, node => {
            return node.keyPath.toString().indexOf(checkedKeyPath.toString()) !== 0
          })
          _.forEach(handlerRecords, recordItem => {
            if (recordItem.keyPath.toString().indexOf(checkedKeyPath.toString()) === 0) {
              removePendingItems.push(handlerNode.userId)
              // 3.2回退 recordItem.invalid = true;
            }
          })
        })
      }
      const preHandlerRecord = _.findLast(handlerRecords, recordItem => recordItem.keyPath.equals(handlerNode.preKeyPath))
      const users = yield getWorkflowNodeUsers({
        payload: {
          workflow,
          keyPath: handlerNode.preKeyPath,
          recordId,
          functionCode
        }
      })
      handlerRecords.push({
        keyPath: handlerNode.keyPath,
        preKeyPath: handlerNode.preKeyPath,
        userId: basicInfo.id,
        userName: basicInfo.name,
        dealWithResult: dealWithResultStatusConst.reject.value,
        // dateTime: moment().format(dateTimeFormatConst),后台取值
        message: verifiedMessage
      })
      currentNodes.push({ keyPath: preHandlerRecord.keyPath, preKeyPath: preHandlerRecord.preKeyPath, users })
      _.map(users, user => {
        addPendingItems.push(user.userId)
      })
    }
    yield request("/sysWorkflow/sysWorkflowNodeController", {
      id: recordId,
      nodeCode: workflow.getIn([...handlerNode.keyPath, "unionid"]),
      callBack: workflow.getIn([...handlerNode.keyPath, "callBackCache"]),
      nodeState: dealWithResultStatusConst.reject.value,
      wfState: wfState,
      removePendingItems: removePendingItems,
      addPendingItems: addPendingItems,
      sysWorkflowInfo: {
        ...data.result,
        wfState: wfState,
        wfContent: JSON.stringify({
          ...wfContent,
          workflow: workflowJSON,
          currentNodes,
          handlerRecords
        })
      }
    })
    message.success(abandonSuccess)
  }
}
// 处理节点
function* handlerWorkflow(action) {
  const { functionCode, recordId, verifiedMessage, workflowResult, successMessage } = action.payload
  //当前用户信息
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  //流程信息
  let data = workflowResult
  if (!data) {
    data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
  }
  // 主表数据
  const detailInfo = yield getFormDetailData({ payload: { id: recordId, functionCode } })
  // 主表名
  const mainTableName = yield select(x => x.getIn(["form", functionCode, "mainTableName"]))
  //流程主体
  if (data && data.result && data.result.wfContent) {
    let wfContent = JSON.parse(data.result.wfContent)
    let {
      workflow: workflowJSON,
      currentNodes, //当前节点
      handlerRecords, //处理记录
      unsuperveneNodes: unsuperveneCheckedNodes, //当前竞争节点
      noticeNodes: currentNoticeNodes //通知
    } = wfContent
    let workflow = fromJS(workflowJSON)
    let addPendingItems = [] //添加节点
    let removePendingItems = [] //删除节点
    let newNoticeNodes = [] //添加通知
    let nodeState = dealWithResultStatusConst.verifiedOne.value //节点状态
    let nodeTitle = ""
    if (currentNodes && currentNodes.length > 0) {
      // 当前节点有值
      handlerRecords = handlerRecords || []
      let handlerNode = _.find(currentNodes, node => {
        //匹配登录人作为处理节点
        return _.find(node.users, { userId: basicInfo.id, handled: false })
      })
      if (currentNodes.length == 1 && currentNodes[0].users.length == 0 && currentNodes[0].keyPath[0] < workflowJSON.length) {
        handlerNode = currentNodes[0]
      }
      // 可处理
      if (handlerNode) {
        const workflowNode = workflow.getIn(handlerNode.keyPath) //处理节点对象
        // 首节点
        if (workflowNode.get("first")) {
          //首节点与之后节点dealWithResult，message不同
          handlerRecords.push({
            keyPath: handlerNode.keyPath,
            preKeyPath: handlerNode.preKeyPath,
            userId: basicInfo.id,
            userName: basicInfo.name,
            dealWithResult: dealWithResultStatusConst.submitted.value
            // dateTime: moment().format(dateTimeFormatConst)后台取值
          })
        } else {
          handlerRecords.push({
            keyPath: handlerNode.keyPath,
            preKeyPath: handlerNode.preKeyPath,
            userId: basicInfo.id,
            userName: basicInfo.name,
            dealWithResult: dealWithResultStatusConst.verified.value,
            message: verifiedMessage
            // dateTime: moment().format(dateTimeFormatConst)后台取值
          })
        }

        // 如果该节点属于竞争节点，去除其当前
        let newCurrentNodes = [...currentNodes]
        if (unsuperveneCheckedNodes && unsuperveneCheckedNodes.length > 0) {
          unsuperveneCheckedNodes.map(unsuperveneNode => {
            if (handlerNode.keyPath.toString().indexOf(unsuperveneNode.toString()) === 0) {
              let tempCurrentNodes = [] //临时处理节点
              newCurrentNodes.map((node, index) => {
                if (node !== handlerNode && node.keyPath.toString().indexOf(unsuperveneNode.toString()) === 0) {
                  removePendingItems = removePendingItems.concat(Array.from(node.users, user => user.userId)) //删除其他节点
                } else {
                  tempCurrentNodes.push(node) //临时处理节点
                }
              })
              newCurrentNodes = tempCurrentNodes //赋值处理节点
            }
          })
        }

        // 判断节点内用户关系，如果是竞争关系，处理直接通过
        let unhanlderUsers = undefined
        _.map(handlerNode.users, user => {
          if (workflowNode.get("nodeAttr") === workFlowNodeAttrModeConst.supervene.value) {
            //并发
            if (user.userId === basicInfo.id) {
              user.handled = true
            }
          } else {
            //竞争
            if (user.userId === basicInfo.id) {
              user.handled = true
            } else {
              user.handled = true
              removePendingItems.push(user.userId)
            }
          }
        })

        unhanlderUsers = _.find(handlerNode.users, { handled: false }) //查找未处理的
        // console.log('unhanlderUsers', unhanlderUsers);
        // if (!unhanlderUsers) {
        //   nodeState = dealWithResultStatusConst.verified.value;
        //   newCurrentNodes.splice(newCurrentNodes.indexOf(handlerNode), 1);
        //   console.log('handlerNode:', handlerNode, 'newCurrentNodes', newCurrentNodes);
        //   const { nextNodes } = yield calculateNextNodes(workflow, handlerNode.keyPath, newCurrentNodes, detailInfo[mainTableName]);
        //   console.log('nextNodes', nextNodes);
        //   if (nextNodes) {
        //     nodeState = dealWithResultStatusConst.verifiedAll.value;
        //     for (var i = 0; i < nextNodes.length; i++) {
        //       let nodeKeyPath = nextNodes[i];
        //       const users = yield getWorkflowNodeUsers({
        //         payload: {
        //           workflow,
        //           keyPath: nodeKeyPath,
        //           recordId,
        //           functionCode
        //         }
        //       });
        //       console.log('users', users);
        //       newCurrentNodes.push({ keyPath: nodeKeyPath, preKeyPath: handlerNode.keyPath, users });
        //       addPendingItems = addPendingItems.concat(Array.from(users, user => user.userId));
        //       const node = workflow.getIn(nodeKeyPath);
        //       nodeTitle = node.get("name");
        //     }
        //   }
        //   console.log('noticeNodes', noticeNodes);
        //   if (noticeNodes) {
        //     for (var i = 0; i < noticeNodes.length; i++) {
        //       let nodeKeyPath = noticeNodes[i];
        //       const users = yield getWorkflowNodeUsers({
        //         payload: {
        //           workflow,
        //           keyPath: nodeKeyPath,
        //           recordId,
        //           functionCode
        //         }
        //       });
        //       newNoticeNodes.push({ keyPath: nodeKeyPath, preKeyPath: handlerNode.keyPath, users });
        //       addPendingItems = addPendingItems.concat(Array.from(users, user => user.userId));
        //     }
        //   }
        //   console.log('addPendingItems', addPendingItems);
        // }
        if (!unhanlderUsers) {
          yield emptyNodeRecurses(handlerNode, newCurrentNodes)
        }
        // 没有未处理的人，将会走到下一个节点
        let isRecurse = false
        // eslint-disable-next-line
        function* emptyNodeRecurses(handlerNodeCustom, newCurrentNodes) {
          handlerNode = handlerNodeCustom
          nodeState = dealWithResultStatusConst.verified.value
          newCurrentNodes.splice(newCurrentNodes.indexOf(handlerNodeCustom), 1)
          // console.log('handlerNode:', handlerNodeCustom, 'newCurrentNodes', newCurrentNodes);
          const { nextNodes } = yield calculateNextNodes(workflow, handlerNodeCustom.keyPath, newCurrentNodes, detailInfo[mainTableName])
          if (nextNodes) {
            nodeState = dealWithResultStatusConst.verifiedAll.value
            for (var i = 0; i < nextNodes.length; i++) {
              let nodeKeyPath = nextNodes[i]
              const users = yield getWorkflowNodeUsers({
                payload: {
                  workflow,
                  keyPath: nodeKeyPath,
                  recordId,
                  functionCode
                }
              })
              newCurrentNodes.push({ keyPath: nodeKeyPath, preKeyPath: handlerNodeCustom.keyPath, users })

              addPendingItems = addPendingItems.concat(Array.from(users, user => user.userId))
              const node = workflow.getIn(nodeKeyPath)
              nodeTitle = node.get("name")
            }
          }
          if (noticeNodes&&noticeNodes.length>0) {
              for (let i = 0; i < noticeNodes.length; i++) {
                let nodeKeyPath = noticeNodes[i]
                const { nextNodes } = calculateNextNodesWithoutChildren(workflow, nodeKeyPath, info)
                const node = workflow.getIn(nodeKeyPath)
                let skipCondition = node.get("ignoreCon")
                let detailInfo = yield getFormDetailData({ payload: { id: recordId, functionCode } })
                const mainTableName = yield select(x => x.getIn(["form", functionCode, "mainTableName"]))
                let info = detailInfo[mainTableName];
                let isIgnore = skipCondition && skipCondition.length > 0 && calculateConds(JSON.parse(skipCondition), info)
                if(isIgnore){
                  if(nextNodes.length==0){
                    const { nextNodes } = calculateNextNodesWithoutChildren(workflow, nodeKeyPath, info)
                    for (let i = 0; i < nextNodes.length; i++) {
                      let nodeKeyPath = nextNodes[i]
                      const users = yield getWorkflowNodeUsers({
                        payload: {
                          workflow,
                          keyPath: nodeKeyPath,
                          recordId,
                          functionCode
                        }
                      })
                    newNoticeNodes.push({ keyPath: nodeKeyPath, preKeyPath: handlerNode.keyPath, users })
                    }
                    
                  }else{
                    newNoticeNodes = []
                  }
                }else{
                  const users = yield getWorkflowNodeUsers({
                    payload: {
                      workflow,
                      keyPath: nodeKeyPath,
                      recordId,
                      functionCode
                    }
                  })
                  if(users&&users.length>0){
                    newNoticeNodes.push({ keyPath: nodeKeyPath, preKeyPath: handlerNode.keyPath, users })
                  }

                }

            }
          }

          // if (addPendingItems.length == 0 && nextNodes) {
          //   isRecurse = true
          //   yield emptyNodeRecurses({ keyPath: nextNodes[0] }, newCurrentNodes);
          // }
        }
        // 数组对象根据关键字转字符转去重
        const quchong = (arr,k)=>{
          
          let narr = _.cloneDeep(arr);
          
          narr.map(item=>{
            item[k] = JSON.stringify(item[k])
          })
          let newList = _.uniqBy(narr,k);
          
          newList.map(item=>{
            item[k]= JSON.parse(item[k]);
          })
          return newList
        }

        // 纯数组转字符转去重 例：[[1,"children",0,0],[1,"children",0,0],[1,"children",0,0]] 
        const quchong1 = (arr)=>{
          let narr = _.cloneDeep(arr);
          let narrr = narr.map(item=>{
            return JSON.stringify(item)
          })
          let narrrr = _.uniq(narrr)
          let narrrrr = narrrr.map(item=>{
            return JSON.parse(item);
          })
          return narrrrr
        }

        // 将知会的人和审核的人汇总，提取出userId;
        // let nl = currentNoticeNodes ?currentNoticeNodes.concat(newNoticeNodes):newNoticeNodes
        let nl = newNoticeNodes
        let allreadpeoList = [...newCurrentNodes, ...nl];
        let pKeyList = allreadpeoList.map(item=>{return item.users[0].userId})
        let addPendingItemsR = [...addPendingItems, ...pKeyList];
        try {
          const handleData = yield request("/sysWorkflow/sysWorkflowNodeController", {
            id: recordId,
            nodeCode: workflow.getIn([...handlerNode.keyPath, "unionid"]),
            callBack: workflow.getIn([...handlerNode.keyPath, "callBackCache"]),
            nodeState, //节点状态
            workflowTitle: nodeTitle,
            wfState: newCurrentNodes.length > 0 ? workflowStatusConst.process.value : workflowStatusConst.done.value,
            addPendingItems:_.uniq(addPendingItemsR), //下一个节点
            removePendingItems,
            currentNodeUsers: handlerNode.users, //当前节点
            sysWorkflowInfo: {
              ...data.result,
              wfState: newCurrentNodes.length > 0 ? workflowStatusConst.process.value : workflowStatusConst.done.value,
              wfContent: JSON.stringify({
                ...wfContent,
                workflow: workflowJSON,
                currentNodes: quchong(newCurrentNodes, "keyPath"),
                handlerRecords,
                unsuperveneNodes:quchong1(unsuperveneNodes),
                noticeNodes: currentNoticeNodes ? quchong(currentNoticeNodes.concat(newNoticeNodes), "keyPath"): quchong(newNoticeNodes, "keyPath") 
              })
            }
          })
          // 提交后如果是首节点且下一节点是当前仅为登录人自动审核
          if (handleData) {
            if (workflowNode.get("first")) {
              if (addPendingItems.length == 1 && handlerNode.users[0].userId == addPendingItems[0]) {
                yield put({ type: "workflow/verfiedWorkflow", payload: { recordIds: [recordId], functionCode } })
                return
              }
            }
          }

          message.success(successMessage || handlerSuccess)
          yield put({ type: "form/fetchFormDetailData", payload: { id: recordId, functionCode } })
          yield put({ type: "workflow/refreshCurrentWorkflow" , payload : {functionCode:functionCode,id:recordId} }) //刷新
        } catch (e) {
          console.log(866,e)
          message.error(work_err)
        }

        if (newCurrentNodes.length == 1 && newCurrentNodes[0].users.length == 0 && newCurrentNodes[0].keyPath[0] < workflowJSON.length) {
          yield put({ type: "workflow/verfiedWorkflow", payload: { recordIds: [recordId], functionCode } })
        }
        // try {
        //   // 串发相同人员自动提交
        //   console.log('handlerNode', handlerNode);
        //   if (workflowNode.get('first')) {
        //     if (addPendingItems.length == 1 && handlerNode.users && handlerNode.users[0].userId == addPendingItems[0] && !isRecurse) {
        //       yield put({ type: 'workflow/verfiedWorkflow', payload: { recordIds: [recordId], functionCode } })
        //       return
        //     }
        //   }
        // } catch (e) {
        //   console.log('828', e);
        // }
      } else {
        message.error(notHandlerPersion)
      }
    } else {
      message.error(notHandlerPersion)
    }
  } else {
    message.error(notHandlerPersion)
  }
}

let unsuperveneNodes = []
let noticeNodes = []
function calculateNextNodes(workflow, currentKeyPath, currentNodes, info) {
  window.localStorage.setItem("yuanNode", JSON.stringify(currentKeyPath));
  unsuperveneNodes = []
  noticeNodes = []
  var result = undefined
  return new Promise(resolve => {
    const loop = keyPath => {
      const childrenIndex = _.lastIndexOf(keyPath, "children")
      if (childrenIndex === -1) {
        resolve({ nextNodes: undefined })
        return
      }
      const parentKeyPath = _.slice(keyPath, 0, childrenIndex)
      if (_.find(currentNodes, node => node.keyPath.toString().indexOf(parentKeyPath.toString()) === 0)) {
        resolve({ nextNodes: undefined })
        result = { nextNodes: undefined }
        return
      } else {
        const { nextNodes } = calculateNextNodesWithoutChildren(workflow, parentKeyPath, info)
        if (!nextNodes) {
          loop([...parentKeyPath])
        } else {
          resolve({ nextNodes })
          return
        }
      }
    }
    const { nextNodes } = calculateNextNodesWithoutChildren(workflow, currentKeyPath, info)
    if (!nextNodes) {
      loop(currentKeyPath)
    } else {
      resolve({ nextNodes })
    }
  })
}

function calculateNextNodesWithoutChildren(workflow, currentKeyPath, info) {
  const newKeyPath = [...currentKeyPath]
  const lastIndex = newKeyPath.pop()
  newKeyPath.push(lastIndex + 1)
  const nextWorkflow = workflow.getIn(newKeyPath)
  const nextNodes = []
  if (nextWorkflow && !nextWorkflow.get("last")) {
    getWorkFlowHeaderNode(nextWorkflow, newKeyPath, nextNodes, info, workflow)
    return { nextNodes }
  } else {
    return { nextNodes: undefined }
  }
}

function getWorkFlowHeaderNode(workflow, parentKeyPath, nodes, info, originalWorkflow) {
  if (List.isList(workflow.get("children"))) {
    if (!workflow.get("isSupervene")) {
      unsuperveneNodes.push(parentKeyPath)
    }
    workflow.getIn(["children"]).map((itemWorkflow, itemWorkflowIndex) => {
      getWorkFlowHeaderNode(itemWorkflow.get(0), [...parentKeyPath, "children", itemWorkflowIndex, 0], nodes, info, originalWorkflow)
    })
  } else {
    // let parentNode = workflow.getIn(parentKeyPath);
    const enterCon = workflow.get("enterCon")//进入
    const ignoreCon = workflow.get("ignoreCon")//跳过
    let isEnter = true
    let isIgnore = false
    if (ignoreCon) {
      let cons = JSON.parse(ignoreCon)
      if (cons.length > 0) {
        isIgnore = cons.length > 0 && calculateConds(cons, info)
      }
    }
    if (enterCon) {
      let cons = JSON.parse(enterCon)
      if (cons.length > 0) {
        isEnter = cons.length > 0 && calculateConds(cons, info)
      }
    }
    if (isEnter) {
      //判断是是否是知会类型，是：加入通知节点，并跳过，查找下个节点返回
      if (workflow.get("handleType") === wfHandleType.notice.value) {
        noticeNodes.push(parentKeyPath)           //加入通知节点
        const { nextNodes } = calculateNextNodesWithoutChildren(originalWorkflow, parentKeyPath, info)            //获取下个节点
        if (nextNodes) {
          // nodes = nodes.concat(nextNodes);
          nextNodes.map(nextNode => {
            nodes.push(nextNode)
          })
        }else{
          let yuanN = JSON.parse(localStorage.getItem("yuanNode"));
          let genList = yuanN.slice(0,yuanN.length-1);
          let genNode  = originalWorkflow.getIn(genList);
          let rGen = parentKeyPath.slice(0, yuanN.length);
          const { nextNodes } = calculateNextNodesWithoutChildren(originalWorkflow, rGen, info);
          if(nextNodes){
            nextNodes.map(nextNode => {
              nodes.push(nextNode)
            })
          }
        }
        return
      }
      if (isIgnore) {
        const { nextNodes } = calculateNextNodesWithoutChildren(originalWorkflow, parentKeyPath, info)
        if (nextNodes) {
          // nodes = nodes.concat(nextNodes);
          nextNodes.map(nextNode => {
            nodes.push(nextNode)
          })
        }
      } else {
        nodes.push(parentKeyPath)
      }
    }
  }
}

function calculateConds(cons, info) {
  let result = false
  if (cons) {
    let currentGroup = ""
    result = false
    let currentGroupResult = false
    try {
      cons.map(con => {
        if (con.group && con.dataIndex && con.relation && con.condition) {
          let value = info[con.dataIndex]
          let conValue = ""
          if (con.type === "treeSelect" || con.type === "treeSelectForbidParent" || con.type === "select") {
            conValue = (con.value && con.value.value) || ""
          } else if (con.type === "popup") {
            if(con.value && con.value.value && con.value.value.length > 0 && con.value.mainKey){
              value = info[con.value.mainKey];//popup要去取mainKey
            }
            conValue = (con.value && con.value.value && con.value.value.length > 0 && con.value.value[0].value) || ""
          } else {
            conValue = con.value
          }
          let conResult = wfConditionConst[con.condition].operation(value, conValue)
          if (currentGroup !== con.group) {
            result = wfRelationConst[con.relation].operation(currentGroupResult, result)
            if (currentGroup) {
              currentGroupResult = wfRelationConst[con.relation].operation(currentGroupResult, conResult)
            } else {
              currentGroupResult = conResult
            }
          } else {
            currentGroupResult = wfRelationConst[con.relation].operation(currentGroupResult, conResult)
          }
          currentGroup = con.group
        }

        return true
      })
      result = result || currentGroupResult
    } catch (e) {
      console.log(1037,e)
      result = false
    }
  }
  return result
}

function* withdrawWorkflow(action) {
  const { functionCode, recordIds } = action.payload
  if (!recordIds || recordIds.length === 0) {
    message.error(notRecord)
    return
  }
  let recordId = recordIds[0]
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
  if (data && data.result && data.result.wfContent) {
    let wfContent = JSON.parse(data.result.wfContent)
    let { workflow: workflowJSON, handlerRecords, currentNodes } = wfContent
    let workflow = fromJS(workflowJSON)
    if (data.result.wfState === workflowStatusConst.new.value) {
      message.error(notSubmit)
      return
    }
    if (_.head(handlerRecords).userId !== basicInfo.id) {
      message.error(notHandlerPersion)
      return
    }
    const handlerRecord = _.findLast(handlerRecords, {
      userId: basicInfo.id,
      dealWithResult: dealWithResultStatusConst.submitted.value
    })
    const afterHandlerRecords = handlerRecords.slice(handlerRecords.indexOf(handlerRecord), handlerRecords.length)
    let verifiedCount = 0
    let recallCount = 0
    let removePendingItems = []
    afterHandlerRecords.map(record => {
      if (record.dealWithResult === dealWithResultStatusConst.verified.value) {
        verifiedCount++
      } else if (record.dealWithResult === dealWithResultStatusConst.recall.value) {
        recallCount++
      }
      // removePendingItems.push(handlerRecord.userId);
      return true
    })
    if (verifiedCount > recallCount) {
      message.error(precessForbitAbandon)
      return
    }
    _.map(currentNodes, currentNode => {
      _.map(currentNode.users, user => removePendingItems.push(user.userId))
    })
    handlerRecords.push({
      keyPath: [0],
      userId: basicInfo.id,
      userName: basicInfo.name,
      dealWithResult: dealWithResultStatusConst.abandon.value
      // dateTime: moment().format(dateTimeFormatConst)后台取值
    })
    yield request("/sysWorkflow/sysWorkflowNodeController", {
      id: recordId,
      nodeCode: workflow.getIn([[0], "unionid"]),
      callBack: workflow.getIn([[0], "callBackCache"]),

      nodeState: dealWithResultStatusConst.abandon.value,
      wfState: workflowStatusConst.new.value,
      removePendingItems: removePendingItems,
      sysWorkflowInfo: {
        ...data.result,
        wfState: workflowStatusConst.new.value,
        wfContent: JSON.stringify({
          ...wfContent,
          workflow: workflowJSON,
          currentNodes: [{ keyPath: [0], users: [{ userId: basicInfo.id, username: basicInfo.name, handled: false }] }],
          handlerRecords
        })
      }
    })
    message.success(withdrawSuccess)
    yield put({ type: "workflow/refreshCurrentWorkflow" , payload : {functionCode:functionCode,id:recordId} }) //刷新
  } else {
    message.error(notHandlerPersion)
  }
}

function* readWorkflow(action) {
  let { result, functionCode, recordId } = action.payload
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  if (result && result.wfContent) {
    let wfContent = JSON.parse(result.wfContent)
    let { noticeNodes: currentNoticeNodes, workflow: workflowJSON } = wfContent
    let workflow = fromJS(workflowJSON)
    if (currentNoticeNodes && currentNoticeNodes.length > 0) {
      let noticeNode = _.find(currentNoticeNodes, node => {
        return _.find(node.users, { userId: basicInfo.id, handled: false })
      })

      if (noticeNode) {
        _.map(currentNoticeNodes, node => {
          _.map(node.users, user => {
            if (user.userId === basicInfo.id) {
              user.handled = true
            }
          })
        })
        let catchsN = workflow.getIn(noticeNode.keyPath)
        yield request("/sysWorkflow/sysWorkflowNodeController", {
          id: recordId,
          nodeCode: catchsN.get("unionid"),
          callBack: catchsN.get("callBackCache"),
          nodeState: dealWithResultStatusConst.read.value,
          wfState: workflowStatusConst.process.value,
          removePendingItems: [basicInfo.id],
          sysWorkflowInfo: {
            ...result,
            wfState: workflowStatusConst.process.value,
            wfContent: JSON.stringify({
              ...wfContent,
              workflow: workflowJSON,
              noticeNodes: currentNoticeNodes
            })
          }
        })
        yield put({ type: "form/fetchFormDetailData", payload: { id: recordId, functionCode } })
        yield put({ type: "workflow/refreshCurrentWorkflow" , payload : {functionCode:functionCode,id:recordId} }) //刷新
      }
    }
  }
}

function* viewWorkflow(action) {
  const { functionCode, recordIds, hideWindow } = action.payload
  if (!recordIds || recordIds.length === 0) {
    message.error(notRecord)
    return
  }
  const recordId = recordIds[0]
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
  if (data && data.result && data.result.wfContent) {
    let wfContent = JSON.parse(data.result.wfContent)
    let { workflow: workflowJSON, currentNodes, handlerRecords, noticeNodes: currentNoticeNodes } = wfContent
    handlerRecords = handlerRecords || []
    currentNodes = currentNodes || []
    let workflow = fromJS(workflowJSON)
    handlerRecords &&
      handlerRecords.map(handerRecord => {
        if (handerRecord.invalid) {
          return false
        }
        workflow = workflow.updateIn(handerRecord.keyPath, child => {
          let childHandlerRecords = child.get("childHandlerRecords") || List([])
          childHandlerRecords = childHandlerRecords.push({
            userId: handerRecord.userId,
            userName: handerRecord.userName,
            message: handerRecord.message,
            date: handerRecord.date
          })
          child = child
            .set("childHandlerRecords", childHandlerRecords)
            .set("dealWithResult", handerRecord.dealWithResult)
            .set(
              "status",
              handerRecord.dealWithResult === dealWithResultStatusConst.submitted.value ||
                handerRecord.dealWithResult === dealWithResultStatusConst.verified.value
                ? workFlowStatusMapping.completed.key
                : workFlowStatusMapping.notArrived.key
            )
          //提交人显示
          if (child.get("first") && child.getIn(["childHandlerRecords", 0])) {
            child = child.set("name", child.getIn(["childHandlerRecords", 0]).userName)
          }
          return child
        })
      })
    currentNodes &&
      _.forEach(currentNodes, currentNode => {
        workflow = workflow.updateIn(currentNode.keyPath, child => {
          child = child.set("status", workFlowStatusMapping.process.key).set("users", currentNode.users)
          if (_.find(currentNode.users, { userId: basicInfo.id })) {
            child = child.set("add", "1")
          }
          return child
        })
      })
    currentNoticeNodes &&
      _.forEach(currentNoticeNodes, noticeNode => {
        let unReadUser = _.find(noticeNode.users, { handled: false })
        let status = ""
        if (!unReadUser) {
          status = workFlowStatusMapping.read.key
        } else {
          status = workFlowStatusMapping.unread.key
        }
        workflow = workflow.updateIn(noticeNode.keyPath, child => {
          child = child.set("status", status).set("users", noticeNode.users)
          return child
        })
      })
    yield put({
      type: "workflow/showWorkflowView",
      payload: {
        functionCode,
        recordId,
        workflowInfo: Map({
          currentNodes,
          handlerRecords,
          workflow
        }),
        hideWindow
      }
    })
  } else {
    message.error(noneWorkflow)
  }
}

function* addNodeToWorkflow(action) {
  const { functionCode, recordId, workflow } = action.payload
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))
  const data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
  if (data && data.result && data.result.wfContent) {
    let wfContent = JSON.parse(data.result.wfContent)
    wfContent.workflow = workflow
    yield request("/sysWorkflow/sysWorkflowNodeController", {
      id: recordId,
      callBack: workflow.getIn([...handlerRecord.keyPath, "callBackCache"]),
      sysWorkflowInfo: {
        ...data.result,
        wfContent: JSON.stringify(wfContent)
      }
    })
    message.success(work_addSuccess)
    yield viewWorkflow({ payload: { functionCode, recordIds: [recordId] } })
  }
}

function* getWorkflowNodeUsers(action) {
  const { workflow, keyPath, handled, recordId, functionCode } = action.payload;
  const node = workflow.getIn(keyPath)
  //001角色 002部门 003岗位 004职务
  if (node.get("type") === workflowNodeType.person.value) {
    return [{ userId: node.get("id"), username: node.get("name"), handled: handled || false }]
  } else {
    const data = yield request("/sysWorkflow/getSysUserByWorkflowNode", {
      funCode: functionCode,
      code: node.get("id"),
      type: node.get("type"),
      id: recordId
    })
    if (data && data.result) {
      return Array.from(data.result, user => {
        return { userId: user.id, username: user.userName, handled: handled || false }
      })
    }
  }
  return []
}

export const checkRecordInWorkFlow = function* (action) {
  const { functionCode, recordId } = action.payload
  const editIgnore = yield select(x => x.getIn(["workflow", functionCode, "editIgnore"]))
  const basicInfo = yield select(x => x.getIn(["system", "basicInfo"]))

  if (editIgnore === "1") {
    return false
  }
  const data = yield request("/sysWorkflow/getSysWorkflowInfo", { funCode: functionCode, fid: recordId })
  if (data && data.result) {
    const { workflow, currentNodes } = JSON.parse(data.result.wfContent)
    let isEdit = false
    currentNodes.map(item => {
      let arr = workflow
      let user = false
      item.keyPath.map((key, keyIdx) => {
        arr = arr[key]
      })
      item.users.map(use => {
        if (use.userId == basicInfo.id) {
          user = true
        }
      })
      if (arr.isEdit && user) {
        isEdit = true
      }
    })
    if (isEdit) {
      return false
    }
  }
  if (data && data.result && data.result.wfState !== workflowStatusConst.new.value) {
    return true
  }
  return false
}

export const closeDetailTab = function* (action) {
  const { functionCode } = action.payload
  const immClose = yield select(x => x.getIn(["workflow", functionCode, "immClose"]))
  const activeTab = yield select(x => x.getIn(["system", "activeTab"]))
  if (immClose === "1") {
    if (activeTab.type === "detail") {
      yield put({ type: "system/tabRemove", payload: activeTab })
      return
    }
    yield put({ type: "list/changeSelectedIdValue", payload: { functionCode: activeTab.functionCode, selectedId: "" } })
    yield put({
      type: "advanceList/changeSelectedId",
      payload: {
        selectedId: "",
        functionCode: activeTab.functionCode
      }
    })
  }
}
