import { fork, takeEvery } from 'redux-saga/effects';
import { List } from 'immutable'
import {
  watchAddTab,
  watchMenuClick,
  watchTabChange,
  watchFetchMenu,
  watchTabRemove,
  watchTabsRemove,
  watchTurnCorpLogin,
  watchJumpToEditPage,
  watchFetchNoticeList,
  watchJumpToDetailPage,
  watchNoticeItemClick,
  watchSaveCurrentTabCache,
  watchAddNewTabWithoutMenu,
  watchChangeActiveTabCache,
  watchRemoveActiveTab,
} from './model/system/sagas';
import {
  watchDeleteTable,
  watchChangeTable,
  watchSaveFormData,
  watchChangeFormData,
  watchFetchDetailData,
  watchChangeFormTableData,
  watchDeleteFormData,
  watchFormButtonClick,
  watchRemoveTabSuccess,
  watchGenerateRelationForm,
  watchCalculateWithFormula,
  watchFetchFormConfigByFunctionCode
} from './model/form/sagas';
import {
  watchListButtonClick,
  watchFetchListDataByParentId,
  watchFetchListDataByFunctionCode,
  watchFetchListConfigByFunctionCode,
} from './model/list/sagas';
import {
  watchFetchReferenceConfig,
  watchFetchReferenceTableData
} from './model/reference/sagas';
import {
  watchFetchDictionary
} from './model/dictionary/sagas';
import {
  watchFetchReportConfig,
  watchFetchReportData,
  watchReportButtonClick
} from './model/report/sagas';
import {
  watchFetchCustomReportConfig,
  watchFetchCustomReportData,
} from './model/customReports/sagas';
import {
  watchSendMessage,
  watchReceiveMessage
} from './model/message/sagas'
import {
  watchFetchNoteListData,
  watchFetchNoteFolderData
} from './model/note/sagas'
import {
  watchFetchAdvanceListConfig,
  watchAdvanceListButtonClick
} from './model/advanceList/sagas'
import {
  watchWatchShortFormAction
} from './model/shortForm/sagas';
import {
  watchInitDashboardConfig,
  watchSaveDashboardConfig,
  watchDashboardActions,
  watchFetchConfigForDashboard
} from './model/dashboard/sagas';
import {
  watchTreeFormAction
} from './model/treeForm/sagas';
import {
  watchWorkflowAction
} from './model/workflow/sagas';

function* watchAndLog() {
  // eslint-disable-next-line
  yield takeEvery('*', function* logger(action) {
    console.warn(action.type, List.isList(action.payload) ? action.payload.toJSON() : action.payload)
  })
}

export default function* rootSaga() {
  // var env = process.env.NODE_ENV || 'development';
  // if (env === 'development') {
  try {
    //yield fork(watchAndLog);
    yield fork(watchAddTab);
    yield fork(watchFetchMenu);
    yield fork(watchRemoveActiveTab);
    yield fork(watchMenuClick);
    yield fork(watchTabChange);
    yield fork(watchTabRemove);
    yield fork(watchTabsRemove);
    yield fork(watchSaveCurrentTabCache);
    yield fork(watchJumpToEditPage);
    yield fork(watchAddNewTabWithoutMenu);
    yield fork(watchSaveFormData);
    yield fork(watchFetchFormConfigByFunctionCode);
    yield fork(watchFetchListDataByFunctionCode);
    yield fork(watchFetchListConfigByFunctionCode);
    yield fork(watchFetchDetailData);
    yield fork(watchDeleteFormData);
    yield fork(watchChangeActiveTabCache);
    yield fork(watchFetchReferenceConfig);
    yield fork(watchFetchReferenceTableData);
    yield fork(watchFetchDictionary);
    yield fork(watchGenerateRelationForm);
    yield fork(watchListButtonClick);
    yield fork(watchFormButtonClick);
    yield fork(watchJumpToDetailPage);
    yield fork(watchFetchListDataByParentId);
    yield fork(watchFetchReportConfig);
    yield fork(watchChangeFormData);
    yield fork(watchChangeTable);
    yield fork(watchChangeFormTableData);
    yield fork(watchDeleteTable);
    yield fork(watchRemoveTabSuccess);
    yield fork(watchFetchReportData);
    yield fork(watchSendMessage);
    yield fork(watchFetchNoteListData);
    yield fork(watchFetchNoteFolderData);
    yield fork(watchReceiveMessage);
    yield fork(watchFetchAdvanceListConfig);
    yield fork(watchAdvanceListButtonClick);
    yield fork(watchCalculateWithFormula);
    yield fork(watchFetchNoticeList);
    yield fork(watchNoticeItemClick);
    yield fork(watchTurnCorpLogin);
    yield fork(watchReportButtonClick);
    yield fork(watchInitDashboardConfig);
    yield fork(watchSaveDashboardConfig);
    yield fork(watchFetchConfigForDashboard);
    yield fork(watchDashboardActions);
    yield fork(watchWatchShortFormAction);
    yield fork(watchTreeFormAction);
    yield fork(watchWorkflowAction);
    yield fork(watchFetchCustomReportConfig);
    yield fork(watchFetchCustomReportData);
  } catch (e) {
    console.log(149,e);
  }
}
