import {
  request as _request,
  download as _download,
  uploadImportFile as _uploadImportFile,
  downloadFromServer as _downloadFromServer,
  get as _get,
  requestPrintUi as _requestPrintUi
} from "./request"
import { quickButtons as _quickButtons } from "./quick-button"
import {
  referenceStyleType,
  groupType,
  defaultValueTypeConst,
  dateFormatConst,
  timeFormatConst,
  dateTimeFormatConst,
  formFieldType,
  monthFormatConst
} from "../common/const"
import { fileServerUrl, reportServerUrl, reportServerPort } from "../common/config"
import { Map, List } from "immutable"
import _ from "lodash"
import moment from "moment"
import { Modal } from "antd"
import { stompConnect as _stompConnect, stompDisconnect as _stompDisconnect } from "./stomp"

export const request = _request
export const get = _get
export const download = _download
export const uploadImportFile = _uploadImportFile
export const stompConnect = _stompConnect
export const stompDisconnect = _stompDisconnect
export const downloadFromServer = _downloadFromServer
export const requestPrintServer = _requestPrintUi
export const quickButtons = _quickButtons

// let cache = localStorage.getItem('cache') ? JSON.parse(localStorage.getItem('cache')) : {};

// export const setCache = (newCache) => {
//   localStorage.setItem('cache', JSON.stringify(newCache));
//   cache = newCache;
// }

let cacheReferenceIds = []
let cacheDictionaryIds = []
let cacheCustomPop = {}

export const resetReferenceAndDictionary = () => {
  cacheReferenceIds = []
  cacheDictionaryIds = []
}
// 暂存组件节点
export const registerCustomPopComponent = (key, C) => {
  cacheCustomPop[key] = { C: C }
}
// 获取暂存组件
export const getCachePopComponent = key => {
  return cacheCustomPop[key]
}

export const buildFileActualUrl = shortUrl => {
  return fileServerUrl + shortUrl
}

export const convertQuery = userLocation => {
  if (userLocation.search && userLocation.search.length > 0) {
    var search = userLocation.search.substring(1)
    const query = JSON.parse('{"' + decodeURI(search).replace(/"/g, '\\"').replace(/&/g, '","').replace(/=/g, '":"') + '"}')
    return query
  }
  return undefined
}

export const getItemSourceFromCache = (sourceKey, dictionary, dispatch) => {
  if (dictionary && dictionary.get(sourceKey)) {
    return dictionary.get(sourceKey)
  } else {
    if (cacheDictionaryIds.indexOf(sourceKey) < 0) {
      cacheDictionaryIds.push(sourceKey)
      dispatch &&
        dispatch({
          type: "dictionary/initDictionary",
          payload: sourceKey
        })
    }
  }
  return []
}

export const getValueFromSource = (source, key) => {
  var result = source.filter(x => x.value === key)
  return result && result.length > 0 ? result[0].label : key
}

export const getValueFormCache = (sourceKey, key, dictionary, dispatch) => {
  const source = getItemSourceFromCache(sourceKey, dictionary, dispatch)
  return getValueFromSource(source, key)
}

// 根绝条件筛选出所需要的内容
export const convertListToTreeSource = (data, treeField = { id: "id", name: "name", pid: "pid" }, scenario, zd) => {

  var result = []

  // 判断 条件内容是否是JSON格式的
  const isJSON = (str) => {
    if (typeof str == 'string') {

      try {
        var obj = JSON.parse(str);
        if (typeof obj == 'object' && obj) {
          return true;
        } else {
          return false;
        }
      } catch (e) {
        // console.log('error：'+str+'!!!'+e);
        return false;
      }
    }
  }

  let newData = [];
  try {
    if (scenario && Object.keys(scenario).length > 0) {//判断是否有筛选条件
      newData = [];
      let obgK = Object.keys(scenario);//将筛选条件的字段转换城数组， 防有多个筛选条件
      obgK.forEach(item => {
        if (isJSON(scenario[item])) {//判断条件的内容是否是多条件JSON
          let newTiaojian = JSON.parse(scenario[item]);
          let newlist = newTiaojian.realValue.split(",");
          newlist.forEach(realItem => {
            newData = [...newData, ..._.filter(data, { [item]: realItem })];
          })
        } else {
          let newl = scenario[item] && scenario[item].split(",");
          newl && newl.length > 0 && newl.forEach(realItem => {
            newData = [...newData, ..._.filter(data, { [item]: realItem })];
          })
        }
      })
    } else {
      newData = [...newData, ...data]
    }
  } catch (e) {
    console.log('e', e);
  }
  // let newData = scenario ? _.filter(data, scenario) : data
  newData = _.cloneDeep(newData)

  if (zd) {
    newData.map(item => {
      var hasParent = false
      newData.map(child => {
        if (item[treeField.id] === child[treeField.pid]) {
          var children = item.children || []
          children.push(child)
          // 组间排序
          children = _.sortBy(children, [zd.gCode, zd.lCode])

          item.children = children;
        }
        if (child[treeField.id] === item[treeField.pid]) {
          hasParent = true
        }
      })
      if (!hasParent) {
        // 组间排序
        item.children = _.sortBy(item.children, [zd.gCode, zd.lCode])

        result.push(item)
      }
      //删除空children
      if (item.children && item.children.length == 0) {
        delete item.children
      }
    })
    result = _.sortBy(result, [zd.gCode, zd.lCode]);
    return result
  }


  newData.map(item => {
    var hasParent = false
    newData.map(child => {
      if (item[treeField.id] === child[treeField.pid]) {
        var children = item.children || []
        children.push(child)
        // 组间排序
        children = _.sortBy(children, (sort) => {
          return sort.blockSortId
        })
        item.children = children
      }
      if (child[treeField.id] === item[treeField.pid]) {
        hasParent = true
      }
    })
    if (!hasParent) {
      // 组间排序
      item.children = _.sortBy(item.children, (sort) => {
        return sort.blockSortId
      })
      result.push(item)
    }
    //删除空children
    if (item.children && item.children.length == 0) {
      delete item.children
    }
  })
  result = _.sortBy(result, (sort) => {
    return sort.blockSortId
  })
  return result
}

// 生成树↑↑↑↑↑↑↑

export const convertNumToList = num => {
  let arr = []
  for (let i = 0; i < num; i++) {
    arr.push(i)
  }
  return arr
}

export const getStatusMappingByKey = (mapping, status) => {
  var filterItem = {}
  Object.keys(mapping).map(function (key) {
    const item = mapping[key]
    if (item.key === status) {
      filterItem = item
    }
    return true
  })
  return filterItem
}

export const getStatusMappingLabelByValue = (mapping, status) => {
  var filterItem = {}
  Object.keys(mapping).map(function (key) {
    const item = mapping[key]
    if (item.value === status) {
      filterItem = item
    }
    return true
  })
  return filterItem ? filterItem.label : ""
}

export const getStatusMappingItemByValue = (mapping, status) => {
  var filterItem = {}
  Object.keys(mapping).map(function (key) {
    const item = mapping[key]
    if (item.value === status) {
      filterItem = item
    }
    return true
  })
  return filterItem
}

export const translateObjectToList = obj => {
  const newList = []
  if (obj) {
    const newObj = { ...obj }
    Object.keys(newObj).map(key => {
      newList.push(newObj[key])
    })
  }
  return newList
}

export const convertListToObject = (list, key) => {
  const obj = {}
  if (list) {
    list.map(item => {
      if (item[key]) obj[item[key]] = item
      return true
    })
  }
  return obj
}

export const getLabelFromReference = (reference, value, field) => {
  if (reference && field && value) {
    if (value && value.indexOf(",") >= 0) {
      const labelArray = []
      const valueArray = value.split(",")
      valueArray &&
        valueArray.map(val => {
          const filteredReference = reference.filter(x => x[field["value"]] == val) //不判断类型
          labelArray.push(filteredReference && filteredReference[0] && filteredReference[0][field["label"]])
        })

      if (labelArray && labelArray.length > 0) {
        return labelArray.join(",")
      }
    } else {
      const filteredReference = reference.filter(x => x[field["value"]] == value) //不判断类型
      if (filteredReference && filteredReference.length > 0) {
        return filteredReference[0][field["label"]]
      }
    }
  }
  return value
}

export const getReferenceByItem = (reference, item, value, dispatch) => {
  if (item.referenceKey) {
    return getReferenceByKey(reference, item.referenceKey, value, dispatch)
  }
  return { itemSource: [], value }
}

export const getReferenceByKey = (reference, key, value, dispatch) => {
  if (reference.get(key)) {
    let itemSource = reference.get(key).get("treeData") || []
    const config = reference.get(key).get("config")
    if (config.refType === referenceStyleType.select.value || config.refType === referenceStyleType.treeSelect.value) {
      value = getLabelFromReference(itemSource, value, config.field)
    }
    return { itemSource, value }
  } else {
    if (cacheReferenceIds.indexOf(key) < 0) {
      cacheReferenceIds.push(key)
      dispatch({
        type: "reference/initReference",
        payload: key
      })
    }
  }
  return { itemSource: [], value }
}
//配置,值,dispatch,参照,字典
export const getItemActualValue = (item, label, dispatch, reference, dictionary) => {
  label = getItemDisplayData(item, label) //初始值
  let itemSource = item.source || (item.sourceKey && getItemSourceFromCache(item.sourceKey, dictionary, dispatch))
  if (itemSource && itemSource.length > 0) {
    // const filterResult = itemSource.filter(({ value }) => value === label);  //2020/07/02 chenfeng 不清楚用途，变量未调用
    let a = false
    label = getValueFromSource(itemSource, label)
  }
  if (item.referenceKey) {
    label = getReferenceByItem(reference, item, label, dispatch).value
  }
  return label === "null" ? "" : label
}
// 获取参照
export const getReferenceSourceByKey = (reference, key, dispatch) => {
  if (reference.get(key)) {
    let itemSource = reference.get(key).get("treeData") || []
    const config = reference.get(key).get("config")
    return { itemSource, config }
  } else {
    if (cacheReferenceIds.indexOf(key) < 0) {
      cacheReferenceIds.push(key)
      dispatch({
        type: "reference/initReference",
        payload: key
      })
    }
  }
  return { itemSource: [], config: {} }
}
// 获取字典或参照
export const getItemSourceByKey = (item, dispatch, reference, dictionary) => {
  let itemSource = item.source || (item.sourceKey && getItemSourceFromCache(item.sourceKey, dictionary, dispatch))
  let config
  if (item.referenceKey) {
    const { itemSource: actualItemSource, config: actualItemConfig } = getReferenceSourceByKey(reference, item.referenceKey, dispatch)
    itemSource = actualItemSource
    config = actualItemConfig
  }
  return { itemSource, config }
}

export const filterEmptyKey = oldObj => {
  let newObj = {}
  if (oldObj) {
    Object.keys(oldObj).map(key => {
      if (oldObj[key]) {
        newObj[key] = oldObj[key]
      }
    })
  }
  return newObj
}

export const mergeObject = (fromObj, toObj) => {
  fromObj &&
    Object.keys(fromObj).map(key => {
      toObj[key] = fromObj[key]
      return true
    })
}

export const newGuid = () => {
  var guid = ""
  for (var i = 1; i <= 32; i++) {
    var n = Math.floor(Math.random() * 16.0).toString(16)
    guid += n
    if (i == 8 || i == 12 || i == 16 || i == 20) guid += "-"
  }
  return guid
}

export const renderAColor = function () {
  var r = Math.floor(Math.random() * 150)
  var g = Math.floor(Math.random() * 200)
  var b = Math.floor(Math.random() * 50)
  return "rgb(" + r + "," + g + "," + b + ")"
}

export const fullScreen = () => {
  var el = document.documentElement
  var rfs = el.requestFullScreen || el.webkitRequestFullScreen || el.mozRequestFullScreen || el.msRequestFullscreen
  if (typeof rfs !== "undefined" && rfs) {
    rfs.call(el)
  }

  return
}
//2021-10-14 16:56:49:快捷日期
export const fastDate = {
  first_month: {
    value: "first_month",
    label: "当月第一天"
  },
  last_month: {
    value: "last_month",
    label: "当月最后一天"
  },
  first_year: {
    value: "first_year",
    label: "当年第一天"
  },
  last_year: {
    value: "last_year",
    label: "当年最后一天"
  },
  today_begin: {
    value: "today_begin",
    label: "今天开始"
  },
  today_end: {
    value: "today_end",
    label: "今天结束"
  },
  today_beginToEnd: {
    value: "today_beginToEnd",
    label: "今天开始到结束"
  },
}

export const getFastDate = value => {
  let td = new Date();
  let bDate = td.getFullYear() + "-" + (td.getMonth() + 1).toString().padStart(2, "0") + "-" + td.getDate().toString().padStart(2, "0") + " 00:00:00";
  let eDate = td.getFullYear() + "-" + (td.getMonth() + 1).toString().padStart(2, "0") + "-" + td.getDate().toString().padStart(2, "0") + " 23:59:59";
  switch (value) {
    case "last_month":
      return moment().endOf("month")
    case "first_month":
      return moment().startOf("month")
    case "last_year":
      return moment().endOf("year")
    case "first_year":
      return moment().startOf("year")
    case "today_begin": {
      return bDate
    }
    case "today_end": {
      return eDate
    }
    case "today_beginToEnd":
      return JSON.stringify([bDate, eDate]);
  }
}

export const getBeginAndEndDate = (value, formatType) => {
  switch (value) {
    case "current_week":
      if (formatType) {
        return [moment().startOf("week").format(formatType), moment().endOf("week").format(formatType)]
      } else {
        return [moment().startOf("week"), moment().endOf("week")]
      }
    case "last_week":
      if (formatType) {
        return [moment().startOf("week").format(formatType), moment().endOf("week").format(formatType)]
      } else {
        return [
          moment()
            .week(moment().week() - 1)
            .startOf("week"),
          moment()
            .week(moment().week() - 1)
            .endOf("week")
        ]
      }
    case "current_month":
      if (formatType) {
        return [moment().startOf("month").format(formatType), moment().endOf("month").format(formatType)]
      } else {
        return [moment().startOf("month"), moment().endOf("month")]
      }
    case "last_month":
      if (formatType) {
        return [
          moment()
            .month(moment().month() - 1)
            .startOf("month")
            .format(formatType),
          moment()
            .month(moment().month() - 1)
            .endOf("month")
            .format(formatType)
        ]
      } else {
        return [
          moment()
            .month(moment().month() - 1)
            .startOf("month"),
          moment()
            .month(moment().month() - 1)
            .endOf("month")
        ]
      }
    case "current_quarter":
      if (formatType) {
        return [moment().startOf("quarter").format(formatType), moment().endOf("quarter").format(formatType)]
      } else {
        return [moment().startOf("quarter"), moment().endOf("quarter")]
      }
    case "last_quarter":
      if (formatType) {
        return [
          moment()
            .quarter(moment().quarter() - 1)
            .startOf("quarter")
            .format(formatType),
          moment()
            .quarter(moment().quarter() - 1)
            .endOf("quarter")
            .format(formatType)
        ]
      } else {
        return [
          moment()
            .quarter(moment().quarter() - 1)
            .startOf("quarter"),
          moment()
            .quarter(moment().quarter() - 1)
            .endOf("quarter")
        ]
      }
    case "current_year":
      if (formatType) {
        return [moment().startOf("year").format(formatType), moment().endOf("year").format(formatType)]
      } else {
        return [moment().startOf("year"), moment().endOf("year")]
      }
    case "last_year":
      if (formatType) {
        return [
          moment()
            .year(moment().year() - 1)
            .startOf("year")
            .format(formatType),
          moment()
            .year(moment().year() - 1)
            .endOf("year")
            .format(formatType)
        ]
      } else {
        return [
          moment()
            .year(moment().year() - 1)
            .startOf("year"),
          moment()
            .year(moment().year() - 1)
            .endOf("year")
        ]
      }
    case "first_half_year":
      if (formatType) {
        return [
          moment().startOf("year").format(formatType),
          moment().month(5).date(30).hours(23).minutes(59).seconds(59).format(formatType)
        ]
      } else {
        return [moment().startOf("year"), moment().month(5).date(30).hours(23).minutes(59).seconds(59)]
      }
    case "second_half_year":
      if (formatType) {
        return [moment().month(6).date(1).hours(0).minutes(0).seconds(0).format(formatType), moment().endOf("year").format(formatType)]
      } else {
        return [moment().month(6).date(1).hours(0).minutes(0).seconds(0), moment().endOf("year")]
      }
    case "yesterday":
      if (formatType) {
        return [moment().add(-1, "days").format(formatType), moment().add(-1, "days").format(formatType)]
      } else {
        return [moment().add(-1, "days"), moment().add(-1, "days")]
      }
    default:
      if (formatType) {
        return [
          moment()
            .add(Number(value || 0), "days")
            .format(formatType),
          moment().format(formatType)
        ]
      } else {
        return [moment().add(Number(value || 0), "days"), moment()]
      }
  }
}

export const getItemDefaultValue = item => {
  if (item.defaultValueType === defaultValueTypeConst.loginParams.value) {
    const basicInfoString = localStorage.getItem("basicInfo")
    if (!basicInfoString) {
      return
    }
    const basicInfo = JSON.parse(basicInfoString)
    return basicInfo[item.defaultValue] || ""
  } else if (item.defaultValueType === defaultValueTypeConst.time.value) {
    return moment().add(item.defaultValue, "h").format(timeFormatConst)
  } else if (item.defaultValueType === defaultValueTypeConst.date.value) {
    if (item.type === formFieldType.dateTime.value) {
      return moment().add(item.defaultValue, "d").format(dateTimeFormatConst)
    }
    return moment().add(item.defaultValue, "d").format(dateFormatConst)
  } else if (item.defaultValueType === defaultValueTypeConst.fastDate.value) {
    if (["today_beginToEnd", "today_begin", "today_end"].indexOf(item.defaultValue) > -1) {
      return getFastDate(item.defaultValue)
    }
    return getFastDate(item.defaultValue).format(dateFormatConst)
  } else if (item.defaultValueType === defaultValueTypeConst.month.value) {
    return moment().add(item.defaultValue, "M").format(monthFormatConst)
  } else if (item.defaultValueType === defaultValueTypeConst.dataRanger.value) {
    if (item.defaultValue) {
      const range = item.defaultValue.split(",")
      if (range && range.length === 2 && range[0] <= range[1]) {
        const defaultValue = JSON.stringify([
          moment().add(range[0], "d").format(dateFormatConst),
          moment().add(range[1], "d").format(dateFormatConst)
        ])
        return defaultValue
      }
    }
    return ""
  } else if (item.defaultValueType === defaultValueTypeConst.customDateRange.value) {
    if (item.defaultValue) {
      const [begin, end] = getBeginAndEndDate(item.defaultValue)
      return JSON.stringify([begin.format(dateFormatConst), end.format(dateFormatConst), item.defaultValue])
    }

    return ""
  }

  return item.defaultValue || ""
}

export const getItemDisplayData = (item, value) => {
  if (!value) {
    return value
  }
  if (item.type === formFieldType.date.value) {
    return moment(value).format(dateFormatConst)
  } else if (item.type === formFieldType.dateTime.value) {
    return moment(value).format(dateTimeFormatConst)
  } else if (item.type === formFieldType.customDateTime1.value) {
    return moment(value).format("YYYY-MM-DD HH")
  } else if (item.type === formFieldType.customDateTime2.value) {
    return moment(value).format("YYYY-MM-DD HH:mm")
  } else if (item.type === formFieldType.customDateTime3.value) {
    return moment(value).format("YYYY-MM-DD HH:mm:ss")
  }
  // else if (item.type === formFieldType.time.value) {
  //   return moment(value).format(timeFormatConst);
  // }
  else if (item.type === formFieldType.amount.value) {
    return Number(value) || value === "0" ? Number(value).toFixed(window.numberFixedConst) : value
  }

  return value
}

export const strToHexCharCode = str => {
  if (str === "") return ""
  var hexCharCode = []
  hexCharCode.push("0x")
  for (var i = 0; i < str.length; i++) {
    hexCharCode.push(str.charCodeAt(i).toString(16))
  }
  return hexCharCode.join("")
}

export const checkButtonCondition = (button, checkValue) => {
  if (button.rowTable && button.rowField && button.rowValue) {
    return checkFieldCondition(button.rowValue, checkValue)
  }
  return true
}

export const checkFieldCondition = (condition, checkValue) => {
  const execScript = (parseFloat(checkValue) || 0) + condition
  try {
    const checkResult = eval(execScript)
    if (typeof checkResult === "boolean") {
      if (!checkResult) {
        return false
      }
    } else if (condition.split(",").indexOf(checkValue) < 0) {
      return false
    }
  } catch (e) {
    if (condition.split(",").indexOf(checkValue) < 0) {
      return false
    }
    console.log(723, e)
  }
  return true
}

export const clearUserLoginInfo = () => {
  localStorage.removeItem("token")
  localStorage.removeItem("menus")
  localStorage.removeItem("permissions")
  localStorage.removeItem("basicInfo")
  localStorage.removeItem("cfgContent")
  localStorage.removeItem("thirdPartyConfiguration") //微信企业微信钉钉配置
  localStorage.removeItem("autoLoginConfig") //自动登录配置信息
}

export const ConfirmModal = config => {
  return new Promise(resolve => {
    Modal.confirm({
      ...config,
      onOk() {
        resolve(true)
      },
      onCancel() {
        resolve(false)
      }
    })
  })
}

export const getBMap = ak => {
  return new Promise(function (resolve, reject) {
    const script = document.createElement("script")
    script.type = "text/javascript"
    script.dataset.name = "map"
    script.src = `https://api.map.baidu.com/api?v=2.0&ak=${ak}&callback=init` //callback调用init函数。
    document.head.appendChild(script)
    window.init = () => {
      // eslint-disable-next-line no-undef
      resolve(BMap)
    }
  })
}

export const validateItemPermission = (permissonStr, basicInfo) => {
  if (permissonStr && permissonStr.length > 0) {
    const showRolePermissions = JSON.parse(permissonStr)
    if (showRolePermissions.length > 0) {
      let hasShowPermission = true
      for (let rolePermissionIndex = 0; rolePermissionIndex < showRolePermissions.length; rolePermissionIndex++) {
        if (basicInfo.roles.indexOf(showRolePermissions[rolePermissionIndex].id) >= 0) {
          hasShowPermission = false
          break
        }
      }
      return hasShowPermission
    }
  }
  return true
}

export const sortTableColumns = columns => {
  if (!columns) {
    return columns
  }
  const left = []
  const right = []
  const middle = []
  columns.map(column => {
    if (column.fixed === "left") {
      left.push(column)
    } else if (column.fixed === "right") {
      right.push(column)
    } else {
      column.fixed = ""
      middle.push(column)
    }
    return true
  })
  return left.concat(middle).concat(right)
}

export const getReportServerUrl = () => {
  //return reportServerUrl.indexOf("debug") === -1 ? reportServerUrl + "cf_report/": reportServerUrl.replace('test','');
  // return reportServerUrl ? reportServerUrl : window.location.protocol + "//" + window.location.hostname + ":" + reportServerPort + "/";
  return reportServerUrl ? reportServerUrl : window.location.protocol + "//" + window.location.hostname + ":" + window.location.port + "/"
}

export const returnMinusWidth = (item, info) => {
  let minusWidth = 0
  if (item.filter && item.filter.filterReg) {
    minusWidth += 20
  }
  if (item.referenceRefresh === "1") {
    minusWidth += 20
  }
  if (item.editDictionary === "1") {
    minusWidth += 20
  }
  if (item.sourceKey && info) {
    minusWidth += 20
  }
  return minusWidth
}

export const isJSON = (str) => {
  if (typeof str == 'string') {
    try {
      let obj = JSON.parse(str);
      if (typeof obj == 'object' && obj) {
        return true;
      } else {
        return false;
      }
    } catch (e) {
      // console.log(837,e);
      return false;
    }
  }
  console.log('It is not a string!')
}
export const ifTrue = (str) => {
  if (str == false || str == 'undefined' || str == 'false' || str == '0') {
    return false;
  }
  if (str == true || str == 'true' || str == '1') {
    return true;
  }
}

// TODO:时间戳转时间格式
export const timestampToTime = (time) => {
  let day = new Date(time)
  return day.getFullYear() + '-' + (day.getMonth() + 1).toString().padStart(2, '0') + '-' + day.getDate().toString().padStart(2, '0');
}

//金额数字千分位展示
export function numFormat(num) {
  // console.log('numFormat',window.thousandthsDisplay)
  let thousandthsDisplay = window.thousandthsDisplay || '0';
  if (thousandthsDisplay === true && num !== '-') {
    var res = num.toString().replace(/\d+/, function (n) { // 先提取整数部分
      return n.replace(/(\d)(?=(\d{3})+$)/g, function ($1) {
        return $1 + ",";
      });
    })
    return res;
  } else {
    return num;
  }
}