import isomorphicFetch from 'isomorphic-fetch';
import { message, Modal, Table } from 'antd';
import React, { Component } from 'react';
import { serverUrl, fileServerUrl, reportServerUrl, reportServerPort } from '../common/config';
import en from "../locale/en_US";
import zh from "../locale/zh_CN";
class FetchTimeOutError extends Error {
}


const CORS_SIMPLE_METHODS = ['GET', 'HEAD'];
let modal = undefined;

let id = '';

export const request = (url, body = {}, loading = true, withReject = false, funCode = '', actionId = '') => {
  if (!id) {
    const basicInfoString = localStorage && localStorage.getItem('basicInfo')
    if (basicInfoString) {
      try {
        const basicInfo = JSON.parse(basicInfoString);
        id = basicInfo.id;
      } catch (e) {
        console.error(e);
      }
    }
  }

  const queryScope = (localStorage && localStorage.getItem(funCode + id + 'listQueryScope')) || window.queryScope;

  const finalOptions = {
    method: 'post',
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
      Accept: 'application/json; charset=utf-8',
      authorization: localStorage.getItem('token') || '',
      accessType: 'backend',
      funCode,
      actionId,
      queryScope
    },
    body: typeof body === 'string' ? body : JSON.stringify(body)
  };
  return fetch(url, finalOptions, loading, withReject)
}

export const get = (url) => {
  return fetch(url, { method: 'get' }, false, true);
}

function destroyMessage() {
  message.destroy();
}

function fetch(url, options, loading = true, withReject = false) {
  //输入http://不拼接serverUrl直接访问接口
  if (url && url.indexOf('http://') == -1 && url.indexOf('https://') == -1) {
    url = serverUrl + url
  }
  const lang = localStorage.getItem("lang")
  loading && window.startLoading && window.startLoading();
  return new Promise((resolve, reject) => {
    const onTimeout = () => reject(new FetchTimeOutError(`调用 ${url} 时间太长!`));
    const timeout = setTimeout(onTimeout, 2000000);
    isomorphicFetch(url, options)
      .then(parseResponse)
      .then((response) => {
        loading && window.stopLoading && window.stopLoading();
        clearTimeout(timeout);
        if (response.code === '0') {
          response.message && message.success(lang == 'en' ? response.messageEN : response.message);
          if (response && response.data && response.data.result && response.data.result.type && response.data.result.type === 'error') {
            let errorContent = <div style={{ marginTop: 10 }}><Table dataSource={response.data.result.data} columns={response.data.result.data1} />
              <div style={{ textAlign: 'center', marginTop: 10, cursor: 'pointer' }} onClick={() => destroyMessage()}>关闭</div></div>
            message.error(errorContent, 0);
          }
          resolve(response.data);
        } else if (response.code === '1') {
          Modal.confirm({
            title: '提醒',
            content: lang == 'en' ? response.messageEN : response.message,
            onOk() {
              let newOptions = { ...options };
              newOptions.headers.doubleConfirm = 1;
              isomorphicFetch(url, newOptions)
                .then(parseResponse)
                .then((confirmResponse) => {
                  if (confirmResponse.code === '0') {
                    confirmResponse.message && message.success(lang == 'en' ? confirmResponse.messageEN : confirmResponse.message);
                    resolve(confirmResponse.data);
                  } else if (confirmResponse.code === '401') {
                    if (!modal) {
                      modal = Modal.warning({
                        title: '登录提醒',
                        content: lang == 'en' ? confirmResponse.messageEN : confirmResponse.message,
                        onOk() {
                          window.logoutSystem && window.logoutSystem();
                        }
                      })
                    }
                  } else {
                    response.message && message.error(lang == 'en' ? confirmResponse.messageEN : confirmResponse.message);
                    withReject && reject(confirmResponse.data)
                  }
                })
            },
            onCancel() {
              withReject && reject(response.data)
            },
          });
        } else if (response.code === '401') {
          if (!modal) {
            modal = Modal.warning({
              title: '登录提醒',
              content: lang == 'en' ? response.messageEN : response.message,
              onOk() {
                window.logoutSystem && window.logoutSystem();
              }
            })
          }
        } else {
          response.message && message.error(lang == 'en' ? response.messageEN : response.message);
          withReject && resolve(response.data)
        }
      })
      .catch((error) => {
        clearTimeout(timeout);
        console.log(error);
        loading && window.stopLoading && window.stopLoading();
        // message.error('系统错误， 请联系统管理员');
        withReject && reject();
      })
  })
}

function parseResponse(rawResponse) {
  return rawResponse.text()
    .then((response) => {
      try {
        return JSON.parse(response);
      } catch (e) {
        return response;
      }
    });
}

export const download = (url, body = {}, fileName) => {
  const fileNameUrl = url.indexOf('/') != -1 ? url.substr(url.lastIndexOf('/') + 1, url.length) : 'TemplateFile';
  const finalOptions = {
    method: 'post',
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
      authorization: localStorage.getItem('token') || '',
      accessType: 'backend',
    },
    mode: 'cors',
    body: typeof body === 'string' ? body : JSON.stringify(body)
  };
  return new Promise((resolve, reject) => {
    isomorphicFetch(fileServerUrl + url, finalOptions)
      .then(res => res.blob().then(blob => {
        const a = document.createElement('a');
        const url = window.URL.createObjectURL(blob);
        a.href = url;
        a.download = fileName || fileNameUrl;
        a.click();
        window.URL.revokeObjectURL(url);
      }))
      .catch(error => {
        message.error(`${fileName}文件下载失败，请稍后再试！`)
      })
  })
}

export const downloadFromServer = (url, body = {}, fileName, funCode) => {
  const finalOptions = {
    method: 'post',
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
      authorization: localStorage.getItem('token') || '',
      accessType: 'backend',
      funCode
    },
    mode: 'cors',
    body: typeof body === 'string' ? body : JSON.stringify(body)
  };
  let key = 'download';
  if(typeof body === 'object' && body.id){
    key = body.id
  }
  message.loading({ content: `准备下载 ${fileName}`, key, duration:0 });
  return new Promise((resolve, reject) => {
    isomorphicFetch(serverUrl + url, finalOptions)
      .then(res => res.blob().then(blob => {
        console.log(blob)
        const actualFileName = res.headers.get("Content-Disposition") ? res.headers.get("Content-Disposition").split(";")[1].split("filename=")[1] : fileName;
        const a = document.createElement('a');
        const url = window.URL.createObjectURL(blob);
        a.href = url;
        //a.download = actualFileName.trim();
        const fileNameUri = actualFileName.trim();
        a.download = decodeURIComponent(fileNameUri);
        a.click();
        window.URL.revokeObjectURL(url);
        message.success({ content: `${fileName} 下载完成!`, key, duration: 3 })
      }))
      .catch(error => {
        console.log(error);
        message.error(`${fileName}文件下载失败，请稍后再试！`,key)
      })
  })
}

export const uploadImportFile = (url, file, data) => {
  let filedata = new FormData();
  filedata.append('file', file);
  if (data) {
    Object.keys(data).map(key => {
      filedata.append(key, data[key]);
      return true;
    });
  }
  const finalOptions = {
    method: 'post',
    headers: {
      authorization: localStorage.getItem('token') || '',
      accessType: 'backend',
    },
    body: filedata,
  };
  window.startLoading && window.startLoading();
  return new Promise((resolve, reject) => {
    isomorphicFetch(serverUrl + url, finalOptions)
      .then(res => {
        window.stopLoading && window.stopLoading();
        const resultString = res.headers.get('result');
        if (resultString) {
          const result = JSON.parse(resultString);
          if (result.code === '0') {
            if (result.msg) {
              message.success(decodeURI(result.msg));
            }
          } else {
            if (result.msg) {
              message.error(decodeURI(result.msg));
            }
          }
          let fileName = result.fileName;
          if (fileName) {
            fileName = decodeURI(fileName);
            res.blob()
              .then(blob => {
                const a = document.createElement('a');
                const url = window.URL.createObjectURL(blob);
                a.href = url;
                a.download = fileName;
                a.click();
                window.URL.revokeObjectURL(url);
              })
          }
        }
      })
      .catch(error => {
        window.stopLoading && window.stopLoading();
        console.log(error);
      })
  })

}

export const requestPrintUi = (params, id) => {
  const fileName = params.newFileName, filePath = params.filePath, templateName = params.templateName, printUrl = params.reportRqurl;
  if (!id) {
    message.warn("缺失单据ID");
    return;
  }
  if (!fileName || !filePath || !printUrl) {
    message.warn("打印模板不存在");
    return;
  }

  //const url = reportServerUrl.indexOf("test") === -1 ? reportServerUrl + "cf_report/" + printUrl + "&iid=" + id : reportServerUrl.replace('test','') + printUrl + "&iid=" + id;
  // const url = reportServerUrl ? reportServerUrl + printUrl + "&iid=" + id : window.location.protocol + "//" + window.location.hostname + ":" + reportServerPort + "/" + printUrl + "&iid=" + id;
  const url = reportServerUrl ? reportServerUrl + printUrl + "&iid=" + id : window.location.protocol + "//" + window.location.hostname + ":" + window.location.port + "/" + printUrl + "&iid=" + id;
  // console.log("requestPrintUi -> url", encodeURI(url))
  window.open(encodeURI(url), templateName ? templateName : "打印");
}

