/**
 * Created by 王友生 on 2018-06-26.
 */

import React from 'react';
import Stomp from 'stompjs';
import { notification, Button } from 'antd';
import { newGuid } from './index';
import { activeMQ_SERVICE, activeMQUserName, activeMQPassword } from '../common/config'
import { messageTypeConst } from '../common/const'
import { get } from './request';

let currentUser = null;
let stompClient = null;
let stompDispatch = null;
export const stompConnect = (dispatch) => {
  if (! activeMQ_SERVICE) {
    get('/get_socket_config')
      .then((data) => {
        if (data) {
          connect(data.url, data.name, data.password, dispatch);
        }
      })
  } else {
    connect(activeMQ_SERVICE, activeMQUserName, activeMQPassword, dispatch);
  }
}

function connect(url, name, password, dispatch) {
  stompDisconnect();
  stompClient = Stomp.client(url);
  stompClient.debug = null;
  const headers = {
    'login': name,
    'passcode': password,
  };
  stompDispatch = dispatch;
  stompClient.connect(headers, () => {
    const basicInfoString = localStorage.getItem('basicInfo');
    if (basicInfoString) {
      currentUser = JSON.parse(basicInfoString);

      stompClient.subscribe(`/queue/${currentUser.id}`, receiveMessage);

      stompClient.subscribe(`/topic/amqTopic`, receiveMessage);

    }
  }, (e) => {
    console.log(49,e)
  });
}

function receiveMessage(message) {
  const dataString = message.body;
  if (dataString) {
    const data = JSON.parse(dataString);
    if (data.type === messageTypeConst.chat.value) {
      stompDispatch && stompDispatch({
        type: 'message/receiveNewMessage',
        payload: data
      })
      return;
    }
    const key = newGuid();
    const btn = <Button type="primary" size="small" onClick={() => {
      notification.close(key);
      stompDispatch({
        type: 'system/noticeItemClick',
        payload: {
          item: {
            msgType: data.type,
            funCode: data.funCode,
            fid: data.fid
          }
        }
      })
    }}>查看</Button>
    if (data.type === messageTypeConst.online.value) {
      notification.open({
        key,
        message: '上线通知',
        description: `${data.content}上线了`
      });
    } else if (data.type === messageTypeConst.warning.value) {
      notification.error({
        key,
        message: data.title,
        description: data.content,
        btn,
      });
    } else if (data.type === messageTypeConst.dealWith.value) {
      notification.warning({
        key,
        message: data.title,
        description: data.content,
        btn,
      });
    } else if (data.type === messageTypeConst.notice.value) {
      notification.info({
        key,
        message: data.title,
        description: data.content,
        btn,
      });
    } else if (data.type === messageTypeConst.alert.value) {
      notification.info({
        key,
        message: data.title,
        description: data.content,
        btn,
      });
    } else if (data.type === messageTypeConst.custom.value) {
      window.eventObj.sendEvent('receiveMessageFromServer', data.customerParams);
    }
    stompDispatch({
      type: 'system/increaseUnreadMessage'
    })
  }
}

export const sendMessage = (toUser, content) => {
  if (stompClient && currentUser) {
    const message = {
      type: messageTypeConst.chat.value,
      fromUser: currentUser,
      content,
    }
    stompClient.send(`/queue/${toUser.id}`, {}, JSON.stringify(message));
  }
}

export const stompDisconnect = () => {
  if (stompClient) {
    stompClient.disconnect();
    stompClient = null;
  }
}

export const stompSendMessageToUser = (userId, message) => {
  if (stompClient) {
    stompClient.send(`/queue/${userId}`, { priority: 9 }, message);
  } else {
    stompConnect();
  }
}

export default stompClient;
