/* eslint-disable */
process.env.BABEL_ENV = "development"
const TerserPlugin = require("terser-webpack-plugin")
const path = require("path")
const webpack = require("webpack")
const ManifestPlugin = require("webpack-manifest-plugin")

const shouldUseSourceMap = false

module.exports = {
  mode: "development",
  bail: true,
  devtool: shouldUseSourceMap ? "source-map" : false,
  entry: { main: "index.js" },
  context: path.resolve("src"),
  output: {
    // The build folder.
    path: path.resolve("dist"),
    filename: "[name].js",
    library: "CFBase",
    libraryTarget: "umd"
  },
  resolve: {
    modules: [path.resolve("src"), path.resolve("node_modules")],

    // These are the reasonable defaults supported by the Node ecosystem.
    // We also include JSX as a common component filename extension to support
    // some tools, although we do not recommend using it, see:
    // https://github.com/facebookincubator/create-react-app/issues/290
    // `web` extension prefixes have been added for better support
    // for React Native Web.
    extensions: [".web.js", ".mjs", ".js", ".json", ".web.jsx", ".jsx"],
    alias: {
      // Support React Native Web
      // https://www.smashingmagazine.com/2016/08/a-glimpse-into-the-future-with-react-native-for-web/
      "react-native": "react-native-web"
    },
    plugins: [
      // Prevents users from importing files from outside of src/ (or node_modules/).
      // This often causes confusion because we only process files within src/ with babel.
      // To fix this, we prevent you from importing files out of src/ -- if you'd like to,
      // please link the files into your node_modules/ and let module-resolution kick in.
      // Make sure your source files are compiled, as they will not be processed in any way.
      // new ModuleScopePlugin(paths.appSrc, [paths.appPackageJson]),
    ]
  },
  module: {
    strictExportPresence: true,
    rules: [
      // TODO: Disable require.ensure as it's not a standard language feature.
      // We are waiting for https://github.com/facebookincubator/create-react-app/issues/2176.
      // { parser: { requireEnsure: false } },

      // First, run the linter.
      // It's important to do this before Babel processes the JS.
      {
        test: /\.(js|jsx|mjs)$/,
        enforce: "pre",
        use: [
          {
            loader: require.resolve("eslint-loader")
          }
        ],
        include: path.resolve("src")
      },
      {
        // "oneOf" will traverse all following loaders until one will
        // match the requirements. When no loader matches it will fall
        // back to the "file" loader at the end of the loader list.
        oneOf: [
          // "url" loader works just like "file" loader but it also embeds
          // assets smaller than specified size as data URLs to avoid requests.
          {
            test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
            loader: require.resolve("url-loader"),
            options: {
              limit: 10000,
              name: "static/media/[name].[hash:8].[ext]",
              esModule: false
            }
          },
          // Process JS with Babel.
          {
            test: /\.(js|jsx|mjs)$/,
            include: path.resolve("src"),
            loader: require.resolve("babel-loader"),
            options: {
              plugins: [["import", { libraryName: "antd", style: "css" }], require.resolve("babel-plugin-transform-decorators-legacy")],
              compact: true
            }
          },
          {
            test: /\.less$/,
            exclude: "/node_modules",
            use: [
              {
                loader: "style-loader"
              },
              {
                loader: "css-loader",
                options: {
                  importLoaders: 1
                }
              },
              {
                loader: "postcss-loader",
                options: {
                  ident: "postcss"
                }
              },
              {
                loader: "less-loader", //
                options: {
                  importLoaders: 1
                }
              }
            ]
          },
          {
            loader: require.resolve("file-loader"),
            exclude: [/\.(js|jsx|mjs)$/, /\.html$/, /\.json$/],
            options: {
              name: "static/media/[name].[hash:8].[ext]",
              esModule: false
            }
          }
        ]
      }
    ]
  },
  plugins: [
    // new BundleAnalyzerPlugin(),
    // new webpack.optimize.UglifyJsPlugin({
    //   beautify: false,
    //   comments: false,
    //   compress: {
    //     warnings: false,
    //     comparisons: false,
    //     drop_console: false,
    //     collapse_vars: true,
    //     reduce_vars: true
    //   },
    //   mangle: {
    //     safari10: true
    //   },
    //   output: {
    //     comments: false,
    //     // Turned on because emoji and regex is not minified properly using default
    //     // https://github.com/facebookincubator/create-react-app/issues/2488
    //     ascii_only: true
    //   },
    //   sourceMap: shouldUseSourceMap
    // }),
    new ManifestPlugin({
      fileName: "asset-manifest.json"
    }),
    new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/)
  ],
  // optimization: {
  //   minimize: true,
  //   minimizer: [
  //     new TerserPlugin({
  //       terserOptions: {
  //         output: {
  //           comments: false,
  //           safari10: true
  //         }
  //       },
  //       extractComments: false
  //     })
  //   ]
  // },
  // Some libraries import Node modules but don't use them in the browser.
  // Tell Webpack to provide empty mocks for them so importing them works.
  node: {
    dgram: "empty",
    fs: "empty",
    net: "empty",
    tls: "empty",
    child_process: "empty"
  },
  externals: {
    react: "react",
    redux: "redux",
    antd: "antd",
    mathjs: "mathjs",
    lodash: "lodash",
    immutable: "immutable",
    "rc-color-picker": "rc-color-picker"
  }
}
